/*
 * $Id: pam_auth_srv.c,v 1.5 2006/06/29 01:17:25 edson Exp $
 * pam_auth_srv
 *      Creates blocked user in passwd to allow bash to run with users
 *      with users authenticated in remote servers.
 *
 * See end for Copyright information */

#define PAM_SM_AUTH

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/resource.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <syslog.h>
#include <stdarg.h>
#include <utmp.h>
#include <time.h>
#include <netinet/in.h>
#include <netdb.h>
#include <fcntl.h>
#include <limits.h>
#include <errno.h>
#include <pwd.h>
#include <grp.h>

#ifdef sun
#include <security/pam_appl.h>
#endif
#include <security/pam_modules.h>

#include <security/pam_appdata.h>

#define PAM_DEBUG_ARG      1
#define PAM_FORCE_ARG      2

/* internal data */
static char *pam_module_name = "pam_auth_srv";

/* we need to save these from open_session to close_session, since
 * when close_session will be called we won't be root anymore and
 * won't be able to access again the radius server configuration file
 * -- cristiang */

static struct pam_appl_data *get_cyc_appl(pam_handle_t *pamh)
{
  struct pam_conv *conv;
  struct pam_appl_data *pappl;

  if (pam_get_item(pamh, PAM_CONV, (const void **)(void *) &conv) != PAM_SUCCESS) { //[GY] 2005/Sep/19  clean warning
    return(NULL);
  }

  pappl = (struct pam_appl_data *) conv->appdata_ptr;

  if (!pappl || pappl->id != 0x1234fedc) {
    return(NULL);
  }

  return(pappl);
}

/* logging */
static void _pam_log(int err, char *format, ...)
{
    va_list args;
    char buffer[1024];

    va_start(args, format);
    vsprintf(buffer, format, args);
    /* don't do openlog or closelog, but put our name in to be friendly */
    syslog(err, "%s: %s", pam_module_name, buffer);
    va_end(args);
}

/* argument parsing */
static int _pam_parse(int argc, const char **argv)
{
  int ctrl=0;

  /*
   *  If either is not there, then we can't parse anything.
   */
  if ((argc == 0) || (argv == NULL)) {
    return ctrl;
  }
  
  /* step through arguments */
  for (ctrl=0; argc-- > 0; ++argv) {
    
    /* generic options */
    if (!strcmp(*argv, "debug")) {
      ctrl |= PAM_DEBUG_ARG;
    } else if (!strcmp(*argv, "force")) {
      ctrl |= PAM_FORCE_ARG;
    } else {
      _pam_log(LOG_WARNING, "unrecognized option '%s'", *argv);
    }
  }
  
  return ctrl;
}

#define USR_MIN 500

int get_user_id(FILE *fp)
{
  int uid = USR_MIN, aux;
  char buf[1024];
  char *p, *x;

  /* stuff to do:  
   * make sure login isn't taken;
   * find free uid
  */
  while(fgets(buf, sizeof(buf), fp)) {
    if (buf[0] == '#' || buf[0] == '\n' ||
    	buf[0] == ' ' || buf[0] == '\t') continue;

	if ((p = strchr(buf, ':')) == NULL) continue;

	if ((p = strchr(p+1, ':')) == NULL) continue;

	if ((x = strchr(++p, ':')) == NULL) continue;

    *x = 0;
	aux = atoi(p);

    if (uid <= aux) uid = aux;
  }

  return uid+1;
}

static void adduser(char *login)
{
  FILE *fp;
  struct passwd pw;
  struct group *pg;
  char pw_dir[40] = "/var/home/";
  char syscmd[60] = "mkdir ";

  if ((fp = fopen("/etc/passwd", "r")) == NULL) {
    return;
  }
 
//  pg = getgrnam("nobody");
  pg = getgrnam("pam");

  strcat(pw_dir, login);

  /* create a passwd struct */
  pw.pw_name = login;
  pw.pw_passwd = "!";
  pw.pw_uid = get_user_id(fp);
  pw.pw_gid = pg ? pg->gr_gid : 99;
  pw.pw_gecos = "AuthSrv User,,,";
  pw.pw_dir = pw_dir;
  pw.pw_shell = "/bin/sh";

  fclose(fp);
  if ((fp = fopen("/etc/passwd", "a")) == NULL) {
    return;
  }
  fseek(fp, 0, SEEK_END);

  fprintf (fp, "%s:%s:%u:%u:%s:%s:%s\n",
    pw.pw_name,
    pw.pw_passwd,
    pw.pw_uid,
    pw.pw_gid,
    pw.pw_gecos,
    pw.pw_dir,
    pw.pw_shell);

  fclose(fp);

  strcat(syscmd, pw_dir);
  system(syscmd);
  sprintf(syscmd, "chown %s:pam %s", login, pw_dir);
  system(syscmd);
  sprintf(syscmd, "chmod 700 %s", pw_dir);
  system(syscmd);

}

//[GY]2006/Jan,27  Group_Auth_Box
static void setadmingid(char *user, struct passwd *pwd, int gid)
{
        struct stat st;
        FILE *fp;
        unsigned char *buf, *t, *s;
        int size, len;
        char *usr;

        if( stat("/etc/passwd", &st) ) return;
        size = st.st_size + 1;
        if( !(buf = (unsigned char *)malloc(size)) ) {
           return;
        }
        if( !(fp = fopen("/etc/passwd", "r")) ) {
           free(buf);
           return;
        }
        fread(buf, 1, st.st_size + 1, fp);
        *(buf+st.st_size) = 0;
        fclose(fp);
        s = buf;
        while( *s ) {
           t = s;
           while( *s && *s != ':' ) s++;
           len = (int) (s - t);
           usr = strndup(t, len);
           if( !strcmp(usr, user) ) {
              while( *s && *s != '\n' ) s++;
              if( *s == 0 || *s++ == 0 ) {
                 *t = 0;
                 break;
              }
              while( *s ) {
                 *t = *s;
                 s++;
                 t++;
              }
              *t = 0;
              break;
           }
           while( *s && *s != '\n' ) s++;
           s++;
        }
        if( (fp = fopen("/etc/passwd", "w")) ) {
           fwrite(buf, strlen(buf), 1, fp);
           pwd->pw_gid = gid;
           fprintf(fp, "%s:%s:%u:%u:%s:%s:%s\n", pwd->pw_name, pwd->pw_passwd, pwd->pw_uid, pwd->pw_gid, pwd->pw_gecos, pwd->pw_dir, pwd->pw_shell);
           fclose(fp);
        }
        free(buf);
}

#define DPRINT if (ctrl & PAM_DEBUG_ARG) _pam_log

PAM_EXTERN int 
pam_sm_authenticate(pam_handle_t *pamh,int flags,int argc,const char **argv)
{
  char *user, *prompt;
  int ctrl, retval;
  struct pam_appl_data *pappl;
  struct passwd *pwd;

  ctrl = _pam_parse(argc, argv);
  DPRINT(LOG_DEBUG, "start");

  pappl = get_cyc_appl(pamh);

  if (pappl && !pappl->flag && !(ctrl & PAM_FORCE_ARG)) {
	// client is PortSlave (pappl is not null) and the protocol is not login
    DPRINT(LOG_DEBUG, "client is portslave and protocol is not login");
    return(PAM_SUCCESS);
  }

  /* grab the user name */
  if (pappl) {
	retval = pam_get_item(pamh, PAM_USER, (const void **)(void *) &user); //[GY] 2005/Sep/19  clean warning
  } else {
	pam_get_item(pamh,PAM_USER_PROMPT,(const void **)(void *)&prompt); //[GY] 2005/Sep/19  clean warning
	if (prompt == NULL) prompt = "Who are you? ";
	retval = pam_get_user(pamh, (const char **)&user, prompt);
  }

  if (retval != PAM_SUCCESS || user == NULL || *user == '\0') {
	DPRINT(LOG_DEBUG, "User name not available");
	return(PAM_IGNORE);
  }
  
  /* Get the password entry */
  pwd = getpwnam(user);
  if (pwd != NULL) {
    DPRINT(LOG_DEBUG, "user %s already in the local database", user);
    return(PAM_SUCCESS);
  }

  DPRINT(LOG_DEBUG, "add user %s", user);
  adduser(user);

  return(PAM_SUCCESS);
}

/*
 * Return a value matching the return value of pam_sm_authenticate, for
 * greatest compatibility. 
 * (Always returning PAM_SUCCESS breaks other authentication modules;
 * always returning PAM_IGNORE breaks PAM when we're the only module.)
 */
PAM_EXTERN int
pam_sm_setcred(pam_handle_t *pamh,int flags,int argc,const char **argv)
{
  return(PAM_SUCCESS);
}

//[GY]2006/Jan,27  Group_Auth_Box
PAM_EXTERN int
pam_sm_acct_mgmt(pam_handle_t *pamh, int flags, int argc, const char **argv)
{

   char *user, *aux;
   int ctrl, retval, gid_adm, gid_pam;
   struct pam_appl_data *pappl;
   struct passwd *pwd;
   struct group *pg;

   ctrl = _pam_parse(argc, argv);
   DPRINT(LOG_DEBUG, "pam_sm_acct_mgmt start");

   pappl = get_cyc_appl(pamh);

   /* grab the user name */
   if( pappl && pappl->port == 0 ) {
      retval = pam_get_item(pamh, PAM_USER, (const void **)(void *) &user);
      if( retval != PAM_SUCCESS || user == NULL || *user == '\0' ) {
         DPRINT(LOG_DEBUG, "unable to obtain username");
         return PAM_USER_UNKNOWN;
      }

      /* Get the password entry */
      pwd = getpwnam(user);
      if( pwd == NULL ) {
         DPRINT(LOG_DEBUG, "user %s isn't in the local database", user);
         return PAM_USER_UNKNOWN;
      }
      pg = getgrnam("admin");
      gid_adm = pg ? pg->gr_gid : 99; 

      pg = getgrnam("pam");
      gid_pam = pg ? pg->gr_gid : 99; 

      if( pappl->group_name ) {
         if( (aux = strstr(pappl->group_name, "admin")) ) {
            if( pwd->pw_gid == gid_pam ) {
            DPRINT(LOG_DEBUG, "set admin (or 99) group_id to the user %s", user);
            setadmingid(user, pwd, gid_adm);
            }
         }
         else {
            if( pwd->pw_gid == gid_adm && !strcmp(pwd->pw_passwd, "!") ) {
               DPRINT(LOG_DEBUG, "set pam (or 99) group_id to the user %s", user);
               setadmingid(user, pwd, gid_pam);
            }
         }
      }
      else {
         DPRINT(LOG_DEBUG, "attribute group_name isn't defined for user %s", user);
         if( pwd->pw_gid == gid_adm && !strcmp(pwd->pw_passwd, "!") ) {
            DPRINT(LOG_DEBUG, "set pam (or 99) group_id to the user %s", user);
            setadmingid(user, pwd, gid_pam);
         }
      }
   }
   return PAM_SUCCESS;
}

PAM_EXTERN int
pam_sm_open_session(pam_handle_t *pamh,int flags,int argc,const char **argv)
{
  return(PAM_SUCCESS);
}

PAM_EXTERN int
pam_sm_close_session(pam_handle_t *pamh,int flags,int argc,const char **argv)
{
  return(PAM_SUCCESS);
}

PAM_EXTERN int
pam_sm_chauthtok(pam_handle_t *pamh,int flags,int argc,const char **argv)
{
  return(PAM_SUCCESS);
}

#ifdef PAM_STATIC

/* static module data */

struct pam_module _pam_auth_srv_modstruct = {
  "pam_auth_srv",
  pam_sm_authenticate,
  pam_sm_setcred,
  pam_sm_acct_mgmt,   //[GY]2006/Jan,26  Group_Auth_Box
  pam_sm_open_session,
  pam_sm_close_session,
  pam_sm_chauthtok,
};
#endif

