#ifndef __ADSAP2_H__
#define __ADSAP2_H__

#include "types.h"

typedef struct sockaddr_in sockaddr_in;

#define SOH				0x1		/* Start of Header		*/
#define SSL_CONTEXT_ANON		 1	/*init_ssl_context - set CTX for ADH	*/
#define SSL_CONTEXT_CERT		 2	/*init_ssl_context - set CTX for X.509	*/
#define SSL_CONTEXT_ANON_OR_CERT 3	/*init_ssl_context - set CTX for either */

#define SSL_SERVER		TRUE
#define SSL_CLIENT		FALSE

#define ADSAP2_MESSAGE_SIZE			4096

#define ADSAP2_MODE_TRUSTALL		0
#define ADSAP2_MODE_UNASSIGNED		1
#define ADSAP2_MODE_ASSIGNED		2

#define ADSAP2_CERT_DATA			0
#define ADSAP2_KEY_DATA				1

#define ADSAP2_MAX_PREAUTH_PORT		4

#define ADSAP2_USERNAME_STR_LEN     32
#define ADSAP2_PASSWORD_STR_LEN     33
#define ADSAP2_USER_ROLE_STR_LEN    32
#define ADSAP2_RIP_ID_STR_LEN       32
#define ADSAP2_SESSION_MODE_STR_LEN 32
#define ADSAP2_PREAUTH_TYPE_STR_LEN 32

#define ADSAP2_NOT_SHAREABLE_BIT    0x01
#define ADSAP2_NOT_STEALTHABLE_BIT  0x02

typedef enum 
{
  ADSAP2_NORMAL = 0,
  ADSAP2_EXCLUSIVE,
  ADSAP2_PREEMPT, 
  ADSAP2_PREEMPT_EXCLUSIVE,  
  ADSAP2_SHARE, 
  ADSAP2_SHARE_FORCE, 
  ADSAP2_SHARE_PASSIVE, 
  ADSAP2_SCAN,
  ADSAP2_STEALTH,
  ADSAP2_SHARE_PASSIVE_FORCE, 
  ADSAP2_MODE_MAX

} ADSAP2_MODE;

#define INVALID_USER			0
#define USER_LEVEL				1
#define USER_ADMIN_LEVEL		2
#define APPLIANCE_ADMIN_LEVEL	3

typedef enum
{
  ADSAP2_USER_ROLE_NONE  = INVALID_USER,
  ADSAP2_USER            = USER_LEVEL,
  ADSAP2_USER_ADMIN      = USER_ADMIN_LEVEL,
  ADSAP2_APPLIANCE_ADMIN = APPLIANCE_ADMIN_LEVEL,
  ADSAP2_USER_ROLE_MAX
  
} ADSAP2_USER_ROLE;

typedef enum
{
  ADSAP2_PREAUTH_TYPE_VIDEO  = 0,
  ADSAP2_PREAUTH_TYPE_VIRTUAL_MEDIA,
  ADSAP2_PREAUTH_TYPE_MAX
} ADSAP2_PREAUTH_TYPE;

// Information sent to ADSAP2 server to request authorization
// to access a RIP, port or socket
typedef struct 
{
  char username[ADSAP2_USERNAME_STR_LEN];
  char password[ADSAP2_PASSWORD_STR_LEN];
  unsigned char *p_Blob;
  int  Blob_len;
  ADSAP2_MODE mode;
  BYTE ripID[ADSAP2_RIP_ID_STR_LEN];
  BYTE cascadeChan;
  BYTE port;
  BYTE spcDevice;
  BYTE spcSocket;
  BYTE testLocalKey;
} ADSAP2_AUTHORIZE_REQ;

typedef struct
{
	char	RIP_ID  [ADSAP2_RIP_ID_STR_LEN];
	char	command [ADSAP2_RIP_ID_STR_LEN];
	BYTE	role;
	BYTE	mode;
	BYTE	preempt;
	BYTE	port;
	BYTE	type;
	BYTE	logdla;
} ADSAP2_AUTHORIZE_REP;

typedef struct adsap2_preauth_cert
{
	unsigned long	der_cert_len;
	unsigned char	*der_cert_data;

	unsigned long	der_key_len;
	unsigned char	*der_key_data;

	unsigned long	cert_preauth_time;
	unsigned long	cert_valid_duration;

	char			username[ADSAP2_USERNAME_STR_LEN];
	char			role[ADSAP2_USER_ROLE_STR_LEN];
	BYTE			preempt;
	BYTE			req_port;
	BYTE			logdla;
  ADSAP2_MODE     mode;
  // Added for DSRs
  char      RIP_ID[ADSAP2_RIP_ID_STR_LEN];
  char      command[ADSAP2_RIP_ID_STR_LEN];
  BYTE      channel;
  BYTE      type;
} ADSAP2_PREAUTH_CERT, *P_ADSAP2_PREAUTH_CERT;


// Structure to hold the data extracted from the certificate extensions
typedef struct adsap2_cert_ext_data
{
	char			  username[ADSAP2_USERNAME_STR_LEN];
	char			  password[ADSAP2_PASSWORD_STR_LEN];
	char			  role    [ADSAP2_USER_ROLE_STR_LEN];
  char        RIP_ID  [ADSAP2_RIP_ID_STR_LEN];
  char        command  [ADSAP2_RIP_ID_STR_LEN];
  BYTE        channel;
  char        mode    [ADSAP2_SESSION_MODE_STR_LEN];
	BYTE	  		preempt;
	BYTE	  		port;
	char			  type    [ADSAP2_PREAUTH_TYPE_STR_LEN];
  int         expiration;
	BYTE	  		logdla;
} ADSAP2_CERT_EXT_DATA, *P_ADSAP2_CERT_EXT_DATA;

typedef struct ds_verify_data
{
	int sock;
	int adsap2_mode;
	void *p_ssl;
}	VERIFY_DS_DATA, *P_VERIFY_DS_DATA;

/*///////////////////////////////////////////////////////////////////////////
// FIELD TYPES
*/
typedef struct ADSAP2Field
{
	BYTE	byFieldID;
	WORD	wFieldLen;
} __attribute__ ((packed)) ADSAP2FIELD, *PADSAP2FIELD;

//************************************************************************************

typedef struct fieldADSAP2Cert
{
	ADSAP2FIELD	ADSAP2FieldHdr;
	BYTE		byCert;
} __attribute__ ((packed)) F_ADSAP2_CERT, *PF_ADSAP2_CERT;

typedef struct fieldADSAP2Username
{
	ADSAP2FIELD	ADSAP2FieldHdr;
	BYTE		byUsername;
} __attribute__ ((packed)) F_ADSAP2_USERNAME, *PF_ADSAP2_USERNAME;

typedef struct fieldADSAP2Password
{
	ADSAP2FIELD	ADSAP2FieldHdr;
	BYTE		byPassword;
} __attribute__ ((packed)) F_ADSAP2_PASSWORD, *PF_ADSAP2_PASSWORD;

typedef struct fieldADSAP2Key
{
	ADSAP2FIELD	ADSAP2FieldHdr;
	BYTE		byKey;
} __attribute__ ((packed)) F_ADSAP2_KEY, *PF_ADSAP2_KEY;

typedef struct fieldADSAP2Mode
{
	ADSAP2FIELD	ADSAP2FieldHdr;
	BYTE		byMode;
} __attribute__ ((packed)) F_ADSAP2_MODE, *PF_ADSAP2_MODE;

typedef struct fieldADSAP2ID
{
	ADSAP2FIELD	ADSAP2FieldHdr;
	BYTE		byID;
} __attribute__ ((packed)) F_ADSAP2_ID, *PF_ADSAP2_ID;

typedef struct fieldADSAP2Role
{
	ADSAP2FIELD	ADSAP2FieldHdr;
	BYTE		byRole;
} __attribute__ ((packed)) F_ADSAP2_ROLE, *PF_ADSAP2_ROLE;

typedef struct fieldADSAP2Preempt
{
	ADSAP2FIELD	ADSAP2FieldHdr;
	BYTE		byPreempt;
} __attribute__ ((packed)) F_ADSAP2_PREEMPT, *PF_ADSAP2_PREEMPT;

typedef struct fieldADSAP2State
{
	ADSAP2FIELD	ADSAP2FieldHdr;
	BYTE		byState;
} __attribute__ ((packed)) F_ADSAP2_STATE, *PF_ADSAP2_STATE;

typedef struct fieldADSAP2Status
{
	ADSAP2FIELD	ADSAP2FieldHdr;
	WORD		wStatus;
} __attribute__ ((packed)) F_ADSAP2_STATUS, *PF_ADSAP2_STATUS;

//************************************************************************************

enum enum_ADSAPStoreRequestFieldType
{
	ADSAP2_STORE_REQ_DS_CERT	= 0x01,
	ADSAP2_STORE_REQ_AP_CERT	= 0x02,
	ADSAP2_STORE_REQ_AP_KEY		= 0x03
};

enum enum_ADSAP2StoreReplyFieldType
{
	ADSAP2_STORE_REP_STATUS		= 0x01
};

//************************************************************************************

enum enum_ADSAP2ClearReplyFieldType
{
	ADSAP2_CLEAR_REP_STATUS		= 0x01
};

//************************************************************************************

enum enum_ADSAP2PreAuthRequestFieldType
{
	ADSAP2_PREAUTH_REQ_CERT		= 0x01,
};

enum enum_ADSAP2PreAuthReplyFieldType
{
	ADSAP2_PREAUTH_REP_STATUS	= 0x01,
	ADSAP2_PREAUTH_REP_USERNAME	= 0x02,
	ADSAP2_PREAUTH_REP_PREEMPT	= 0x03,
	ADSAP2_PREAUTH_REP_STATE	= 0x04
};

//************************************************************************************

enum enum_ADSAP2AuthRequestFieldType
{
	ADSAP2_AUTH_REQ_CERT		= 0x01,
	ADSAP2_AUTH_REQ_USERNAME	= 0x02,
	ADSAP2_AUTH_REQ_PASSWORD	= 0x03,
	ADSAP2_AUTH_REQ_KEY			= 0x04,
	ADSAP2_AUTH_REQ_MODE		= 0x05,
	ADSAP2_AUTH_REQ_ID			= 0x80
};

enum enum_ADSAP2AuthReplyFieldType
{
	ADSAP2_AUTH_REP_STATUS		= 0x01,
	ADSAP2_AUTH_REP_PREEMPT		= 0x02,
	ADSAP2_AUTH_REP_ROLE		= 0x03
};

//************************************************************************************

enum enum_ADSAP2StatusCodes
{
	ADSAP2_NOERROR				= 0x0000,
	ADSAP2_ERROR				= 0x0100,
	ADSAP2_RESOURCE_BUSY		= 0x0101,
	ADSAP2_RESOURCE_BLOCKED		= 0x0102,
	ADSAP2_UNAUTHORIZED			= 0x0103,
	ADSAP2_INTRUST_ALL			= 0x0104,
	ADSAP2_RESOURCE_UNAVAILABLE	= 0x0105,
	ADSAP2_TOO_MANY_PENDING		= 0x0106,
	ADSAP2_RESOURCE_UPGRADING   = 0x0107,
	ADSAP2_TOO_MANY_SESSIONS    = 0x0108
};

//************************************************************************************

/* REQUEST Messages */
enum enum_ADSAPRequestMessageType
{
	ADSAP2_MSG_STORE_REQ		= 0x01,
	ADSAP2_MSG_CLEAR_REQ		= 0x02,
	ADSAP2_MSG_PREAUTH_REQ		= 0x03,
	ADSAP2_MSG_AUTH_REQ			= 0x04
};

#define CPS_ADSAP2_SIGNATURE_SIZE	6			/* Length of CPS_ADSDAP2_SIGNATURE (without NULL) */
#define CPS_ADSAP2_SIGNATURE		"ADSAP2"	/* Actual Signature string */

/*///////////////////////////////////////////////////////////////////////////
// Store Credentials Command
*/

typedef struct tagADSAP2StoreRequest
{
	PF_ADSAP2_CERT		pDS_Cert;
	PF_ADSAP2_CERT		pAP_Cert;
	PF_ADSAP2_KEY		pAP_Priv_Key;
} __attribute__ ((packed)) ADSAP2_STORE_REQ, *P_ADSAP2_STORE_REQ;

typedef struct tagADSAP2StoreReply
{
	PF_ADSAP2_STATUS	pStatus;
} __attribute__ ((packed)) ADSAP2_STORE_REPLY, *P_ADSAP2_STORE_REPLY;

/*///////////////////////////////////////////////////////////////////////////
// Clear Credentials Command
*/

typedef struct tagADSAP2ClearReply
{
	PF_ADSAP2_STATUS	pStatus;
} __attribute__ ((packed)) ADSAP2_CLEAR_REPLY, *P_ADSAP2_CLEAR_REPLY;

/*///////////////////////////////////////////////////////////////////////////
// Authorize Command
*/
typedef struct tagADSAP2AuthRequest
{
	PF_ADSAP2_CERT		pCert;
	PF_ADSAP2_USERNAME	pUsername;
	PF_ADSAP2_PASSWORD	pPassword;
	PF_ADSAP2_KEY		pKey;
	PF_ADSAP2_MODE		pMode;
	PF_ADSAP2_ID		pID;
} __attribute__ ((packed)) ADSAP2_AUTH_REQ, *P_ADSAP2_AUTH_REQ;

typedef struct tagADSAP2AuthReply
{
	PF_ADSAP2_STATUS	pStatus;
	PF_ADSAP2_PREEMPT	pPreempt;
	PF_ADSAP2_ROLE		pRole;
} __attribute__ ((packed)) ADSAP2_AUTH_REPLY, *P_ADSAP2_AUTH_REPLY;

/*///////////////////////////////////////////////////////////////////////////
// Pre-Authorize Command
*/
typedef struct tagADSAP2PreAuthRequest
{
	PF_ADSAP2_CERT		pCert;
} __attribute__ ((packed)) ADSAP2_PREAUTH_REQ, *P_ADSAP2_PREAUTH_REQ;

typedef struct tagADSAP2PreAuthReply
{
	PF_ADSAP2_STATUS	pStatus;
	PF_ADSAP2_USERNAME	pUsername;
	PF_ADSAP2_PREEMPT	pPreempt;
	PF_ADSAP2_STATE		pState;
} __attribute__ ((packed)) ADSAP2_PREAUTH_REPLY, *P_ADSAP2_PREAUTH_REPLY;

/*///////////////////////////////////////////////////////////////////////////
// Header for ADSAP2 Message
*/

typedef struct tagADSAP2Header
{
	BYTE				bySOH;									/* SOH */
	BYTE				szUniqueID[CPS_ADSAP2_SIGNATURE_SIZE];	/* Unique ID (Always ASCII representation of "ADSAP") */
	BYTE				byMsgType;								/* enum AIDPMessageType */
	DWORD				dwMsgLen;								/* Length  */
} __attribute__ ((packed)) ADSAP2_HEADER, *P_ADSAP2_HEADER;

/*///////////////////////////////////////////////////////////////////////////
// Trailer for ADSAP2 Message
*/

typedef struct tagADSAP2RecTerm
{
	BYTE				byRecTerm;								/* Record terminator */
} __attribute__ ((packed)) ADSAP2_REC_TERM, *P_ADSAP2_REC_TERM;

typedef struct tagADSAPTrailer
{
	BYTE				byEOM;									/* End of Message */
} __attribute__ ((packed)) ADSAP2_TRAILER, *P_ADSAP2_TRAILER;

#define ADSAP2_REC_TERM_VALUE	0xFF
#define ADSAP2_TRAILER_VALUE	0x0D

//*****************************************************************************
typedef struct ap_credential
{
	unsigned long	der_priv_key_len;
	unsigned char	*der_priv_key_data;
	unsigned long	der_cert_len;
	unsigned char	*der_cert_data;
} __attribute__ ((packed)) AP_CREDENTIAL, *P_AP_CREDENTIAL;

typedef struct ds_credential
{
	unsigned long	der_cert_len;
	unsigned char	*der_cert_data;
} __attribute__ ((packed)) DS_CREDENTIAL, *P_DS_CREDENTIAL;

#define ADSAP2_DSAUTHENTICATION_SUCCESS      0
#define ADSAP2_DSAUTHENTICATION_FAILED       1
#define ADSAP2_DSAUTHENTICATION_UNAVAILABLE  2

//*****************************************************************************

int ADSAP2_authorize(	ADSAP2_AUTHORIZE_REQ * authorizeRequest, ADSAP2_AUTHORIZE_REP *authorizeReply);

void					*accept_ssl					(int sock, int adsap2_mode);

void					processADSAP2Connect		(void *p_ssl, int *p_adsap2_mode);

void					process_store_credentials	(P_ADSAP2_HEADER p_header, int *p_adsap2_mode);
void					process_clear_credentials	(P_ADSAP2_HEADER p_header, int *p_adsap2_mode);

BOOLEAN       ADSAP2_credentials_present(void);
void                    resetSecurityKeys();
BOOLEAN				test_ds_credential			(unsigned char *p_cert_data, unsigned long cert_len, unsigned char *p_test_data, unsigned long test_len);
int						verify_ds_credentials		(void *p_store, void *data);
int           verify_preauth_credentials(void *p_store, void *data);
BOOLEAN       init_adsap2_credentials(void *p_ssl);

int						get_ADSAP2_mode				(void);
void * adsap2ConnectServer( void * threadarg );
unsigned long authenticate_ds(ADSAP2_AUTHORIZE_REQ * pAuthReq, ADSAP2_AUTHORIZE_REP *authorizeReply);

//*****************************************************************************
//	Functions to modify for portability

#define ADSAP2_printf		printf

void			ADSAP2_initialize_globals	(void);


#endif	/* __ADSAP_H__ */
