//---------------------------------------------------------------------
// Copyright 2003 Avoocent Corporation
//---------------------------------------------------------------------
// FILE         : avoprint.h
//
// DESCRIPTION  : Functions to perform debug printing
//
//---------------------------------------------------------------------
// AUTHOR       : David Stafford
// DATE         : Dec 12-2003
//
//---------------------------------------------------------------------
// HISTORY
//
// DATE         PROGRAMMER        MODIFICATIONS
// Dec 12-2003  David Stafford    Initial version
//
//---------------------------------------------------------------------

#ifndef __AVOPRINT_H
#define __AVOPRINT_H

/*======================================================================== 
 * Debug significance levels that are used with avoPrintf() levelAndMask param
 */

#define AVODBG_ERROR_LEVEL    1 /* Error display */
#define AVODBG_PROGRESS_LEVEL 2 /* Progress display */
#define AVODBG_INFO_LEVEL     3 /* Information display */

/*======================================================================== 
 * Debug mask controls that are used with avoPrintf() levelAndMask param
 */

#define _BIT_( n )        (0x01 << (n))

/* severity level controls */

#define AVODBG_FAILURE      _BIT_(31)
#define AVODBG_WARNING      _BIT_(30)
#define AVODBG_INFO         _BIT_(29)
#define AVODBG_ADHOC        _BIT_(28)  /* for temporary / experimental use */

/* protocol debug controls */

#define AVODBG_BDTP         _BIT_(27)
#define AVODBG_AIDP         _BIT_(26)
#define AVODBG_ASMP         _BIT_(25)
#define AVODBG_AVSP         _BIT_(24)
#define AVODBG_ADSAP        _BIT_(23)
#define AVODBG_PPP          _BIT_(22)
#define AVODBG_SSL          _BIT_(21)

/* sub-system debug controls */

#define AVODBG_VIDFPGA      _BIT_(20)
#define AVODBG_VIDMGR       _BIT_(19)
#define AVODBG_ADC          _BIT_(18)
#define AVODBG_RIP_NAMES    _BIT_(17)
#define AVODBG_RIP_STATUS   _BIT_(16)
#define AVODBG_SPC          _BIT_(15)

/* functional debug controls */

#define AVODBG_KEYBOARD     _BIT_(14)
#define AVODBG_VIDEO        _BIT_(13)
#define AVODBG_MOUSE        _BIT_(12)
#define AVODBG_LOCAL        _BIT_(11)
#define AVODBG_REMOTE       _BIT_(10)
#define AVODBG_CHANSELECT   _BIT_( 9)
#define AVODBG_LOGIN        _BIT_( 8)
#define AVODBG_STATUS_POLL  _BIT_( 7) 
#define AVODBG_MIB          _BIT_( 6) 
#define AVODBG_VIRT_MEDIA   _BIT_( 5)
#define AVODBG_ALWAYS      _BIT_( 4)

/* for future use */

#define AVODBG_SPARE_3      _BIT_( 3)
#define AVODBG_SPARE_2      _BIT_( 2)

#define AVODBG_RESERVED_FOR_DBG_LEVEL (_BIT_( 1) | _BIT_( 0))

//====================================================================
// Suggestions for use of debug control bits 

//  To make best use of these debug control bits it is suggested to use no
//  more than 2 or 3 at a time for a given type of debug message.  For
//  example, a message about mouse buffer overflows might be coded like
//  this...
//
//    #define MOUSE_WARNING     (AVODBG_WARNING | AVODBG_MOUSE)
//  
//    avoPrintf( MOUSE_WARNING, "The mouse buffer is overflowed!\n" );
//
//  To enable this message for console printing the global debug control mask must
//  include both AVODBG_WARNING and AVODBG_MOUSE. This is defined by the "avoPrintMask"
//  flash file or via test mode on the config port. 
//
//  You can also include a numeric debug level as well but this it is not required.
//
//    avoPrintf( MOUSE_WARNING | 2, "The mouse buffer is overflowed!\n" );
//
//  When you omit the debug level it is zero. In this case only the control bits 
//  will affect when this message is printed. 


#ifdef __cplusplus
extern "C" {
#endif 

//---------------------------------------------------------------------
// FUNCTION         : avoPrintf
//
// DESCRIPTION      : This function is used to print diagnostic information
//
// PARAMETERS       : levelAndMask - debug level and mask 
//                    fmt   - format control string like printf
//                    ...   - parameter list 
//
// RETURNS          : none
//
// NOTES            : 
//
//---------------------------------------------------------------------

void avoPrintf( unsigned long level, const char *fmt, ... );
void avoPrintfSetLevel(unsigned long level);


#ifdef __cplusplus
}
#endif 

#endif /* __AVOPRINT_H */

/* end */

