//------------------------------------------------------------------------
// Copyright 2002-2005, Avocent Corporation
//------------------------------------------------------------------------
// FILE        :  ds2mib.c
//
// DESCRIPTION :  This modules provides an interface between the DS appliance
//                code and the MIB structure.
//
//------------------------------------------------------------------------
// AUTHOR      :  Jamie Collins
//
// DATE        :  January 2002
//
//------------------------------------------------------------------------
// HISTORY
//
// DATE           PROGRAMMER              MODIFICATIONS
// January 2002  Jamie Collins    Initial version
//
//------------------------------------------------------------------------

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <arpa/inet.h>

#include "types.h"
#include "dstypes.h"
#include "ds2mib.h"
#include "utf8.h"
#include "eid_util.h"
#include "avoprint.h"
#include "adsap2.h"

#define MIB_WARNING     (AVODBG_WARNING | AVODBG_MIB)
#define MIB_FAILURE     (AVODBG_FAILURE | AVODBG_MIB)
#define MIB_INFO        (AVODBG_INFO    | AVODBG_MIB)
#define MIB_ADHOC       (AVODBG_ADHOC   | AVODBG_MIB)

static  dsAppliance g_dsAppliance = {0};

//---------------------------------------------------------------------
// FUNCTION         : DsmibGetAuthServerAddress
//
// DESCRIPTION      : This function gets an auth server IP address from the MIB.
//
// PARAMETERS       : inx - auth server table index [ 0..SNMP_AUTHSVR_ADDR_COUNT-1 ]
//                    ptr - pointer to IP_ADDR
//
// RETURNS          : void
//
// NOTES            :
//
//---------------------------------------------------------------------

void DsmibGetAuthServerAddress( unsigned inx, IP_ADDR * ptr )
{
  if( inx < DSMIB_AUTHSERVER_ADDRESS_COUNT )
  {
    ptr->s_addr =
      g_dsAppliance.dsAuthServerAddressEntry[ inx ].dsAuthServerAddressIpAddress;
  }
  else
  {
    avoPrintf( MIB_ADHOC, "DsmibGetAuthServerAddress %d > %d\n", inx, DSMIB_AUTHSERVER_ADDRESS_COUNT );
  }
}


//---------------------------------------------------------------------
// FUNCTION         : DsmibInitAuthServerAddress
//
// DESCRIPTION      : This function sets an auth server IP address in the MIB.
//
// PARAMETERS       : inx - manager table index [ 0..SNMP_AUTHSVR_ADDR_COUNT-1 ]
//                    ptr - pointer to IP_ADDR
//
// RETURNS          : void
//
// NOTES            :
//
//---------------------------------------------------------------------

void DsmibInitAuthServerAddress( unsigned inx, IP_ADDR * ptr )
{
  if( inx < DSMIB_AUTHSERVER_ADDRESS_COUNT )
  {
     g_dsAppliance.dsAuthServerAddressEntry[ inx ].dsAuthServerAddressIpAddress =
       ptr->s_addr;
    avoPrintf( MIB_ADHOC, "DsmibInitAuthServerAddress %d: %s\n", inx, inet_ntoa(*ptr));

  // does NOT update flash
  }
  else
  {
    avoPrintf( MIB_ADHOC, "DsmibInitAuthServerAddress %d > %d\n", inx, DSMIB_AUTHSERVER_ADDRESS_COUNT );
  }
}

//---------------------------------------------------------------------
// FUNCTION         : DsmibGetSecurityMode
//
// DESCRIPTION      :
//
//
// PARAMETERS       : none
//
// RETURNS          :
//
// NOTES            :
//
//---------------------------------------------------------------------
int DsmibGetSecurityMode( void )
{
  int retVal;

  retVal = g_dsAppliance.dsSecurityMode;

  avoPrintf(MIB_ADHOC, "DsmibGetSecurityMode: %s\n", ( ( g_dsAppliance.dsSecurityMode == DSMIB_SECURITY_MODE_TRUSTALL ) ? "TRUST-ALL" : "SECURE" ) );

  return( retVal );
}

//---------------------------------------------------------------------
// FUNCTION         : DsmibInitSecurityMode
//
// DESCRIPTION      :
//
//
// PARAMETERS       :
//
// RETURNS          : void
//
// NOTES            :
//
//---------------------------------------------------------------------
void DsmibInitSecurityMode( unsigned long mode )
{
  g_dsAppliance.dsSecurityMode = mode;
  avoPrintf(MIB_ADHOC, "DsmibInitSecurityMode: %s\n", ( ( g_dsAppliance.dsSecurityMode == DSMIB_SECURITY_MODE_TRUSTALL ) ? "TRUST-ALL" : "SECURE" ) );

}

//---------------------------------------------------------------------
// FUNCTION         : DsmibGetSecurityMode
//
// DESCRIPTION      :
//
//
// PARAMETERS       : none
//
// RETURNS          :
//
// NOTES            :
//
//---------------------------------------------------------------------
void DsmibSetSecurityMode( unsigned long mode )
{
  DsmibInitSecurityMode( mode );
}

/* end */

