//---------------------------------------------------------------------
// Copyright 2001, Avocent Corporation
//---------------------------------------------------------------------
// FILE         : eid_util.c
//
// DESCRIPTION  : Functions for manipulating EIDs
//
//---------------------------------------------------------------------
// AUTHOR       : David Stafford
// DATE         : Feb 28-2002
//
//---------------------------------------------------------------------

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "types.h"
#include "eid_util.h"

//---------------------------------------------------------------------
// FUNCTION         : eid2Str
//
// DESCRIPTION      : This function formats a 13 character NULL terminated 
//                    string with hexascii digits from an EID array. 
//
// PARAMETERS       : buffer - pointer of buffer to receive string
//                    eid    - pointer of byte array containing EID bytes
//
// RETURNS          : pointer to buffer containing string
//
// NOTES            : Formated as follows: 520255-123456
//
//---------------------------------------------------------------------

char * eid2Str( char * buffer, byte eid[ 6 ] ) 
{
  sprintf( buffer, "%02X%02X%02X-%02X%02X%02X", 
           eid[ 0 ], eid[ 1 ], eid[ 2 ], eid[ 3 ], 
           eid[ 4 ], eid[ 5 ]  );
  return buffer;
}

//---------------------------------------------------------------------
// FUNCTION         : str2Eid
//
// DESCRIPTION      : This function decodes an EID string as formted by 
//                    eid2Str() and fills a 6 byte EID array.
//
// PARAMETERS       : eid   - dst pointer of byte array to receive EID bytes
//                    str   - src pointer of hexAscii string
//
// RETURNS          : pointer of destination buffer 
//
// NOTES            : 
//
//---------------------------------------------------------------------

byte * str2Eid( byte eid[ 6 ], char * str )
{
  unsigned n;
  static char hexDigits[] = "0123456789ABCDEF";

  for ( n = 0; n < 6; ++n ) 
  {
    char *cp;
    unsigned c;
    
    if ( *str == '-' ) 
    {
      ++str;
    }
    c = *str++;
    c = toupper(c);
    cp = strchr( hexDigits, c );
    eid[ n ] = (( cp ) ? cp - hexDigits : 0) << 4;

    c = *str++;
    c = toupper(c);
    cp = strchr( hexDigits, c );
    eid[ n ] |= (( cp ) ? cp - hexDigits : 0);
  }
  return eid;
}

/* end */

