
#ifndef LIST_ITEM_H
#define LIST_ITEM_H
#include "types.h"

//**************************************************************************************

typedef struct list_item
{
	struct list_item	*p_prev;
	void				*p_data;
	struct list_item	*p_next;
} LIST_ITEM, *PLIST_ITEM;

typedef struct global_list
{
  pthread_mutex_t list_lock;
	//HANDLE		list_lock;				// Lock for list
	PLIST_ITEM	list_head;				// Pointer to the beginning of doubly linked list
	PLIST_ITEM	list_tail;				// Pointer to the end of doubly linked list

	BOOLEAN		(*compare_cb)();		// Compare function callback
	BOOLEAN		(*valid_cb)();			// Valid function callback
	BOOLEAN		(*free_cb)();			// Free function callback

	WORD		(*port_avail_cb)();		// Port available function callback
//	BOOLEAN		(*port_reserve_cb)();	// Port reserve function callback
//	BOOLEAN		(*port_release_cb)();	// Port release function callback
}	GLOBAL_LIST, *PGLOBAL_LIST;

//**************************************************************************************

PGLOBAL_LIST	new_list						(void);
PLIST_ITEM		new_list_item					(void *p_data);
void			free_list_item					(PGLOBAL_LIST p_list, PLIST_ITEM p_item, BOOLEAN list_locked);
WORD			avail_list_item					(PGLOBAL_LIST p_list, PLIST_ITEM p_item, BOOLEAN list_locked);

void			set_item_compare_callback		(PGLOBAL_LIST p_list, BOOLEAN (*cb)(void *, void *, int));
void			set_item_valid_callback			(PGLOBAL_LIST p_list, BOOLEAN (*cb)(void *));
void			set_item_free_callback			(PGLOBAL_LIST p_list, BOOLEAN (*cb)(void *));
void			set_item_avail_callback			(PGLOBAL_LIST p_list, WORD    (*cb)(void *));

void			add_list_item					(PGLOBAL_LIST p_list, PLIST_ITEM p_item, BOOLEAN list_locked);
PLIST_ITEM		del_list_item					(PGLOBAL_LIST p_list, PLIST_ITEM p_item, BOOLEAN list_locked);
PLIST_ITEM		find_list_item					(PGLOBAL_LIST p_list, PLIST_ITEM p_start, void *p_data, BOOLEAN forward, int data_type, BOOLEAN list_locked);

void			purge_list						(PGLOBAL_LIST p_list, BOOLEAN list_locked);
BOOLEAN			item_valid						(PGLOBAL_LIST p_list, PLIST_ITEM p_item, BOOLEAN list_locked);
void			lock_unlock_list				(PGLOBAL_LIST p_list, BOOLEAN lock);

#endif

