/*                                                                      */
/* SNMPUTIL.C - SNMP Utility Functions                                  */
/*                                                                      */
/* EBS - RTIP                                                           */
/*                                                                      */
/* Copyright Peter Van Oudenaren , 1993                                 */
/* All rights reserved.                                                 */
/* This code may not be redistributed in source or linkable object form */
/* without the consent of its author.                                   */
/*                                                                      */
/* Module description:                                                  */
/*    This module contains various utilities for SNMP.                  */

//#include "rtip.h"
#include "types.h"
#include "snmputil.h"

/* a1rwoid.c - a1readoid, a1writeoid   */
/*------------------------------------------------------------------------
 * a1readoid - convert an ASN.1 encoded object id into internal form
 *------------------------------------------------------------------------
 */
/* pack     = input  = ANS1 format                                     */
/* objid    = output = internal format                                 */
/* objidlen = input  = length of pack                                  */
/* one value in the ASN.1 data (x.y.z) is stored as follows in base 2: */
/*    1aaaabbb 1bccccdd 1ddeeeef 0fffgggg                              */
/* which represents the binary number aaaabbbbccccddddeeeeffffgggg     */
int a1readoid(PFBYTE pack, int objidlen, POID objid)
{
int   val;
PFBYTE  pp;
byte c;

    objid->len = 0;
    pp = pack;

    for (; pp < pack + objidlen; objid->len++)
    {
		if (objid->len >= SMAXOBJID)
		{
		/* This OID is too long, we must not continue or we will overwrite	*/
		/* the oid short list.												*/
			return(SERR_GEN_ERR);
		}

        /* if short form   */
        if (! (*pp & CHAR_HIBIT))
        {
            objid->id[objid->len] = *pp++;
            continue;
        }

        /* if long form, where bits 6 - 0 of each      */
        /* octet are used; (bit 7 == 0) ==> last octet */
        val = 0;
        do
        {
            c = *pp;
            val = (val << 7) | (int) (c & ~CHAR_HIBIT);
        } while (*pp++ & CHAR_HIBIT);   /* high bit set */
        objid->id[objid->len] = (word)val;
    }

    return SNMP_OK;
}

