#ifndef __SNMPUTIL_H
#define __SNMPUTIL_H

#define SNMP_OK 0
#define CHAR_HIBIT      0x80
#define SERR_GEN_ERR    5

typedef unsigned char *PFBYTE;
typedef char *PFCHAR;

#define SMAXOBJID   128                  /* max # of sub object ids    */
#define OBJSUBIDTYPE word               /* type of sub object ids     */

/* ********************************************************************    */
/* internal representation of an ASN.1 object id (a1readoid and a1writeoid */
/* convert from ASN.1 to internal and vice versa)                          */
struct oid
{
    OBJSUBIDTYPE id[SMAXOBJID]; /* array of sub-identifiers     */
    int          len;           /* length of this object id */
};

typedef struct oid *POID;

/* ********************************************************************   */
/* Structure that contains the value of an SNMP variable.                 */
struct snval
{
    byte           sv_type;      /* variable type                 */
    union                        /* value of var is one of these  */
    {
        long       sv_int;       /* variable is one of: integer,  */
                                 /* counter, gauge, timeticks        */
                                 /* NOTE: could be dword             */
        struct                   /* variable is a (octet) string  */
        {
            PFCHAR sv_str;       /* string's contents             */
            int    sv_len;       /* string's length               */
        } sv_str;
        struct oid sv_oid;       /* variable is an object id      */
        byte       sv_ipaddr[4]; /* variable is an IP address     */
    } sv_val;
};

int a1readoid(PFBYTE pack, int objidlen, POID objid);

#endif /* __SNMPUTIL_H */

/* end */

