#!/bin/sh

# parameters:
#
# ${1} - action/event
#
#  start: udhcpc was started no action required.
#
#  bound: A new IP was given by the sever. The script should configure
#         the interface, and set any other relavent parameters, such as,
#         default gateway, dns server, etc.
#
#  deconfig: the lease was lost. The script must put the interface
#            in an up, but deconfigured state, ie:
#            ifconfig ${interface} 0.0.0.0.
#
#  renew:  lease was renewed. The IP address will not change, however,
#          the other DHCP paramaters, such as the default gateway,
#          subnet mask, and dns server may have changed.
#
#  nak: This argument is used with udhcpc receives a NAK message. 
#
#  rt: called by cy_ras (no environment variables set)
#
#  Evironment variables used, set by udhcpc
#
#    interface   - The interface this was obtained on
#    ip          - The obtained IP
#    subnet      - The assigend subnet mask
#    router      - A list of routers
#    dns         - A list of DNS server
#    hostname    - The assigned hostname
#    domain      - The domain name of the network
#    broadcast   - The broadcast address for this network
#    message     - Reason for a DHCPNAK
#

DHCP=0
verbose=0

if [ "x${1}" = "xconsole" ] ; then
  shift 
  exec  > /dev/console 2>&1
fi

if [ "x${1}" = "xsilent" ] ; then
  shift 
  exec  > /dev/null 2>&1
fi

if [ "x${1}" = "xverbose" ] ; then
  shift 
  verbose=1
fi

ST_ROUTES=/etc/network/st_routes
FILE_SAVE="/etc/dhcpcd-eth0.save"
FILE_INFO="/tmp/dhcpcd-eth0.save"
FILE_LOCK="/tmp/dhcp_lock"

INTF=eth0

set_static_routes ()
{
  GATEWAY=
  if [ -n "${1}" ] ; then
	if [ "${1}" = "-f" ] ; then
      GATEWAY=$(grep ^GATEWAY ${2} | cut -f 2 -d =) 
    elif [ "${1}" = "-r" ] ; then
      GATEWAY=${2}
    fi
  fi
  if [ -n "${GATEWAY}" ] ; then
    route add default gw ${GATEWAY}
    grep -v default ${ST_ROUTES} > /tmp/st_routes
    /bin/sh /tmp/st_routes
    rm -f /tmp/st_routes
  else
    /bin/sh ${ST_ROUTES}
  fi
}

set_dns()
{
  echo "search" ${1} > /etc/resolv.conf
  tmp=$(echo ${2} | sed "s/,/ /g")
  for i in $tmp 
  do
    echo "nameserver" ${i} >> /etc/resolv.conf
  done
}

process_rt()
{
  ifconfig ${INTF} down
  ifconfig ${INTF} up
  if [ -f ${FILE_SAVE} ]; then
    set_static_routes -f ${FILE_SAVE}
    return
  fi
  set_static_routes
}

process_down()
{
  DHCP=$(teste -C dhcp_client)

  if [ ${DHCP} -eq 0 ] ; then
    return
  fi

  grep -q "DOWN=" ${FILE_SAVE} && return
  echo "DOWN=yes" >> ${FILE_SAVE}

  ifconfig ${INTF} down
  ifconfig ${INTF} 0.0.0.0 up

  if [ ${DHCP} -eq 2 ] ; then
    # use the DHCP information saved before
    if [ -f ${FILE_SAVE} ]; then
      # check if IP save is in use by another device
      IP_INFO=$(grep ^IPADDR ${FILE_SAVE} | cut -f 2 -d =) 
      IP_BRO=$(grep ^BROADCAST ${FILE_SAVE} | cut -f 2 -d =) 
      IP_MASK=$(grep ^NETMASK ${FILE_SAVE} | cut -f 2 -d =) 
      ifconfig ${INTF} ${IP_INFO} netmask ${IP_MASK} broadcast ${IP_BRO} up
      DOMAIN=$(grep -i "^domain" ${FILE_SAVE} | cut -f 2 -d =)
      DNS=$(grep -i "^dns" ${FILE_SAVE} | cut -f 2 -d =)
      set_dns ${DOMAIN} ${DNS}
      set_static_routes -f ${FILE_SAVE}
      /bin/signal_ras usr1 &
    fi
  fi
}

process_up()
{
#
# save the values given by the server
#
echo IPADDR=${ip}				>  ${FILE_INFO}
echo NETMASK=${subnet}			>> ${FILE_INFO}
echo BROADCAST=${broadcast}		>> ${FILE_INFO}
echo GATEWAY=${router}			>> ${FILE_INFO}
echo DOMAIN=${domain}			>> ${FILE_INFO}
echo DNS=${dns}					>> ${FILE_INFO}
echo INTERFACE=${interface}		>> ${FILE_INFO}
echo HOSTNAME=${hostname}		>> ${FILE_INFO}

cmp -s ${FILE_INFO} ${FILE_SAVE} && return

killall aidpd > /dev/null 2>&1

ifconfig ${interface} down
ifconfig ${interface} ${ip} netmask ${subnet} broadcast ${broadcast} up

set_static_routes -r ${router}
set_dns ${domain} ${dns}
/bin/signal_ras usr1 &

while true ; do
  if [ -f ${FILE_LOCK} ] ; then
    kill -0 $(cat ${FILE_LOCK})
    if [ $? -eq 0 ]; then
        sleep 1
    else
        break
    fi
  else
    break
  fi
done

echo $$ > ${FILE_LOCK}

cp ${FILE_INFO} ${FILE_SAVE}
/bin/updatefiles ${FILE_SAVE} 2>&1 > /dev/null
rm -f ${FILE_LOCK}
}

echo ${0}: $*

if [ ${verbose} -eq 1 ] ; then
  env
fi

case ${1} in
rt)
  process_rt
  ;;
start)
  ifconfig ${INTF} down
  ifconfig ${INTF} 0.0.0.0 up
  if [ -f ${FILE_SAVE} ] ; then
    mv ${FILE_SAVE} ${FILE_SAVE}.tmp
	sed "s/DOWN=/START=/" ${FILE_SAVE}.tmp > ${FILE_SAVE}
	grep -q "START=" ${FILE_SAVE} || echo "START=yes" >> ${FILE_SAVE}
  fi
  ;;
nak)
  echo "${0}: ${1} - message [${message}]"
  ;;
renew|bound)
  process_up
  ;;
deconfig|leasefail)
  process_down
  ;;
*)
  echo "${0} : error - invalid parameter '${1}'"
  ;;
esac

