/*
 *  8/00  -  CPM SPI works, so just use that
 *
 *  Copyright (c) 2000  Montavista Software, Inc <source@mvista.com>
 *  based on the following
 *
 */

/*-----------------------------------------------------------------------------
 *
 * Copyright (C) Cyclades Corporation, 1997 - 2000
 * All rights reserved.
 *
 * TSx100/NL1000 v1.00 for Linux
 *
 * This file :      cyc_rtc.c
 *
 * Description:     This file contains the routines to access external real time
 *                  clock via SPI protocol
 *
 * Complies with Cyclades SW coding Standard rev 1.2
 *-----------------------------------------------------------------------------
 */

 /*----------------------------------------------------------------------------
  * Change History
  *----------------------------------------------------------------------------
  * 11/8/01   Arnaldo Zimmermann     Initial implementation
  *----------------------------------------------------------------------------
  * Aug/09/02 Edson Seabra           Port to linux kernel
  *----------------------------------------------------------------------------
  */

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/signal.h>
#include <linux/errno.h>
#include <linux/mm.h>
#include <linux/poll.h>
#include <linux/miscdevice.h>
#include <linux/random.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/timex.h>
#include <linux/time.h>

#include <asm/uaccess.h>
#include <asm/mpc8xx.h>
#include <asm/system.h>
#include <asm/irq.h>
#include <asm/io.h>
#include <asm/prxk.h>

#ifdef CONFIG_SM100
#include <linux/mc146818rtc.h>
#include <linux/delay.h>
#include "commproc.h"
#else
#include <asm/delay.h>
#include <asm/commproc.h>
#include <asm/time.h>
#include <asm/todc.h>
#endif

#define RTC_GET_BCD_SEC(x)      ((x & 0x0f) + (((x & 0x70) >> 4) * 10))
#define RTC_GET_BCD_MIN(x)      ((x & 0x0f) + (((x & 0x70) >> 4) * 10))
#define RTC_GET_BCD_HR(x)       ((x & 0x0f) + (((x & 0x30) >> 4) * 10))
#define RTC_GET_BCD_MDAY(x)     ((x & 0x0f) + (((x & 0x30) >> 4) * 10))
#define RTC_GET_BCD_MONTH(x)    ((x & 0x0f) + (((x & 0x10) >> 4) * 10))
#define RTC_GET_BCD_YEAR(x)     ((x & 0x0f) + (((x & 0xf0) >> 4) * 10))

#define RTC_PUT_BCD_SEC(x,y)    ((((x % 10) + ((x / 10) << 4)) & 0x7f) | (y & ~0x7f))
#define RTC_PUT_BCD_MIN(x,y)    ((((x % 10) + ((x / 10) << 4)) & 0x7f) | (y & ~0x7f))
#define RTC_PUT_BCD_HR(x,y)     ((((x % 10) + ((x / 10) << 4)) & 0x3f) | (y & ~0x3f))
#define RTC_PUT_BCD_MDAY(x,y)   ((((x % 10) + ((x / 10) << 4)) & 0x3f) | (y & ~0x3f))
#define RTC_PUT_BCD_MONTH(x,y)  ((((x % 10) + ((x / 10) << 4)) & 0x1f) | (y & ~0x1f))
#define RTC_PUT_BCD_YEAR(x)     (((x % 10) + ((x / 10) << 4)) & 0xff)

extern ssize_t cyc_cpm_spi_read(char *buf, size_t count);
extern ssize_t cyc_cpm_spi_write(char *buf, size_t count);
extern void __init cyc_cpm_spi_init(void);

void
last_reboot(void) 
{
	bd_t *binfo = (bd_t *)res;
	struct rtc_time	tm;
	char i;

	tm.tm_sec  = RTC_GET_BCD_SEC(binfo->hw_info.rtc_value[1]);
	tm.tm_min  = RTC_GET_BCD_MIN(binfo->hw_info.rtc_value[2]);
	tm.tm_hour = RTC_GET_BCD_HR(binfo->hw_info.rtc_value[3]);
	tm.tm_mday = RTC_GET_BCD_MDAY(binfo->hw_info.rtc_value[5]);
	tm.tm_mon  = RTC_GET_BCD_MONTH(binfo->hw_info.rtc_value[6]);
	tm.tm_year = RTC_GET_BCD_YEAR(binfo->hw_info.rtc_value[7]);

	//printk("RTC on boottime: ");
	//for (i = 0; i<20; i++) printk("%02x", binfo->hw_info.rtc_value[i]);
	//printk("\r\n");

	tm.tm_year = tm.tm_year + 1900;
	if (tm.tm_year < 1970) {
		tm.tm_year += 100;
	}
	printk("Last reboot %d/%d/%04d at %02d:%02d:%02d GST\r\n", tm.tm_mon, tm.tm_mday, tm.tm_year, tm.tm_hour, tm.tm_min, tm.tm_sec);
}

void __init
cyc_rtc_init(void)
{
	cyc_cpm_spi_init();
	printk("RTC init done.\r\n");
	last_reboot();
}

ulong
cyc_get_rtc_time(void)
{
	uint	year, mon, day, hour, min, sec;
	char    buf[32];

	memset(buf,0, sizeof(buf));

	buf[0] = 0x00;					/* first position should have register addr on the RTC */
	cyc_cpm_spi_read(buf, 16);		/* read 15 bytes + 1 reg addr */
									/* buf[1-15] are valid returned data */
	sec = RTC_GET_BCD_SEC(buf[2]);
	min = RTC_GET_BCD_MIN(buf[3]);
	hour= RTC_GET_BCD_HR(buf[4]);
	day = RTC_GET_BCD_MDAY(buf[6]);
	mon = RTC_GET_BCD_MONTH(buf[7]);
	year= RTC_GET_BCD_YEAR(buf[8]);

	year = year + 1900;
	if (year < 1970) {
		year += 100;
	}

	return mktime(year, mon, day, hour, min, sec);
}

int
cyc_set_rtc_time(unsigned long nowtime)
{
	struct rtc_time	tm;
	char buf[32];

	to_tm(nowtime, &tm);

	tm.tm_year = (tm.tm_year - 1900) % 100;

	memset(buf,0, sizeof(buf));
	buf[0] = 0x00;					/* first position should have register addr on the RTC */
	cyc_cpm_spi_read(buf, 16);		/* read 15 bytes + 1 red addr */

	buf[2] = RTC_PUT_BCD_SEC(tm.tm_sec, buf[2]);
	buf[3] = RTC_PUT_BCD_MIN(tm.tm_min, buf[3]);
	buf[4] = RTC_PUT_BCD_HR(tm.tm_hour, buf[4]);
	buf[6] = RTC_PUT_BCD_MDAY(tm.tm_mday, buf[6]);
	buf[7] = RTC_PUT_BCD_MONTH(tm.tm_mon, buf[7]);
	buf[8] = RTC_PUT_BCD_YEAR(tm.tm_year);

	buf[0] = 0x00;					/* first position should have register addr on the RTC */
	cyc_cpm_spi_write(buf, 10);

	return 0;
}

