/*
 *  UART driver for MPC860 CPM SCC or SMC
 *  Copyright (c) 1997 Dan Malek (dmalek@jlc.net)
 *
 * I used the serial.c driver as the framework for this driver.
 * Give credit to those guys.
 * The original code was written for the MBX860 board.  I tried to make
 * it generic, but there may be some assumptions in the structures that
 * have to be fixed later.
 * To save porting time, I did not bother to change any object names
 * that are not accessed outside of this file.
 * It still needs lots of work........When it was easy, I included code
 * to support the SCCs, but this has never been tested, nor is it complete.
 * Only the SCCs support modem control, so that is not complete either.
 *
 * This module exports the following rs232 io functions:
 *
 *	int rs_8xx_init(void);
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/interrupt.h>
#include <linux/tty.h>
#include <linux/tty_flip.h>
#include <linux/serial.h>
#include <linux/serialP.h>
#include <linux/major.h>
#include <linux/string.h>
#include <linux/fcntl.h>
#include <linux/ptrace.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <asm/uaccess.h>
#include <asm/8xx_immap.h>
#include <asm/mpc8xx.h>
#include "commproc.h"
#ifdef CONFIG_TS1H
#include <asm/frame_mode.h>
#endif

#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
#include <linux/serial_reg.h>
#include <linux/joystick.h>
#define	modem_control
#define	notdef
#define DO_THIS_LATER
#define set_current_state(x) __set_current_state(x)
#endif

#ifdef CONFIG_KGDB
extern void breakpoint(void);
extern void set_debug_traps(void);
extern int  kgdb_output_string (const char* s, unsigned int count);
#endif

#if defined(CONFIG_UART_CTS_CONTROL_SCC1) || \
    defined(CONFIG_UART_RTS_CONTROL_SCC1) || \
    defined(CONFIG_UART_CD_CONTROL_SCC1)  || \
    defined(CONFIG_UART_DTR_CONTROL_SCC1) || \
    defined(CONFIG_UART_CTS_CONTROL_SCC2) || \
    defined(CONFIG_UART_RTS_CONTROL_SCC2) || \
    defined(CONFIG_UART_CD_CONTROL_SCC2)  || \
    defined(CONFIG_UART_DTR_CONTROL_SCC2) || \
    defined(CONFIG_UART_CTS_CONTROL_SCC3) || \
    defined(CONFIG_UART_RTS_CONTROL_SCC3) || \
    defined(CONFIG_UART_CD_CONTROL_SCC3)  || \
    defined(CONFIG_UART_DTR_CONTROL_SCC3) || \
    defined(CONFIG_UART_CTS_CONTROL_SCC4) || \
    defined(CONFIG_UART_RTS_CONTROL_SCC4) || \
    defined(CONFIG_UART_CD_CONTROL_SCC4)  || \
    defined(CONFIG_UART_DTR_CONTROL_SCC4)
#define CPM_UART_HANDSHAKING
#endif

#ifdef CPM_UART_HANDSHAKING
#define PORT_A 1
#define PORT_B 2
#define PORT_C 3
#define PORT_D 4

#if (CONFIG_UART_DTR_CONTROL_SCC1 == PORT_A)
#define PORT_DTR1_PAR immap->im_ioport.iop_papar
#define PORT_DTR1_DAT immap->im_ioport.iop_padat
#define PORT_DTR1_DIR immap->im_ioport.iop_padir
#endif
#if (CONFIG_UART_DTR_CONTROL_SCC1 == PORT_B)
#define PORT_DTR1_PAR immap->im_cpm.cp_pbpar
#define PORT_DTR1_DAT immap->im_cpm.cp_pbdat
#define PORT_DTR1_DIR immap->im_cpm.cp_pbdir
#define DTR1_PIN (1 << (31 - CONFIG_DTR1_PIN))
#endif
#if (CONFIG_UART_DTR_CONTROL_SCC1 == PORT_C)
#define PORT_DTR1_PAR immap->im_ioport.iop_pcpar
#define PORT_DTR1_DAT immap->im_ioport.iop_pcdat
#define PORT_DTR1_DIR immap->im_ioport.iop_pcdir
#endif
#if (CONFIG_UART_DTR_CONTROL_SCC1 == PORT_D)
#define PORT_DTR1_PAR immap->im_ioport.iop_pdpar
#define PORT_DTR1_DAT immap->im_ioport.iop_pddat
#define PORT_DTR1_DIR immap->im_ioport.iop_pddir
#endif
#if (CONFIG_UART_DTR_CONTROL_SCC2 == PORT_A)
#define PORT_DTR2_PAR immap->im_ioport.iop_papar
#define PORT_DTR2_DAT immap->im_ioport.iop_padat
#define PORT_DTR2_DIR immap->im_ioport.iop_padir
#endif
#if (CONFIG_UART_DTR_CONTROL_SCC2 == PORT_B)
#define PORT_DTR2_PAR immap->im_cpm.cp_pbpar
#define PORT_DTR2_DAT immap->im_cpm.cp_pbdat
#define PORT_DTR2_DIR immap->im_cpm.cp_pbdir
#define DTR2_PIN (1 << (31 - CONFIG_DTR2_PIN))
#endif
#if (CONFIG_UART_DTR_CONTROL_SCC2 == PORT_C)
#define PORT_DTR2_PAR immap->im_ioport.iop_pcpar
#define PORT_DTR2_DAT immap->im_ioport.iop_pcdat
#define PORT_DTR2_DIR immap->im_ioport.iop_pcdir
#endif
#if (CONFIG_UART_DTR_CONTROL_SCC2 == PORT_D)
#define PORT_DTR2_PAR immap->im_ioport.iop_pdpar
#define PORT_DTR2_DAT immap->im_ioport.iop_pddat
#define PORT_DTR2_DIR immap->im_ioport.iop_pddir
#endif
#if (CONFIG_UART_DTR_CONTROL_SCC3 == PORT_A)
#define PORT_DTR3_PAR immap->im_ioport.iop_papar
#define PORT_DTR3_DAT immap->im_ioport.iop_padat
#define PORT_DTR3_DIR immap->im_ioport.iop_padir
#endif
#if (CONFIG_UART_DTR_CONTROL_SCC3 == PORT_B)
#define PORT_DTR3_PAR immap->im_cpm.cp_pbpar
#define PORT_DTR3_DAT immap->im_cpm.cp_pbdat
#define PORT_DTR3_DIR immap->im_cpm.cp_pbdir
#define DTR3_PIN (1 << (31 - CONFIG_DTR3_PIN))
#endif
#if (CONFIG_UART_DTR_CONTROL_SCC3 == PORT_C)
#define PORT_DTR3_PAR immap->im_ioport.iop_pcpar
#define PORT_DTR3_DAT immap->im_ioport.iop_pcdat
#define PORT_DTR3_DIR immap->im_ioport.iop_pcdir
#endif
#if (CONFIG_UART_DTR_CONTROL_SCC3 == PORT_D)
#define PORT_DTR3_PAR immap->im_ioport.iop_pdpar
#define PORT_DTR3_DAT immap->im_ioport.iop_pddat
#define PORT_DTR3_DIR immap->im_ioport.iop_pddir
#endif
#if (CONFIG_UART_DTR_CONTROL_SCC4 == PORT_A)
#define PORT_DTR4_PAR immap->im_ioport.iop_papar
#define PORT_DTR4_DAT immap->im_ioport.iop_padat
#define PORT_DTR4_DIR immap->im_ioport.iop_padir
#endif
#if (CONFIG_UART_DTR_CONTROL_SCC4 == PORT_B)
#define PORT_DTR4_PAR immap->im_cpm.cp_pbpar
#define PORT_DTR4_DAT immap->im_cpm.cp_pbdat
#define PORT_DTR4_DIR immap->im_cpm.cp_pbdir
#define DTR4_PIN (1 << (31 - CONFIG_DTR4_PIN))
#endif
#if (CONFIG_UART_DTR_CONTROL_SCC4 == PORT_C)
#define PORT_DTR4_PAR immap->im_ioport.iop_pcpar
#define PORT_DTR4_DAT immap->im_ioport.iop_pcdat
#define PORT_DTR4_DIR immap->im_ioport.iop_pcdir
#endif
#if (CONFIG_UART_DTR_CONTROL_SCC4 == PORT_D)
#define PORT_DTR4_PAR immap->im_ioport.iop_pdpar
#define PORT_DTR4_DAT immap->im_ioport.iop_pddat
#define PORT_DTR4_DIR immap->im_ioport.iop_pddir
#endif
#ifdef CONFIG_UART_DTR_CONTROL_SCC1
#if (CONFIG_UART_DTR_CONTROL_SCC1 != PORT_B)
#define DTR1_PIN (1 << (15 - CONFIG_DTR1_PIN))
#endif
#endif
#ifdef CONFIG_UART_DTR_CONTROL_SCC2
#if (CONFIG_UART_DTR_CONTROL_SCC2 != PORT_B)
#define DTR2_PIN (1 << (15 - CONFIG_DTR2_PIN))
#endif
#endif
#ifdef CONFIG_UART_DTR_CONTROL_SCC3
#if (CONFIG_UART_DTR_CONTROL_SCC3 != PORT_B)
#define DTR3_PIN (1 << (15 - CONFIG_DTR3_PIN))
#endif
#endif
#ifdef CONFIG_UART_DTR_CONTROL_SCC4
#if (CONFIG_UART_DTR_CONTROL_SCC4 != PORT_B)
#define DTR4_PIN (1 << (15 - CONFIG_DTR4_PIN))
#endif
#endif

#endif

#ifdef CONFIG_SERIAL_CONSOLE
#include <linux/console.h>

/* this defines the index into rs_table for the port to use
*/
# ifndef CONFIG_SERIAL_CONSOLE_PORT
#  ifdef CONFIG_SCC3_ENET
#   ifdef CONFIG_CONS_SMC2
#    define CONFIG_SERIAL_CONSOLE_PORT	0	/* Console on SMC2 is 1st port */
#   else
#    error "Can't use SMC1 for console with Ethernet on SCC3"
#   endif
#  else	/* ! CONFIG_SCC3_ENET */
#   ifdef CONFIG_CONS_SMC2			/* Console on SMC2 */
#    define CONFIG_SERIAL_CONSOLE_PORT	1
#   else					/* Console on SMC1 */
#    define CONFIG_SERIAL_CONSOLE_PORT	0
#   endif /* CONFIG_CONS_SMC2 */
#  endif  /* CONFIG_SCC3_ENET */
# endif	  /* CONFIG_SERIAL_CONSOLE_PORT */
#endif	  /* CONFIG_SERIAL_CONSOLE */

#if defined(CONFIG_LANTEC)
/* SCC3 for console	(ugly hack)
*/
#undef CONFIG_SERIAL_CONSOLE_PORT
#define CONFIG_SERIAL_CONSOLE_PORT	1
#endif

#define TX_WAKEUP	ASYNC_SHARE_IRQ

static ushort	smc_chan_map[] = {
	CPM_CR_CH_SMC1,
	CPM_CR_CH_SMC2
};

static ushort	scc_chan_map[] = {
	CPM_CR_CH_SCC1,
	CPM_CR_CH_SCC2,
	CPM_CR_CH_SCC3,
	CPM_CR_CH_SCC4
};

#if defined(CONFIG_TS1H)
static struct timeval  rtuend;  //used for Modbus RTU framming
#endif
#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
struct timer_list wait_idle_line;
#endif

static char *serial_name = "CPM UART driver";
static char *serial_version = "0.03";

static DECLARE_TASK_QUEUE(tq_serial);

static struct tty_driver serial_driver, callout_driver;
static int serial_refcount;
static int serial_console_setup(struct console *co, char *options);
static void do_serial_hangup(void *private_);

/*
 * Serial driver configuration section.  Here are the various options:
 */
#define SERIAL_PARANOIA_CHECK
#define CONFIG_SERIAL_NOPAUSE_IO
#define SERIAL_DO_RESTART

/* Set of debugging defines */

#undef SET_DEBUG_IOCTL
#undef SET_DEBUG_IOCTL1
#undef SERIAL_DEBUG_INTR
#undef SERIAL_DEBUG_OPEN // WATCH OUT: this flag has to be set ONLY when the JUMPER
			 // for console is not connected (single user). It crashes
			 // if used in single user. It's OK to use in multiuser
#undef SERIAL_DEBUG_CLOSE
#undef SERIAL_DEBUG_WRITE
#undef SERIAL_DEBUG_READ
#undef SERIAL_DEBUG_IOCTL
#undef SERIAL_DEBUG_TERMIOS
#undef SERIAL_DEBUG_FLOW
#undef SERIAL_DEBUG_RS_WAIT_UNTIL_SENT

#define _INLINE_ inline
  
#define DBG_CNT(s)

/* We overload some of the items in the data structure to meet our
 * needs.  For example, the port address is the CPM parameter ram
 * offset for the SCC or SMC.  The maximum number of ports is 4 SCCs and
 * 2 SMCs.  The "hub6" field is used to indicate the channel number, with
 * a flag indicating SCC or SMC, and the number is used as an index into
 * the CPM parameter area for this device.
 * The "type" field is currently set to 0, for PORT_UNKNOWN.  It is
 * not currently used.  I should probably use it to indicate the port
 * type of SMC or SCC.
 * The SMCs do not support any modem control signals.
 */
#define smc_scc_num	hub6
#define NUM_IS_SCC	((int)0x00010000)
#define PORT_NUM(P)	((P) & 0x0000ffff)

volatile	smc_t		*my_cp_smc;
volatile	scc_t		*my_cp_scc;
cbd_t *my_smc_tx_bdp, *my_smc_rx_bdp;
cbd_t *my_scc_tx_bdp, *my_scc_rx_bdp;

/* Processors other than the 860 only get SMCs configured by default.
 * Either they don't have SCCs or they are allocated somewhere else.
 * Of course, there are now 860s without some SCCs, so we will need to
 * address that someday.
 * The Embedded Planet Multimedia I/O cards use TDM interfaces to the
 * stereo codec parts, and we use SMC2 to help support that.
 */
static struct serial_state rs_table[] = {
	/* UART CLK   PORT          IRQ      FLAGS  NUM   */
#ifndef CONFIG_TS1H
#ifndef CONFIG_SCC3_ENET	/* SMC1 not usable with Ethernet on SCC3 */
  	{ 0,     0, PROFF_SMC1, CPMVEC_SMC1,   0,    0 },    /* SMC1 ttyS0 */
#endif
#endif
#if !defined(CONFIG_USB_MPC8xx) && !defined(CONFIG_USB_CLIENT_MPC8xx)
#ifdef CONFIG_SMC2_UART
	{ 0,     0, PROFF_SMC2, CPMVEC_SMC2,   0,    1 },    /* SMC2 ttyS1 */
#endif
#endif	/* CONFIG_USB_xxx */
#ifdef CONFIG_SCC1_UART
	{ 0,     0, PROFF_SCC1, CPMVEC_SCC1,   0,    (NUM_IS_SCC | 0) },    /* SCC1 ttyS2 */
#endif
#ifdef CONFIG_SCC2_UART
	{ 0,     0, PROFF_SCC2, CPMVEC_SCC2,   0,    (NUM_IS_SCC | 1) },    /* SCC2 ttyS2 */
#endif
#ifdef CONFIG_SCC3_UART
	{ 0,     0, PROFF_SCC3, CPMVEC_SCC3,   0,    (NUM_IS_SCC | 2) },    /* SCC3 ttyS3 */
#endif
#ifdef CONFIG_SCC4_UART
	{ 0,     0, PROFF_SCC4, CPMVEC_SCC4,   0,    (NUM_IS_SCC | 3) },    /* SCC4 ttyS4 */
#endif
};

#define NR_PORTS	(sizeof(rs_table)/sizeof(struct serial_state))

/* map rs_table index to baud rate generator number */
/* if USB is turned on, we need to use BRG1 for SOF generation */
static int rs_brg_map[] = {
#ifdef CONFIG_USB_MPC8xx
  1,	/* BRG2 */
  2,	/* BRG3 */
  3,	/* BRG4 */
	/* must share remaining BRG's */
  1,	/* BRG2 */
  2,	/* BRG3 */
  3,	/* BRG4 */
#else
  0,	/* BRG1 */
  1,	/* BRG2 */
  2,	/* BRG3 */
  3,	/* BRG4 */
	/* must share remaining BRG's */
  2,	/* BRG3 */
  3,	/* BRG4 */
#endif
};

static int rx_bd_nums[6] = {
#ifdef CONFIG_SCC1_UART_RX_BDNUM
#if defined(CONFIG_TS1H)
	FRAMEMODE_RXBUFFERS,
#else
	CONFIG_SCC1_UART_RX_BDNUM,
#endif
#else
  0,
#endif
#ifdef CONFIG_SCC2_UART_RX_BDNUM
  CONFIG_SCC2_UART_RX_BDNUM,
#else
  0,
#endif
#ifdef CONFIG_SCC3_UART_RX_BDNUM
  CONFIG_SCC3_UART_RX_BDNUM,
#else
  0,
#endif
#ifdef CONFIG_SCC4_UART_RX_BDNUM
  CONFIG_SCC4_UART_RX_BDNUM,
#else
  0,
#endif
#ifdef CONFIG_SMC1_UART_RX_BDNUM
  CONFIG_SMC1_UART_RX_BDNUM,
#else
  0,
#endif
#ifdef CONFIG_SMC2_UART_RX_BDNUM
  CONFIG_SMC2_UART_RX_BDNUM
#else
  0
#endif
};

static int tx_bd_nums[6] = {
#ifdef CONFIG_SCC1_UART_TX_BDNUM
#if defined(CONFIG_TS1H)
	FRAMEMODE_TXBUFFERS,
#else
	CONFIG_SCC1_UART_TX_BDNUM,
#endif
#else
  0,
#endif
#ifdef CONFIG_SCC2_UART_TX_BDNUM
  CONFIG_SCC2_UART_TX_BDNUM,
#else
  0,
#endif
#ifdef CONFIG_SCC3_UART_TX_BDNUM
  CONFIG_SCC3_UART_TX_BDNUM,
#else
  0,
#endif
#ifdef CONFIG_SCC4_UART_TX_BDNUM
  CONFIG_SCC4_UART_TX_BDNUM,
#else
  0,
#endif
#ifdef CONFIG_SMC1_UART_TX_BDNUM
  CONFIG_SMC1_UART_TX_BDNUM,
#else
  0,
#endif
#ifdef CONFIG_SMC2_UART_TX_BDNUM
  CONFIG_SMC2_UART_TX_BDNUM
#else
  0
#endif
};

static int rx_bd_sizes[6] = {
#ifdef CONFIG_SCC1_UART_RX_BDSIZE
#if defined(CONFIG_TS1H)
	FRAMEMODE_FRAMELEN,
#else
  	CONFIG_SCC1_UART_RX_BDSIZE,
#endif
#else
  0,
#endif
#ifdef CONFIG_SCC2_UART_RX_BDSIZE
  CONFIG_SCC2_UART_RX_BDSIZE,
#else
  0,
#endif
#ifdef CONFIG_SCC3_UART_RX_BDSIZE
  CONFIG_SCC3_UART_RX_BDSIZE,
#else
  0,
#endif
#ifdef CONFIG_SCC4_UART_RX_BDSIZE
  CONFIG_SCC4_UART_RX_BDSIZE,
#else
  0,
#endif
#ifdef CONFIG_SMC1_UART_RX_BDSIZE
  CONFIG_SMC1_UART_RX_BDSIZE,
#else
  0,
#endif
#ifdef CONFIG_SMC2_UART_RX_BDSIZE
  CONFIG_SMC2_UART_RX_BDSIZE
#else
  0
#endif
};

static int tx_bd_sizes[6] = {
#ifdef CONFIG_SCC1_UART_TX_BDSIZE
#if defined(CONFIG_TS1H)
	FRAMEMODE_FRAMELEN,
#else
	CONFIG_SCC1_UART_TX_BDSIZE,
#endif
#else
  0,
#endif
#ifdef CONFIG_SCC2_UART_TX_BDSIZE
  CONFIG_SCC2_UART_TX_BDSIZE,
#else
  0,
#endif
#ifdef CONFIG_SCC3_UART_TX_BDSIZE
  CONFIG_SCC3_UART_TX_BDSIZE,
#else
  0,
#endif
#ifdef CONFIG_SCC4_UART_TX_BDSIZE
  CONFIG_SCC4_UART_TX_BDSIZE,
#else
  0,
#endif
#ifdef CONFIG_SMC1_UART_TX_BDSIZE
  CONFIG_SMC1_UART_TX_BDSIZE,
#else
  0,
#endif
#ifdef CONFIG_SMC2_UART_TX_BDSIZE
  CONFIG_SMC2_UART_TX_BDSIZE
#else
  0
#endif
};

static struct tty_struct *serial_table[NR_PORTS];
static struct termios *serial_termios[NR_PORTS];
static struct termios *serial_termios_locked[NR_PORTS];

#define	RX_BUF_SIZE	32

#ifndef MIN
#define MIN(a,b)	((a) < (b) ? (a) : (b))
#endif

#if !defined(CONFIG_TS1H) && !defined(CONFIG_SM100)
typedef struct wait_queue * wait_queue_head_t;
#endif

/* The async_struct in serial.h does not really give us what we
 * need, so define our own here.
 */
typedef struct serial_info {
	int			magic;
	int			flags;
	struct serial_state	*state;
	struct tty_struct 	*tty;
	int			read_status_mask;
	int			ignore_status_mask;
	int			timeout;
	int			line;
	int			close_delay;
	unsigned short		closing_wait;
	unsigned short		closing_wait2;
	unsigned long		event;
	unsigned long		last_active;
	int			blocked_open; /* # of blocked opens */
	long			session; /* Session of opening process */
	long			pgrp; /* pgrp of opening process */
	struct tq_struct	tqueue;
	struct tq_struct	tqueue_hangup;
	wait_queue_head_t	open_wait;
	wait_queue_head_t	close_wait;

	/* CPM Buffer Descriptor pointers.
	*/
	cbd_t			*rx_bd_base;
	cbd_t			*rx_cur;
	cbd_t			*tx_bd_base;
	cbd_t			*tx_cur;
	unsigned int		MCR;
	unsigned int		MSR;
	unsigned int		IER;
	wait_queue_head_t	delta_msr_wait;
#if defined(CONFIG_TS1H)
	struct frame_mode_cfg frame_cfg;
#endif
#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
	unsigned char   hw_stopped; // Half duplex CTS ctrl
	char            media;      // RS485
	unsigned char   terminator;	// RS485 with terminator
	cbd_t           *bdtx;
#endif
} ser_info_t;

static void change_speed(ser_info_t *info);
static void rs_8xx_wait_until_sent(struct tty_struct *tty, int timeout);
#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
static void serial_out(ser_info_t *info, int val, int val2);
#endif

static int set_modem_info(ser_info_t *info, unsigned int cmd, unsigned int *value)
{
	int error;
	unsigned int arg;

	error = get_user(arg, value);
	if (error)
		return error;

#ifdef modem_control
#ifdef CONFIG_SM100
	if (!(info->state->smc_scc_num & NUM_IS_SCC))
		return 0;
#endif
	switch (cmd) {
	case TIOCMBIS: 
		if (arg & TIOCM_RTS)
			info->MCR |= UART_MCR_RTS;
		if (arg & TIOCM_DTR)
			info->MCR |= UART_MCR_DTR;
#ifdef TIOCM_OUT1
		if (arg & TIOCM_OUT1)
			info->MCR |= UART_MCR_OUT1;
		if (arg & TIOCM_OUT2)
			info->MCR |= UART_MCR_OUT2;
#endif
		break;
	case TIOCMBIC:
		if (arg & TIOCM_RTS)
			info->MCR &= ~UART_MCR_RTS;
		if (arg & TIOCM_DTR)
			info->MCR &= ~UART_MCR_DTR;
#ifdef TIOCM_OUT1
		if (arg & TIOCM_OUT1)
			info->MCR &= ~UART_MCR_OUT1;
		if (arg & TIOCM_OUT2)
			info->MCR &= ~UART_MCR_OUT2;
#endif
		break;
	case TIOCMSET:
		info->MCR = ((info->MCR & ~(UART_MCR_RTS |
#ifdef TIOCM_OUT1
					    UART_MCR_OUT1 |
					    UART_MCR_OUT2 |
#endif
					    UART_MCR_DTR))
			     | ((arg & TIOCM_RTS) ? UART_MCR_RTS : 0)
#ifdef TIOCM_OUT1
			     | ((arg & TIOCM_OUT1) ? UART_MCR_OUT1 : 0)
			     | ((arg & TIOCM_OUT2) ? UART_MCR_OUT2 : 0)
#endif
			     | ((arg & TIOCM_DTR) ? UART_MCR_DTR : 0));
		break;
	default:
		return -EINVAL;
	}
	cli();
	serial_out(info, UART_MCR, info->MCR);
	sti();
#endif	/* modem_control */
	return 0;
}

#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)

void serial_out(ser_info_t *info, int val, int val2)
{
	immap_t *immap = (immap_t *)IMAP_ADDR;
	int         pin_value;
	volatile	scc_t	*scp;
	
	if (val == UART_MCR) { // modem control register
#ifdef CONFIG_SM100
		if (!(info->state->smc_scc_num & NUM_IS_SCC)) return;
#endif
		pin_value = immap->im_cpm.cp_pbdat;
		
		if (val2 & UART_MCR_DTR) { 
			pin_value &= ~0x800;
		} else {
			pin_value |= 0x800;
		}
	
		if (!info->media) { // RS232
			if (val2 & UART_MCR_RTS) {
				pin_value &= ~(1 << (31 - CONFIG_RTS1_PIN));
			} else {
				pin_value |= (1 << (31 - CONFIG_RTS1_PIN));
			}
		}

		immap->im_cpm.cp_pbdat = pin_value;

#ifdef CONFIG_TS1H
		if (info->media && (info->flags & ASYNC_LEGACY_RTS)) { // RS485-Half
			if (val2 & UART_MCR_RTS) {
				if (info->terminator)
					immap->im_ioport.iop_pcdat |= (1 << 9); // termination OFF
				immap->im_ioport.iop_padat &= ~0x80; /*D_SEL*/
				scp = &immap->im_cpm.cp_scc[PORT_NUM(info->state->smc_scc_num)];
				scp->scc_sccm &= ~(UART_SCCM_RX | UART_SCCM_BSY);
				scp->scc_gsmrl &= ~SCC_GSMRL_ENR;
			} else {
				immap->im_ioport.iop_padat |= 0x80;
				if (info->terminator)
					immap->im_ioport.iop_pcdat &= ~(1 << 9); // termination ON
				scp = &immap->im_cpm.cp_scc[PORT_NUM(info->state->smc_scc_num)];
				scp->scc_sccm |= (UART_SCCM_RX | UART_SCCM_BSY);
				scp->scc_gsmrl |= SCC_GSMRL_ENR;
			}
		}
#endif
	} else {
		//[RK]Mar/29/05 - fixed burnin problem
		//if ((val == UART_IER) && !info->media)  
		if (val == UART_IER) { 
			// Modem Interrupt and RS232
			pin_value = 0;
			immap->im_ioport.iop_pcso &= 0xffef; // CTS1 as GP I/O 
			if (val2 & UART_IER_MSI) { 
				pin_value = 0x10; // CTS1 connected to the pin and can generate interrupt 
			} else {
				pin_value = 0x00;
			}
			immap->im_ioport.iop_pcso |= pin_value;
		}
	}
	return;
}

int serial_in(ser_info_t *info, int val)
{
	immap_t *immap = (immap_t *)IMAP_ADDR;
	int	status = 0;

	if (val == UART_MSR) {
		status =  ((immap->im_ioport.iop_pcdat & 0x20) ? 0 : UART_MSR_DCD)
			| ((immap->im_ioport.iop_pcdat & 0x8) ? 0 : UART_MSR_DSR)
			| ((immap->im_ioport.iop_pcdat & 0x10) ? 0 : UART_MSR_CTS)
			;
		if ((status & UART_MSR_DCD)!=(info->MSR & UART_MSR_DCD)) {
			status |= UART_MSR_DDCD;
			
		}
		if ((status & UART_MSR_DSR)!=(info->MSR & UART_MSR_DSR)) {
			status |= UART_MSR_DDSR;
		}
		if ((status & UART_MSR_CTS)!=(info->MSR & UART_MSR_CTS)) {
			status |= UART_MSR_DCTS;
		}
		info->MSR = status;
	}
	if (val == UART_MCR) {
		status = ((immap->im_cpm.cp_pbdat & 0x800) ? 0 : UART_MCR_DTR)
		| ((immap->im_cpm.cp_pbdat & 0x1000) ? 0 : UART_MCR_RTS)
		;
	}
	if (val == UART_LSR) {
		status = 0;
	}

	return(status);
}

#endif

static inline int serial_paranoia_check(ser_info_t *info,
					kdev_t device, const char *routine)
{
#ifdef SERIAL_PARANOIA_CHECK
	static const char *badmagic =
		"Warning: bad magic number for serial struct (%s) in %s\n";
	static const char *badinfo =
		"Warning: null async_struct for (%s) in %s\n";

	if (!info) {
		printk(badinfo, kdevname(device), routine);
		return 1;
	}
	if (info->magic != SERIAL_MAGIC) {
		printk(badmagic, kdevname(device), routine);
		return 1;
	}
#endif
	return 0;
}

/*
 * This is used to figure out the divisor speeds and the timeouts,
 * indexed by the termio value.  The generic CPM functions are responsible
 * for setting and assigning baud rate generators for us.
 */
static int baud_table[] = {
	0, 50, 75, 110, 134, 150, 200, 300, 600, 1200, 1800, 2400, 4800,
	9600, 14400, 19200, 28800, 38400, 57600, 76800, 115200, 230400, 460800, 500000, 576000,
	921600, 1000000, 1152000, 1500000, 2000000, 2500000, 3000000, 
	3500000, 4000000, 0 };

/*
 * ------------------------------------------------------------
 * rs_stop() and rs_start()
 *
 * This routines are called before setting or resetting tty->stopped.
 * They enable or disable transmitter interrupts, as necessary.
 * ------------------------------------------------------------
 */
static void rs_8xx_stop(struct tty_struct *tty)
{
	ser_info_t *info = (ser_info_t *)tty->driver_data;
	int	idx;
	unsigned long flags;
	volatile scc_t	*sccp;
	volatile smc_t	*smcp;

	if (serial_paranoia_check(info, tty->device, "rs_stop"))
		return;

	save_flags(flags); cli();
	idx = PORT_NUM(info->state->smc_scc_num);
	if (info->state->smc_scc_num & NUM_IS_SCC) {
		sccp = &cpmp->cp_scc[idx];
		sccp->scc_sccm &= ~UART_SCCM_TX;
	}
	else {
		smcp = &cpmp->cp_smc[idx];
		smcp->smc_smcm &= ~SMCM_TX;
	}
	restore_flags(flags);
}

static void rs_8xx_start(struct tty_struct *tty)
{
	ser_info_t *info = (ser_info_t *)tty->driver_data;
	int	idx;
	unsigned long flags;
	volatile scc_t	*sccp;
	volatile smc_t	*smcp;

	if (serial_paranoia_check(info, tty->device, "rs_stop"))
		return;
	
	idx = PORT_NUM(info->state->smc_scc_num);
	save_flags(flags); cli();
	if (info->state->smc_scc_num & NUM_IS_SCC) {
		sccp = &cpmp->cp_scc[idx];
		sccp->scc_sccm |= UART_SCCM_TX;
	}
	else {
		smcp = &cpmp->cp_smc[idx];
		smcp->smc_smcm |= SMCM_TX;
	}
	restore_flags(flags);
}

/*
 * ----------------------------------------------------------------------
 *
 * Here starts the interrupt handling routines.  All of the following
 * subroutines are declared as inline and are folded into
 * rs_interrupt().  They were separated out for readability's sake.
 *
 * Note: rs_interrupt() is a "fast" interrupt, which means that it
 * runs with interrupts turned off.  People who may want to modify
 * rs_interrupt() should try to keep the interrupt handler as fast as
 * possible.  After you are done making modifications, it is not a bad
 * idea to do:
 * 
 * gcc -S -DKERNEL -Wall -Wstrict-prototypes -O6 -fomit-frame-pointer serial.c
 *
 * and look at the resulting assemble code in serial.s.
 *
 * 				- Ted Ts'o (tytso@mit.edu), 7-Mar-93
 * -----------------------------------------------------------------------
 */

/*
 * This routine is used by the interrupt handler to schedule
 * processing in the software interrupt portion of the driver.
 */
static _INLINE_ void rs_sched_event(ser_info_t *info,
				  int event)
{
	info->event |= 1 << event;

	/* patch from: Pavel Bartusek <pavel.bartusek@tiscali.cz> */
#ifdef CONFIG_SERIAL
	/* if the serial driver is also selected, we have to use another tq */
	queue_task(&info->tqueue, &tq_immediate);
	mark_bh(IMMEDIATE_BH);
#else
	queue_task(&info->tqueue, &tq_serial);
	mark_bh(SERIAL_BH);
#endif
}

#if defined(CONFIG_TS1H)
static _INLINE_ void receive_frame(ser_info_t *info)
{
	struct tty_struct *tty = info->tty;
	unsigned char ch, *cp, *dest;
	int	i, control_char = 0;
	ushort	status;
	struct	async_icount *icount;
	volatile cbd_t	*bdp;
	volatile scc_t	*sccp;

	icount = &info->state->icount;

	if (info->state->smc_scc_num & NUM_IS_SCC) {
		sccp = &cpmp->cp_scc[PORT_NUM(info->state->smc_scc_num)];
	}

	/* Just loop through the closed BDs and copy the characters into
	 * the buffer.
	 */
	bdp = info->rx_cur;
	for (;;) {
		volatile sysconf8xx_t *psiu = &(((immap_t *)IMAP_ADDR)->im_siu_conf);
		psiu->sc_swsr = 0x556c;
		psiu->sc_swsr = 0xaa39;

		if (bdp->cbd_sc & BD_SC_EMPTY) {	/* If this one is empty */
			break;			        /* we are all done */
		}

		/* Get the number of characters and the buffer pointer.
		*/
		i = bdp->cbd_datlen;
		
		cp = (unsigned char *)__va(bdp->cbd_bufaddr);
		status = bdp->cbd_sc;
	
		/* Check to see if there is room in the tty buffer for
		 * the characters in our BD buffer.  If not, we exit
		 * now, leaving the BD with the characters.  We'll pick
		 * them up again on the next receive interrupt (which could
		 * be a timeout).
		 */
		if ((tty->flip.count + i) >= TTY_FLIPBUF_SIZE)
			break;

		if ((tty->flip.count + i) >= tty->ldisc.receive_room(tty)) 
			break;

		if (*cp == STOP_CHAR(tty) || *cp == START_CHAR(tty)) {
			control_char = 1;
			goto process_char;
		}

		// if we receive a ':' char it is a start-of-frame
		if (!info->frame_cfg.frame_started) {
			do {
				if ( *cp++ == 0x3a ) {
					info->frame_cfg.rxcount += 2;	
					dest = tty->flip.char_buf_ptr; 	
					tty->flip.char_buf_ptr += 2;
					tty->flip.flag_buf_ptr += 2;
					tty->flip.count += 2;
					info->frame_cfg.frame_started = 1;
				}
				else
				{
					printk("uart.c:receive_frame() - wrong start character\n");
				}
			} while (--i && !info->frame_cfg.frame_started);
		}

		if (!i) {
			bdp->cbd_sc |= BD_SC_EMPTY;
			bdp->cbd_sc &= ~(BD_SC_BR | BD_SC_FR | BD_SC_PR | BD_SC_OV);
			if (bdp->cbd_sc & BD_SC_WRAP)
				bdp = info->rx_bd_base;
			else
				bdp++;
			info->rx_cur = (cbd_t *)bdp;
			return;
		}
				
process_char:
		
		while (i-- > 0) {
			ch = *cp++;
			*tty->flip.char_buf_ptr = ch;
			//printk("%x ", ch);
			icount->rx++;
			info->frame_cfg.rxcount++;

#ifdef SERIAL_DEBUG_INTR
			printk("DR%02x:%02x...", ch, status);
#endif

			*tty->flip.flag_buf_ptr = 0;
			if (status & (BD_SC_BR | BD_SC_FR |
				       BD_SC_PR | BD_SC_OV)) {
				/*
				 * For statistics only
				 */
				if (status & BD_SC_BR)
					icount->brk++;
				else if (status & BD_SC_PR)
					icount->parity++;
				else if (status & BD_SC_FR)
					icount->frame++;
				if (status & BD_SC_OV)
					icount->overrun++;
				status &= info->read_status_mask;
		
				if (status & (BD_SC_BR)) {
#ifdef SERIAL_DEBUG_INTR
					printk("handling break....");
#endif
					*tty->flip.flag_buf_ptr = TTY_BREAK;
					if (info->flags & ASYNC_SAK)
						do_SAK(tty);
				} else if (status & BD_SC_PR)
					*tty->flip.flag_buf_ptr = TTY_PARITY;
				else if (status & BD_SC_FR)
					*tty->flip.flag_buf_ptr = TTY_FRAME;
				if (status & BD_SC_OV) {
					/*
					 * Overrun is special, since it's
					 * reported immediately, and doesn't
					 * affect the current character
					 */
					if (tty->flip.count < TTY_FLIPBUF_SIZE) {
						tty->flip.count++;
						tty->flip.flag_buf_ptr++;
						tty->flip.char_buf_ptr++;
						*tty->flip.flag_buf_ptr =
								TTY_OVERRUN;
					}
				}
			}
			if (tty->flip.count >= TTY_FLIPBUF_SIZE)
				break;

			tty->flip.flag_buf_ptr++;
			tty->flip.char_buf_ptr++;
			tty->flip.count++;
		}

		if ( !control_char &&
			 *((char *)tty->flip.char_buf_ptr-1) == 0x0a && 
			 *((char *)tty->flip.char_buf_ptr-2) == 0x0d ) {
			* (short *) dest = info->frame_cfg.rxcount - 2;
			tty->flip.flag_buf_ptr -= 2;
			tty->flip.char_buf_ptr -= 2;
			tty->flip.count -= 2;
			info->frame_cfg.rxcount = 0;
			info->frame_cfg.frame_started = 0;
			info->frame_cfg.rx_stat.success++;
		}	
	
		/* This BD is ready to be used again.  Clear status.
		 * Get next BD.
		 */
		bdp->cbd_sc |= BD_SC_EMPTY;
		bdp->cbd_sc &= ~(BD_SC_BR | BD_SC_FR | BD_SC_PR | BD_SC_OV);

	if (bdp->cbd_sc & BD_SC_WRAP)
			bdp = info->rx_bd_base;
		else
			bdp++;
	}

	info->rx_cur = (cbd_t *)bdp;

	queue_task(&tty->flip.tqueue, &tq_timer);
}
#endif

static _INLINE_ void receive_chars(ser_info_t *info)
{
	struct tty_struct *tty = info->tty;
	unsigned char ch, *cp;
	/*int	ignored = 0;*/
	int	i;
	ushort	status;
	struct	async_icount *icount;
	volatile cbd_t	*bdp;

	icount = &info->state->icount;

	/* Just loop through the closed BDs and copy the characters into
	 * the buffer.
	 */
	bdp = info->rx_cur;
	for (;;) {
		volatile sysconf8xx_t *psiu = &(((immap_t *)IMAP_ADDR)->im_siu_conf);
		psiu->sc_swsr = 0x556c;
		psiu->sc_swsr = 0xaa39;

		if (bdp->cbd_sc & BD_SC_EMPTY) {	/* If this one is empty */
			break;			/*   we are all done */
		}

		/* Get the number of characters and the buffer pointer.
		*/
		i = bdp->cbd_datlen;

		cp = (unsigned char *)__va(bdp->cbd_bufaddr);
		status = bdp->cbd_sc;

		/* Check to see if there is room in the tty buffer for
		 * the characters in our BD buffer.  If not, we exit
		 * now, leaving the BD with the characters.  We'll pick
		 * them up again on the next receive interrupt (which could
		 * be a timeout).
		 */
		if ((tty->flip.count + i) >= TTY_FLIPBUF_SIZE) {
#if 0
			bdp->cbd_sc |= BD_SC_EMPTY;
			bdp->cbd_sc &=
			~(BD_SC_BR | BD_SC_FR | BD_SC_PR | BD_SC_OV);
			if (bdp->cbd_sc & BD_SC_WRAP)
				bdp = info->rx_bd_base;
			else
				bdp++;
			continue;
#else
			break;
#endif
		}

		if ((tty->flip.count + i) >= tty->ldisc.receive_room(tty)) {
#if 0
			bdp->cbd_sc |= BD_SC_EMPTY;
			bdp->cbd_sc &=
			~(BD_SC_BR | BD_SC_FR | BD_SC_PR | BD_SC_OV);
			if (bdp->cbd_sc & BD_SC_WRAP)
				bdp = info->rx_bd_base;
			else
				bdp++;
			continue;
#else
			break;
#endif
		}
#if defined(CONFIG_TS1H)
		if ( info->frame_cfg.status == FRAMEMODE_ON ) { //surely it is RTU
			short aux = i + 2;
			
			*(short *) tty->flip.char_buf_ptr = aux;
			*(short *) tty->flip.flag_buf_ptr = (short) 0;
			
			tty->flip.flag_buf_ptr += 2;
			tty->flip.char_buf_ptr += 2;
			tty->flip.count += 2;
			info->frame_cfg.rx_stat.success++;
		}
#endif
		
		while (i-- > 0) {
			ch = *cp++;
			*tty->flip.char_buf_ptr = ch;
			icount->rx++;

#ifdef SERIAL_DEBUG_INTR
			printk("DR%02x:%02x...", ch, status);
#endif

			*tty->flip.flag_buf_ptr = 0;
			if (status & (BD_SC_BR | BD_SC_FR |
				       BD_SC_PR | BD_SC_OV)) {
				/*
				 * For statistics only
				 */
				if (status & BD_SC_BR)
					icount->brk++;
				else if (status & BD_SC_PR)
					icount->parity++;
				else if (status & BD_SC_FR)
					icount->frame++;
				if (status & BD_SC_OV)
					icount->overrun++;

				status &= info->read_status_mask;
		
				if (status & (BD_SC_BR)) {
#ifdef SERIAL_DEBUG_INTR
					printk("handling break....");
#endif
					*tty->flip.flag_buf_ptr = TTY_BREAK;
					if (info->flags & ASYNC_SAK)
						do_SAK(tty);
				} else if (status & BD_SC_PR)
					*tty->flip.flag_buf_ptr = TTY_PARITY;
				else if (status & BD_SC_FR)
					*tty->flip.flag_buf_ptr = TTY_FRAME;
				if (status & BD_SC_OV) {
					/*
					 * Overrun is special, since it's
					 * reported immediately, and doesn't
					 * affect the current character
					 */
					if (tty->flip.count < TTY_FLIPBUF_SIZE) {
						tty->flip.count++;
						tty->flip.flag_buf_ptr++;
						tty->flip.char_buf_ptr++;
						*tty->flip.flag_buf_ptr =
								TTY_OVERRUN;
					}
				}
			}
			if (tty->flip.count >= TTY_FLIPBUF_SIZE)
				break;

			tty->flip.flag_buf_ptr++;
			tty->flip.char_buf_ptr++;
			tty->flip.count++;
		}

		/* This BD is ready to be used again.  Clear status.
		 * Get next BD.
		 */
		bdp->cbd_sc |= BD_SC_EMPTY;
		bdp->cbd_sc &= ~(BD_SC_BR | BD_SC_FR | BD_SC_PR | BD_SC_OV);

		if (bdp->cbd_sc & BD_SC_WRAP)
			bdp = info->rx_bd_base;
		else
			bdp++;

	}

	info->rx_cur = (cbd_t *)bdp;

	queue_task(&tty->flip.tqueue, &tq_timer);
}

static _INLINE_ void receive_break(ser_info_t *info)
{
	struct tty_struct *tty = info->tty;

	info->state->icount.brk++;
	/* Check to see if there is room in the tty buffer for
	 * the break.  If not, we exit now, losing the break.  FIXME
	 */
	if ((tty->flip.count + 1) >= TTY_FLIPBUF_SIZE)
		return;
	*(tty->flip.flag_buf_ptr++) = TTY_BREAK;
	*(tty->flip.char_buf_ptr++) = 0;
	tty->flip.count++;

	queue_task(&tty->flip.tqueue, &tq_timer);
}

#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
static _INLINE_ void transmit_chars(volatile ser_info_t *info)
{
	volatile cbd_t *bdp;
	int aux; 

	if (!info->tty) return;
	
	if ((info->flags & ASYNC_LEGACY_RTS) && info->bdtx) { // Half
		bdp = info->bdtx;
		if (!info->media && (bdp->cbd_sc & BD_SC_CD)) { // CTS Lost
			bdp->cbd_sc |= BD_SC_READY | BD_SC_CR;
			return;
		}
		// verify next BD 	
		if (bdp->cbd_sc & BD_SC_WRAP)
			bdp = info->tx_bd_base;
		else
			bdp++;
		
		if ((bdp != info->tx_cur) && bdp->cbd_datlen) {
			info->bdtx->cbd_datlen = 0;
			info->bdtx = bdp;
			bdp->cbd_sc |= BD_SC_READY | BD_SC_CR; //transmit next
		} else {
			info->bdtx = 0;
			info->MCR &= ~UART_MCR_RTS; //RTS down
			serial_out(info,UART_MCR, info->MCR);
		}
	}
	
	if ((info->flags & TX_WAKEUP) ||
	    (info->tty->flags & (1 << TTY_DO_WRITE_WAKEUP))) {
		rs_sched_event(info, RS_EVENT_WRITE_WAKEUP);
	}
}

#else // defined(CONFIG_TS1H) || defined(CONFIG_SM100)

static _INLINE_ void transmit_chars(ser_info_t *info)
{
	if (!info->tty) return;
	
	if ((info->flags & TX_WAKEUP) ||
	    (info->tty->flags & (1 << TTY_DO_WRITE_WAKEUP))) {
		rs_sched_event(info, RS_EVENT_WRITE_WAKEUP);
	}

#ifdef SERIAL_DEBUG_INTR
	printk("THRE...");
#endif
}
#endif // defined TS100 or SM100

#ifdef notdef
	/* I need to do this for the SCCs, so it is left as a reminder.
	*/
static _INLINE_ void check_modem_status(ser_info_t *info)
{
	int	status;
	struct	async_icount *icount;
	
	status = serial_in(info, UART_MSR);

#if !defined(CONFIG_TS1H) && !defined(CONFIG_SM100)
	if (status & UART_MSR_ANY_DELTA) {
		icount = &info->state->icount;
		/* update input line counters */
		if (status & UART_MSR_TERI)
			icount->rng++;
		if (status & UART_MSR_DDSR)
			icount->dsr++;
		if (status & UART_MSR_DDCD) {
			icount->dcd++;
#ifdef CONFIG_HARD_PPS
			if ((info->flags & ASYNC_HARDPPS_CD) &&
			    (status & UART_MSR_DCD))
				hardpps();
#endif
		}
		if (status & UART_MSR_DCTS)
			icount->cts++;
		wake_up_interruptible(&info->delta_msr_wait);
	}
#endif

	if ((info->flags & ASYNC_CHECK_CD) && (status & UART_MSR_DDCD)) {
		if (status & UART_MSR_DCD) { // DCD was OFF, now is ON
#ifdef SERIAL_DEBUG_INTR
			printk("ttys%d CD now %s...", info->line, (status & UART_MSR_DCD) ? "on" : "off");
#endif		
			wake_up_interruptible(&info->open_wait);
		}
		else if (!((info->flags & ASYNC_CALLOUT_ACTIVE) &&
			   (info->flags & ASYNC_CALLOUT_NOHUP))) {
#ifdef SERIAL_DEBUG_INTR
			printk("ttys%d CD now %s...", info->line, (status & UART_MSR_DCD) ? "on" : "off");
			printk("scheduling hangup...");
#endif
			MOD_INC_USE_COUNT;
#if !defined(CONFIG_TS1H) && !defined(CONFIG_SM100)
			if (schedule_task(&info->tqueue_hangup) == 0)
				MOD_DEC_USE_COUNT;
#else
			do_serial_hangup(info);
#endif
		}
	}

#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
	if ((info->flags & ASYNC_LEGACY_CTS) &&
	    (status & UART_MSR_DCTS)) {
		if (status & UART_MSR_CTS) { // CTS UP
			info->hw_stopped = 1;
			if (!info->bdtx && 
			    (serial_in(info,UART_MCR) & UART_MCR_RTS)) {
				info->MCR &= ~UART_MCR_RTS; // RTS OFF
				serial_out(info,UART_MCR,info->MCR);
			}
		} else { // CTS down
			info->hw_stopped = 0;
			if ((info->flags & TX_WAKEUP) ||
	    		    (info->tty->flags & (1 << TTY_DO_WRITE_WAKEUP))) {
				rs_sched_event(info, RS_EVENT_WRITE_WAKEUP);
			}
		}
	}
#endif
	
#if !defined(CONFIG_TS1H) && !defined(CONFIG_SM100)
	if (info->flags & ASYNC_CTS_FLOW) {
		if (info->tty->hw_stopped) {
			if (status & UART_MSR_CTS) {
#if (defined(SERIAL_DEBUG_INTR) || defined(SERIAL_DEBUG_FLOW))
				printk("CTS tx start...");
#endif
				info->tty->hw_stopped = 0;
				info->IER |= UART_IER_THRI;
				serial_out(info, UART_IER, info->IER);
				rs_sched_event(info, RS_EVENT_WRITE_WAKEUP);
				return;
			}
		} else {
			if (!(status & UART_MSR_CTS)) {
#if (defined(SERIAL_DEBUG_INTR) || defined(SERIAL_DEBUG_FLOW))
				printk("CTS tx stop...");
#endif
				info->tty->hw_stopped = 1;
				info->IER &= ~UART_IER_THRI;
				serial_out(info, UART_IER, info->IER);
			}
		}
	}
#endif
}
#endif	/* notdef */

#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
static void rs_8xx_dcd_interrupt(void *dev_id)
{
	int idx;
	ser_info_t *info;

	info = (ser_info_t *) dev_id;

#ifdef modem_control
	if (!info->media && (info->flags & ASYNC_CHECK_CD)) {
		check_modem_status(info);
	}
#endif	
	info->last_active = jiffies;
}

static void rs_8xx_cts_interrupt(void *dev_id)
{
	ser_info_t *info;

	info = (ser_info_t *) dev_id;

#ifdef modem_control
	if (!info->media && (info->flags & ASYNC_LEGACY_CTS)) {
		check_modem_status(info);
	}
#endif	
	info->last_active = jiffies;
}
#endif

/*
 * This is the serial driver's interrupt routine for a single port
 */
static void rs_8xx_interrupt(void *dev_id)
{
	u_char	events;
	int	idx;
	ser_info_t *info;
	volatile smc_t	*smcp;
	volatile scc_t	*sccp;
	
	info = (ser_info_t *)dev_id;

	idx = PORT_NUM(info->state->smc_scc_num);

	if (info->state->smc_scc_num & NUM_IS_SCC) {
		sccp = &cpmp->cp_scc[idx];
		events = sccp->scc_scce;
		sccp->scc_scce = events;

		if (events & (SCCM_RX | SCCM_BSY)) {
#if defined(CONFIG_TS1H)
			if ( info->frame_cfg.status == FRAMEMODE_ON  &&
				info->frame_cfg.mode == FRAMEMODE_ASCII ) {
				receive_frame(info);
			} else
#endif
				receive_chars(info);
		}
		if (events & SCCM_TX) {
#ifdef CONFIG_TS1H
			if ((info->frame_cfg.status == FRAMEMODE_ON) &&
				(info->frame_cfg.mode == FRAMEMODE_RTU)) {
				long rtui;
				
				do_gettimeofday(&rtuend);
				
				// (FRAMEMODE_IDLEWORDS + 0,5) chars at 8N1 last "rtui" microseconds
				rtui = (FRAMEMODE_IDLEWORDS * 10 + 5) * 1042 * 960 /
					baud_table[info->tty->termios->c_cflag & CBAUD];
				
				if ((rtuend.tv_usec += rtui) > 999999) {
					rtuend.tv_usec -= 1000000;
					rtuend.tv_sec++;
				}					
			}
#endif
			transmit_chars(info);
		}

		//sccp->scc_scce = events;
	}
	else {
		smcp = &cpmp->cp_smc[idx];
		events = smcp->smc_smce;
		smcp->smc_smce = events;

		if (events & SMCM_BRKE)
			receive_break(info);
		if (events & (SCCM_RX | SCCM_BSY))
			receive_chars(info);
		if (events & SMCM_TX)
			transmit_chars(info);

		//smcp->smc_smce = events;
	}

#ifdef SERIAL_DEBUG_INTR
	printk("rs_interrupt_single(%d, %x)...",
					info->state->smc_scc_num, events);
#endif
#ifdef modem_control
#ifdef CONFIG_SM100
	if (info->state->smc_scc_num & NUM_IS_SCC)
#endif
		if (!info->media) check_modem_status(info); //RS232
#endif
	info->last_active = jiffies;
}


/*
 * -------------------------------------------------------------------
 * Here ends the serial interrupt routines.
 * -------------------------------------------------------------------
 */

/*
 * This routine is used to handle the "bottom half" processing for the
 * serial driver, known also the "software interrupt" processing.
 * This processing is done at the kernel interrupt level, after the
 * rs_interrupt() has returned, BUT WITH INTERRUPTS TURNED ON.  This
 * is where time-consuming activities which can not be done in the
 * interrupt driver proper are done; the interrupt driver schedules
 * them using rs_sched_event(), and they get done here.
 */
static void do_serial_bh(void)
{
	run_task_queue(&tq_serial);
}

static void do_softint(void *private_)
{
	ser_info_t	*info = (ser_info_t *) private_;
	struct tty_struct	*tty;
	
	tty = info->tty;
	if (!tty)
		return;

	if (test_and_clear_bit(RS_EVENT_WRITE_WAKEUP, &info->event)) {
		if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) &&
		    tty->ldisc.write_wakeup)
			(tty->ldisc.write_wakeup)(tty);
		wake_up_interruptible(&tty->write_wait);
	}
}

/*
 * This routine is called from the scheduler tqueue when the interrupt
 * routine has signalled that a hangup has occurred.  The path of
 * hangup processing is:
 *
 * 	serial interrupt routine -> (scheduler tqueue) ->
 * 	do_serial_hangup() -> tty->hangup() -> rs_hangup()
 * 
 */
static void do_serial_hangup(void *private_)
{
	ser_info_t *info = (ser_info_t *) private_;
	struct tty_struct	*tty;
	
#ifdef SERIAL_DEBUG_INTR
	printk("do_serial_hangup\n");
#endif
	tty = info->tty;
	if (tty)
		tty_hangup(tty);
	MOD_DEC_USE_COUNT;
}

static int startup(ser_info_t *info)
{
	unsigned long flags;
	int	retval=0;
	int	idx;
	struct serial_state *state= info->state;
	volatile smc_t		*smcp;
	volatile scc_t		*sccp;
	volatile smc_uart_t	*up;
	volatile scc_uart_t	*scup;
#ifdef CPM_UART_HANDSHAKING
	volatile immap_t	*immap;
#endif

	save_flags(flags); cli();

	if (info->flags & ASYNC_INITIALIZED) {
		goto errout1;
	}

#ifdef maybe
	if (!state->port || !state->type) {
		if (info->tty)
			set_bit(TTY_IO_ERROR, &info->tty->flags);
		goto errout;
	}
#endif

#ifdef SERIAL_DEBUG_OPEN
	printk("starting up ttys%d (irq %d)...", info->line, state->irq);
#endif

#ifdef modem_control
#ifdef CONFIG_SM100
	if (info->state->smc_scc_num & NUM_IS_SCC)
#endif	//sm100
	{
		info->MCR = 0;
		info->MSR = 0;
		if (info->tty->termios->c_cflag & CBAUD)
			info->MCR = UART_MCR_DTR | UART_MCR_RTS;
	}
#endif
#ifdef CPM_UART_HANDSHAKING
	immap = (immap_t *)IMAP_ADDR;
	idx = PORT_NUM(state->smc_scc_num);
	if (state->smc_scc_num & NUM_IS_SCC) {
	//az - for full HW flow control
#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
		immap->im_cpm.cp_pbdat &= ~(1 << (31 - CONFIG_RTS1_PIN));
		info->MCR |= UART_MCR_RTS;
#endif
		if (info->tty->termios->c_cflag & CBAUD) {
#ifdef CONFIG_UART_DTR_CONTROL_SCC1
			if ( idx == 0 )
				PORT_DTR1_DAT &= ~DTR1_PIN;
#endif
#ifdef CONFIG_UART_DTR_CONTROL_SCC2
			if ( idx == 1 )
				PORT_DTR2_DAT &= ~DTR2_PIN;
#endif
#ifdef CONFIG_UART_DTR_CONTROL_SCC3
			if ( idx == 2 )
				PORT_DTR3_DAT &= ~DTR3_PIN;
#endif
#ifdef CONFIG_UART_DTR_CONTROL_SCC4
			if ( idx == 3 )
				PORT_DTR4_DAT &= ~DTR4_PIN;
#endif
                }
        }
#endif
	
	if (info->tty)
		clear_bit(TTY_IO_ERROR, &info->tty->flags);

	/*
	 * and set the speed of the serial port
	 */
	change_speed(info);
	
errout1:
	idx = PORT_NUM(info->state->smc_scc_num);
	if (info->state->smc_scc_num & NUM_IS_SCC) {
		sccp = &cpmp->cp_scc[idx];
		scup = (scc_uart_t *)&cpmp->cp_dparam[state->port];
#if defined(CONFIG_TS1H)
		scup->scc_genscc.scc_mrblr = CONFIG_SCC1_UART_RX_BDSIZE;
		scup->scc_maxidl = CONFIG_SCC1_UART_RX_BDSIZE;
#else
		scup->scc_genscc.scc_mrblr = rx_bd_sizes[idx];
		scup->scc_maxidl = rx_bd_sizes[idx];
#endif

#ifdef CONFIG_UART_MAXIDL_SCC1
		if ( idx == 0 )
			scup->scc_maxidl = CONFIG_UART_MAXIDL_SCC1;
#endif
#ifdef CONFIG_UART_MAXIDL_SCC2
		if ( idx == 1 )
			scup->scc_maxidl = CONFIG_UART_MAXIDL_SCC2;
#endif
#ifdef CONFIG_UART_MAXIDL_SCC3
		if ( idx == 2 )
			scup->scc_maxidl = CONFIG_UART_MAXIDL_SCC3;
#endif
#ifdef CONFIG_UART_MAXIDL_SCC4
		if ( idx == 3 )
			scup->scc_maxidl = CONFIG_UART_MAXIDL_SCC4;
#endif

		sccp->scc_sccm |= (UART_SCCM_TX | UART_SCCM_RX | UART_SCCM_BSY);
		sccp->scc_gsmrl |= (SCC_GSMRL_ENR | SCC_GSMRL_ENT);
	}
	else {
		smcp = &cpmp->cp_smc[idx];

		/* Enable interrupts and I/O.
		*/
		smcp->smc_smcm |= (SMCM_RX | SMCM_TX | SMCM_BSY);
		smcp->smc_smcmr |= (SMCMR_REN | SMCMR_TEN);

		/* We can tune the buffer length and idle characters
		 * to take advantage of the entire incoming buffer size.
		 * If mrblr is something other than 1, maxidl has to be
		 * non-zero or we never get an interrupt.  The maxidl
		 * is the number of character times we wait after reception
		 * of the last character before we decide no more characters
		 * are coming.
		 */
		up = (smc_uart_t *)&cpmp->cp_dparam[state->port];
		up->smc_mrblr = rx_bd_sizes[idx+4];

		up->smc_maxidl = rx_bd_sizes[idx+4];

#ifdef CONFIG_UART_MAXIDL_SMC2
		if ( idx == 1 )
			up->smc_maxidl = CONFIG_UART_MAXIDL_SMC2;
#endif

		up->smc_brkcr = 1;	/* number of break chars */
	}

	info->flags |= ASYNC_INITIALIZED;
	restore_flags(flags);
	return 0;
	
errout:
	restore_flags(flags);
	return retval;
}

/*
 * This routine will shutdown a serial port; interrupts are disabled, and
 * DTR is dropped if the hangup on close termio flag is on.
 */
static void shutdown(ser_info_t * info)
{
	unsigned long	flags;
	struct serial_state *state;
	int		idx;
	volatile smc_t	*smcp;
	volatile scc_t	*sccp;
#ifdef CPM_UART_HANDSHAKING
	volatile immap_t	*immap;
#endif

	if (!(info->flags & ASYNC_INITIALIZED))
		return;

	state = info->state;

#ifdef SERIAL_DEBUG_OPEN
	printk("Shutting down serial port %d (irq %d)....", info->line, state->irq);
#endif
	
	save_flags(flags); cli(); /* Disable interrupts */

	idx = PORT_NUM(state->smc_scc_num);
	if (state->smc_scc_num & NUM_IS_SCC) {
		sccp = &cpmp->cp_scc[idx];
		sccp->scc_gsmrl &= ~(SCC_GSMRL_ENR | SCC_GSMRL_ENT);
#ifdef CONFIG_SERIAL_CONSOLE
		/* We can't disable the transmitter if this is the
		 * system console.
		 */
		if ((state - rs_table) != CONFIG_SERIAL_CONSOLE_PORT)
#endif
			sccp->scc_sccm &= ~(UART_SCCM_TX | UART_SCCM_RX | UART_SCCM_BSY);
	}
	else {
		smcp = &cpmp->cp_smc[idx];

		/* Disable interrupts and I/O.
		*/
		smcp->smc_smcm &= ~(SMCM_RX | SMCM_TX | SMCM_BSY);
#ifdef CONFIG_SERIAL_CONSOLE
		/* We can't disable the transmitter if this is the
		 * system console.
		 */
		if ((state - rs_table) != CONFIG_SERIAL_CONSOLE_PORT)
#endif
			smcp->smc_smcmr &= ~(SMCMR_REN | SMCMR_TEN);
	}
#ifdef CPM_UART_HANDSHAKING
	immap = (immap_t *)IMAP_ADDR;
	idx = PORT_NUM(state->smc_scc_num);
	if (state->smc_scc_num & NUM_IS_SCC) {
		if (!info->tty || (info->tty->termios->c_cflag & HUPCL)) {
#ifdef CONFIG_UART_DTR_CONTROL_SCC1
			if ( idx == 0 ) {
				PORT_DTR1_DAT |= DTR1_PIN;
				immap->im_cpm.cp_pbdat |= (1 << (31 - CONFIG_RTS1_PIN));
				info->MCR &= ~UART_MCR_RTS;
				//mp, March2003: avoid leaving RTS active
			}
#endif
#ifdef CONFIG_UART_DTR_CONTROL_SCC2
			if ( idx == 1 )
				PORT_DTR2_DAT |= DTR2_PIN;
#endif
#ifdef CONFIG_UART_DTR_CONTROL_SCC3
			if ( idx == 2 )
				PORT_DTR3_DAT |= DTR3_PIN;
#endif
#ifdef CONFIG_UART_DTR_CONTROL_SCC4
			if ( idx == 3 )
				PORT_DTR4_DAT |= DTR4_PIN;
#endif
		}
	}
#endif
	
	if (info->tty)
		set_bit(TTY_IO_ERROR, &info->tty->flags);

	info->flags &= ~ASYNC_INITIALIZED;
	restore_flags(flags);
}

/*
 * This routine is called to set the UART divisor registers to match
 * the specified baud rate for a serial port.
 */
static void change_speed(ser_info_t *info)
{
	int	baud_rate;
	unsigned cflag, cval, scval, prev_mode;
	int	i, bits, sbits, idx;
	unsigned long	flags;
	struct serial_state *state;
	volatile smc_t	*smcp;
	volatile scc_t	*sccp;

	if (!info->tty || !info->tty->termios)
		return;
	cflag = info->tty->termios->c_cflag;

	state = info->state;

	/* Character length programmed into the mode register is the
	 * sum of: 1 start bit, number of data bits, 0 or 1 parity bit,
	 * 1 or 2 stop bits, minus 1.
	 * The value 'bits' counts this for us.
	 */
	cval = 0;
	scval = 0;

#ifdef CPM_UART_HANDSHAKING
	if (state->smc_scc_num & NUM_IS_SCC) {
		idx = PORT_NUM(state->smc_scc_num);
#ifdef CONFIG_UART_CTS_CONTROL_SCC1
		if ( idx == 0 )
			scval = SCU_PMSR_FLC;
#endif
#ifdef CONFIG_UART_CTS_CONTROL_SCC2
		if ( idx == 1 )
			scval = SCU_PMSR_FLC;
#endif
#ifdef CONFIG_UART_CTS_CONTROL_SCC3
		if ( idx == 2 )
			scval = SCU_PMSR_FLC;
#endif
#ifdef CONFIG_UART_CTS_CONTROL_SCC4
		if ( idx == 3 )
			scval = SCU_PMSR_FLC;
#endif
	}
#endif

	/* byte size and parity */
	switch (cflag & CSIZE) {
	      case CS5: bits = 5; break;
	      case CS6: bits = 6; break;
	      case CS7: bits = 7; break;
	      case CS8: bits = 8; break;
	      /* Never happens, but GCC is too dumb to figure it out */
	      default:  bits = 8; break;
	}
	sbits = bits - 5;

	if (cflag & CSTOPB) {
		cval |= SMCMR_SL;	/* Two stop bits */
		scval |= SCU_PMSR_SL;
		bits++;
	}

	if (cflag & PARENB) { // parity selected
		bits++;
		cval |= SMCMR_PEN;
		scval |= SCU_PMSR_PEN;
		if (cflag & PARODD) { // odd parity
			scval |= (SCU_PMSR_RODP | SCU_PMSR_TODP);
		} else {
			cval |= SMCMR_PM_EVEN;
			scval |= (SCU_PMSR_REVP | SCU_PMSR_TEVP);
		}
	}

	/* Determine divisor based on baud rate */
	i = cflag & CBAUD;
	if (i >= (sizeof(baud_table)/sizeof(int)))
		baud_rate = 9600;
	else
		baud_rate = baud_table[i];

	info->timeout = (tx_bd_sizes[ info->state->smc_scc_num & NUM_IS_SCC  ?
	                              PORT_NUM(info->state->smc_scc_num)     :
	                              PORT_NUM(info->state->smc_scc_num) + 4
	                            ]*HZ*bits);
	info->timeout += HZ/50;		/* Add .02 seconds of slop */

#ifdef modem_control
#ifdef CONFIG_SM100
	if (info->state->smc_scc_num & NUM_IS_SCC)
#endif	//sm100
	{
		/* CTS flow control flag and modem status interrupts */
		info->IER &= ~UART_IER_MSI;
		if (info->flags & ASYNC_HARDPPS_CD)
			info->IER |= UART_IER_MSI;
		if (cflag & CRTSCTS) {
			info->flags |= ASYNC_CTS_FLOW;
			info->IER |= UART_IER_MSI;
		} else
			info->flags &= ~ASYNC_CTS_FLOW;
		if (cflag & CLOCAL)
			info->flags &= ~ASYNC_CHECK_CD;
		else {
			info->flags |= ASYNC_CHECK_CD;
			info->IER |= UART_IER_MSI;
		}
		serial_out(info, UART_IER, info->IER);
	}
#endif

	/*
	 * Set up parity check flag
	 */
#define RELEVANT_IFLAG(iflag) (iflag & (IGNBRK|BRKINT|IGNPAR|PARMRK|INPCK))

	info->read_status_mask = (BD_SC_EMPTY | BD_SC_OV);
	if (I_INPCK(info->tty))
		info->read_status_mask |= BD_SC_FR | BD_SC_PR;
	if (I_BRKINT(info->tty) || I_PARMRK(info->tty))
		info->read_status_mask |= BD_SC_BR;
	
	/*
	 * Characters to ignore
	 */
	info->ignore_status_mask = 0;
	if (I_IGNPAR(info->tty))
		info->ignore_status_mask |= BD_SC_PR | BD_SC_FR;
	if (I_IGNBRK(info->tty)) {
		info->ignore_status_mask |= BD_SC_BR;
		/*
		 * If we're ignore parity and break indicators, ignore 
		 * overruns too.  (For real raw support).
		 */
		if (I_IGNPAR(info->tty))
			info->ignore_status_mask |= BD_SC_OV;
	}
	/*
	 * !!! ignore all characters if CREAD is not set
	 */
	if ((cflag & CREAD) == 0)
		info->read_status_mask &= ~BD_SC_EMPTY;

	save_flags(flags); cli();

	/* Start bit has not been added (so don't, because we would just
	 * subtract it later), and we need to add one for the number of
	 * stops bits (there is always at least one).
	 */
	bits++;
	idx = PORT_NUM(state->smc_scc_num);
	if (state->smc_scc_num & NUM_IS_SCC) {
		sccp = &cpmp->cp_scc[idx];
		sccp->scc_pmsr = (sbits << 12) | scval;
	}
	else {
		smcp = &cpmp->cp_smc[idx];

		/* Set the mode register.  We want to keep a copy of the
		 * enables, because we want to put them back if they were
		 * present.
		 */
		prev_mode = smcp->smc_smcmr;
		smcp->smc_smcmr = smcr_mk_clen(bits) | cval |  SMCMR_SM_UART;
		smcp->smc_smcmr |= (prev_mode & (SMCMR_REN | SMCMR_TEN));
	}

	m8xx_cpm_setbrg(rs_brg_map[state - rs_table], baud_rate);

	restore_flags(flags);
}

static void rs_8xx_put_char(struct tty_struct *tty, unsigned char ch)
{
	ser_info_t *info = (ser_info_t *)tty->driver_data;
	volatile cbd_t	*bdp;

	if (serial_paranoia_check(info, tty->device, "rs_put_char"))
		return;

	if (!tty)
		return;

	bdp = info->tx_cur;
	/* Get next BD.
	*/
	if (bdp->cbd_sc & BD_SC_WRAP)
		info->tx_cur = info->tx_bd_base;
	else
		info->tx_cur++;

	while (bdp->cbd_sc & BD_SC_READY);

	*((char *)__va(bdp->cbd_bufaddr)) = ch;
	bdp->cbd_datlen = 1;
#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
	bdp->cbd_sc |= BD_SC_READY | BD_SC_LAST;
#else
	bdp->cbd_sc |= BD_SC_READY;
#endif

#if 0
	/* Get next BD.
	*/
	if (bdp->cbd_sc & BD_SC_WRAP)
		bdp = info->tx_bd_base;
	else
		bdp++;

	info->tx_cur = (cbd_t *)bdp;
#endif

}

#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
void *line_is_idle(ser_info_t *info)
{
	rs_sched_event(info, RS_EVENT_WRITE_WAKEUP);
}
#endif

#if defined(CONFIG_TS1H)
static int rs_8xx_framemode_write(struct tty_struct * tty, int from_user,
		    const unsigned char *buf, int count)
{
	int	c, ret = 0;
	int tx_buf_size;
	ser_info_t *info = (ser_info_t *)tty->driver_data;
	volatile cbd_t *bdp;
	immap_t *immap = (immap_t *)IMAP_ADDR;

#ifdef CONFIG_KGDB
        /* Try to let stub handle output. Returns true if it did. */ 
        if (kgdb_output_string(buf, count))
            return ret;
#endif

	if (serial_paranoia_check(info, tty->device, "rs_write"))
		return 0;

#ifdef SERIAL_DEBUG_WRITE
	printk("rs_write ttys%d, count = %d\n", info->line, count);
#endif
	if (!tty) 
		return 0;

	if (info->media && (info->flags & ASYNC_LEGACY_RTS)) { 
		//RS485-Half Duplex	
		if (!(immap->im_cpm.cp_scc[PORT_NUM(info->state->smc_scc_num)]
			  .scc_sccs & 0x01)) {   /* The line is not idle */

			init_timer(&wait_idle_line);
			wait_idle_line.function = line_is_idle;
			wait_idle_line.data = info;
			wait_idle_line.expires = HZ / 100; //10ms
			add_timer(&wait_idle_line);

			return 0;
		}
	}
	
	if (!info->media && info->hw_stopped) {
		//RS232- CTS ctrl 
		info->flags |= TX_WAKEUP;
		if (!info->bdtx && (serial_in(info,UART_MCR) & UART_MCR_RTS)) {
			cli();
			info->MCR &= ~UART_MCR_RTS; // RTS OFF
			serial_out(info,UART_MCR,info->MCR);
			sti();
		}
		return 0;
	}

	if (info->frame_cfg.mode == FRAMEMODE_RTU) {
		if (rtuend.tv_usec || rtuend.tv_usec) {
			struct timeval time;
			long interval;
			
			do_gettimeofday(&time);
			
			if ((interval = time.tv_sec - rtuend.tv_sec) <= 0) {
				if (!interval) {
					if ((time.tv_usec - rtuend.tv_usec) < 0) {
						// waiting 3,5 chars time...
						return 0;
					}
				}
				else {
					// waiting 3,5 chars time...
					return 0;
				}
			}
		}
	}

	bdp = info->tx_cur;

	tx_buf_size = tx_bd_sizes[ info->state->smc_scc_num & NUM_IS_SCC  ?
	                           PORT_NUM(info->state->smc_scc_num)     :
	                           PORT_NUM(info->state->smc_scc_num) + 4 ];
	
	if (bdp->cbd_sc & BD_SC_READY) {
		info->flags |= TX_WAKEUP;
		return 0;
	}			
	
	if ( count > tx_buf_size ) {
		printk("framemode_write(): tx buffer is too short\n");
		return 0;
	}

	if ( info->frame_cfg.txcount == 0 ) {
		short aux;
		
		if (from_user) {
			if (copy_from_user( (char *) &aux, buf, 2)) {
				if (!ret)
					return -EFAULT;
			}
		} else {
			memcpy( (char *) &aux, buf, 2);
		}
		ret += 2;
		count -= 2;
		buf += 2;
		
		if ( info->frame_cfg.mode == FRAMEMODE_ASCII ) {
			* (char *) __va(bdp->cbd_bufaddr) = 0x3a;	// (:)
			info->frame_cfg.txcount ++;
			bdp->cbd_datlen = aux - 1;
		} else {
			bdp->cbd_datlen = aux - 2;
		}
	} else {
		printk("uart.c:write() - Buffer existing\n");
	}
	c = count;
	
	if (from_user) {
		if (copy_from_user( ((char *) __va(bdp->cbd_bufaddr)) + 
								info->frame_cfg.txcount, buf, c)) {
			if (!ret)
				ret = -EFAULT;
			return ret;;
		}
	} else {
		memcpy(((char *)__va(bdp->cbd_bufaddr)) + info->frame_cfg.txcount, 
						buf, c);
	}

	info->frame_cfg.txcount += c;
	ret += c;

	if ( info->frame_cfg.txcount == bdp->cbd_datlen) {
		if ( info->frame_cfg.mode == FRAMEMODE_ASCII ) {
			* ((char *)__va(bdp->cbd_bufaddr)+info->frame_cfg.txcount) = 0x0d;
			* ((char *)__va(bdp->cbd_bufaddr)+info->frame_cfg.txcount+1) = 0x0a;
			bdp->cbd_datlen += 2;
		}

		if (info->flags & ASYNC_LEGACY_RTS) { // Half
			if (!info->bdtx) {
				cli();
				info->bdtx = bdp;
				info->MCR |= UART_MCR_RTS; // RTS UP
				serial_out(info,UART_MCR,info->MCR);
				bdp->cbd_sc |= (BD_SC_READY | BD_SC_CR);
				sti();
			}
		} else {
			bdp->cbd_sc |= (BD_SC_READY | BD_SC_CR);
		}

		info->frame_cfg.txcount = 0;
		info->frame_cfg.tx_stat.success++;

		/* Get next BD.
		*/
		if (bdp->cbd_sc & BD_SC_WRAP)
			bdp = info->tx_bd_base;
		else
			bdp++;
		info->tx_cur = (cbd_t *)bdp;

	}

	return ret;
}
#endif		// CONFIG_TS1H

static int rs_8xx_write(struct tty_struct * tty, int from_user,
		    const unsigned char *buf, int count)
{
	int	c, ret = 0;
	int tx_buf_size;
	ser_info_t *info = (ser_info_t *)tty->driver_data;
	volatile cbd_t *bdp;
#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
	immap_t *immap = (immap_t *)IMAP_ADDR;
#endif

#ifdef CONFIG_KGDB
        /* Try to let stub handle output. Returns true if it did. */ 
        if (kgdb_output_string(buf, count))
            return ret;
#endif

	if (serial_paranoia_check(info, tty->device, "rs_write"))
		return 0;

#ifdef SERIAL_DEBUG_WRITE
	printk("rs_write ttys%d, info %x count = %d\n", info->line, count);
#endif
	if (!tty) 
		return 0;

#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
	if (info->media && (info->flags & ASYNC_LEGACY_RTS)) { //RS485-halfd
		if (!(immap->im_cpm.cp_scc[PORT_NUM(info->state->smc_scc_num)]
			  .scc_sccs & 0x01)) {   /* The line is not idle */

			init_timer(&wait_idle_line);
			wait_idle_line.function = line_is_idle;
			wait_idle_line.data = info;
			wait_idle_line.expires = HZ / 100; //10ms
			add_timer(&wait_idle_line);

			return 0;
		}
	}

	if (!info->media && info->hw_stopped) {//RS232 CTS ctrl
		info->flags |= TX_WAKEUP;
		if (!info->bdtx && (serial_in(info,UART_MCR) & UART_MCR_RTS)) {
			cli();
			info->MCR &= ~UART_MCR_RTS; // RTS OFF
			serial_out(info,UART_MCR,info->MCR);
			sti();
		}
		return 0;
	}
#endif

	bdp = info->tx_cur;

	tx_buf_size = tx_bd_sizes[ info->state->smc_scc_num & NUM_IS_SCC  ?
	                           PORT_NUM(info->state->smc_scc_num)     :
	                           PORT_NUM(info->state->smc_scc_num) + 4 ];
	while (1) {
		c = MIN(count, tx_buf_size);

		if (c <= 0)
			break;

		if (bdp->cbd_sc & BD_SC_READY) {
			info->flags |= TX_WAKEUP;
			break;
		}

		if (from_user) {
			if (copy_from_user(__va(bdp->cbd_bufaddr), buf, c)) {
				if (!ret)
					ret = -EFAULT;
				break;
			}
		} else {
			memcpy(__va(bdp->cbd_bufaddr), buf, c);
		}

		bdp->cbd_datlen = c;
#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
		if (info->flags & ASYNC_LEGACY_RTS) { // Half
			if (!info->bdtx) {
				cli();
				info->bdtx = bdp;
				info->MCR |= UART_MCR_RTS; // RTS UP
				serial_out(info,UART_MCR,info->MCR);
				bdp->cbd_sc |= (BD_SC_READY | BD_SC_CR);
				sti();
			}
		} else {
			bdp->cbd_sc |= (BD_SC_READY | BD_SC_CR);
		}
#else
		bdp->cbd_sc |= BD_SC_READY;
#endif

		buf += c;
		count -= c;
		ret += c;

		/* Get next BD.
		*/
		if (bdp->cbd_sc & BD_SC_WRAP)
			bdp = info->tx_bd_base;
		else
			bdp++;
		info->tx_cur = (cbd_t *)bdp;
	}
	return ret;
}

static int rs_8xx_write_room(struct tty_struct *tty)
{
	ser_info_t *info = (ser_info_t *)tty->driver_data;
	int	ret;

	if (serial_paranoia_check(info, tty->device, "rs_write_room"))
		return 0;

	if ((info->tx_cur->cbd_sc & BD_SC_READY) == 0) {
		info->flags &= ~TX_WAKEUP;
		ret = tx_bd_sizes[ info->state->smc_scc_num & NUM_IS_SCC  ?
		                   PORT_NUM(info->state->smc_scc_num)     :
		                   PORT_NUM(info->state->smc_scc_num) + 4 ];
	}
	else {
		info->flags |= TX_WAKEUP;
		ret = 0;
	}
	return ret;
}

/* I could track this with transmit counters....maybe later.
*/
static int rs_8xx_chars_in_buffer(struct tty_struct *tty)
{
	ser_info_t *info = (ser_info_t *)tty->driver_data;
				
	if (serial_paranoia_check(info, tty->device, "rs_chars_in_buffer"))
		return 0;
	return 0;
}

static void rs_8xx_flush_buffer(struct tty_struct *tty)
{
	ser_info_t *info = (ser_info_t *)tty->driver_data;
				
	if (serial_paranoia_check(info, tty->device, "rs_flush_buffer"))
		return;

	/* There is nothing to "flush", whatever we gave the CPM
	 * is on its way out.
	 */
	wake_up_interruptible(&tty->write_wait);
	if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) &&
	    tty->ldisc.write_wakeup)
		(tty->ldisc.write_wakeup)(tty);
	info->flags &= ~TX_WAKEUP;
}

/*
 * This function is used to send a high-priority XON/XOFF character to
 * the device
 */
static void rs_8xx_send_xchar(struct tty_struct *tty, char ch)
{
	volatile cbd_t	*bdp;

	ser_info_t *info = (ser_info_t *)tty->driver_data;

	if (serial_paranoia_check(info, tty->device, "rs_send_char"))
		return;

#ifdef SERIAL_DEBUG_WRITE
	printk("rs_putchar ttys%d \n", info->line);
#endif
	bdp = info->tx_cur;
	while (bdp->cbd_sc & BD_SC_READY);

	*((char *)__va(bdp->cbd_bufaddr)) = ch;
	bdp->cbd_datlen = 1;
#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
	bdp->cbd_sc |= BD_SC_READY | BD_SC_CR;
#else
	bdp->cbd_sc |= BD_SC_READY;
#endif

	/* Get next BD.
	*/
	if (bdp->cbd_sc & BD_SC_WRAP)
		bdp = info->tx_bd_base;
	else
		bdp++;

	info->tx_cur = (cbd_t *)bdp;
}

/*
 * ------------------------------------------------------------
 * rs_throttle()
 * 
 * This routine is called by the upper-layer tty layer to signal that
 * incoming characters should be throttled.
 * ------------------------------------------------------------
 */
static void rs_8xx_throttle(struct tty_struct * tty)
{
	ser_info_t *info = (ser_info_t *)tty->driver_data;
#ifdef SERIAL_DEBUG_THROTTLE
	char	buf[64];
	
	printk("throttle %s: %d....\n", _tty_name(tty, buf),
	       tty->ldisc.chars_in_buffer(tty));
#endif

	if (serial_paranoia_check(info, tty->device, "rs_throttle"))
		return;
	
	cli();

	if (I_IXOFF(tty))
		rs_8xx_send_xchar(tty, STOP_CHAR(tty));

#ifdef modem_control
#ifdef CONFIG_SM100
	if (info->state->smc_scc_num & NUM_IS_SCC)
#endif
	{
		if (tty->termios->c_cflag & CRTSCTS) 
			info->MCR &= ~UART_MCR_RTS;
		serial_out(info, UART_MCR, info->MCR);
	}
#endif
	sti();
}

static void rs_8xx_unthrottle(struct tty_struct * tty)
{
	ser_info_t *info = (ser_info_t *)tty->driver_data;
#ifdef SERIAL_DEBUG_THROTTLE
	char	buf[64];
	
	printk("unthrottle %s: %d....\n", _tty_name(tty, buf),
	       tty->ldisc.chars_in_buffer(tty));
#endif

	if (serial_paranoia_check(info, tty->device, "rs_unthrottle"))
		return;
		
	cli();
	
	receive_chars(info);

	if (I_IXOFF(tty))
		rs_8xx_send_xchar(tty, START_CHAR(tty));
	
#ifdef modem_control
#ifdef CONFIG_SM100
	if (info->state->smc_scc_num & NUM_IS_SCC)
#endif
	{
		if (tty->termios->c_cflag & CRTSCTS)
			info->MCR |= UART_MCR_RTS;
		serial_out(info, UART_MCR, info->MCR);
	}
#endif
	sti();
}

/*
 * ------------------------------------------------------------
 * rs_ioctl() and friends
 * ------------------------------------------------------------
 */

#ifdef maybe
/*
 * get_lsr_info - get line status register info
 *
 * Purpose: Let user call ioctl() to get info when the UART physically
 * 	    is emptied.  On bus types like RS485, the transmitter must
 * 	    release the bus after transmitting. This must be done when
 * 	    the transmit shift register is empty, not be done when the
 * 	    transmit holding register is empty.  This functionality
 * 	    allows an RS485 driver to be written in user space. 
 */
static int get_lsr_info(ser_info_t * info, unsigned int *value)
{
	unsigned char status;
	unsigned int result;

	cli();
	status = serial_in(info, UART_LSR);
	sti();
	result = ((status & UART_LSR_TEMT) ? TIOCSER_TEMT : 0);
	return put_user(result,value);
}
#endif	/* maybe */

int get_modem_info(ser_info_t *info, unsigned int *value)
{
	unsigned int status = 0;
#ifdef modem_control
#ifdef CONFIG_SM100
	if (info->state->smc_scc_num & NUM_IS_SCC)
#endif
	{
		unsigned char control;

		control = info->MCR;
		cli();
		status = serial_in(info, UART_MSR);
		sti();
		status =  ((control & UART_MCR_RTS) ? TIOCM_RTS : 0)
			| ((control & UART_MCR_DTR) ? TIOCM_DTR : 0)
#ifdef TIOCM_OUT1
			| ((control & UART_MCR_OUT1) ? TIOCM_OUT1 : 0)
			| ((control & UART_MCR_OUT2) ? TIOCM_OUT2 : 0)
#endif
			| ((status  & UART_MSR_DCD) ? TIOCM_CAR : 0)
			| ((status  & UART_MSR_RI) ? TIOCM_RNG : 0)
			| ((status  & UART_MSR_DSR) ? TIOCM_DSR : 0)
			| ((status  & UART_MSR_CTS) ? TIOCM_CTS : 0);
	}
#endif	/* modem_control */
#ifdef SET_DEBUG_IOCTL
	printk("get_modem_status retval=%08x\n", status);
	print_pin("get_modem_status", (immap_t *)IMAP_ADDR);
#endif
	return put_user(status,value);
}

/* Sending a break is a two step process on the SMC/SCC.  It is accomplished
 * by sending a STOP TRANSMIT command followed by a RESTART TRANSMIT
 * command.  We take advantage of the begin/end functions to make this
 * happen.
 */

static void begin_break(ser_info_t *info)
{
	volatile cpm8xx_t *cp;
	ushort	chan;
	int	idx;

	cp = cpmp;

	idx = PORT_NUM(info->state->smc_scc_num);
	if (info->state->smc_scc_num & NUM_IS_SCC)
		chan = scc_chan_map[idx];
	else
		chan = smc_chan_map[idx];
	cp->cp_cpcr = mk_cr_cmd(chan, CPM_CR_STOP_TX) | CPM_CR_FLG;
	while (cp->cp_cpcr & CPM_CR_FLG);
}

static void end_break(ser_info_t *info)
{
	volatile cpm8xx_t *cp;
	ushort	chan;
	int	idx;

	cp = cpmp;

	idx = PORT_NUM(info->state->smc_scc_num);
	if (info->state->smc_scc_num & NUM_IS_SCC)
		chan = scc_chan_map[idx];
	else
		chan = smc_chan_map[idx];
	cp->cp_cpcr = mk_cr_cmd(chan, CPM_CR_RESTART_TX) | CPM_CR_FLG;
	while (cp->cp_cpcr & CPM_CR_FLG);
}

/*
 * This routine sends a break character out the serial port.
 */
static void send_break(ser_info_t *info, int duration)
{
    // mpeccin:07Fev2003
	// The idea of starting and stopping break with the functions begin_ and
	// end_break() seems to me not correct for the MPC860 because once
	// we STOP_TX the SCC automatically inserts the number of break characters
	// specified by its BRKCR parameter.
    // I tried with a big BRKCR rely on the RESTART_TX to stop sending break, but
    // found out that SCC will not in fact restart TX until all BRKCR characters
    // are sent.
    // So, the only strategy is to program BRKCR in a case by case basis...
	((scc_uart_t *)&cpmp->cp_dparam[info->state->port])->scc_brkcr =
		baud_table[info->tty->termios->c_cflag & CBAUD] / 40;

	current->state = TASK_INTERRUPTIBLE;
#ifdef SERIAL_DEBUG_SEND_BREAK
	printk("rs_send_break(%d) jiff=%lu... ", duration, jiffies);
#endif
	begin_break(info);
	schedule_timeout(duration);
	end_break(info);
#ifdef SERIAL_DEBUG_SEND_BREAK
	printk("done: jiffies=%lu\n", jiffies);
#endif
}

#if 1
void print_pin(char *str, volatile immap_t *immap)
{
printk("%s pcdat=%x, pbdat=%x, padat=%x\n", str, immap->im_ioport.iop_pcdat, immap->im_cpm.cp_pbdat, immap->im_ioport.iop_padat);
printk("%s pcpar=%x, pbpar=%x, papar=%x\n", str, immap->im_ioport.iop_pcpar, immap->im_cpm.cp_pbpar, immap->im_ioport.iop_papar);
printk("%s pcdir=%x, pbdir=%x, padir=%x, pcso=%x, siumcr=%x\n", str, immap->im_ioport.iop_pcdir, immap->im_cpm.cp_pbdir, immap->im_ioport.iop_padir, immap->im_ioport.iop_pcso, immap->im_siu_conf.sc_siumcr);
#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
printk("pmsr=%x\r\n", immap->im_cpm.cp_scc[0].scc_pmsr);
#endif
}
#endif

#if defined(CONFIG_TS1H) || defined(CONFIG_SM100) 
//[RK]Jul/23/03 - Cyclades IOCTL command
static int cyc_set_command(ser_info_t *info, unsigned int * arg)
{
	immap_t *immap = (immap_t *)IMAP_ADDR;
	unsigned long flags;
	unsigned int command, aux, media;

	aux = get_user(command,(unsigned int *) arg);
	if (aux) return aux;

	aux = tty_check_change(info->tty);
	if (aux) return aux;
	
#ifdef CONFIG_TS1H
	if (!((bd_t *)res)->hw_info.serial_itf) { /* console mode */
    		return -EIO;
	}
#endif
#ifdef CONFIG_SM100
	if (!(info->state->smc_scc_num & NUM_IS_SCC)) {
		return -EIO;
	}							
#endif
	if ((command & 0xffff0000) == CYC_GET_MEDIA) {
		//get media value
		media = info->media;
		if (info->flags & ASYNC_LEGACY_RTS) {
			media |= 4;
		}
		if (info->flags & ASYNC_LEGACY_CTS) {
			media |= 8;
		}
		aux = put_user(media, (int *)arg);
		return 0;
	}

	if ((command & 0xffff0000) == CYC_SET_MEDIA) {
		media = command & 0xffff;

		if (media & 1) { // RS485
			info->media = media & 9;
		} else {
			info->media = 0; // RS232
		}
#ifdef CONFIG_TS1H
//[RK]Apr/13/04 - Bug 1557  tty_wait_until_sent(info->tty, 0);
		immap->im_ioport.iop_pcdat |= (1 << 9); // termination OFF
		info->terminator = 0;

		/* PA10=retval:31, PA9=retval:30, PA8=retval:29 */
		/* AB_SEL, C_SEL, D_SEL, respectively */
		immap->im_ioport.iop_padat &=  ~(7 << 5);
		if (media & 1) { // RS485
			immap->im_ioport.iop_padat |=  (media & 7) << 5;

			if (!(info->media & 8)) {
				/* PC6=retval:28 - termination resistor */
				info->terminator = 1;
				immap->im_ioport.iop_pcdat &= ~(1 << 9); // termination ON
			}
		}
#endif
		
		info->flags &= ~(ASYNC_LEGACY_RTS | ASYNC_LEGACY_CTS);
		info->hw_stopped = 0;
		
		if (!info->media && ((media & 0xc) == 0xc)) { 
			// RS232 - RTS/CTS control
			info->flags |= ASYNC_LEGACY_RTS | ASYNC_LEGACY_CTS;
		} else { 
			if ((media & 0x6) == 0x4) { // Halfd - RTS ctrl
				info->flags |= ASYNC_LEGACY_RTS;
			}
		}

#ifdef CONFIG_SM100
		if (!(info->state->smc_scc_num & NUM_IS_SCC)) return -EIO;
#endif

		if (info->flags & ASYNC_LEGACY_RTS) {
			// do RTS down 
			info->MCR &= ~UART_MCR_RTS;
			cli();
			serial_out(info, UART_MCR, info->MCR);
			sti();
			
			if (info->flags & ASYNC_LEGACY_CTS) {
				info->IER |= UART_IER_MSI;
				immap->im_ioport.iop_pcso &= ~(0x10);
				cli();
				serial_out(info, UART_IER, info->IER);
				sti();
				aux = serial_in(info, UART_MSR);
				if (aux & UART_MSR_CTS) {
					info->hw_stopped = 1;
				}
			}
		} else { 
			// Full  ==> do RTS UP
			info->MCR |= UART_MCR_RTS;
			if (!(info->flags & (ASYNC_CHECK_CD|ASYNC_CTS_FLOW))) {
				info->IER &= ~UART_IER_MSI;
			}
			immap->im_ioport.iop_pcso |= 0x10;
			cli();
			serial_out(info, UART_MCR, info->MCR);
			serial_out(info, UART_IER, info->IER);
			sti();
		}
		return 0;
	}
	return -EINVAL;
}
#endif

static int rs_8xx_ioctl(struct tty_struct *tty, struct file * file,
		    unsigned int cmd, unsigned long arg)
{
	int error;
	ser_info_t *info = (ser_info_t *)tty->driver_data;
	int retval;
	struct async_icount cnow, cprev;	/* kernel counter temps */
	struct serial_icounter_struct *p_cuser;	/* user space */

	if (serial_paranoia_check(info, tty->device, "rs_ioctl"))
		return -ENODEV;

	if ((cmd != TIOCMIWAIT) && (cmd != TIOCGICOUNT)) {
		if (tty->flags & (1 << TTY_IO_ERROR))
		    return -EIO;
	}
	
#ifdef SERIAL_DEBUG_IOCTL
	printk("rs_8xx_ioctl: cmd=%x\n", cmd);
#endif

	switch (cmd) {
		case TCSBRK:	/* SVID version: non-zero arg --> no break */
			retval = tty_check_change(tty);
			if (retval)
				return retval;
			tty_wait_until_sent(tty, 0);
			if (signal_pending(current))
				return -EINTR;
			if (!arg) {
				send_break(info, HZ/4);	/* 1/4 second */
				if (signal_pending(current))
					return -EINTR;
			}
			return 0;
		case TCSBRKP:	/* support for POSIX tcsendbreak() */
			retval = tty_check_change(tty);
			if (retval)
				return retval;
			tty_wait_until_sent(tty, 0);
			if (signal_pending(current))
				return -EINTR;
			send_break(info, arg ? arg*(HZ/10) : HZ/4);
			if (signal_pending(current))
				return -EINTR;
			return 0;
		case TIOCSERCONFIG:
			return 0;
		case TIOCSBRK:
			retval = tty_check_change(tty);
			if (retval)
				return retval;
			tty_wait_until_sent(tty, 0);
			begin_break(info);
			return 0;
		case TIOCCBRK:
			retval = tty_check_change(tty);
			if (retval)
				return retval;
			end_break(info);
			return 0;
		case TIOCGSOFTCAR:
			return put_user(C_CLOCAL(tty) ? 1 : 0, (int *) arg);
		case TIOCSSOFTCAR:
			error = get_user(arg, (unsigned int *) arg);
			if (error)
				return error;
			tty->termios->c_cflag =
				((tty->termios->c_cflag & ~CLOCAL) |
				 (arg ? CLOCAL : 0));
			return 0;
		case TIOCMGET:
			return get_modem_info(info, (unsigned int *) arg);
		case TIOCMBIS:
		case TIOCMBIC:
		case TIOCMSET:
			return set_modem_info(info, cmd, (unsigned int *) arg);
#ifdef maybe
		case TIOCSERGETLSR: /* Get line status register */
			return get_lsr_info(info, (unsigned int *) arg);
#endif
		/*
		 * Wait for any of the 4 modem inputs (DCD,RI,DSR,CTS) to change
		 * - mask passed in arg for lines of interest
 		 *   (use |'ed TIOCM_RNG/DSR/CD/CTS for masking)
		 * Caller should use TIOCGICOUNT to see which one it was
		 */
		 case TIOCMIWAIT:
#ifdef modem_control
#ifdef CONFIG_SM100
			if (!(info->state->smc_scc_num & NUM_IS_SCC))
				return 0;
#endif
			cli();
			/* note the counters on entry */
			cprev = info->state->icount;
			sti();
			while (1) {
				interruptible_sleep_on(&info->delta_msr_wait);
				/* see if a signal did it */
				if (signal_pending(current))
					return -ERESTARTSYS;
				cli();
				cnow = info->state->icount; /* atomic copy */
				sti();
				if (cnow.rng == cprev.rng && cnow.dsr == cprev.dsr && 
				    cnow.dcd == cprev.dcd && cnow.cts == cprev.cts)
					return -EIO; /* no change => error */
				if ( ((arg & TIOCM_RNG) && (cnow.rng != cprev.rng)) ||
				     ((arg & TIOCM_DSR) && (cnow.dsr != cprev.dsr)) ||
				     ((arg & TIOCM_CD)  && (cnow.dcd != cprev.dcd)) ||
				     ((arg & TIOCM_CTS) && (cnow.cts != cprev.cts)) ) {
					return 0;
				}
				cprev = cnow;
			}
			/* NOTREACHED */
#else
			return 0;
#endif	/* modem_control */

		/* 
		 * Get counter of input serial line interrupts (DCD,RI,DSR,CTS)
		 * Return: write counters to the user passed counter struct
		 * NB: both 1->0 and 0->1 transitions are counted except for
		 *     RI where only 0->1 is counted.
		 */
		case TIOCGICOUNT:
			cli();
			cnow = info->state->icount;
			sti();
			p_cuser = (struct serial_icounter_struct *) arg;
			error = put_user(cnow.cts, &p_cuser->cts);
			if (error) return error;
			error = put_user(cnow.dsr, &p_cuser->dsr);
			if (error) return error;
			error = put_user(cnow.rng, &p_cuser->rng);
			if (error) return error;
			error = put_user(cnow.dcd, &p_cuser->dcd);
			if (error) return error;
			return 0;

#if defined(CONFIG_TS1H)
		// 	Frame-Mode Support
		case FRAMEMODESCFG:
			{
				struct		serial_state *ser = info->state;
				volatile	cpm8xx_t	 *cp = cpmp;
				volatile	scc_uart_t	 *sup;
				volatile    immap_t      *immap = (immap_t *)IMAP_ADDR;
				volatile	scc_t		 *scp;
				            ushort        chan = scc_chan_map[PORT_NUM(info->state->smc_scc_num)];

				scp = &immap->im_cpm.cp_scc[PORT_NUM(info->state->smc_scc_num)];

				if (ser->smc_scc_num & NUM_IS_SCC) {
					sup = (scc_uart_t *)&cp->cp_dparam[ser->port];
				}

				// Getting the configuration structure
				error = copy_from_user((void *)&info->frame_cfg, 
					(void *) arg, sizeof(struct frame_mode_cfg));
				if (error != 0) {
					printk("    error on copy.from.user\n");
					return error;
				}

				if ( info->frame_cfg.status == FRAMEMODE_ON ) {	
	
					// Resetting framemode configuration
					info->frame_cfg.txcount = 0;
					info->frame_cfg.rxcount = 0;
					info->frame_cfg.frame_started = 0;

					// Changing the write routine...
					tty->driver.write = rs_8xx_framemode_write;

					scp->scc_gsmrl &= ~SCC_GSMRL_ENR;
					// ...and SCC parameters
					sup->scc_genscc.scc_mrblr = FRAMEMODE_FRAMELEN;
					if (info->frame_cfg.mode == FRAMEMODE_RTU) {
						sup->scc_maxidl = FRAMEMODE_IDLEWORDS;
						sup->scc_char1 = 0x8000;
					} else {   // ascii
						sup->scc_maxidl = 0;
						sup->scc_char1 = 0x000A;
					}
					cp->cp_cpcr = mk_cr_cmd(chan, CPM_CR_INIT_RX) | CPM_CR_FLG;
					while (cp->cp_cpcr & CPM_CR_FLG);
					scp->scc_gsmrl |= SCC_GSMRL_ENR;
					
				} else {

					// Changing the write routine...
					tty->driver.write = rs_8xx_write;
					
					scp->scc_gsmrl &= ~SCC_GSMRL_ENR;
					// ...and SCC parameters
					sup->scc_genscc.scc_mrblr = CONFIG_SCC1_UART_RX_BDSIZE;
					sup->scc_maxidl = CONFIG_UART_MAXIDL_SCC1;
					sup->scc_char1 = 0x8000;

					cp->cp_cpcr = mk_cr_cmd(chan, CPM_CR_INIT_RX) | CPM_CR_FLG;
					while (cp->cp_cpcr & CPM_CR_FLG);
					scp->scc_gsmrl |= SCC_GSMRL_ENR;

				}
				//because CPM_CR_INIT_RX causes next buffer descriptor to be scc_rbase
				info->rx_cur = info->rx_bd_base;

				return 0;
			}
		
			case FRAMEMODEGCFG:
				// Getting the configuration structure
				error = copy_to_user((void *) arg, (void *)&info->frame_cfg, 
							sizeof(struct frame_mode_cfg));
				if (error != 0) {
					printk("uart.c:ioctl() - error on copy.from.user\n");
					return error;
				}
				break;
#endif	//TS1H				

#if defined(CONFIG_TS1H) || defined(CONFIG_SM100) 
		case TIOCSERCYC :
			return cyc_set_command(info, (unsigned int *)arg);
#endif
		default:
			return -ENOIOCTLCMD;
		}
	return 0;
}

/* FIX UP modem control here someday......
*/
static void rs_8xx_set_termios(struct tty_struct *tty, struct termios *old_termios)
{
	ser_info_t *info = (ser_info_t *)tty->driver_data;

#ifdef SERIAL_DEBUG_TERMIOS
	printk("rs_8xx_set_termios ttys%d cflag = %x old cflag = %x\n", info->line, tty->termios->c_cflag, old_termios->c_cflag);
#endif
	if (   (tty->termios->c_cflag == old_termios->c_cflag)
	    && (   RELEVANT_IFLAG(tty->termios->c_iflag) 
		== RELEVANT_IFLAG(old_termios->c_iflag)))
	  return;

	change_speed(info);

#ifdef modem_control
#ifdef CONFIG_SM100
	if (!(info->state->smc_scc_num & NUM_IS_SCC))
		return;
#endif
	/* Handle transition to B0 status */
	if ((old_termios->c_cflag & CBAUD) &&
	    !(tty->termios->c_cflag & CBAUD)) {
		info->MCR &= ~(UART_MCR_DTR|UART_MCR_RTS);
		cli();
		serial_out(info, UART_MCR, info->MCR);
		sti();
	}
	
	/* Handle transition away from B0 status */
	if (!(old_termios->c_cflag & CBAUD) &&
	    (tty->termios->c_cflag & CBAUD)) {
		info->MCR |= UART_MCR_DTR;
		if (!tty->hw_stopped ||
		    !(tty->termios->c_cflag & CRTSCTS)) {
			info->MCR |= UART_MCR_RTS;
		}
		cli();
		serial_out(info, UART_MCR, info->MCR);
		sti();
	}
	
	/* Handle turning off CRTSCTS */
	if ((old_termios->c_cflag & CRTSCTS) &&
	    !(tty->termios->c_cflag & CRTSCTS)) {
		tty->hw_stopped = 0;
		rs_8xx_start(tty);
	}
#endif

}

/*
 * ------------------------------------------------------------
 * rs_close()
 * 
 * This routine is called when the serial port gets closed.  First, we
 * wait for the last remaining data to be sent.  Then, we unlink its
 * async structure from the interrupt chain if necessary, and we free
 * that IRQ if nothing is left in the chain.
 * ------------------------------------------------------------
 */
static void rs_8xx_close(struct tty_struct *tty, struct file * filp)
{
	ser_info_t *info = (ser_info_t *)tty->driver_data;
	struct serial_state *state;
	unsigned long	flags;
	int		idx;
	volatile smc_t	*smcp;
	volatile scc_t	*sccp;

	if (!info || serial_paranoia_check(info, tty->device, "rs_close"))
		return;

	state = info->state;
	
	save_flags(flags); cli();
	
	if (tty_hung_up_p(filp)) {
		DBG_CNT("before DEC-hung");
		MOD_DEC_USE_COUNT;
		restore_flags(flags);
		return;
	}
	
#ifdef SERIAL_DEBUG_CLOSE
	printk("rs_close ttys%d, count = %d\n", info->line, state->count);
#endif
	if ((tty->count == 1) && (state->count != 1)) {
		/*
		 * Uh, oh.  tty->count is 1, which means that the tty
		 * structure will be freed.  state->count should always
		 * be one in these conditions.  If it's greater than
		 * one, we've got real problems, since it means the
		 * serial port won't be shutdown.
		 */
		printk("rs_close: bad serial port count; tty->count is 1, "
		       "state->count is %d\n", state->count);
		state->count = 1;
	}
	if (--state->count < 0) {
		printk("rs_close: bad serial port count for ttys%d: %d\n",
		       info->line, state->count);
		state->count = 0;
	}
	if (state->count) {
		DBG_CNT("before DEC-2");
		MOD_DEC_USE_COUNT;
		restore_flags(flags);
		return;
	}
	info->flags |= ASYNC_CLOSING;
	/*
	 * Save the termios structure, since this port may have
	 * separate termios for callout and dialin.
	 */
	if (info->flags & ASYNC_NORMAL_ACTIVE)
		info->state->normal_termios = *tty->termios;
	if (info->flags & ASYNC_CALLOUT_ACTIVE)
		info->state->callout_termios = *tty->termios;
	/*
	 * Now we wait for the transmit buffer to clear; and we notify 
	 * the line discipline to only process XON/XOFF characters.
	 */
	tty->closing = 1;
	if (info->closing_wait != ASYNC_CLOSING_WAIT_NONE)
		tty_wait_until_sent(tty, info->closing_wait);
	/*
	 * At this point we stop accepting input.  To do this, we
	 * disable the receive line status interrupts, and tell the
	 * interrupt driver to stop checking the data ready bit in the
	 * line status register.
	 */
	info->read_status_mask &= ~BD_SC_EMPTY;
	if (info->flags & ASYNC_INITIALIZED) {
		idx = PORT_NUM(info->state->smc_scc_num);
		if (info->state->smc_scc_num & NUM_IS_SCC) {
			sccp = &cpmp->cp_scc[idx];
			sccp->scc_sccm &= ~(UART_SCCM_RX | UART_SCCM_BSY);
			sccp->scc_gsmrl &= ~SCC_GSMRL_ENR;
		}
		else {
			smcp = &cpmp->cp_smc[idx];
			smcp->smc_smcm &= ~(SMCM_RX|SMCM_BSY);
			smcp->smc_smcmr &= ~SMCMR_REN;
		}
		/*
		 * Before we drop DTR, make sure the UART transmitter
		 * has completely drained; this is especially
		 * important if there is a transmit FIFO!
		 */
		rs_8xx_wait_until_sent(tty, info->timeout);
	}
	shutdown(info);
	if (tty->driver.flush_buffer)
		tty->driver.flush_buffer(tty);
	if (tty->ldisc.flush_buffer)
		tty->ldisc.flush_buffer(tty);
	tty->closing = 0;
	info->event = 0;
	info->tty = 0;
	if (info->blocked_open) {
		if (info->close_delay) {
			current->state = TASK_INTERRUPTIBLE;
			schedule_timeout(info->close_delay);
		}
		wake_up_interruptible(&info->open_wait);
	}
	info->flags &= ~(ASYNC_NORMAL_ACTIVE|ASYNC_CALLOUT_ACTIVE|
			 ASYNC_CLOSING);
	wake_up_interruptible(&info->close_wait);

#if defined(CONFIG_TS1H)
	// Disabling frame-mode
	info->frame_cfg.status = FRAMEMODE_OFF;
	info->frame_cfg.txcount = 0;
	info->frame_cfg.rxcount = 0;
	info->frame_cfg.frame_started = 0;

	info->media = 0;
	info->terminator = 0;
	info->bdtx = 0;
#endif
	
	MOD_DEC_USE_COUNT;
	restore_flags(flags);
}

/*
 * rs_wait_until_sent() --- wait until the transmitter is empty
 */
static void rs_8xx_wait_until_sent(struct tty_struct *tty, int timeout)
{
	ser_info_t *info = (ser_info_t *)tty->driver_data;
	unsigned long orig_jiffies, char_time;
	/*int lsr;*/
	volatile cbd_t *bdp;
	int bd_num;
	
	if (serial_paranoia_check(info, tty->device, "rs_wait_until_sent"))
		return;

#ifdef maybe
	if (info->state->type == PORT_UNKNOWN)
		return;
#endif

	orig_jiffies = jiffies;
	/*
	 * Set the check interval to be 1/5 of the estimated time to
	 * send a single character, and make it at least 1.  The check
	 * interval should also be less than the timeout.
	 * 
	 * Note: we have to use pretty tight timings here to satisfy
	 * the NIST-PCTS.
	 */
	char_time = 1;
	if (timeout)
		char_time = MIN(char_time, timeout);
#ifdef SERIAL_DEBUG_RS_WAIT_UNTIL_SENT
	printk("In rs_wait_until_sent(%d) check=%lu...", timeout, char_time);
	printk("jiff=%lu...", jiffies);
#endif

	/* We go through the loop at least once because we can't tell
	 * exactly when the last character exits the shifter.  There can
	 * be at least two characters waiting to be sent after the buffers
	 * are empty.
	 */
	do {
#ifdef SERIAL_DEBUG_RS_WAIT_UNTIL_SENT
		printk("lsr = %d (jiff=%lu)...", lsr, jiffies);
#endif
		current->state = TASK_INTERRUPTIBLE;
/*		current->counter = 0;	 make us low-priority */
		schedule_timeout(char_time);
		if (signal_pending(current))
			break;
		if (timeout && ((orig_jiffies + timeout) < jiffies))
			break;

		/* The 'tx_cur' is really the next buffer to send.  We
		 * have to back up to the previous BD and wait for it
		 * to go.  This isn't perfect, because all this indicates
		 * is the buffer is available.  There are still characters
		 * in the CPM FIFO.
		 */
		bd_num = tx_bd_nums[ info->state->smc_scc_num & NUM_IS_SCC  ?
		                     PORT_NUM(info->state->smc_scc_num)     :
		                     PORT_NUM(info->state->smc_scc_num) + 4 ];
		bdp = info->tx_cur;
		if (bdp == info->tx_bd_base)
			bdp += bd_num - 1;
		else
			bdp--;
	} while (bdp->cbd_sc & BD_SC_READY);
	current->state = TASK_RUNNING;
#ifdef SERIAL_DEBUG_RS_WAIT_UNTIL_SENT
	printk("lsr = %d (jiff=%lu)...done\n", lsr, jiffies);
#endif
}

/*
 * rs_hangup() --- called by tty_hangup() when a hangup is signaled.
 */
static void rs_8xx_hangup(struct tty_struct *tty)
{
	ser_info_t *info = (ser_info_t *)tty->driver_data;
	struct serial_state *state = info->state;
	
	if (serial_paranoia_check(info, tty->device, "rs_hangup"))
		return;

	state = info->state;
	
	rs_8xx_flush_buffer(tty);
	shutdown(info);
	info->event = 0;
	state->count = 0;
	info->flags &= ~(ASYNC_NORMAL_ACTIVE|ASYNC_CALLOUT_ACTIVE);
	info->tty = 0;
	wake_up_interruptible(&info->open_wait);
}

/*
 * ------------------------------------------------------------
 * rs_open() and friends
 * ------------------------------------------------------------
 */
static int block_til_ready(struct tty_struct *tty, struct file * filp,
			   ser_info_t *info)
{
#ifdef DO_THIS_LATER
	DECLARE_WAITQUEUE(wait, current);
#endif
	struct serial_state *state = info->state;
	int		retval;
	int		do_clocal = 0;

	/*
	 * If the device is in the middle of being closed, then block
	 * until it's done, and then try again.
	 */
	if (tty_hung_up_p(filp) ||
	    (info->flags & ASYNC_CLOSING)) {
		if (info->flags & ASYNC_CLOSING)
			interruptible_sleep_on(&info->close_wait);
#ifdef SERIAL_DO_RESTART
		if (info->flags & ASYNC_HUP_NOTIFY)
			return -EAGAIN;
		else
			return -ERESTARTSYS;
#else
		return -EAGAIN;
#endif
	}

	/*
	 * If this is a callout device, then just make sure the normal
	 * device isn't being used.
	 */
	if (tty->driver.subtype == SERIAL_TYPE_CALLOUT) {
		if (info->flags & ASYNC_NORMAL_ACTIVE)
			return -EBUSY;
		if ((info->flags & ASYNC_CALLOUT_ACTIVE) &&
		    (info->flags & ASYNC_SESSION_LOCKOUT) &&
		    (info->session != current->session))
		    return -EBUSY;
		if ((info->flags & ASYNC_CALLOUT_ACTIVE) &&
		    (info->flags & ASYNC_PGRP_LOCKOUT) &&
		    (info->pgrp != current->pgrp))
		    return -EBUSY;
		info->flags |= ASYNC_CALLOUT_ACTIVE;
		return 0;
	}
	
	/*
	 * If non-blocking mode is set, or the port is not enabled,
	 * then make the check up front and then exit.
	 * If this is an SMC port, we don't have modem control to wait
	 * for, so just get out here.
	 */
	if ((filp->f_flags & O_NONBLOCK) ||
	    (tty->flags & (1 << TTY_IO_ERROR)) ||
	    !(info->state->smc_scc_num & NUM_IS_SCC)) {
		if (info->flags & ASYNC_CALLOUT_ACTIVE)
			return -EBUSY;
		info->flags |= ASYNC_NORMAL_ACTIVE;
		return 0;
	}

	if (info->flags & ASYNC_CALLOUT_ACTIVE) {
		if (state->normal_termios.c_cflag & CLOCAL)
			do_clocal = 1;
	} else {
		if (tty->termios->c_cflag & CLOCAL)
			do_clocal = 1;
	}
	
	/*
	 * Block waiting for the carrier detect and the line to become
	 * free (i.e., not in use by the callout).  While we are in
	 * this loop, state->count is dropped by one, so that
	 * rs_close() knows when to free things.  We restore it upon
	 * exit, either normal or abnormal.
	 */
	retval = 0;
#ifdef DO_THIS_LATER
	add_wait_queue(&info->open_wait, &wait);
#ifdef SERIAL_DEBUG_OPEN
	printk("block_til_ready before block: ttys%d, count = %d\n", state->line, state->count);
#endif
	cli();
	if (!tty_hung_up_p(filp)) 
		state->count--;
	sti();
	info->blocked_open++;
	while (1) {
		cli();
		if (!(info->flags & ASYNC_CALLOUT_ACTIVE) &&
		    (tty->termios->c_cflag & CBAUD))
			serial_out(info, UART_MCR,
				   serial_in(info, UART_MCR) |
				   (UART_MCR_DTR | UART_MCR_RTS));
		sti();
		set_current_state(TASK_INTERRUPTIBLE);
		if (tty_hung_up_p(filp) ||
		    !(info->flags & ASYNC_INITIALIZED)) {
#ifdef SERIAL_DO_RESTART
			if (info->flags & ASYNC_HUP_NOTIFY)
				retval = -EAGAIN;
			else
				retval = -ERESTARTSYS;	
#else
			retval = -EAGAIN;
#endif
			break;
		}
		if (!(info->flags & ASYNC_CALLOUT_ACTIVE) &&
		    !(info->flags & ASYNC_CLOSING) &&
		    (do_clocal || (serial_in(info, UART_MSR) &
				   UART_MSR_DCD)))
			break;
		if (signal_pending(current)) {
			retval = -ERESTARTSYS;
			break;
		}
#ifdef SERIAL_DEBUG_OPEN
		printk("block_til_ready blocking: ttys%d, count = %d\n", info->line, state->count);
#endif
		schedule();
	}
	current->state = TASK_RUNNING;
	remove_wait_queue(&info->open_wait, &wait);
	if (!tty_hung_up_p(filp))
		state->count++;
	info->blocked_open--;
#ifdef SERIAL_DEBUG_OPEN
	printk("block_til_ready after blocking: ttys%d, count = %d\n", info->line, state->count);
#endif
#endif /* DO_THIS_LATER */
	if (retval)
		return retval;
	info->flags |= ASYNC_NORMAL_ACTIVE;
	return 0;
}

static int get_async_struct(int line, ser_info_t **ret_info)
{
	struct serial_state *sstate;

	sstate = rs_table + line;
	if (sstate->info) {
		sstate->count++;
		*ret_info = (ser_info_t *)sstate->info;
		return 0;
	}
	else {
		return -ENOMEM;
	}
}

/*
 * This routine is called whenever a serial port is opened.  It
 * enables interrupts for a serial port, linking in its async structure into
 * the IRQ chain.   It also performs the serial-specific
 * initialization for the tty structure.
 */
static int rs_8xx_open(struct tty_struct *tty, struct file * filp)
{
	ser_info_t	*info;
	int 		retval, line;

	line = MINOR(tty->device) - tty->driver.minor_start;
	if ((line < 0) || (line >= NR_PORTS))
		return -ENODEV;
	retval = get_async_struct(line, &info);
	if (retval)
		return retval;
	if (serial_paranoia_check(info, tty->device, "rs_open"))
		return -ENODEV;

#ifdef SERIAL_DEBUG_OPEN
	printk("rs_open %s%d, count = %d\n", tty->driver.name, info->line,
	       info->state->count);
#endif
	tty->driver_data = info;
	info->tty = tty;

	/*
	 * Start up serial port
	 */
	retval = startup(info);
	if (retval)
		return retval;

	MOD_INC_USE_COUNT;
	retval = block_til_ready(tty, filp, info);
	if (retval) {
#ifdef SERIAL_DEBUG_OPEN
		printk("rs_open returning after block_til_ready with %d\n", retval);
#endif
		MOD_DEC_USE_COUNT;
		return retval;
	}

	if ((info->state->count == 1) &&
	    (info->flags & ASYNC_SPLIT_TERMIOS)) {
		if (tty->driver.subtype == SERIAL_TYPE_NORMAL)
			*tty->termios = info->state->normal_termios;
		else 
			*tty->termios = info->state->callout_termios;
		change_speed(info);
	}

	info->session = current->session;
	info->pgrp = current->pgrp;

#ifdef SERIAL_DEBUG_OPEN
	printk("rs_open ttys%d successful...", info->line);
#endif
	return 0;
}

/*
 * /proc fs routines....
 */

static int inline line_info(char *buf, struct serial_state *state)
{
// [Fuji] - Dec/30/02 - Bug #165 fix
	ser_info_t *info = (ser_info_t *)state->info;
#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
	char	stat_buf[30], control, status;
#endif
	int	ret;

	ret = sprintf(buf, "%d: uart:%s port:%X irq:%d",
		      state->line,
		      (state->smc_scc_num & NUM_IS_SCC) ? "SCC" : "SMC",
		      (unsigned int)(state->port), state->irq);

#ifdef maybe
	if (!state->port || (state->type == PORT_UNKNOWN)) {
		ret += sprintf(buf+ret, "\n");
		return ret;
	}
#endif

#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
	/*
	 * Figure out the current RS-232 lines
	 */
	stat_buf[0] = 0;
	stat_buf[1] = 0;
	cli();
	status = serial_in((ser_info_t *)info, UART_MSR);
	control = info ? info->MCR : serial_in((ser_info_t *)info, UART_MCR);
	sti();
	if (control & UART_MCR_RTS)
		strcat(stat_buf, "|RTS");
	if (status & UART_MSR_CTS)
		strcat(stat_buf, "|CTS");
	if (control & UART_MCR_DTR)
		strcat(stat_buf, "|DTR");
	if (status & UART_MSR_DSR)
		strcat(stat_buf, "|DSR");
	if (status & UART_MSR_DCD)
		strcat(stat_buf, "|CD");
	if (status & UART_MSR_RI)
		strcat(stat_buf, "|RI");
#endif	/* defined(CONFIG_TS1H) || defined(CONFIG_SM100) */
	
	if (info && info->tty && info->tty->termios) {
		int i, baud_rate;
		i = info->tty->termios->c_cflag & CBAUD;
		if (i >= (sizeof(baud_table)/sizeof(int)))
			baud_rate = 9600;
		else
			baud_rate = baud_table[i];
		ret += sprintf(buf+ret, " baud:%d", baud_rate);
	}
	
	ret += sprintf(buf+ret, " tx:%d rx:%d",
		      state->icount.tx, state->icount.rx);

	if (state->icount.frame)
		ret += sprintf(buf+ret, " fe:%d", state->icount.frame);
	
	if (state->icount.parity)
		ret += sprintf(buf+ret, " pe:%d", state->icount.parity);
	
	if (state->icount.brk)
		ret += sprintf(buf+ret, " brk:%d", state->icount.brk);	

	if (state->icount.overrun)
		ret += sprintf(buf+ret, " oe:%d", state->icount.overrun);

	/*
	 * Last thing is the RS-232 status lines
	 */

#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
	ret += sprintf(buf+ret, " %s", stat_buf+1);

#endif	/* defined(CONFIG_TS1H) || defined(CONFIG_SM100) */

	ret += sprintf(buf+ret, "\n");
	return ret;
}

int rs_8xx_read_proc(char *page, char **start, off_t off, int count,
		 int *eof, void *data)
{
	int i, len;
	off_t	begin = 0;

	len = sprintf(page, "serinfo:1.0 driver:%s\n", serial_version);

	for (i = 0; i < NR_PORTS && len < 3900; i++) {
		len += line_info(page + len, &rs_table[i]);
		if (len + begin > count + off) {
			goto done;
		}
		if (len + begin < off) {
			begin += len;
			len = 0;
		}
	}
	*eof = 1;
done:
	if (len + begin < off) {
		return 0;
	}
	*start = page + (off - begin);
	return (count < len - (off - begin)) ? count : len - (off - begin);
}

/*
 * ---------------------------------------------------------------------
 * rs_init() and friends
 *
 * rs_init() is called at boot-time to initialize the serial driver.
 * ---------------------------------------------------------------------
 */

/*
 * This routine prints out the appropriate serial driver version
 * number, and identifies which options were configured into this
 * driver.
 */
static _INLINE_ void show_serial_version(void)
{
 	printk(KERN_INFO "%s version %s\n", serial_name, serial_version);
}


/*
 * The serial console driver used during boot.  Note that these names
 * clash with those found in "serial.c", so we currently can't support
 * the 16xxx uarts and these at the same time.  I will fix this to become
 * an indirect function call from tty_io.c (or something).
 */

#ifdef CONFIG_SERIAL_CONSOLE

/*
 * Print a string to the serial port trying not to disturb any possible
 * real use of the port...
 */
static void my_console_write(int idx, const char *s,
				unsigned count)
{
	struct		serial_state	*ser;
	ser_info_t			*info;
	unsigned			i;
	volatile	cbd_t		*bdp, *bdbase;
	volatile	smc_uart_t	*up;
	volatile	scc_uart_t	*sup;
	volatile	u_char		*cp;

	ser = rs_table + idx;

	/* If the port has been initialized for general use, we have
	 * to use the buffer descriptors allocated there.  Otherwise,
	 * we simply use the single buffer allocated.
	 */
	if ((info = (ser_info_t *)ser->info) != NULL) {
		bdp = info->tx_cur;
		bdbase = info->tx_bd_base;
	} else {
		/* Pointer to UART in parameter ram.
		*/
		if (ser->smc_scc_num & NUM_IS_SCC) {
			sup = (scc_uart_t *)&cpmp->cp_dparam[ser->port];
			bdp = bdbase = (cbd_t *)&cpmp->cp_dpmem[sup->scc_genscc.scc_tbase];
		} else {
			up = (smc_uart_t *)&cpmp->cp_dparam[ser->port];

			/* Get the address of the host memory buffer.
			 */
			bdp = bdbase = (cbd_t *)&cpmp->cp_dpmem[up->smc_tbase];
		}
	}

	/*
	 * We need to gracefully shut down the transmitter, disable
	 * interrupts, then send our bytes out.
	 */

	/*
	 * Now, do each character.  This is not as bad as it looks
	 * since this is a holding FIFO and not a transmitting FIFO.
	 * We could add the complexity of filling the entire transmit
	 * buffer, but we would just wait longer between accesses......
	 */
	for (i = 0; i < count; i++, s++) {
		/* Wait for transmitter fifo to empty.
		 * Ready indicates output is ready, and xmt is doing
		 * that, not that it is ready for us to send.
		 */
		while (bdp->cbd_sc & BD_SC_READY);

		/* Send the character out.
		 * If the buffer address is in the CPM DPRAM, don't
		 * convert it.
		 */
		if ((uint)(bdp->cbd_bufaddr) > (uint)IMAP_ADDR)
			cp = (u_char *)(bdp->cbd_bufaddr);
		else
			cp = __va(bdp->cbd_bufaddr);
		*cp = *s;
		
		bdp->cbd_datlen = 1;
#ifdef CONFIG_TS1H
		bdp->cbd_sc |= BD_SC_READY | BD_SC_CR;
#else
		bdp->cbd_sc |= BD_SC_READY;
#endif

		if (bdp->cbd_sc & BD_SC_WRAP)
			bdp = bdbase;
		else
			bdp++;

		/* if a LF, also do CR... */
		if (*s == 10) {
			while (bdp->cbd_sc & BD_SC_READY);
			cp = __va(bdp->cbd_bufaddr);
			*cp = 13;
			bdp->cbd_datlen = 1;
#ifdef CONFIG_TS1H
			bdp->cbd_sc |= BD_SC_READY | BD_SC_CR;
#else
			bdp->cbd_sc |= BD_SC_READY;
#endif

			if (bdp->cbd_sc & BD_SC_WRAP) {
				bdp = bdbase;
			}
			else {
				bdp++;
			}
		}
	}

	/*
	 * Finally, Wait for transmitter & holding register to empty
	 *  and restore the IER
	 */
	while (bdp->cbd_sc & BD_SC_READY);

	if (info)
		info->tx_cur = (cbd_t *)bdp;
}

static void serial_console_write(struct console *c, const char *s,
				unsigned count)
{
#ifdef CONFIG_KGDB
	/* Try to let stub handle output. Returns true if it did. */ 
	if (kgdb_output_string(s, count))
		return;
#endif	/* CONFIG_KGDB */
	my_console_write(c->index, s, count);
}

#ifdef CONFIG_XMON
int
xmon_8xx_write(const char *s, unsigned count)
{
	my_console_write(0, s, count);
	return(count);
}
#endif	/* CONFIG_XMON */

#ifdef CONFIG_KGDB
void
putDebugChar(char ch)
{
	my_console_write(0, &ch, 1);
}
#endif	/* CONFIG_KGDB */

/*
 * Receive character from the serial port.  This only works well
 * before the port is initialized for real use.
 */
static int my_console_wait_key(int idx, int xmon, char *obuf)
{
	struct serial_state		*ser;
	u_char				c, *cp;
	ser_info_t			*info;
	volatile	cbd_t		*bdp;
	volatile	smc_uart_t	*up;
	volatile	scc_uart_t	*sup;
	int				i;

	ser = rs_table + idx;

	/* Pointer to UART in parameter ram.
	*/
	if (ser->smc_scc_num & NUM_IS_SCC) {
		sup = (scc_uart_t *)&cpmp->cp_dparam[ser->port];
	} else {
		up = (smc_uart_t *)&cpmp->cp_dparam[ser->port];
	}

	/* Get the address of the host memory buffer.
	 * If the port has been initialized for general use, we must
	 * use information from the port structure.
	 */
	if ((info = (ser_info_t *)ser->info)) {
		bdp = info->rx_cur;
	} else {
		if (ser->smc_scc_num & NUM_IS_SCC) {
			bdp = (cbd_t *)&cpmp->cp_dpmem[sup->scc_genscc.scc_rbase];
		} else {
			bdp = (cbd_t *)&cpmp->cp_dpmem[up->smc_rbase];
		}
	}

	/*
	 * We need to gracefully shut down the receiver, disable
	 * interrupts, then read the input.
	 * XMON just wants a poll.  If no character, return -1, else
	 * return the character.
	 */
	if (!xmon) {
		while (bdp->cbd_sc & BD_SC_EMPTY);
	}
	else {
		if (bdp->cbd_sc & BD_SC_EMPTY)
			return -1;
	}

	/* If the buffer address is in the CPM DPRAM, don't
	 * convert it.
	 */
	if ((uint)(bdp->cbd_bufaddr) > (uint)IMAP_ADDR)
		cp = (u_char *)(bdp->cbd_bufaddr);
	else
		cp = __va(bdp->cbd_bufaddr);

	if (obuf) {
		i = c = bdp->cbd_datlen;
		while (i-- > 0)
			*obuf++ = *cp++;
	}
	else {
		c = *cp;
	}
	bdp->cbd_sc |= BD_SC_EMPTY;

	if (info) {
		if (bdp->cbd_sc & BD_SC_WRAP) {
			bdp = info->rx_bd_base;
		}
		else {
			bdp++;
		}
		info->rx_cur = (cbd_t *)bdp;
	}

	return((int)c);
}

static int serial_console_wait_key(struct console *co)
{
	return(my_console_wait_key(co->index, 0, NULL));
}

#ifdef CONFIG_XMON
int
xmon_8xx_read_poll(void)
{
	return(my_console_wait_key(0, 1, NULL));
}

int
xmon_8xx_read_char(void)
{
	return(my_console_wait_key(0, 0, NULL));
}
#endif	/* CONFIG_XMON */

#ifdef CONFIG_KGDB
static char kgdb_buf[RX_BUF_SIZE], *kgdp;
static int kgdb_chars;

unsigned char
getDebugChar(void)
{
	if (kgdb_chars <= 0) {
		kgdb_chars = my_console_wait_key(0, 0, kgdb_buf);
		kgdp = kgdb_buf;
	}
	kgdb_chars--;

	return(*kgdp++);
}

void kgdb_interruptible(int state)
{
}
void kgdb_map_scc(void)
{
	struct		serial_state *ser;
	uint		mem_addr;
	volatile	cbd_t		*bdp;
	volatile	smc_uart_t	*up;

	cpmp = (cpm8xx_t *)&(((immap_t *)IMAP_ADDR)->im_cpm);

	/* To avoid data cache CPM DMA coherency problems, allocate a
	 * buffer in the CPM DPRAM.  This will work until the CPM and
	 * serial ports are initialized.  At that time a memory buffer
	 * will be allocated.
	 * The port is already initialized from the boot procedure, all
	 * we do here is give it a different buffer and make it a FIFO.
	 */

	ser = rs_table;

	/* Right now, assume we are using SMCs.
	*/
	up = (smc_uart_t *)&cpmp->cp_dparam[ser->port];

	/* Allocate space for an input FIFO, plus a few bytes for output.
	 * Allocate bytes to maintain word alignment.
	 */
	mem_addr = (uint)(&cpmp->cp_dpmem[0x1000]);

	/* Set the physical address of the host memory buffers in
	 * the buffer descriptors.
	 */
	bdp = (cbd_t *)&cpmp->cp_dpmem[up->smc_rbase];
	bdp->cbd_bufaddr = mem_addr;

	bdp = (cbd_t *)&cpmp->cp_dpmem[up->smc_tbase];
	bdp->cbd_bufaddr = mem_addr+RX_BUF_SIZE;

	up->smc_mrblr = RX_BUF_SIZE;		/* receive buffer length */
	up->smc_maxidl = RX_BUF_SIZE;
}
#endif	/* CONFIG_KGDB */

static kdev_t serial_console_device(struct console *c)
{
	return MKDEV(TTY_MAJOR, 64 + c->index);
}


static struct console sercons = {
	name:		"ttyS",
	write:		serial_console_write,
	device:		serial_console_device,
	wait_key:	serial_console_wait_key,
	setup:		serial_console_setup,
	flags:		CON_PRINTBUFFER,
	index:		CONFIG_SERIAL_CONSOLE_PORT,
};

/*
 *	Register console.
 */
long __init console_8xx_init(long kmem_start, long kmem_end)
{
	register_console(&sercons);
	return kmem_start;
}

#endif	/* CONFIG_SERIAL_CONSOLE */

/* Index in baud rate table of the default console baud rate.
*/
static	int	baud_idx;

/*
 * The serial driver boot-time initialization code!
 */
int __init rs_8xx_init(void)
{
	struct serial_state * state;
	ser_info_t	*info;
	uint		mem_addr, dp_addr, iobits;
	int		i, j, idx;
	ushort		chan;
	volatile	cbd_t		*bdp;
	volatile	cpm8xx_t	*cp;
	volatile	smc_t		*sp;
	volatile	smc_uart_t	*up;
	volatile	scc_t		*scp;
	volatile	scc_uart_t	*sup;
	volatile	immap_t		*immap;
	
	/* patch from: Pavel Bartusek <pavel.bartusek@tiscali.cz> */
#ifndef CONFIG_SERIAL
	/* if the serial driver is also selected, SERIAL_BH can't be used */
	init_bh(SERIAL_BH, do_serial_bh);
#endif

	show_serial_version();

	/* Initialize the tty_driver structure */
	
	__clear_user(&serial_driver,sizeof(struct tty_driver));
	serial_driver.magic = TTY_DRIVER_MAGIC;
	serial_driver.driver_name = "serial";
	serial_driver.name = "ttyS";
	serial_driver.major = TTY_MAJOR;
	serial_driver.minor_start = 64;
	serial_driver.num = NR_PORTS;
	serial_driver.type = TTY_DRIVER_TYPE_SERIAL;
	serial_driver.subtype = SERIAL_TYPE_NORMAL;
	serial_driver.init_termios = tty_std_termios;
	serial_driver.init_termios.c_cflag =
		baud_idx | CS8 | CREAD | HUPCL | CLOCAL;
	serial_driver.flags = TTY_DRIVER_REAL_RAW;
	serial_driver.refcount = &serial_refcount;
	serial_driver.table = serial_table;
	serial_driver.termios = serial_termios;
	serial_driver.termios_locked = serial_termios_locked;

	serial_driver.open = rs_8xx_open;
	serial_driver.close = rs_8xx_close;
	serial_driver.write = rs_8xx_write;
	serial_driver.put_char = rs_8xx_put_char;
	serial_driver.write_room = rs_8xx_write_room;
	serial_driver.chars_in_buffer = rs_8xx_chars_in_buffer;
	serial_driver.flush_buffer = rs_8xx_flush_buffer;
	serial_driver.ioctl = rs_8xx_ioctl;
	serial_driver.throttle = rs_8xx_throttle;
	serial_driver.unthrottle = rs_8xx_unthrottle;
	serial_driver.send_xchar = rs_8xx_send_xchar;
	serial_driver.set_termios = rs_8xx_set_termios;
	serial_driver.stop = rs_8xx_stop;
	serial_driver.start = rs_8xx_start;
	serial_driver.hangup = rs_8xx_hangup;
	serial_driver.wait_until_sent = rs_8xx_wait_until_sent;
	serial_driver.read_proc = rs_8xx_read_proc;
	
	/*
	 * The callout device is just like normal device except for
	 * major number and the subtype code.
	 */
	callout_driver = serial_driver;
	callout_driver.name = "cua";
	callout_driver.major = TTYAUX_MAJOR;
	callout_driver.subtype = SERIAL_TYPE_CALLOUT;
	callout_driver.read_proc = 0;
	callout_driver.proc_entry = 0;

	if (tty_register_driver(&serial_driver))
		panic("Couldn't register serial driver\n");
	if (tty_register_driver(&callout_driver))
		panic("Couldn't register callout driver\n");
	
	cp = cpmp;	/* Get pointer to Communication Processor */
	immap = (immap_t *)IMAP_ADDR;	/* and to internal registers */


	/* Configure SCC2, SCC3, and SCC4 instead of port A parallel I/O.
	 */
	iobits = 0
#if defined(CONFIG_SCC1_UART)
		| 0x0003
#endif
#if defined(CONFIG_SCC2_UART)
		| 0x000C
#endif
#if defined(CONFIG_SCC3_UART)	&& \
   !defined(CONFIG_MBX)		&& \
   !defined(CONFIG_IP860)	&& \
   !defined(CONFIG_TQM850L)	&& \
   !defined(CONFIG_SM850)
		| 0x0030
#endif
#if defined(CONFIG_SCC4_UART) && !defined(CONFIG_IP860)
		| 0x00C0
#endif
		;

	/* The "standard" configuration through the 860.
	*/
	immap->im_ioport.iop_papar |=  iobits;
	immap->im_ioport.iop_padir &= ~iobits;
	immap->im_ioport.iop_paodr &= ~iobits;

#if defined(CONFIG_SCC3_UART) && (defined(CONFIG_TQM850L) || defined(CONFIG_SM850))
	/* The MPC850 has SCC3 on Port B */
	cp->cp_pbpar |=  0x06;
	cp->cp_pbdir &= ~0x06;
	cp->cp_pbodr &= ~0x06;
#endif

#if defined(CONFIG_MBX) || defined(CONFIG_IP860)
	/*
	 * On the MBX,   SCC3 is through Port D.
	 * On the IP860, SCC3 and SCC4 are through Port D.
	 */
	iobits = 0
# if defined(CONFIG_SCC3_UART)
		| 0x0030
# endif
# if defined(CONFIG_SCC4_UART)
		| 0x00C0
# endif
		;
	immap->im_ioport.iop_pdpar |= iobits;
#endif	/* CONFIG_MBX, CONFIG_IP860 */

	/* Since we don't yet do modem control, connect the port C pins
	 * as general purpose I/O.  This will assert CTS and CD for the
	 * SCC ports.
	 */
	iobits = 0
#if defined(CONFIG_SCC1_UART)

#if CONFIG_PRxK

#ifndef CONFIG_TS1H
		| 0x0030	/* CD1, CTS1 */
#else
		| 0x0001	/* CD1 INPUT, CTS1 INPUT, RTS1 OUTPUT */
#endif

#endif // CONFIG_PRxK

#endif

#if defined(CONFIG_SCC2_UART)
		| 0x00C2	/* CD2, CTS2, RTS2 */
#endif
#if defined(CONFIG_SCC3_UART)
		| 0x0304	/* CD3, CTS3, RTS3 */
#endif
#if defined(CONFIG_SCC4_UART)
		| 0x0C08	/* CD4, CTS4, RTS4 */
#endif
		;

	immap->im_ioport.iop_pcdir =  iobits;
	immap->im_ioport.iop_pcpar &= ~iobits;

#if defined(CONFIG_PRxK) && defined(CONFIG_SCC1_UART)
    /* Assert DSR using GP I/O.
    */
    iobits = 0x0008;
    immap->im_ioport.iop_pcdir &= ~iobits;
    immap->im_ioport.iop_pcpar &= ~iobits;
#endif

	/* Here we connect handshaking signals CTS, RTS and CD to 
	 * the SCC to be controlled by CPM, and DTR as
	 * a general purpose output to be controlled by software.
	 */
#if defined(CPM_UART_HANDSHAKING)
#if (CONFIG_UART_RTS_CONTROL_SCC1 == PORT_B)
	//az immap->im_cpm.cp_pbpar |= (1 << (31 - CONFIG_RTS1_PIN));
	immap->im_cpm.cp_pbpar &= ~(1 << (31 - CONFIG_RTS1_PIN));
	immap->im_cpm.cp_pbdir |= (1 << (31 - CONFIG_RTS1_PIN));
#endif
#if (CONFIG_UART_RTS_CONTROL_SCC2 == PORT_B)
	immap->im_cpm.cp_pbpar |= (1 << (31 - CONFIG_RTS2_PIN));
	immap->im_cpm.cp_pbdir |= (1 << (31 - CONFIG_RTS2_PIN));
#endif
#if (CONFIG_UART_RTS_CONTROL_SCC3 == PORT_B)
	immap->im_cpm.cp_pbpar |= (1 << (31 - CONFIG_RTS3_PIN));
	immap->im_cpm.cp_pbdir |= (1 << (31 - CONFIG_RTS3_PIN));
#endif
#if (CONFIG_UART_RTS_CONTROL_SCC4 == PORT_B)
	immap->im_cpm.cp_pbpar |= (1 << (31 - CONFIG_RTS4_PIN));
	immap->im_cpm.cp_pbdir |= (1 << (31 - CONFIG_RTS4_PIN));
#endif
#if (CONFIG_UART_RTS_CONTROL_SCC1 == PORT_C)
	immap->im_ioport.iop_pcpar |=  (1 << (15 - CONFIG_RTS1_PIN));
	immap->im_ioport.iop_pcdir &= ~(1 << (15 - CONFIG_RTS1_PIN));
#endif
#if (CONFIG_UART_RTS_CONTROL_SCC2 == PORT_C)
	immap->im_ioport.iop_pcpar |=  (1 << (15 - CONFIG_RTS2_PIN));
	immap->im_ioport.iop_pcdir &= ~(1 << (15 - CONFIG_RTS2_PIN));
#endif
#if (CONFIG_UART_RTS_CONTROL_SCC3 == PORT_C)
	immap->im_ioport.iop_pcpar |=  (1 << (15 - CONFIG_RTS3_PIN));
	immap->im_ioport.iop_pcdir &= ~(1 << (15 - CONFIG_RTS3_PIN));
#endif
#if (CONFIG_UART_RTS_CONTROL_SCC4 == PORT_C)
	immap->im_ioport.iop_pcpar |=  (1 << (15 - CONFIG_RTS4_PIN));
	immap->im_ioport.iop_pcdir &= ~(1 << (15 - CONFIG_RTS4_PIN));
#endif
#if (CONFIG_UART_RTS_CONTROL_SCC3 == PORT_D)
	immap->im_ioport.iop_pdpar |=  (1 << (15 - CONFIG_RTS3_PIN));
#endif
#if (CONFIG_UART_RTS_CONTROL_SCC4 == PORT_D)
	immap->im_ioport.iop_pdpar |=  (1 << (15 - CONFIG_RTS4_PIN));
#endif

        iobits = 0
#ifndef CONFIG_TS1H
#if defined(CONFIG_UART_CTS_CONTROL_SCC1)
		| 0x0010
#endif

#if defined(CONFIG_UART_CD_CONTROL_SCC1)
		| 0x0020
#endif
#endif

#if defined(CONFIG_UART_CTS_CONTROL_SCC2)
		| 0x0040
#endif
#if defined(CONFIG_UART_CD_CONTROL_SCC2)
		| 0x0080
#endif
#if defined(CONFIG_UART_CTS_CONTROL_SCC3)
		| (1 << (15 - CONFIG_CTS3_PIN))
#endif
#if defined(CONFIG_UART_CD_CONTROL_SCC3)
		| (1 << (15 - CONFIG_CD3_PIN))
#endif
#if defined(CONFIG_UART_CTS_CONTROL_SCC4)
		| 0x0400
#endif
#if defined(CONFIG_UART_CD_CONTROL_SCC4)
		| 0x0800
#endif
		;
	immap->im_ioport.iop_pcpar &= ~iobits;
	immap->im_ioport.iop_pcso |=  iobits;
	immap->im_ioport.iop_pcdir &= ~iobits;

#if defined(CONFIG_UART_DTR_CONTROL_SCC1)
	PORT_DTR1_PAR &= ~DTR1_PIN;	/* DTR1 */
	PORT_DTR1_DAT |=  DTR1_PIN;
	PORT_DTR1_DIR |=  DTR1_PIN;
#endif
#if defined(CONFIG_UART_DTR_CONTROL_SCC2)
	PORT_DTR2_PAR &= ~DTR2_PIN;	/* DTR2 */
	PORT_DTR2_DAT |=  DTR2_PIN;
	PORT_DTR2_DIR |=  DTR2_PIN;
#endif
#if defined(CONFIG_UART_DTR_CONTROL_SCC3)
	PORT_DTR3_PAR &= ~DTR3_PIN;	/* DTR3 */
	PORT_DTR3_DAT |=  DTR3_PIN;
	PORT_DTR3_DIR |=  DTR3_PIN;
#endif
#if defined(CONFIG_UART_DTR_CONTROL_SCC4)
	PORT_DTR4_PAR &= ~DTR4_PIN;	/* DTR4 */
	PORT_DTR4_DAT |=  DTR4_PIN;
	PORT_DTR4_DIR |=  DTR4_PIN;
#endif

#endif

#ifdef CONFIG_PP04
	/* Frequentis PP04 forced to RS-232 until we know better.
	 * Port C 12 and 13 low enables RS-232 on SCC3 and SCC4.
	 */
	immap->im_ioport.iop_pcdir |= 0x000c;
	immap->im_ioport.iop_pcpar &= ~0x000c;
	immap->im_ioport.iop_pcdat &= ~0x000c;

	/* This enables the TX driver.
	*/
	cp->cp_pbpar &= ~0x6000;
	cp->cp_pbdat &= ~0x6000;
#endif	/* CONFIG_PP04 */

	immap->im_ioport.iop_pcdir |= 0x0200;  // makes PC6 un output pin

	for (i = 0, state = rs_table; i < NR_PORTS; i++,state++) {
		state->magic = SSTATE_MAGIC;
		state->line = i;
		state->type = PORT_UNKNOWN;
		state->custom_divisor = 0;
		state->close_delay = 5*HZ/10;
		state->closing_wait = 30*HZ;
		state->callout_termios = callout_driver.init_termios;
		state->normal_termios = serial_driver.init_termios;
		state->icount.cts = state->icount.dsr = 
			state->icount.rng = state->icount.dcd = 0;
		state->icount.rx = state->icount.tx = 0;
		state->icount.frame = state->icount.parity = 0;
		state->icount.overrun = state->icount.brk = 0;
		printk (KERN_INFO "ttyS%d on %s%d at 0x%04x, BRG%d\n",
			i,
			(state->smc_scc_num & NUM_IS_SCC) ? "SCC" : "SMC",
			PORT_NUM(state->smc_scc_num) + 1,
			(unsigned int)(state->port),
			rs_brg_map[i] + 1);
#ifdef CONFIG_SERIAL_CONSOLE
		/* If we just printed the message on the console port, and
		 * we are about to initialize it for general use, we have
		 * to wait a couple of character times for the CR/NL to
		 * make it out of the transmit buffer.
		 */
		if (i == CONFIG_SERIAL_CONSOLE_PORT)
			mdelay(2);
#endif
		info = kmalloc(sizeof(ser_info_t), GFP_KERNEL);
		if (info) {
			__clear_user(info,sizeof(ser_info_t));
#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
			init_waitqueue_head(&info->open_wait);
			init_waitqueue_head(&info->close_wait);
#endif
			info->magic = SERIAL_MAGIC;
			info->flags = state->flags;
			info->tqueue.routine = do_softint;
			info->tqueue.data = info;
			info->tqueue_hangup.routine = do_serial_hangup;
			info->tqueue_hangup.data = info;
			info->line = i;
			info->state = state;
			info->MCR = info->MSR = 0;
			state->info = (struct async_struct *)info;

#if defined(CONFIG_TS1H)
			info->frame_cfg.status = FRAMEMODE_OFF;
			info->frame_cfg.rx_stat.success = 0;
			info->frame_cfg.rx_stat.error = 0;
			info->frame_cfg.tx_stat.success = 0;
			info->frame_cfg.tx_stat.error = 0;
#endif

#if defined(CONFIG_TS1H) || defined(CONFIG_SM100) 
			info->bdtx = 0;
			info->media = 0;
			info->hw_stopped = 0;
#endif
			
			/* We need to allocate a transmit and receive buffer
			 * descriptors from dual port ram, and a character
			 * buffer area from host mem.
			 */
			dp_addr = m8xx_cpm_dpalloc(sizeof(cbd_t) *
			            rx_bd_nums[ state->smc_scc_num & NUM_IS_SCC ?
			                        PORT_NUM(state->smc_scc_num)    :
			                        PORT_NUM(state->smc_scc_num) + 4]
			          );

			if (dp_addr== CPM_DP_NOSPACE)
				panic( "uart.c: could not allocate BD" );

			/* Allocate space for FIFOs in the host memory.
			*/
			mem_addr = m8xx_cpm_hostalloc(
			             rx_bd_nums[ state->smc_scc_num & NUM_IS_SCC ?
			                         PORT_NUM(state->smc_scc_num)    :
			                         PORT_NUM(state->smc_scc_num) + 4  ] *
			             rx_bd_sizes[state->smc_scc_num & NUM_IS_SCC ?
			                         PORT_NUM(state->smc_scc_num)    :
			                         PORT_NUM(state->smc_scc_num) + 4  ]
			           );

			if (!mem_addr)
				panic( "uart.c: could not allocate serial buffer" );

			/* Set the physical address of the host memory
			 * buffers in the buffer descriptors, and the
			 * virtual address for us to work with.
			 */
			bdp = (cbd_t *)&cp->cp_dpmem[dp_addr];
			info->rx_cur = info->rx_bd_base = (cbd_t *)bdp;

			for (j=0;
				 j<rx_bd_nums[ state->smc_scc_num & NUM_IS_SCC  ?
				               PORT_NUM(state->smc_scc_num)     :
				               PORT_NUM(state->smc_scc_num) + 4   ] - 1;
				 j++) {
				bdp->cbd_bufaddr = __pa(mem_addr);
				bdp->cbd_sc = BD_SC_EMPTY | BD_SC_INTRPT;
				bdp->cbd_datlen = 0;
				mem_addr += rx_bd_sizes[ state->smc_scc_num & NUM_IS_SCC ?
				                         PORT_NUM(state->smc_scc_num)    :
				                         PORT_NUM(state->smc_scc_num) + 4 ];
				bdp++;
			}
			bdp->cbd_bufaddr = __pa(mem_addr);
			bdp->cbd_sc = BD_SC_WRAP | BD_SC_EMPTY | BD_SC_INTRPT;

			idx = PORT_NUM(info->state->smc_scc_num);
			if (info->state->smc_scc_num & NUM_IS_SCC) {
				scp = &cp->cp_scc[idx];
				my_cp_scc = scp;
				my_scc_rx_bdp = (cbd_t *)&cp->cp_dpmem[dp_addr];
				sup = (scc_uart_t *)&cp->cp_dparam[state->port];
				sup->scc_genscc.scc_rbase = dp_addr;
			}
			else {
				sp = &cp->cp_smc[idx];
				my_cp_smc = sp;
				my_smc_rx_bdp = (cbd_t *)&cp->cp_dpmem[dp_addr];
				up = (smc_uart_t *)&cp->cp_dparam[state->port];
				up->smc_rbase = dp_addr;
			}

			dp_addr = m8xx_cpm_dpalloc(sizeof(cbd_t) * 
			            tx_bd_nums[ info->state->smc_scc_num & NUM_IS_SCC  ?
			                        idx : idx + 4 ]
			          );

			if (dp_addr == CPM_DP_NOSPACE)
			  panic( "uart.c: could not allocate BD" );

			/* Allocate space for FIFOs in the host memory.
			*/
			mem_addr = m8xx_cpm_hostalloc(
			             tx_bd_nums[ info->state->smc_scc_num & NUM_IS_SCC ?
			                         idx : idx + 4 ] *
			             tx_bd_sizes[info->state->smc_scc_num & NUM_IS_SCC ?
			                         idx : idx + 4 ]
			           );

			if (!mem_addr)
			  panic( "uart.c: could not allocate serial buffer" );

			/* Set the physical address of the host memory
			 * buffers in the buffer descriptors, and the
			 * virtual address for us to work with.
			 */
			bdp = (cbd_t *)&cp->cp_dpmem[dp_addr];
			info->tx_cur = info->tx_bd_base = (cbd_t *)bdp;

			for (j=0;
				 j<tx_bd_nums[ info->state->smc_scc_num & NUM_IS_SCC ?
				               idx : idx + 4 ] - 1;
				 j++) {
				bdp->cbd_bufaddr = __pa(mem_addr);
				bdp->cbd_sc = BD_SC_INTRPT;
				bdp->cbd_datlen = 0;
				mem_addr += tx_bd_sizes[info->state->smc_scc_num & NUM_IS_SCC ?
				                        idx : idx + 4 ];
				bdp++;
			}
			bdp->cbd_bufaddr = __pa(mem_addr);
			bdp->cbd_sc = (BD_SC_WRAP | BD_SC_INTRPT);

			if (info->state->smc_scc_num & NUM_IS_SCC) {

				/* Connect SCC to NMSI.
				 * Connect corresponding BRG, too.
				 */
				iobits = 0x000000FF << (idx*8);	/* clear GRx, SCx */
				cp->cp_sicr &= ~iobits;
				iobits  = (rs_brg_map[i] & 0x03) << 3;	/* Rx BRG */
				iobits |= (rs_brg_map[i] & 0x03);	/* Tx BRG */
				iobits <<= (idx*8);
				cp->cp_sicr |= iobits;

				my_scc_tx_bdp = (cbd_t *)&cp->cp_dpmem[dp_addr];
				sup->scc_genscc.scc_tbase = dp_addr;

				/* Set up the uart parameters in the
				 * parameter ram.
				 */
				sup->scc_genscc.scc_rfcr = SMC_EB;
				sup->scc_genscc.scc_tfcr = SMC_EB;

				/* Set this to 1 for now, so we get single
				 * character interrupts.  Using idle character
				 * time requires some additional tuning.
				 */
				sup->scc_genscc.scc_mrblr = 1;
				sup->scc_maxidl = 0;
				sup->scc_brkcr = 1;
				sup->scc_parec = 0;
				sup->scc_frmec = 0;
				sup->scc_nosec = 0;
				sup->scc_brkec = 0;
				sup->scc_uaddr1 = 0;
				sup->scc_uaddr2 = 0;
				sup->scc_toseq = 0;
				sup->scc_char1 = 0x8000;
				sup->scc_char2 = 0x8000;
				sup->scc_char3 = 0x8000;
				sup->scc_char4 = 0x8000;
				sup->scc_char5 = 0x8000;
				sup->scc_char6 = 0x8000;
				sup->scc_char7 = 0x8000;
				sup->scc_char8 = 0x8000;
				sup->scc_rccm = 0xc0ff;

				/* Send the CPM an initialize command.
				*/
				chan = scc_chan_map[idx];

				cp->cp_cpcr = mk_cr_cmd(chan,
						CPM_CR_INIT_TRX) | CPM_CR_FLG;
				while (cp->cp_cpcr & CPM_CR_FLG);

				/* Set UART mode, 8 bit, no parity, one stop.
				 * Enable receive and transmit.
				 */
#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
				scp->scc_gsmrh = SCC_GSMRH_RTSM;
#else
				scp->scc_gsmrh = 0;
#endif
				scp->scc_gsmrl = 
//[RK]Test SM100 has problem with 9600bps #if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
#if defined(CONFIG_TS1H) 
//mp, March2003: change TDCR & RDCR from 16 to 8, since
//it is better to adjust higher baud rates; see commproc.c
					(SCC_GSMRL_MODE_UART | SCC_GSMRL_TDCR_8 | SCC_GSMRL_RDCR_8);
#else
					(SCC_GSMRL_MODE_UART | SCC_GSMRL_TDCR_16 | SCC_GSMRL_RDCR_16);
#endif

				/* Disable all interrupts and clear all pending
				 * events.
				 */
				scp->scc_sccm = 0;
				scp->scc_scce = 0xffff;
				scp->scc_dsr  = 0x7e7e;
				scp->scc_pmsr = 0x3000;

				/* If the port is the console, enable Rx and Tx.
				*/
#if defined(CONFIG_SERIAL_CONSOLE) && !defined(CONFIG_SM100)
				if (i == CONFIG_SERIAL_CONSOLE_PORT)
					scp->scc_gsmrl |= (SCC_GSMRL_ENR | SCC_GSMRL_ENT);
#endif

				/* Install interrupt handler.
				*/
				cpm_install_handler(state->irq, rs_8xx_interrupt, info);
#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
				cpm_install_handler(CPMVEC_PIO_PC10, rs_8xx_dcd_interrupt, info);
				cpm_install_handler(CPMVEC_PIO_PC11, rs_8xx_cts_interrupt, info);
#elif defined(CONFIG_UART_CD_CONTROL_SCC1)
				cpm_install_handler(14, rs_8xx_interrupt, info);
#endif
			}
			else {
				/* Configure SMCs Tx/Rx instead of port B
				 * parallel I/O.  On 823/850 these are on
				 * port A for SMC2.
				 */
#ifndef CONFIG_ALTSMC2
				iobits = 0xc0 << (idx * 4);
				cp->cp_pbpar |= iobits;
				cp->cp_pbdir &= ~iobits;
				cp->cp_pbodr &= ~iobits;
#else
				iobits = 0xc0;
				if (idx == 0) {
					/* SMC1 on Port B, like all 8xx.
					*/
					cp->cp_pbpar |= iobits;
					cp->cp_pbdir &= ~iobits;
					cp->cp_pbodr &= ~iobits;
				}
				else {
					/* SMC2 is on Port A.
					*/
					immap->im_ioport.iop_papar |= iobits;
					immap->im_ioport.iop_padir &= ~iobits;
					immap->im_ioport.iop_paodr &= ~iobits;
				}
#endif

				/* Connect the baud rate generator to the
				 * SMC based upon index in rs_table.  Also
				 * make sure it is connected to NMSI.
				 */
				cp->cp_simode &= ~(0xffff << (idx * 16));
				cp->cp_simode |=
				  (rs_brg_map[i] << ((idx * 16) + 12));

				my_smc_tx_bdp = (cbd_t *)&cp->cp_dpmem[dp_addr];
				up->smc_tbase = dp_addr;

				/* Set up the uart parameters in the
				 * parameter ram.
				 */
				up->smc_rfcr = SMC_EB;
				up->smc_tfcr = SMC_EB;

				/* Set this to 1 for now, so we get single
				 * character interrupts.  Using idle character
				 * time requires some additional tuning.
				 */
				up->smc_mrblr = 1;
				up->smc_maxidl = 0;
				up->smc_brkcr = 1;

				/* Send the CPM an initialize command.
				*/
				chan = smc_chan_map[idx];

				cp->cp_cpcr = mk_cr_cmd(chan,
						CPM_CR_INIT_TRX) | CPM_CR_FLG;
				while (cp->cp_cpcr & CPM_CR_FLG);

				/* Set UART mode, 8 bit, no parity, one stop.
				 * Enable receive and transmit.
				 */
				sp->smc_smcmr = smcr_mk_clen(9) | SMCMR_SM_UART;

				/* Disable all interrupts and clear all pending
				 * events.
				 */
				sp->smc_smcm = 0;
				sp->smc_smce = 0xff;

				/* If the port is the console, enable Rx and Tx.
				*/
#ifdef CONFIG_SERIAL_CONSOLE
				if (i == CONFIG_SERIAL_CONSOLE_PORT)
					sp->smc_smcmr |= SMCMR_REN | SMCMR_TEN;
#endif

				/* Install interrupt handler.
				*/
				cpm_install_handler(state->irq, rs_8xx_interrupt, info);
			}

			/* Set up the baud rate generator.
			*/
			m8xx_cpm_setbrg(rs_brg_map[i], baud_table[baud_idx]);
		}
	}

	return 0;
}

/* This must always be called before the rs_8xx_init() function, otherwise
 * it blows away the port control information.
*/
static int __init serial_console_setup(struct console *co, char *options)
{
	struct		serial_state *ser;
	uint		mem_addr, dp_addr, bidx, idx;
	ushort		chan;
	volatile	cbd_t		*bdp;
	volatile	cpm8xx_t	*cp;
	volatile	smc_t		*sp;
	volatile	scc_t		*scp;
	volatile	smc_uart_t	*up;
	volatile	scc_uart_t	*sup;
	bd_t				*bd;
	immap_t *immap = (immap_t *)IMAP_ADDR;

	bd = (bd_t *)res;

	for (bidx = 0; bidx < (sizeof(baud_table) / sizeof(int)); bidx++)
		if (bd->bi_baudrate == baud_table[bidx])
			break;
	/* make sure we have a useful value */
	if (bidx == (sizeof(baud_table) / sizeof(int)))
		bidx = 13;	/* B9600 */

	co->cflag = CREAD|CLOCAL|bidx|CS8;
	baud_idx = bidx;

	ser = rs_table + co->index;

	cp = cpmp;	/* Get pointer to Communication Processor */

	idx = PORT_NUM(ser->smc_scc_num);
	if (ser->smc_scc_num & NUM_IS_SCC) {
		scp = &cp->cp_scc[idx];
		sup = (scc_uart_t *)&cp->cp_dparam[ser->port];
	}
	else {
		sp = &cp->cp_smc[idx];
		up = (smc_uart_t *)&cpmp->cp_dparam[ser->port];
	}

	/* When we get here, the CPM has been reset, so we need
	 * to configure the port.
	 * We need to allocate a transmit and receive buffer descriptor
	 * from dual port ram, and a character buffer area from host mem.
	 */

	/* Allocate space for two buffer descriptors in the DP ram.
	*/
	dp_addr = m8xx_cpm_dpalloc(sizeof(cbd_t) * 2);

	if( dp_addr == CPM_DP_NOSPACE )
	  panic( "uart.c: could not allocate BD" );

	/* Allocate space for two 2 byte FIFOs in the host memory.
	*/
	mem_addr = m8xx_cpm_hostalloc(8);

	if( !mem_addr )
	  panic( "uart.c: could not allocate serial buffer" );

	/* Set the physical address of the host memory buffers in
	 * the buffer descriptors.
	 */
	bdp = (cbd_t *)&cp->cp_dpmem[dp_addr];
	bdp->cbd_bufaddr = __pa(mem_addr);
	(bdp+1)->cbd_bufaddr = __pa(mem_addr+4);

	/* For the receive, set empty and wrap.
	 * For transmit, set wrap.
	 */
	bdp->cbd_sc = BD_SC_EMPTY | BD_SC_WRAP;
	(bdp+1)->cbd_sc = BD_SC_WRAP;

	/* Set up the uart parameters in the parameter ram.
	*/
	if (ser->smc_scc_num & NUM_IS_SCC) {

		sup->scc_genscc.scc_rbase = dp_addr;
		sup->scc_genscc.scc_tbase = dp_addr + sizeof(cbd_t);

		/* Set up the uart parameters in the
		 * parameter ram.
		 */
		sup->scc_genscc.scc_rfcr = SMC_EB;
		sup->scc_genscc.scc_tfcr = SMC_EB;

		/* Set this to 1 for now, so we get single
		 * character interrupts.  Using idle charater
		 * time requires some additional tuning.
		 */
		sup->scc_genscc.scc_mrblr = 1;
		sup->scc_maxidl = 0;
		sup->scc_brkcr = 1;
		sup->scc_parec = 0;
		sup->scc_frmec = 0;
		sup->scc_nosec = 0;
		sup->scc_brkec = 0;
		sup->scc_uaddr1 = 0;
		sup->scc_uaddr2 = 0;
		sup->scc_toseq = 0;
		sup->scc_char1 = 0x8000;
		sup->scc_char2 = 0x8000;
		sup->scc_char3 = 0x8000;
		sup->scc_char4 = 0x8000;
		sup->scc_char5 = 0x8000;
		sup->scc_char6 = 0x8000;
		sup->scc_char7 = 0x8000;
		sup->scc_char8 = 0x8000;
		sup->scc_rccm = 0xc0ff;

		/* Send the CPM an initialize command.
		*/
		chan = scc_chan_map[idx];

		cp->cp_cpcr = mk_cr_cmd(chan, CPM_CR_INIT_TRX) | CPM_CR_FLG;
		while (cp->cp_cpcr & CPM_CR_FLG);

		/* Set UART mode, 8 bit, no parity, one stop.
		 * Enable receive and transmit.
		 */
		scp->scc_gsmrh = 0;
		scp->scc_gsmrl = 
#if defined(CONFIG_TS1H) || defined(CONFIG_SM100)
//mp, March2003: change TDCR & RDCR from 16 to 8, since
//it is better to adjust higher baud rates; see commproc.c
			(SCC_GSMRL_MODE_UART | SCC_GSMRL_TDCR_8 | SCC_GSMRL_RDCR_8);
#else
			(SCC_GSMRL_MODE_UART | SCC_GSMRL_TDCR_16 | SCC_GSMRL_RDCR_16);
#endif

		/* Disable all interrupts and clear all pending
		 * events.
		 */
		scp->scc_sccm = 0;
		scp->scc_scce = 0xffff;
		scp->scc_dsr = 0x7e7e;
		scp->scc_pmsr = 0x3000;

		scp->scc_gsmrl |= (SCC_GSMRL_ENR | SCC_GSMRL_ENT);

	}
	else {
		up->smc_rbase = dp_addr;	/* Base of receive buffer desc. */
		up->smc_tbase = dp_addr+sizeof(cbd_t);	/* Base of xmt buffer desc. */
		up->smc_rfcr = SMC_EB;
		up->smc_tfcr = SMC_EB;

		/* Set this to 1 for now, so we get single character interrupts.
		*/
		up->smc_mrblr = 1;		/* receive buffer length */
		up->smc_maxidl = 0;		/* wait forever for next char */

		/* Send the CPM an initialize command.
		*/
		chan = smc_chan_map[idx];
		cp->cp_cpcr = mk_cr_cmd(chan, CPM_CR_INIT_TRX) | CPM_CR_FLG;
		printk("%s", "");
		while (cp->cp_cpcr & CPM_CR_FLG);

		/* Set UART mode, 8 bit, no parity, one stop.
		 * Enable receive and transmit.
		 */
		sp->smc_smcmr = smcr_mk_clen(9) |  SMCMR_SM_UART;

		/* And finally, enable Rx and Tx.
		*/
		sp->smc_smcmr |= SMCMR_REN | SMCMR_TEN;
	}

	/* Set up the baud rate generator.
	*/
	m8xx_cpm_setbrg(rs_brg_map[ser - rs_table], bd->bi_baudrate);

	immap->im_ioport.iop_pcso &= 0xffef; // CTS1 as GP I/O 

#ifdef CONFIG_TS1H
	if (((bd_t *)res)->hw_info.serial_itf) { /* serial mode */
		return(1);
	}
#endif
#ifdef CONFIG_SM100
	if (ser->smc_scc_num & NUM_IS_SCC) {
		return 1;
	}
	else {
		return 0;
	}
#endif
	return 0;
}
