/*
 * arch/ppc/k2boot/head.S
 * 
 * Initial board bringup code for SBS K2
 *
 * Author: Matt Porter
 *         mporter@mvista.com
 *
 * Derived from arch/ppc/boot/head.S
 *
 * Copyright 2000 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR   IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "../kernel/ppc_defs.h"
#include "../kernel/ppc_asm.tmpl"
#include <asm/processor.h>
#include <asm/cache.h>
#include <asm/cpc710.h>
#include <asm/k2.h>

/* This is a kludge for the bootloader */
#define	_IO_BASE	0x80000000

/*
 * Board initialization macros
 */

#define	DELAY \
	lis	r3,0x000f; \
	ori	r3,r3,0xffff; \
0:	subi	r3,r3,1; \
	cmpi	0,0,r3,0; \
	bne	0b

#define WRITE_MEM_32(address, data) \
	lis	r3, address@h; \
	ori	r3, r3, address@l; \
	lis	r4, data@h; \
	ori	r4, r4, data@l; \
	stw	r4, 0x0000(r3); \
	sync; \
	eieio; \
	DELAY

#define	WRITE_MEM_8(address, data) \
	lis	r3, address@h; \
	ori	r3, r3, address@l; \
	li	r4, data; \
	stb	r4, 0(r3); \
	sync; \
	eieio; \
	DELAY

#define WRITE_PCI64_CONFIG_32(address, data) \
	lis	r3, K2_PCI64_CONFIG_ADDR@h; \
	ori	r3, r3, K2_PCI64_CONFIG_ADDR@l; \
	lis	r4, address@h; \
	ori	r4, r4, address@l; \
	stw	r4, 0(r3); \
	lis	r3, K2_PCI64_CONFIG_DATA@h; \
	ori	r3, r3, K2_PCI64_CONFIG_DATA@l; \
	lis	r4, data@h; \
	ori	r4, r4, data@l; \
	stw	r4, 0(r3); \
	sync; \
	eieio; \
	DELAY

#define WRITE_PCI32_CONFIG_32(address, data) \
	lis	r3, K2_PCI32_CONFIG_ADDR@h; \
	ori	r3, r3, K2_PCI32_CONFIG_ADDR@l; \
	lis	r4, address@h; \
	ori	r4, r4, address@l; \
	stw	r4, 0(r3); \
	lis	r3, K2_PCI32_CONFIG_DATA@h; \
	ori	r3, r3, K2_PCI32_CONFIG_DATA@l; \
	lis	r4, data@h; \
	ori	r4, r4, data@l; \
	stw	r4, 0(r3); \
	sync; \
	eieio; \
	DELAY

#define WRITE_PCI64_CONFIG_16(address, data) \
	lis	r3, K2_PCI64_CONFIG_ADDR@h; \
	ori	r3, r3, K2_PCI64_CONFIG_ADDR@l; \
	lis	r4, address@h; \
	ori	r4, r4, address@l; \
	stw	r4, 0(r3); \
	lis	r3, K2_PCI64_CONFIG_DATA@h; \
	ori	r3, r3, K2_PCI64_CONFIG_DATA@l; \
	lis	r4, data@h; \
	ori	r4, r4, data@l; \
	sth	r4, 0(r3); \
	sync; \
	eieio; \
	DELAY

#define WRITE_PCI32_CONFIG_16(address, data) \
	lis	r3, K2_PCI32_CONFIG_ADDR@h; \
	ori	r3, r3, K2_PCI32_CONFIG_ADDR@l; \
	lis	r4, address@h; \
	ori	r4, r4, address@l; \
	stw	r4, 0(r3); \
	lis	r3, K2_PCI32_CONFIG_DATA@h; \
	ori	r3, r3, K2_PCI32_CONFIG_DATA@l; \
	lis	r4, data@h; \
	ori	r4, r4, data@l; \
	sth	r4, 0(r3); \
	sync; \
	eieio; \
	DELAY

	.text

/*
 *
 *      Begin at some arbitrary location in RAM or Flash
 *	  Initialize core registers
 *	  Configure memory controller (Not executing from RAM)
 *	Move the boot code to the link address (8M)
 *	  Setup C stack
 *	  Initialize UART
 *      Decompress the kernel to 0x0
 *      Jump to the kernel entry
 *
 */

	.globl	start
start:
	bl	start_

start_:
/*
 * Configure core registers
 */

	/* Establish default MSR value, exception prefix 0xFFF */
	li	r3,MSR_IP|MSR_FP
	mtmsr	r3

	/* Clear BATS */
        li      r8,0
        mtspr   DBAT0U,r8
	mtspr	DBAT0L,r8
        mtspr   DBAT1U,r8
	mtspr	DBAT1L,r8
        mtspr   DBAT2U,r8
	mtspr	DBAT2L,r8
        mtspr   DBAT3U,r8
	mtspr	DBAT3L,r8
        mtspr   IBAT0U,r8
	mtspr	IBAT0L,r8
        mtspr   IBAT1U,r8
	mtspr	IBAT1L,r8
	mtspr	IBAT2U,r8
        mtspr   IBAT2L,r8
        mtspr   IBAT3U,r8
	mtspr	IBAT3L,r8
	isync
	sync
	sync

	/* Set segment registers */
	lis	r8, 0x8000
	isync
	mtsr	SR0,r8
	mtsr	SR1,r8
	mtsr	SR2,r8
	mtsr	SR3,r8
	mtsr	SR4,r8
	mtsr	SR5,r8
	mtsr	SR6,r8
	mtsr	SR7,r8
	mtsr	SR8,r8
	mtsr	SR9,r8
	mtsr	SR10,r8
	mtsr	SR11,r8
	mtsr	SR12,r8
	mtsr	SR13,r8
	mtsr	SR14,r8
	mtsr	SR15,r8
	isync
	sync
	sync

	/* Disable L1 icache/dcache */
	li	r4,0x0000
	isync
	mtspr	HID0,r4
	sync
	isync

	/* Invalidate and enable L1 icache */
	mfspr	r4,HID0
	ori	r4,r4,0x8800
	isync
	mtspr	HID0,r4
	sync
	isync

	/*
	 * If we are executing from flash, then do the full init as
	 * follows.  Otherwise, skip the memory controller reset, etc.
	 */
	lis	r4,0xfff0
	ori	r4,r4,0x0104
	mflr	r3
	cmp	0,r3,r4
	bne	brdgcnfg

/*
 * Configure CPC710 bridge, memory controller, and ALI M1543C PIB
 */
	/* Initialize CPC710 core registers */
	WRITE_MEM_32(RSTR, 0xf0000000)

	/* Dummy read */
	lis	r3, MPSR@h
	ori	r3, r3, MPSR@l
	lwz	r4, 0(r3)
	sync
	eieio

	WRITE_MEM_32(SIOC, 0x00000000)
	WRITE_MEM_32(PIDR, 0x00000000)
	WRITE_MEM_32(UCTL, 0x00780080)
	WRITE_MEM_32(ABCNTL, 0x70000000)
	WRITE_MEM_32(SRST, 0x00000000)
	WRITE_MEM_32(ERRC, 0x00000000)
	WRITE_MEM_32(SESR, 0x00000000)
	WRITE_MEM_32(SEAR, 0x00000000)
	WRITE_MEM_32(GPDIR, 0x00000000)
	WRITE_MEM_32(ATAS, 0x709c2508)
	WRITE_MEM_32(AVDG, 0x00000000)
	WRITE_MEM_32(MESR, 0x00000000)
	WRITE_MEM_32(MEAR, 0x00000000)
	WRITE_MEM_32(MCER0, 0x00000000)
	WRITE_MEM_32(MCER1, 0x00000000)
	WRITE_MEM_32(MCER2, 0x00000000)
	WRITE_MEM_32(MCER3, 0x00000000)
	WRITE_MEM_32(MCER4, 0x00000000)
	WRITE_MEM_32(MCER5, 0x00000000)
	WRITE_MEM_32(MCER6, 0x00000000)
	WRITE_MEM_32(MCER7, 0x00000000)
	WRITE_MEM_32(MCCR, 0x52b06000)

brdgcnfg:
	WRITE_MEM_32(PGCHP, 0x808000e0)		/* Set FPHB modes */
	/* Enable and map PCI32 and PCI64 registers to a base address */
	WRITE_MEM_32(CNFR, 0x80000003)		/* Activate CNFR - PCI64 */
	WRITE_MEM_32(BAR, K2_PCI64_BAR)		/* Set BAR - PCI64_BAR_VALUE */
	WRITE_MEM_32(PCIENB, 0x80000000)	/* Enable PCI Control Space */
	WRITE_MEM_32(CNFR, 0x00000000)		/* Deactivate CNFR - PCI64 */
	WRITE_MEM_32(CNFR, 0x80000002)		/* Activate CNFR - PCI32 */
	WRITE_MEM_32(BAR, K2_PCI32_BAR)		/* Set BAR - PCI32_BAR_VALUE */
	WRITE_MEM_32(PCIENB, 0x80000000)	/* Enable PCI Control Space */
	WRITE_MEM_32(CNFR, 0x00000000)		/* Deactivate CNFR - PCI32 */

	/* PCI64 Config space init */
	WRITE_PCI64_CONFIG_32(0x04000080,0x07038002)	/* SERR,BMAS,I/O,MEM */
	WRITE_PCI64_CONFIG_32(0x68000080,0x0f000000)	/* Reset INTA-D */
	WRITE_PCI64_CONFIG_16(0x40000080,0x000f) 	/* Set BUS_NO */

	/* PCI32 Config space init */
	WRITE_PCI32_CONFIG_32(0x04000080,0x07038002)	/* SERR,BMAS,I/O,MEM */
	WRITE_PCI32_CONFIG_32(0x68000080,0x0f000000)	/* Reset INTA-D */
	WRITE_PCI32_CONFIG_16(0x40000080,0x000f) 	/* Set BUS_NO */

	/* PCI64 memory mappings */

	WRITE_MEM_32(0xff4f6110, 0x00000000)	/* PSEA */
	WRITE_MEM_32(0xff4f6120, 0xc0000000)	/* PCIDG */
	WRITE_MEM_32(0xff4f7800, 0x08000000)	/* PIBAR - PCI I/O base */
	WRITE_MEM_32(0xff4f7810, 0x08000000)	/* PMBAR - PCI Mem base */
	WRITE_MEM_32(0xff4f7f20, 0x80008000)	/* PR - MRU parking,queue on */
	WRITE_MEM_32(0xff4f7f30, 0xff000000)	/* ACR - Enable all levels */
	WRITE_MEM_32(0xff4f7f40, 0xf8000000)	/* MSIZE - 128MB */
	WRITE_MEM_32(0xff4f7f60, 0xf8000000)	/* IOSIZE - 128MB */
	WRITE_MEM_32(0xff4f7f80, 0xc8000000)	/* SMBAR - Base@0xc8000000 */
	WRITE_MEM_32(0xff4f7fc0, 0x88000000)	/* SIBAR - Base@0x88000000 */
	WRITE_MEM_32(0xff4f7fd0, 0x02000000)	/* CTLRW - ISA contiguous */
	WRITE_MEM_32(0xff4f8100, 0x000000c0)	/* PSSIZE - 1GB space */
	WRITE_MEM_32(0xff4f8110, 0x000000c0)	/* PPSIZE - 1GB space */
	WRITE_MEM_32(0xff4f8120, 0x00000000)	/* BARPS - Base@0x00000000 */
	WRITE_MEM_32(0xff4f8130, 0x00000000)	/* BARPP - Base@0x00000000 */
	WRITE_MEM_32(0xff4f8310, 0x00000000)	/* Init INT_SET */

	/* PCI32 memory mappings */
	WRITE_MEM_32(0xff5f6110, 0x00000000)	/* Clear PSEA */
	WRITE_MEM_32(0xff5f6120, 0x40000000)	/* PIDG */
	WRITE_MEM_32(0xff5f7800, 0x00000000)	/* PIBAR - PCI I/O Base */
	WRITE_MEM_32(0xff5f7810, 0x00000000)	/* PMBAR - PCI Mem Base */
	WRITE_MEM_32(0xff5f7f20, 0xa000c000)	/* PR - Enable translation */
	WRITE_MEM_32(0xff5f7f30, 0xfc000000)	/* ACR - Enable all levels */
	WRITE_MEM_32(0xff5f7f40, 0xf8000000)	/* MSIZE - 128MB */
	WRITE_MEM_32(0xff5f7f60, 0xf8000000)	/* IOSIZE - 128MB */
	WRITE_MEM_32(0xff5f7f80, 0xc0000000)	/* SMBAR - Base@0xc0000000 */
	WRITE_MEM_32(0xff5f7fc0, 0x80000000)	/* SIBAR - Base@0x80000000 */
	WRITE_MEM_32(0xff5f7fd0, 0x00000000)	/* CTLRW - ISA contiguous */
	WRITE_MEM_32(0xff5f8100, 0x000000c0)	/* PSSIZE - 1GB space */
	WRITE_MEM_32(0xff5f8110, 0x000000c0)	/* PPSIZE - 1GB space */
	WRITE_MEM_32(0xff5f8120, 0x00000000)	/* BARPS - Base@0x00000000 */
	WRITE_MEM_32(0xff5f8130, 0x00000000)	/* BARPP - Base@0x00000000 */

	/* PCI64 specific memory mappings */
	WRITE_PCI64_CONFIG_32(0x10000080, 0x00000080)	/* PSBAR - Base@0x80 */

	/* PCI32 specific memory mappings */
	WRITE_MEM_32(0xff5f8140, 0x00000080)	/* PSBAR - Base@0x80 */
	WRITE_MEM_32(0xff5f8150, 0x00000000)	/* PPBAR */
	WRITE_MEM_32(0xff5f8200, 0xcc000000)	/* BPMDLK */
	WRITE_MEM_32(0xff5f8210, 0xff000000)	/* TPMDLK */
	WRITE_MEM_32(0xff5f8220, 0x81000000)	/* BIODLK */
	WRITE_MEM_32(0xff5f8230, 0xc0000000)	/* TIODLK */

	/* Turn off PCI64/PCI32 resets */
	WRITE_MEM_32(0xff4f7ef0, 0xfc000000)
	WRITE_MEM_32(0xff5f7ef0, 0xfc000000)

	/* ALI 1543C PCI-ISA bridge config */
	WRITE_PCI32_CONFIG_32(0x40400080, 0x00000230)	/* Set PIC,IORC,ISAC */
	WRITE_PCI32_CONFIG_32(0x44400080, 0x00800000)	/* Enable PCI polling */
	WRITE_PCI32_CONFIG_32(0x50400080, 0x00000040)	/* Disable USB */
	WRITE_PCI32_CONFIG_32(0x54400080, 0x00030800)	
	WRITE_PCI32_CONFIG_32(0x58400080, 0x000d0e03)
	WRITE_PCI32_CONFIG_32(0x5c400080, 0x0000000c)
	WRITE_PCI32_CONFIG_32(0x70400080, 0x00000000)
	WRITE_PCI32_CONFIG_32(0x78400080, 0x00000000)

	/* Turn on board fail light and turn off other lights */
	WRITE_MEM_8(0x80000804, 0x22);

	/* Turn off board fail light and turn on User1 light */
	WRITE_MEM_8(0x80000804, 0x24)

	/* Turn on hotswap light and unmask ENUM */
	WRITE_MEM_8(0x8000080c, 0x00);

	/* Enable ALI 1543C UARTs */
	WRITE_MEM_8(K2_ISAPNP_CONFIG, 0x51)	/* Write config key */
	WRITE_MEM_8(K2_ISAPNP_CONFIG, 0x23)	/* Write config key */
	WRITE_MEM_8(K2_ISAPNP_INDEX, 0x22)	/* Access powerdown control */
	WRITE_MEM_8(K2_ISAPNP_DATA, 0x00)	/* Wake up UARTs */
	WRITE_MEM_8(K2_ISAPNP_INDEX, 0x07)	/* Access logical device reg */
	WRITE_MEM_8(K2_ISAPNP_DATA, 0x04)	/* Point to UART1 */
	WRITE_MEM_8(K2_ISAPNP_INDEX, 0x30)	/* Access reg 30 */
	WRITE_MEM_8(K2_ISAPNP_DATA, 0x01)	/* Enable UART1 */
	WRITE_MEM_8(K2_ISAPNP_INDEX, 0x07)	/* Access logical device reg */
	WRITE_MEM_8(K2_ISAPNP_DATA, 0x05)	/* Point to UART3 */
	WRITE_MEM_8(K2_ISAPNP_INDEX, 0x30)	/* Access reg 30 */
	WRITE_MEM_8(K2_ISAPNP_DATA, 0x01)	/* Enable UART3 */
	WRITE_MEM_8(K2_ISAPNP_CONFIG, 0xbb)	/* Exit config mode */

	/*
	 * Configure CPC710 memory controller
	 */
	lis	r3,K2_MSIZ_GEO_REG@h
	ori	r3,r3,K2_MSIZ_GEO_REG@l
	lbz	r4,0(r3)
	mr	r20,r4				/* Save MSIZ reg */
	andi.	r4,r4,0x00e0

	/*
	 * If we are executing from flash, then do the full init as
	 * follows.  Otherwise, skip the memory size configuration.
	 */
	lis	r4,0xfff0
	ori	r4,r4,0x0104
	mflr	r3
	cmp	0,r3,r4
	bne	endmemcfg

	cmpli	0,0,r4,0x0040
	beq	config_512x2

	cmpli	0,0,r4,0x0060
	beq	config_512x1

	cmpli	0,0,r4,0x0080
	beq	config_256x2

	cmpli	0,0,r4,0x00a0
	beq	config_256x1

	cmpli	0,0,r4,0x00c0
	beq	config_128x2

	b	config_128x1	/* 0x00e0 or others default to 128x1 */

config_512x2:
	WRITE_MEM_32(MCER0, 0x80000080)	/* Bank 0 MER - 512MB */
	WRITE_MEM_32(MCER1, 0x82000080)	/* Bank 1 MER - 512MB */
	b	mccr

config_512x1:
	WRITE_MEM_32(MCER0, 0x80000080) /* Bank 0 MER - 512MB */
	b	mccr

config_256x2:
	WRITE_MEM_32(MCER0, 0x800000d4) /* Bank 0 MER - 256MB */
	WRITE_MEM_32(MCER1, 0x810000d4)	/* Bank 1 MER - 256MB */
	b	mccr

config_256x1:
	WRITE_MEM_32(MCER0, 0x800000d4)	/* Bank 0 MER - 256MB */
	b	mccr

config_128x2:
	WRITE_MEM_32(MCER0, 0x800080c0)	/* Bank 0 MER - 128MB */
	WRITE_MEM_32(MCER1, 0x808080c0)	/* Bank 1 MER - 128MB */
	b	mccr

config_128x1:
	WRITE_MEM_32(MCER0, 0x800080c0)	/* Bank 0 MER - 128MB */

mccr:
	/* Enable memory controller configuration */
	WRITE_MEM_32(MCCR, 0xf2b06000)

sdwait:
	/* Wait until SDRAM initialization is complete */
	nop
	lis	r3, MCCR@h
	ori	r3, r3, MCCR@l
	lwz	r4, 0(r3)
	lis	r5, 0x2000
	ori	r5, r5, 0x0000
	and	r4, r4, r5
	cmpl	0, 0, r5, r4
	bne	sdwait

endmemcfg:
	/* Configure M5229 IDE PIB settings */
	WRITE_PCI32_CONFIG_32(0x40400080, 0x00000233)	/* 3 wait states */
	WRITE_PCI32_CONFIG_32(0x48400080, 0x580731b9)	/* PCI IRQ routes */
	WRITE_PCI32_CONFIG_32(0x4c400080, 0x00000000)	/* Disable SIRQ maps */
	WRITE_PCI32_CONFIG_32(0x44400080, 0x0d800000)	/* primary IDE->14 */
	WRITE_PCI32_CONFIG_32(0x74400080, 0x000d0000)	/* secondary IDE->14 */

	/* Configure PIC level/edge settings */	
	WRITE_MEM_8(0x800004d0, 0x62)
	WRITE_MEM_8(0x800004d1, 0xde)

	/* Configure M5229 IDE settings */
	WRITE_PCI32_CONFIG_32(0x58400080, 0x480d0e03)	/* Enable IDE */
	WRITE_PCI32_CONFIG_32(0x50880080, 0x03000000)	/* Enable IDE */
	WRITE_PCI32_CONFIG_32(0x04880080, 0x04008002)	/* I/O accesses on */
	WRITE_PCI32_CONFIG_32(0x08880080, 0xc1df0101)	/* Native, sec only */

	/* Turn on User2 LED */
	WRITE_MEM_8(0x80000804, 0x21)

/* check if we need to relocate ourselves to the link addr or were we
   loaded there to begin with -- Cort */
	lis	r4,start@h
	ori	r4,r4,start@l
	mflr	r3
	subi	r3,r3,4		/* we get the nip, not the ip of the branch */
	mr	r8,r3
	cmp	0,r3,r4
	bne	1010f
/* compute size of whole image in words.  this should be moved to
 * start_ldr() -- Cort
 */
	lis	r4,start@h
	ori	r4,r4,start@l
	lis	r5,end@h
	ori	r5,r5,end@l
	addi	r5,r5,3		/* round up */
	sub	r5,r5,r4
	srwi	r5,r5,2
	mr	r7,r5
	b	start_ldr
1010:
/* 
 * no matter where we're loaded, move ourselves to -Ttext address
 */
relocate:
	mflr	r3		/* Compute code bias */
	subi	r3,r3,4
	mr	r8,r3
	lis	r4,start@h
	ori	r4,r4,start@l
	lis	r5,end@h
	ori	r5,r5,end@l
	addi	r5,r5,3		/* Round up - just in case */
	sub	r5,r5,r4	/* Compute # longwords to move */
	srwi	r5,r5,2
	mtctr	r5
	mr	r7,r5
	li	r6,0
	subi	r3,r3,4		/* Set up for loop */
	subi	r4,r4,4
00:	lwzu	r5,4(r3)
	stwu	r5,4(r4)
	xor	r6,r6,r5
	bdnz	00b
  	lis	r3,start_ldr@h
	ori	r3,r3,start_ldr@l
	mtlr	r3		/* Easiest way to do an absolute jump */
	blr
start_ldr:
/* Clear all of BSS and set up stack for C calls */
	lis	r3,edata@h
	ori	r3,r3,edata@l
	lis	r4,end@h
	ori	r4,r4,end@l
	subi	r3,r3,4
	subi	r4,r4,4
	li	r0,0
50:	stwu	r0,4(r3)
	cmp	0,r3,r4
	bne	50b
90:	mr	r9,r1		/* Save old stack pointer (in case it matters) */
	lis	r1,.stack@h
	ori	r1,r1,.stack@l
	addi	r1,r1,4096*2
	subi	r1,r1,256
	li	r2,0x000F	/* Mask pointer to 16-byte boundary */
	andc	r1,r1,r2

/*
 * Exec kernel loader
 */
	mr	r3,r8			/* Load point */
	mr	r4,r7			/* Program length */
	mr	r5,r6			/* Checksum */
	bl	decompress_kernel
	
	lis	r6,cmd_line@h
	ori	r6,r6,cmd_line@l
	lwz	r6, 0(r6)
	subi	r7,r6,1
00:	lbzu	r2,1(r7)
	cmpi	0,r2,0
	bne	00b

	/* r4,r5 have initrd_start, size */
	lis	r2,initrd_start@h
	ori	r2,r2,initrd_start@l
	lwz	r4,0(r2)
	lis	r2,initrd_end@h
	ori	r2,r2,initrd_end@l
	lwz	r5,0(r2)
	
	li	r9,0x4
	mtlr	r9
	li	r9,0
	stb	r20,0(r9)			/* Save MSIZ reg for kernel */

	/*
	 * Jump to kernel start 
	 */
	blr

hang:
	b	hang	

/*
 * Delay for a number of microseconds
 * -- Use the BUS timer (assumes 66MHz)
 */
	.globl	udelay
udelay:		
	mfspr	r4,PVR
	srwi	r4,r4,16
	cmpi	0,r4,1		/* 601 ? */
	bne	.udelay_not_601
00:	li	r0,86	/* Instructions / microsecond? */
	mtctr	r0
10:	addi	r0,r0,0 /* NOP */
	bdnz	10b
	subic.	r3,r3,1
	bne	00b
	blr

.udelay_not_601:		
	mulli	r4,r3,1000	/* nanoseconds */
	addi	r4,r4,59
	li	r5,60
	divw	r4,r4,r5	/* BUS ticks */
1:	mftbu	r5
	mftb	r6
	mftbu	r7
	cmp	0,r5,r7
	bne	1b		/* Get [synced] base time */
	addc	r9,r6,r4	/* Compute end time */
	addze	r8,r5
2:	mftbu	r5
	cmp	0,r5,r8
	blt	2b
	bgt	3f
	mftb	r6
	cmp	0,r6,r9
	blt	2b
3:	blr		

.globl _get_HID0
_get_HID0:		
	mfspr	r3,HID0
	blr

.globl _put_HID0
_put_HID0:		
	mtspr	HID0,r3
	blr
		
.globl _get_MSR
_get_MSR:		
	mfmsr	r3
	blr
	
.globl _put_MSR
_put_MSR:		
	mtmsr	r3
	blr

/*
 * Flush instruction cache
 * *** I'm really paranoid here!
 */
_GLOBAL(flush_instruction_cache)
	mflr	r5
	bl	flush_data_cache
	mfspr	r3,HID0	/* Caches are controlled by this register */
	li	r4,0
	ori	r4,r4,(HID0_ICE|HID0_ICFI)
	or	r3,r3,r4	/* Need to enable+invalidate to clear */
	mtspr	HID0,r3
	andc	r3,r3,r4
	ori	r3,r3,HID0_ICE	/* Enable cache */
	mtspr	HID0,r3
	mtlr	r5
	blr
	
#define NUM_CACHE_LINES 128*8
#define CACHE_LINE_SIZE 32 
#define cache_flush_buffer 0x1000

/*
 * Flush data cache
 * *** I'm really paranoid here!
 */
_GLOBAL(flush_data_cache)
	lis	r3,cache_flush_buffer@h
	ori	r3,r3,cache_flush_buffer@l
	li	r4,NUM_CACHE_LINES
	mtctr	r4
00:	lwz	r4,0(r3)
	addi	r3,r3,CACHE_LINE_SIZE	/* Next line, please */
	bdnz	00b	
10:	blr
	.comm	.stack,4096*2,4

