/*
 * $Id: m8xx_setup.c,v 1.12 2002/08/22 22:54:23 arnaldo Exp $
 *
 *  linux/arch/ppc/kernel/setup.c
 *
 *  Copyright (C) 1995  Linus Torvalds
 *  Adapted from 'alpha' version by Gary Thomas
 *  Modified by Cort Dougan (cort@cs.nmt.edu)
 *  Modified for MBX using prep/chrp/pmac functions by Dan (dmalek@jlc.net)
 *  Further modified for generic 8xx by Dan.
 */

/*
 * bootup setup stuff..
 */

#include <linux/config.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/stddef.h>
#include <linux/unistd.h>
#include <linux/ptrace.h>
#include <linux/malloc.h>
#include <linux/user.h>
#include <linux/a.out.h>
#include <linux/tty.h>
#include <linux/major.h>
#include <linux/interrupt.h>
#include <linux/reboot.h>
#include <linux/init.h>
#include <linux/blk.h>
#include <linux/ioport.h>
#include <linux/delay.h>

#include <asm/mmu.h>
#include <asm/processor.h>
#include <asm/residual.h>
#include <asm/io.h>
#include <asm/pgtable.h>
#include <asm/ide.h>
#include <asm/mpc8xx.h>
#include <asm/8xx_immap.h>
#include <asm/machdep.h>

#ifdef CONFIG_PRxK
#include <asm/prxk.h>
#ifdef CONFIG_TSxK
#include <asm/cpld.h>
#endif
#ifdef CONFIG_PR3K
#include <asm/fpga.h>
#endif
#include <asm/flash.h>
#endif

#include "time.h"
#include "ppc8xx_pic.h"

static int m8xx_set_rtc_time(unsigned long time);
unsigned long m8xx_get_rtc_time(void);
void m8xx_calibrate_decr(void);

#ifdef CONFIG_SM100
extern void cyc_rtc_init(void);
extern int cyc_set_rtc_time(unsigned long time);
extern unsigned long cyc_get_rtc_time(void);
#endif

#if 0
extern int mackbd_setkeycode(unsigned int scancode, unsigned int keycode);
extern int mackbd_getkeycode(unsigned int scancode);
extern int mackbd_translate(unsigned char scancode, unsigned char *keycode,
			   char raw_mode);
extern char mackbd_unexpected_up(unsigned char keycode);
extern void mackbd_leds(unsigned char leds);
extern void mackbd_init_hw(void);
#endif

extern unsigned long loops_per_sec;

unsigned long empty_zero_page[1024];

#ifdef CONFIG_BLK_DEV_RAM
extern int rd_doload;		/* 1 = load ramdisk, 0 = don't load */
extern int rd_prompt;		/* 1 = prompt for ramdisk, 0 = don't prompt */
extern int rd_image_start;	/* starting block # of image */
#endif

extern char saved_command_line[256];

extern unsigned long find_available_memory(void);
extern void m8xx_cpm_reset(uint);

#ifdef CONFIG_PRxK	
#ifdef CONFIG_TSxK
volatile void *uarts;
volatile cpld_regs *cpld;
unsigned char *cflash_addr;
#endif
#ifdef CONFIG_PR3K
FPGA_REG *fpga;
unsigned char *board_addr;
#endif
unsigned char *flash_addr;

void prxk_init_memory(void)
{
	bd_t    *bd = (bd_t *)res;
#ifdef CONFIG_TSxK
	unsigned int	i_uart, i_cpld;
	unsigned int	i_cflash, cflash_size;
#endif
	unsigned int	i_flash, flash_size;

#ifdef CONFIG_PR3K
	board_addr = ioremap_nocache((unsigned int)DB_FPGA_ADDR, DB_FPGA_ADDR_SIZE);
	fpga = (FPGA_REG *)ioremap_nocache(MB_FPGA_ADDR, 0x10000);
#endif

#ifdef CONFIG_TSxK
	if (CYC_BOARD_TYPE == BOARD_TS3K) {
		i_cpld = (((immap_t *)IMAP_ADDR)->im_memctl.memc_br6 & 0xffff8000L);
		i_uart = (((immap_t *)IMAP_ADDR)->im_memctl.memc_br7 & 0xffff8000L);
		i_cflash = (((immap_t *)IMAP_ADDR)->im_memctl.memc_br5 & 0xffff8000L);
		cflash_size = 0x1000;
	} else {
		i_cpld = (((immap_t *)IMAP_ADDR)->im_memctl.memc_br3 & 0xffff8000L);
		i_uart = (((immap_t *)IMAP_ADDR)->im_memctl.memc_br4 & 0xffff8000L);
	}

	uarts = (void *)ioremap_nocache(i_uart, DB_UART_ADDR_SIZE);
	cpld = (cpld_regs *)ioremap_nocache(i_cpld, DB_CPLD_ADDR_SIZE);

	if (CYC_BOARD_TYPE == BOARD_TS3K) {
		cflash_addr = ioremap_nocache(i_cflash, cflash_size);
	}

#endif

	i_flash = (((immap_t *)IMAP_ADDR)->im_memctl.memc_br0 & 0xffff8000L);
	flash_size = bd->hw_info.flash_size;

	flash_addr = ioremap_nocache(i_flash, flash_size);

#ifdef CONFIG_PR3K
	printk("Flash addr: %x, FPGA addr: %x, Board Addr: %x\n",
		(int)flash_addr, (int)fpga, (int)board_addr);
#endif

#ifdef CONFIG_TSxK
	if (CYC_BOARD_TYPE == BOARD_TS3K) {
		printk("CF+ addr: %x Flash Addr %x, CPLD addr: %x, UART Addr: %x\n",
			(int)cflash_addr, (int)flash_addr, (int)cpld, (int)uarts);
	} else {
		printk("Flash addr: %x, CPLD addr: %x, UART Addr: %x\n",
			(int)flash_addr, (int)cpld, (int)uarts);
	}
#endif

#if defined(CONFIG_PR1K) || defined(CONFIG_NL1K) || defined(CONFIG_TS1H) || defined(CONFIG_SM100)
	printk("Flash addr: %x\n", (int)flash_addr);
#endif

}

#endif

void __init adbdev_init(void)
{
}

__initfunc(void
m8xx_setup_arch(unsigned long * memory_start_p, unsigned long * memory_end_p))
{
	int	cpm_page;
	extern char cmd_line[];

	cpm_page = *memory_start_p;
	*memory_start_p += PAGE_SIZE;
	
	printk("Boot arguments: %s\n", cmd_line);

	/* Reset the Communication Processor Module.
	*/
	m8xx_cpm_reset(cpm_page);
#ifdef CONFIG_RPXLCD
	lcd_fb_setpage(memory_start_p);
#endif

#ifdef notdef
	ROOT_DEV = to_kdev_t(0x0301); /* hda1 */
#endif
	
#ifdef CONFIG_BLK_DEV_INITRD
#if 0
	ROOT_DEV = to_kdev_t(0x0200); /* floppy */  
	rd_prompt = 1;
	rd_doload = 1;
	rd_image_start = 0;
#endif
	/* initrd_start and size are setup by boot/head.S and kernel/head.S */
	if ( initrd_start )
	{
		if (initrd_end > *memory_end_p)
		{
			printk("initrd extends beyond end of memory "
			       "(0x%08lx > 0x%08lx)\ndisabling initrd\n",
			       initrd_end,*memory_end_p);
			initrd_start = 0;
		}
	}
#endif

}

void
abort(void)
{
#ifdef CONFIG_XMON
	xmon(0);
#endif
	machine_restart(NULL);
}

/* The decrementer counts at the system (internal) clock frequency divided by
 * sixteen, or external oscillator divided by four.  We force the processor
 * to use system clock divided by sixteen.
 */
__initfunc(void m8xx_calibrate_decr(void))
{
	bd_t	*binfo = (bd_t *)res;
	int freq, fp, divisor;

	/* Unlock the SCCR.
	*/
	((volatile immap_t *)IMAP_ADDR)->im_clkrstk.cark_sccrk = ~KAPWR_KEY;
	((volatile immap_t *)IMAP_ADDR)->im_clkrstk.cark_sccrk = KAPWR_KEY;

	/* Force all 8xx processors to use divide by 16 processor clock.
	*/
	((volatile immap_t *)IMAP_ADDR)->im_clkrst.car_sccr |= 0x02000000;

	/* Processor frequency is MHz.
	 * The value 'fp' is the number of decrementer ticks per second.
	 */
	fp = (binfo->bi_intfreq * 1000000) / 16;
	freq = fp*60;	/* try to make freq/1e6 an integer */
        divisor = 60;
        printk("time_init: decrementer frequency = %d/%d\n", freq, divisor);
        decrementer_count = freq / HZ / divisor;
#ifdef CONFIG_TS1H
		if (binfo->hw_info.clock_freq != 4) {
			decrementer_count -= 18;
		}
#endif
        count_period_num = divisor;
        count_period_den = freq / 1000000;
}

/* A place holder for time base interrupts, if they are ever enabled.
*/
void timebase_interrupt(int irq, void * dev, struct pt_regs * regs)
{
	printk("timebase_interrupt()\n");
}

/* The RTC on the MPC8xx is an internal register.
 * We want to protect this during power down, so we need to unlock,
 * modify, and re-lock.
 */
static int
m8xx_set_rtc_time(unsigned long time)
{
	((volatile immap_t *)IMAP_ADDR)->im_sitk.sitk_rtck = KAPWR_KEY;
	((volatile immap_t *)IMAP_ADDR)->im_sit.sit_rtc = time;
	((volatile immap_t *)IMAP_ADDR)->im_sitk.sitk_rtck = ~KAPWR_KEY;
	return(0);
}

__initfunc(unsigned long
m8xx_get_rtc_time(void))
{
	/* First, unlock all of the registers we are going to modify.
	 * To protect them from corruption during power down, registers
	 * that are maintained by keep alive power are "locked".  To
	 * modify these registers we have to write the key value to
	 * the key location associated with the register.
	 * Some boards power up with these unlocked, while others
	 * are locked.  Writing anything (including the unlock code?)
	 * to the unlocked registers will lock them again.  So, here
	 * we guarantee the registers are locked, then we unlock them
	 * for our use.
	 */
	((volatile immap_t *)IMAP_ADDR)->im_sitk.sitk_tbscrk = ~KAPWR_KEY;
	((volatile immap_t *)IMAP_ADDR)->im_sitk.sitk_rtcsck = ~KAPWR_KEY;
	((volatile immap_t *)IMAP_ADDR)->im_sitk.sitk_tbk = ~KAPWR_KEY;
	((volatile immap_t *)IMAP_ADDR)->im_sitk.sitk_tbscrk = KAPWR_KEY;
	((volatile immap_t *)IMAP_ADDR)->im_sitk.sitk_rtcsck = KAPWR_KEY;
	((volatile immap_t *)IMAP_ADDR)->im_sitk.sitk_tbk = KAPWR_KEY;

	/* Disable the RTC one second and alarm interrupts.
	*/
	((volatile immap_t *)IMAP_ADDR)->im_sit.sit_rtcsc &=
						~(RTCSC_SIE | RTCSC_ALE);

	/* Enabling the decrementer also enables the timebase interrupts
	 * (or from the other point of view, to get decrementer interrupts
	 * we have to enable the timebase).  The decrementer interrupt
	 * is wired into the vector table, nothing to do here for that.
	 */
	((volatile immap_t *)IMAP_ADDR)->im_sit.sit_tbscr =
				((mk_int_int_mask(DEC_INTERRUPT) << 8) |
					 (TBSCR_TBF | TBSCR_TBE));
	if (request_8xxirq(DEC_INTERRUPT, timebase_interrupt, 0, "tbint", NULL) != 0)
		panic("Could not allocate timer IRQ!");

	/* Get time from the RTC.
	*/
	return((unsigned long)(((immap_t *)IMAP_ADDR)->im_sit.sit_rtc));
}

void
m8xx_restart(char *cmd)
{
#ifndef CONFIG_PRxK
	extern void m8xx_gorom(void);
#endif
#ifndef CONFIG_PRxK
	m8xx_gorom();
#else
#ifdef CONFIG_PR3K
    fpga = (FPGA_REG *)ioremap_nocache(MB_FPGA_ADDR, 0x10000);
    fpga->rcr |= 0x8000;  /* system reset */
#else
#ifdef CONFIG_SM100
    int i;
    /* RTC chip enable pin set to level 1  - pin PB31*/
    ((volatile immap_t *)IMAP_ADDR)->im_cpm.cp_pbpar &= ~0x0001;
    ((volatile immap_t *)IMAP_ADDR)->im_cpm.cp_pbdat |=  0x0001;
    ((volatile immap_t *)IMAP_ADDR)->im_cpm.cp_pbdir |=  0x0001;
    for (i = 0; i < 1000; i++) {
        /* delay 200ms */
        udelay(200);
    }    
    /* RTC chip enable pin set to level 0  - pin PB31*/
    ((volatile immap_t *)IMAP_ADDR)->im_cpm.cp_pbdat &= ~0x0001;
    for (i = 0; i < 1000; i++) {
        /* delay 200ms */
        udelay(200);
    }    
    /* reset output set to level 0 - pin PA3 */
    ((volatile immap_t *)IMAP_ADDR)->im_ioport.iop_padat &= ~0x1000;
    ((volatile immap_t *)IMAP_ADDR)->im_ioport.iop_padir |= 0x1000;
    for (i = 0; i < 1000; i++) {
        /* delay 200ms */
        udelay(200);
    }    
    ((volatile immap_t *)IMAP_ADDR)->im_ioport.iop_padat |= 0x1000;
#else
    /* reset output set to level 0 - pin PA3 */
    ((volatile immap_t *)IMAP_ADDR)->im_ioport.iop_padat &= ~0x1000;
    ((volatile immap_t *)IMAP_ADDR)->im_ioport.iop_padir |= 0x1000;
#endif
#endif
#endif
}

void
m8xx_power_off(void)
{
   m8xx_restart(NULL);
}

void
m8xx_halt(void)
{
   m8xx_restart(NULL);
}


int m8xx_setup_residual(char *buffer)
{
        int     len = 0;
	bd_t	*bp;
	extern	RESIDUAL *res;
			
	bp = (bd_t *)res;
			
	len += sprintf(len+buffer,"clock\t\t: %dMHz\n"
		       "bus clock\t: %dMHz\n",
		       bp->bi_intfreq /*/ 1000000*/,
		       bp->bi_busfreq /*/ 1000000*/);

	return len;
}

/* Initialize the internal interrupt controller.  The number of
 * interrupts supported can vary with the processor type, and the
 * 82xx family can have up to 64.
 * External interrupts can be either edge or level triggered, and
 * need to be initialized by the appropriate driver.
 */
__initfunc(void
m8xx_init_IRQ(void))
{
	int i;

        ppc8xx_pic.irq_offset = 0;
        for ( i = 0 ; i < NR_SIU_INTS ; i++ )
                irq_desc[i].ctl = &ppc8xx_pic;
	
	/* We could probably incorporate the CPM into the multilevel
	 * interrupt structure.
	 */
	cpm_interrupt_init();
        unmask_irq(CPM_INTERRUPT);

#if 0
#ifdef CONFIG_MBX
        for ( i = NR_SIU_INTS ; i < (NR_SIU_INTS + NR_8259_INTS) ; i++ )
                irq_desc[i].ctl = &i8259_pic;
        i8259_pic.irq_offset = NR_SIU_INTS;
        i8259_init();
        request_8xxirq(ISA_BRIDGE_INT, mbx_i8259_action, 0, "8259 cascade", NULL);
        enable_irq(ISA_BRIDGE_INT);
#endif
#endif
}

#if defined(CONFIG_BLK_DEV_IDE) || defined(CONFIG_BLK_DEV_IDE_MODULE)

/* Define this to make a PCMCIA ATA Flash card work.
*/
#define ATA_FLASH 1

/*
 * IDE stuff.
 */
void
m8xx_ide_insw(ide_ioreg_t port, void *buf, int ns)
{
#ifdef ATA_FLASH
	ide_insw(port, buf, ns);
#else
	ide_insw(port+_IO_BASE, buf, ns);
#endif
}

void
m8xx_ide_outsw(ide_ioreg_t port, void *buf, int ns)
{
#ifdef ATA_FLASH
	ide_outsw(port, buf, ns);
#else
	ide_outsw(port+_IO_BASE, buf, ns);
#endif
}

int
m8xx_ide_default_irq(ide_ioreg_t base)
{
#ifdef ATA_FLASH
	return PCMCIA_INTERRUPT;
#else
        return 14;
#endif
}

ide_ioreg_t
m8xx_ide_default_io_base(int index)
{
        return index;
}

int
m8xx_ide_check_region(ide_ioreg_t from, unsigned int extent)
{
        return 0;
}

void
m8xx_ide_request_region(ide_ioreg_t from,
			unsigned int extent,
			const char *name)
{
}

void
m8xx_ide_release_region(ide_ioreg_t from,
			unsigned int extent)
{
}

int
m8xx_ide_request_irq(unsigned int irq,
		       void (*handler)(int, void *, struct pt_regs *),
		       unsigned long flags, 
		       const char *device,
		       void *dev_id)
{
#ifdef ATA_FLASH
	return request_8xxirq(irq, handler, flags, device, dev_id);
#else
	return request_irq(irq, handler, flags, device, dev_id);
#endif
}

void
m8xx_ide_fix_driveid(struct hd_driveid *id)
{
        ppc_generic_ide_fix_driveid(id);
}

/* We can use an external IDE controller or wire the IDE interface to
 * the internal PCMCIA controller.
 */
void __init m8xx_ide_init_hwif_ports(ide_ioreg_t *p, ide_ioreg_t base, int *irq)
{
	ide_ioreg_t port = base;
	int i;
#ifdef ATA_FLASH
	volatile pcmconf8xx_t	*pcmp;
#endif

#ifdef ATA_FLASH
	*p = 0;
	*irq = 0;

	if (base != 0)		/* Only map the first ATA flash drive */
		return;

#if defined(CONFIG_RPXLITE) || defined(CONFIG_RPXCLASSIC)
	if (rpx_pcmcia_config())
		return;
#else
	pcmp = (pcmconf8xx_t *)(&(((immap_t *)IMAP_ADDR)->im_pcmcia));
	if (pcmp->pcmc_pipr & 0x18000000)
		return;		/* No card in slot */
#endif

	base = (unsigned long) ioremap(PCMCIA_MEM_ADDR, 0x200);

	/* For the M-Systems ATA card, the first 8 registers map 1:1.
	 * The following register, control/Altstatus, is located at 0x0e.
	 * Following that, the irq offset, is not used, so we place it in
	 * an unused location, 0x0a.
	 */
	*p++ = base + 8;
	for (i = 1; i < 8; ++i)
		*p++ = base + i;
	*p++ = base + 0x0e;		/* control/altstatus */
	*p = base + 0x0a;		/* IRQ, not used */
	if (irq)
		*irq = PCMCIA_INTERRUPT;

#if 0
	/* Configure the interface for this interrupt.
	*/
	pcmp->pcmc_pgcra = (mk_int_int_mask(PCMCIA_INTERRUPT) << 24) |
		(mk_int_int_mask(PCMCIA_INTERRUPT) << 16);

	/* Enable status change interrupt from slot A.
	*/
	pcmp->pcmc_per = 0xff100000;
	pcmp->pcmc_pscr = ~0;
#endif
#else
	
	/* Just a regular IDE drive on some I/O port.
	*/
	i = 8;
	while (i--)
		*p++ = port++;
	*p++ = base + 0x206;
	if (irq != NULL)
		*irq = 0;
#endif
}
#endif

__initfunc(void
m8xx_init(unsigned long r3, unsigned long r4, unsigned long r5,
	 unsigned long r6, unsigned long r7))
{

	if ( r3 )
		memcpy( (void *)res,(void *)(r3+KERNELBASE), sizeof(bd_t) );
	
#ifdef CONFIG_PCI
	m8xx_setup_pci_ptrs();
#endif

#ifdef CONFIG_BLK_DEV_INITRD
	/* take care of initrd if we have one */
	if ( r4 )
	{
		initrd_start = r4 + KERNELBASE;
		initrd_end = r5 + KERNELBASE;
	}
#endif /* CONFIG_BLK_DEV_INITRD */
	/* take care of cmd line */
	if ( r6 )
	{
		
		*(char *)(r7+KERNELBASE) = 0;
		strcpy(cmd_line, (char *)(r6+KERNELBASE));
	}

	ppc_md.setup_arch     = m8xx_setup_arch;
	ppc_md.setup_residual = m8xx_setup_residual;
	ppc_md.get_cpuinfo    = NULL;
	ppc_md.irq_cannonicalize = NULL;
	ppc_md.init_IRQ       = m8xx_init_IRQ;
	ppc_md.do_IRQ         = m8xx_do_IRQ;
	ppc_md.init           = NULL;

	ppc_md.restart        = m8xx_restart;
	ppc_md.power_off      = m8xx_power_off;
	ppc_md.halt           = m8xx_halt;

#ifdef CONFIG_SM100
	ppc_md.time_init      = cyc_rtc_init;
	ppc_md.set_rtc_time   = cyc_set_rtc_time;
	ppc_md.get_rtc_time   = cyc_get_rtc_time;
#else
	ppc_md.time_init      = NULL;
	ppc_md.set_rtc_time   = m8xx_set_rtc_time;
	ppc_md.get_rtc_time   = m8xx_get_rtc_time;
#endif
	ppc_md.calibrate_decr = m8xx_calibrate_decr;

#if 0
	ppc_md.kbd_setkeycode    = pckbd_setkeycode;
	ppc_md.kbd_getkeycode    = pckbd_getkeycode;
	ppc_md.kbd_translate     = pckbd_translate;
	ppc_md.kbd_unexpected_up = pckbd_unexpected_up;
	ppc_md.kbd_leds          = pckbd_leds;
	ppc_md.kbd_init_hw       = pckbd_init_hw;
#ifdef CONFIG_MAGIC_SYSRQ
	ppc_md.kbd_sysrq_xlate	 = pckbd_sysrq_xlate;
#endif
#else
	ppc_md.kbd_setkeycode    = NULL;
	ppc_md.kbd_getkeycode    = NULL;
	ppc_md.kbd_translate     = NULL;
	ppc_md.kbd_unexpected_up = NULL;
	ppc_md.kbd_leds          = NULL;
	ppc_md.kbd_init_hw       = NULL;
#ifdef CONFIG_MAGIC_SYSRQ
	ppc_md.kbd_sysrq_xlate	 = NULL;
#endif
#endif

#if defined(CONFIG_BLK_DEV_IDE) || defined(CONFIG_BLK_DEV_IDE_MODULE)
        ppc_ide_md.insw = m8xx_ide_insw;
        ppc_ide_md.outsw = m8xx_ide_outsw;
        ppc_ide_md.default_irq = m8xx_ide_default_irq;
        ppc_ide_md.default_io_base = m8xx_ide_default_io_base;
        ppc_ide_md.check_region = m8xx_ide_check_region;
        ppc_ide_md.request_region = m8xx_ide_request_region;
        ppc_ide_md.release_region = m8xx_ide_release_region;
        ppc_ide_md.fix_driveid = m8xx_ide_fix_driveid;
        ppc_ide_md.ide_init_hwif = m8xx_ide_init_hwif_ports;
        ppc_ide_md.ide_request_irq = m8xx_ide_request_irq;

        ppc_ide_md.io_base = _IO_BASE;
#endif		
}

#if defined(CONFIG_RPXLITE) || defined(CONFIG_RPXCLASSIC)
int
rpx_pcmcia_config()
{
	volatile pcmconf8xx_t	*pcmp;
	int i;

	pcmp = (pcmconf8xx_t *)(&(((immap_t *)IMAP_ADDR)->im_pcmcia));

	/* For RPX-Lite (850/823) Slot B.
	*/

	/* Make sure there is a card in the slot, then configure the
	 * interface.
	 */
	if (pcmp->pcmc_pipr & 0x00001800)
		return(1);		/* No card in slot B */

	/* Enable power the card.  I don't know if this is exactly
	 * correct, but it works for the cards I have tested.
	 */

	/* Disable PCMCIA buffers (isolate the interface).
	*/
	pcmp->pcmc_pgcrb |= 0x00000080;

	/* Remove any power.
	*/
	*(volatile uint *)RPX_CSR_ADDR &= ~BCSR0_PCMCIAVOLT;
	udelay(5000);

	/* Power On.
	*/
	if ((pcmp->pcmc_pipr & 0x0000c000) == 0x0000c000) {
		*(volatile uint *)RPX_CSR_ADDR |= BCSR0_PCMCIA5VOLT;
printk("PCMCIA Power on 5 Volt\n");
	}
	else {
		*(volatile uint *)RPX_CSR_ADDR |= BCSR0_PCMCIA3VOLT;
printk("PCMCIA Power on 3 Volt\n");
	}
	udelay(500000);

#if 0
	/* Deassert Reset.
	*/
	pcmp->pcmc_pgcrb &= ~0x00000040;

	/* Enable output buffers.
	*/
	pcmp->pcmc_pgcrb &= ~0x00000080;
	udelay(10000);

	/* Assert reset and wait a moment.
	*/
	pcmp->pcmc_pgcrb |= 0x00000040;
	udelay(20);

	/* Deassert Reset and wait a moment.
	*/
	pcmp->pcmc_pgcrb &= ~0x00000040;
	udelay(20000);
#else
	pcmp->pcmc_pgcrb |= 0x00000080;
	udelay(300);
	pcmp->pcmc_pgcrb |= 0x00000040;
	udelay(500);
	pcmp->pcmc_pgcrb &= ~0x00000040;
	udelay(500);
	pcmp->pcmc_pgcrb &= ~0x00000080;
	udelay(300);
#endif
	
	/* Wait for card ready.
	*/
	for (i=0; i<5000; i++) {
		udelay(2000);
		if (pcmp->pcmc_pipr & 0x00000100) {
			break;
		}
	}
	if ((pcmp->pcmc_pipr & 0x00000100) == 0) {
		*(volatile uint *)RPX_CSR_ADDR &= ~BCSR0_PCMCIAVOLT;
		printk("PCMCIA - No rdy after power.\n");
		return(1);
	}

	pcmp->pcmc_por0 = pcmp->pcmc_por1 = pcmp->pcmc_por2 = 0;
	pcmp->pcmc_por3 = pcmp->pcmc_por4 = pcmp->pcmc_por5 = 0;
	pcmp->pcmc_por6 = pcmp->pcmc_por7 = 0;

printk("PCMCIA Addr %x\n", &(pcmp->pcmc_pbr0));
	/* We have to set up the base, option, and control registers
	 * for a 16-bit memory window.
	 */
#if 1
	pcmp->pcmc_pbr0 = PCMCIA_MEM_ADDR;
#else
	pcmp->pcmc_pbr0 = PCMCIA_IO_ADDR;
#endif

	/* Maximum slow (until we know better), 16-bit, commom memory, slot B.
	*/
#if 1
#if 1
	pcmp->pcmc_por0 = 0xc00ff045;
#else
	pcmp->pcmc_por0 = 0xc00ff005;
#endif
#else
#if 0
	pcmp->pcmc_por0 = 0xc00ff05d;	/* 16-bit */
#else
	pcmp->pcmc_por0 = 0xc00ff01d;	/* 8-bit */
#endif
#endif

	/* Map attribute space.
	*/
	pcmp->pcmc_pbr1 = PCMCIA_ATTRB_ADDR;
	pcmp->pcmc_por1 = 0xc0024815;

	/* Set interupt number in SIU.
	*/
	pcmp->pcmc_pgcrb = (mk_int_int_mask(PCMCIA_INTERRUPT) << 24) |
		(mk_int_int_mask(PCMCIA_INTERRUPT) << 16);

	/* Enable status change interrupt from slot B.
	*/
	pcmp->pcmc_per = 0x0000ff10;

	/* Clear all interrupt status.
	*/
	pcmp->pcmc_pscr = ~0;

	return(0);
}
#endif

#if 0
int
rpx_pcmcia_config()
{
	volatile pcmconf8xx_t	*pcmp;
	int i;

	pcmp = (pcmconf8xx_t *)(&(((immap_t *)IMAP_ADDR)->im_pcmcia));

	/* For RPX-Classic (860) Slot A.
	*/

	/* Make sure there is a card in the slot, then configure the
	 * interface.
	 */
	if (pcmp->pcmc_pipr & 0x18000000)
		return(1);		/* No card in slot A */

	/* Enable power the card.  I don't know if this is exactly
	 * correct, but it works for the cards I have tested.
	 */

	/* Disable PCMCIA buffers (isolate the interface).
	*/
	pcmp->pcmc_pgcra |= 0x00000080;

	/* Remove any power.
	*/
	*(volatile uint *)RPX_CSR_ADDR &= ~BCSR0_PCMCIAVOLT;
	udelay(5000);

	/* Power On.
	*/
	if ((pcmp->pcmc_pipr & 0xc0000000) == 0xc0000000) {
		*(volatile uint *)RPX_CSR_ADDR |= BCSR0_PCMCIA5VOLT;
printk("PCMCIA Power on 5 Volt\n");
	}
	else {
		*(volatile uint *)RPX_CSR_ADDR |= BCSR0_PCMCIA3VOLT;
printk("PCMCIA Power on 3 Volt\n");
	}
	udelay(500000);

#if 0
	/* Deassert Reset.
	*/
	pcmp->pcmc_pgcra &= ~0x00000040;

	/* Enable output buffers.
	*/
	pcmp->pcmc_pgcra &= ~0x00000080;
	udelay(10000);

	/* Assert reset and wait a moment.
	*/
	pcmp->pcmc_pgcra |= 0x00000040;
	udelay(20);

	/* Deassert Reset and wait a moment.
	*/
	pcmp->pcmc_pgcra &= ~0x00000040;
	udelay(20000);
#else
	pcmp->pcmc_pgcra |= 0x00000080;
	udelay(300);
	pcmp->pcmc_pgcra |= 0x00000040;
	udelay(500);
	pcmp->pcmc_pgcra &= ~0x00000040;
	udelay(500);
	pcmp->pcmc_pgcra &= ~0x00000080;
	udelay(300);
#endif
	
	/* Wait for card ready.
	*/
	for (i=0; i<5000; i++) {
		udelay(2000);
		if (pcmp->pcmc_pipr & 0x01000000) {
			break;
		}
	}
	if ((pcmp->pcmc_pipr & 0x01000000) == 0) {
		*(volatile uint *)RPX_CSR_ADDR &= ~BCSR0_PCMCIAVOLT;
		printk("PCMCIA - No rdy after power.\n");
		return(1);
	}

	pcmp->pcmc_por0 = pcmp->pcmc_por1 = pcmp->pcmc_por2 = 0;
	pcmp->pcmc_por3 = pcmp->pcmc_por4 = pcmp->pcmc_por5 = 0;
	pcmp->pcmc_por6 = pcmp->pcmc_por7 = 0;

	/* We have to set up the base, option, and control registers
	 * for a 16-bit memory window.
	 */
#if 1
	pcmp->pcmc_pbr0 = PCMCIA_MEM_ADDR;
#else
	pcmp->pcmc_pbr0 = PCMCIA_IO_ADDR;
#endif

	/* Maximum slow (until we know better), 16-bit, commom memory, slot A.
	*/
	pcmp->pcmc_por0 = 0xc00ff041;

	/* Map attribute space.
	*/
	pcmp->pcmc_pbr1 = PCMCIA_ATTRB_ADDR;
	pcmp->pcmc_por1 = 0xc0024811;

	/* Set interupt number in SIU.
	*/
	pcmp->pcmc_pgcra = (mk_int_int_mask(PCMCIA_INTERRUPT) << 24) |
		(mk_int_int_mask(PCMCIA_INTERRUPT) << 16);

	/* Enable status change interrupt from slot A.
	*/
	pcmp->pcmc_per = 0xff100000;

	/* Clear all interrupt status.
	*/
	pcmp->pcmc_pscr = ~0;

	return(0);
}
#endif
