
/* Board specific functions for those embedded 8xx boards that do
 * not have boot monitor support for board information.
 */
#ifndef CONFIG_PRxK
#include <linux/types.h>
#include "asm/mpc8xx.h"
#else
#include <linux/config.h>
#include <linux/types.h>
#include <asm/uaccess.h>
#include <asm/mpc8xx.h>
#include "../8xx_io/commproc.h"
#include <asm/prxk.h>
#include <asm/cyc_config.h>
#include <asm/flash.h>
#endif

/* IIC functions.
 * These are just the basic master read/write operations so we can
 * examine serial EEPROM.
 */
extern void	iic_read(uint devaddr, u_char *buf, uint offset, uint count);
extern u_char	aschex_to_byte(u_char *cp);

static void	rpx_eth(bd_t *bd, u_char *cp);
static void	rpx_brate(bd_t *bd, u_char *cp);
static void	rpx_memsize(bd_t *bd, u_char *cp);
static void	rpx_cpuspeed(bd_t *bd, u_char *cp);

#ifndef CONFIG_MBX
bd_t local_bd_info = {
     0
    ,0x01000000
    ,64
    ,64
};
#endif

/* Supply a default Ethernet address for those eval boards that don't
 * ship with one.  This is an address from the MBX board I have, so
 * it is unlikely you will find it on your network.
 */
static	ushort	def_enet_addr[] = { 0x0800, 0x3e26, 0x1559 };

#if defined(CONFIG_RPXLITE) || defined(CONFIG_RPXCLASSIC)
/* Read the EEPROM on the RPX-Lite board.
*/
void
rpx_cfg(bd_t *bd)
{
	u_char	eebuf[256], *cp;

	/* Read the first 256 bytes of the EEPROM.  I think this
	 * is really all there is, and I hope if it gets bigger the
	 * info we want is still up front.
	 */
#if 1
	iic_read(0xa8, eebuf, 0, 128);
	iic_read(0xa8, &eebuf[128], 128, 128);
	{
		int i;
		cp = (u_char *)0xfa000000;

		for (i=0; i<256; i++)
			*cp++ = eebuf[i];
	}

	/* We look for two things, the Ethernet address and the
	 * serial baud rate.  The records are separated by
	 * newlines.
	 */
	cp = eebuf;
	for (;;) {
		if (*cp == 'E') {
			cp++;
			if (*cp == 'A') {
				cp += 2;
				rpx_eth(bd, cp);
			}
		}
		if (*cp == 'S') {
			cp++;
			if (*cp == 'B') {
				cp += 2;
				rpx_brate(bd, cp);
			}
		}
		if (*cp == 'D') {
			cp++;
			if (*cp == '1') {
				cp += 2;
				rpx_memsize(bd, cp);
			}
		}
		if (*cp == 'H') {
			cp++;
			if (*cp == 'Z') {
				cp += 2;
				rpx_cpuspeed(bd, cp);
			}
		}

		/* Scan to the end of the record.
		*/
		while ((*cp != '\n') && (*cp != 0xff))
			cp++;

		/* If the next character is a 0 or ff, we are done.
		*/
		cp++;
		if ((*cp == 0) || (*cp == 0xff))
			break;
	}
	bd->bi_memstart = 0;

#else
	bd->bi_memstart = 0;
	bd->bi_memsize = (4 * 1024 * 1024);
	bd->bi_intfreq = 48;
	bd->bi_busfreq = 48;
	bd->bi_baudrate = 9600;
#endif
}

static void
rpx_eth(bd_t *bd, u_char *cp)
{
	int	i;

	for (i=0; i<6; i++) {
		bd->bi_enetaddr[i] = aschex_to_byte(cp);
		cp += 2;
	}
}

static void
rpx_brate(bd_t *bd, u_char *cp)
{
	uint	rate;

	rate = 0;

	while (*cp != '\n') {
		rate *= 10;
		rate += (*cp) - '0';
		cp++;
	}

	bd->bi_baudrate = rate * 100;
}

static void
rpx_memsize(bd_t *bd, u_char *cp)
{
	uint	size;

	size = 0;

	while (*cp != '\n') {
		size *= 10;
		size += (*cp) - '0';
		cp++;
	}

	bd->bi_memsize = size * 1024 * 1024;
}

static void
rpx_cpuspeed(bd_t *bd, u_char *cp)
{
	uint	num, den;

	num = den = 0;

	while (*cp != '\n') {
		num *= 10;
		num += (*cp) - '0';
		cp++;
		if (*cp == '/') {
			cp++;
			den = (*cp) - '0';
			break;
		}
	}

	/* I don't know why the RPX just can't state the actual
	 * CPU speed.....
	 */
	if (den) {
		num /= den;
		num *= den;
	}
	bd->bi_intfreq = bd->bi_busfreq = num;

	/* The 8xx can only run a maximum 50 MHz bus speed (until
	 * Motorola changes this :-).  Greater than 50 MHz parts
	 * run internal/2 for bus speed.
	 */
	if (num > 50)
		bd->bi_busfreq /= 2;
}
#endif

#ifdef CONFIG_BSEIP
/* Build a board information structure for the BSE ip-Engine.
 * There is more to come since we will add some environment
 * variables and a function to read them.
 */
void
bseip_cfg(bd_t *bd)
{
	u_char	*cp;
	int	i;

	/* Baud rate and processor speed will eventually come
	 * from the environment variables.
	 */
	bd->bi_baudrate = 9600;
	bd->bi_intfreq = bd->bi_busfreq = 48;

	/* Get the Ethernet station address from the Flash ROM.
	*/
	cp = (u_char *)0xfe003ffa;
	for (i=0; i<6; i++) {
		bd->bi_enetaddr[i] = *cp++;
	}
}
#endif

#ifdef CONFIG_PRxK

static void reset_ports(int board)
{
	volatile immap_t	*pm = (volatile immap_t *)IMAP_ADDR;

#ifdef CONFIG_TSxK
	if (board == BOARD_TS3K) {
		pm->im_ioport.iop_padir |= 0xAFFF;
		pm->im_ioport.iop_papar &= ~0xEFFF;
		pm->im_ioport.iop_paodr &= ~0xEFFF;
		pm->im_ioport.iop_padat &= ~0xEFFF;

		pm->im_cpm.cp_pbdir &= ~0x1000;
		pm->im_cpm.cp_pbodr &= ~0x1000;
		pm->im_cpm.cp_pbpar &= ~0x1000;

		pm->im_ioport.iop_pcdir &= ~0x0700;
		pm->im_ioport.iop_pcpar &= ~0x0700;
		pm->im_ioport.iop_pcdat &= ~0x0700;
	} else {
		pm->im_ioport.iop_padir |= 0xAFFF;
		pm->im_ioport.iop_papar &= ~0xAFFF;
		pm->im_ioport.iop_paodr &= ~0xAFFF;
		pm->im_ioport.iop_padat &= ~0xAFFF;

		pm->im_cpm.cp_pbdir &= ~0x1C3E;
		pm->im_cpm.cp_pbodr &= ~0x1C3E;
		pm->im_cpm.cp_pbpar &= ~0x1C3E;

		pm->im_ioport.iop_pcdir &= ~0x0728;
		pm->im_ioport.iop_pcpar &= ~0x0728;
		pm->im_ioport.iop_pcdat &= ~0x0728;
	}
#endif

#if defined(CONFIG_PR1K) || defined(CONFIG_NL1K)
	pm->im_ioport.iop_padir |= 0xAFFF;
	pm->im_ioport.iop_papar &= ~0xAFFF;
	pm->im_ioport.iop_paodr &= ~0xAFFF;
	pm->im_ioport.iop_padat &= ~0xAFFF;

	pm->im_cpm.cp_pbdir &= ~0x1C3E;
	pm->im_cpm.cp_pbodr &= ~0x1C3E;
	pm->im_cpm.cp_pbpar &= ~0x1C3E;

	pm->im_ioport.iop_pcdir &= ~0x0728;
	pm->im_ioport.iop_pcpar &= ~0x0728;
	pm->im_ioport.iop_pcdat &= ~0x0728;
#endif

#ifdef CONFIG_SM100
	pm->im_ioport.iop_padir &= ~0x0180;
	pm->im_ioport.iop_padir |= 0xC61E;
	pm->im_ioport.iop_papar &= ~0xFFFC;
	pm->im_ioport.iop_paodr &= ~0x2FED;
	pm->im_ioport.iop_padat &= ~0xFFE4;

	pm->im_cpm.cp_pbdir |= 0xFC30;

	pm->im_cpm.cp_pbodr |= 0x8430;
	pm->im_cpm.cp_pbodr &= ~0x7800;
	pm->im_cpm.cp_pbpar |= 0x0030;
	pm->im_cpm.cp_pbdat |= 0x0001;
	pm->im_cpm.cp_pbdat &= ~0x1800;

	pm->im_ioport.iop_pcdir |=  0x07C8;
	pm->im_ioport.iop_pcpar &= ~0x0030;
	pm->im_ioport.iop_pcso |= 0x0030;
#endif

#ifdef CONFIG_TS1H
	pm->im_ioport.iop_padir |= 0xEF1C;
	pm->im_ioport.iop_papar &= ~0xEF1C;
	pm->im_ioport.iop_paodr &= ~0xEF1C;
	pm->im_ioport.iop_padat &= ~0xEF3C;

	pm->im_cpm.cp_pbdir |=  0xE4FE;
	pm->im_cpm.cp_pbodr &= ~0xE4FE;
	pm->im_cpm.cp_pbpar &= ~0xE4FE;

	pm->im_ioport.iop_pcdir |=  0x0F47;
	pm->im_ioport.iop_pcpar &= ~0x0F47;
	pm->im_ioport.iop_pcdat &= ~0x0D47;
#endif

#ifdef CONFIG_PR3K
	pm->im_ioport.iop_padir = 0;
	pm->im_ioport.iop_papar = 0;
	pm->im_ioport.iop_paodr = 0;
	pm->im_ioport.iop_padat = 0;

	pm->im_cpm.cp_pbdir = 0;
	pm->im_cpm.cp_pbodr = 0;
	pm->im_cpm.cp_pbpar = 0x40;

	pm->im_ioport.iop_pcdir = 0;
	pm->im_ioport.iop_pcpar = 0;
	pm->im_ioport.iop_pcdat = 0;

	pm->im_ioport.iop_pddir = 0;
	pm->im_ioport.iop_pdpar = 0;
	pm->im_ioport.iop_pddat = 0;
#endif
}

static unsigned short calc_crc(unsigned long byte_n, unsigned short crc, char *buffer)
{
        int j;

        while (byte_n--) {
                crc = (((crc >> 8) ^ *buffer++) << 8) | (0xFF & crc) ;
                for(j = 0; j < 8; j++) {
                        if (crc & 0x8000)  {
                                crc <<= 1;
                                crc ^= 0x1021;
                        } else {
                                crc <<= 1;
                        }
        }
    }
    return(crc);
}

static unsigned char *get_config_vector(unsigned char *pflash, int type, int tries)
{
	FLASH_CONFIG_VECTOR_HEADER *p;
	int cnt = 0;

	while(cnt++ < tries) {
		p = (FLASH_CONFIG_VECTOR_HEADER *)pflash;
		if (p->type == 9999) {
			break;
		}
		if ((p->type > 200 && p->type != 9000) || p->size > 10000) {
			break;
		}
		if (p->crc != calc_crc(p->size, 0, p->data)) {
			break;
		}
		if (p->type == type) {
			return(p->data);
		}
		pflash += p->size + 6;
	}
	return(NULL);
}

char * _bcopy_str(char *src, char *dst);

#include "cyc_banner.h"

/* Build a board information structure for the PRXK.
 */
void
prxk_cfg(bd_t *bd)
{
	unsigned char *pflash, *pf, *aux;
	struct prxk_boot_info *p = (struct prxk_boot_info *)0x00002000;
	car8xx_t *pcar = &(((immap_t *)IMAP_ADDR)->im_clkrst);
	struct HW_SETUP *pcnf;
	OM_INFO *p_om;
	int speed[6] = {4800, 9600, 19200, 38400, 57600, 115200};
#ifdef CONFIG_TS1H
	bd_t *res = bd;	// SECTOR_SIZE requires
#endif

	memcpy(&bd->hw_info, p, sizeof(struct prxk_boot_info));

#ifdef CONFIG_TS1H
	if (bd->hw_info.clock_freq != 4) {
		bd->bi_busfreq = ((pcar->car_plprcr >> 20) + 1) * 25; // 25 MHz crystal
	} else {
		bd->bi_busfreq = ((pcar->car_plprcr >> 20) + 1) * 4;  // 4 MHz crystal
	}
#else
	bd->bi_busfreq = ((pcar->car_plprcr >> 20) + 1) * 4;  // 4 MHz crystal
#endif
	bd->bi_intfreq = bd->bi_busfreq;

	bd->bi_memsize = p->memory_detected;

	if (p->board_type == BOARD_TS3K ||
		p->board_type == BOARD_TS1H ||
		p->board_type == BOARD_TS110 ||
		p->board_type == BOARD_SM100 ||
		p->board_type == BOARD_NL1K) {
		pflash = FLASH_ADDR + 3 * SECTOR_SIZE;
		pf= OM_ADDRESS1 + FLASH_ADDR;
	} else {
		pflash = FLASH_ADDR + 2 * SECTOR_SIZE;
		pf= OM_ADDRESS2 + FLASH_ADDR;
	}

	if ((pcnf = (struct HW_SETUP *)get_config_vector(pflash, TYPE0068, 1000))) {
		bd->bi_baudrate = speed[pcnf->console_speed];
		memcpy(bd->bi_enetaddr, pcnf->physical_addr, EP_ALEN);
		bd->mii_operation = pcnf->mii_operation;
		bd->max_int_work = pcnf->max_int_work;
	} else {
		bd->mii_operation = 0;
		bd->bi_baudrate = 9600;
	}
	switch (bd->hw_info.wdt_config) {
		case 0:
		case 1:
			bd->wdt = 1;
			break;
		case 2:
			bd->wdt = 0;
			break;
	}

	p_om = (OM_INFO *)get_config_vector(pf, TYPE9000, 1);

	if (!bd->bi_enetaddr[1] && !bd->bi_enetaddr[2] && p_om) {
		memcpy(bd->bi_enetaddr, p_om->mac_addr, EP_ALEN);
	}

	if (!bd->bi_enetaddr[1] && !bd->bi_enetaddr[2]) {
		bd->bi_enetaddr[0] = 0x00;
		bd->bi_enetaddr[1] = 0x60;
		bd->bi_enetaddr[2] = 0x2e;
		bd->bi_enetaddr[3] = 0x00;
		bd->bi_enetaddr[4] = 0x00;
		bd->bi_enetaddr[5] = 0x01;
	}

	_bcopy_str(CYCLADES_VERSION, bd->cyc_version);
	//[RK]Jun/07/06
	_bcopy_str(CYCLADES_DSVIEW_PLUGIN, bd->dsview_plugin);
	_bcopy_str(machine_txt[p->board_type], bd->board_name);
	_bcopy_str("UNKNOWN", bd->card_name);
	_bcopy_str(CYCLADES_TXT, bd->vendor_name);
	_bcopy_str(PROD_LINE, bd->prod_line);
	bd->num_ports = 0;

	if (p_om && p_om->banner[0]) {
		aux = _bcopy_str(p_om->banner, bd->cyc_banner);
		*aux++ = '\r';
		*aux = '\n';
	} else {
		aux = _bcopy_str(CYCLADES_TXT, bd->cyc_banner);
		*aux++ = '-';
		aux = _bcopy_str(bd->board_name, aux);
		*aux++ = '-';
		aux = _bcopy_str(CYCLADES_BANNER, aux);
		aux = _bcopy_str(CYCLADES_VERSION, aux);
	}

#ifdef CONFIG_TS1H
	_bcopy_str("TS1H", bd->card_name);
	bd->num_ports = 1;
#endif
#ifdef CONFIG_TSxK
	_bcopy_str("TS", bd->prod_name);
	_bcopy_str("TSxK", bd->card_name);
	switch(p->board_type) {
	case BOARD_TS3K:		// TS3000
		_bcopy_str("TS3K", bd->card_name);
		bd->num_ports = 48;
		break;
	case BOARD_TS1K:		// TS1000
		bd->num_ports = 16;
		break;
	case BOARD_TS2K:		// TS2000
		bd->num_ports = 32;
		break;
	case BOARD_TS4H:		// TS400
		bd->num_ports = 4;
		break;
	case BOARD_TS8H:		// TS800
		bd->num_ports = 8;
		break;
	}
#endif

	reset_ports(p->board_type);

}

#endif

#ifdef CONFIG_EST8xx
/* Build a board information structure for the EST8xx.
 */
void
est8xx_cfg(bd_t *bd)
{
	u_char	*cp;
	int	i;

#if 1
	bd->bi_baudrate = 115200;
	bd->bi_intfreq = bd->bi_busfreq = 50;
	bd->bi_memsize = 8 * 1024 * 1024;
#endif

	cp = (u_char *)def_enet_addr;
	for (i=0; i<6; i++) {
		bd->bi_enetaddr[i] = *cp++;
	}
}
#endif

