/*----------------------------------------------------------------------------
 *      Copyright (C) Cyclades Corporation, 1998.
 *      All rights reserved
 *      PR-3000 TS
 *
 *      This file:      cy_wdt.c
 *      Description:    Routines to handle WDT and CPU led.
 *---------------------------------------------------------------------------
 *---------------------------------------------------------------------------
 *      Change History
 *---------------------------------------------------------------------------
 *      Sep/28/2000     V 1.0.0    Edson Ap. Seabra - Initial Implementation
 *--------------------------------------------------------------------------*/

#include <linux/config.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/miscdevice.h>
#include <linux/malloc.h>
#include <linux/ioport.h>
#include <linux/fcntl.h>
#include <asm/uaccess.h>
#include <asm/system.h>

#include <asm/mpc8xx.h>
#include <asm/8xx_immap.h>

#ifdef CONFIG_PR3K
#include <asm/fpga.h>
#endif

static int wdt_is_open=0;
static int led_is_open=0;

#ifdef CONFIG_SM100
static int pa8pa7old = 0;
static int pa8pa7new = 0;
#define RS_STATUS_SMC1	(0x0080)
#define RS_STATUS_SCC1	(0x0100)
#define RS_STATUS		(RS_STATUS_SMC1|RS_STATUS_SCC1)
#endif

static long long wdt_llseek(struct file *file, long long offset, int origin)
{
	return -ESPIPE;
}

static ssize_t wdt_write(struct file *file, const char *buf, size_t count, loff_t *ppos)
{
        struct inode *inode;
#ifndef CONFIG_PR3K
	volatile immap_t *pm = (volatile immap_t *)IMAP_ADDR;
#endif


        inode = file->f_dentry->d_inode;

	switch(MINOR(inode->i_rdev))
	{
		case WATCHDOG_MINOR:
			if(!wdt_is_open) {
				return -ESPIPE;
			} else {
               		 	volatile sysconf8xx_t *psiu = &(((immap_t *)IMAP_ADDR)->im_siu_conf);
               		 	psiu->sc_swsr = 0x556c;
                		psiu->sc_swsr = 0xaa39;
        		}
			break;
		case TEMP_MINOR:
#ifndef CONFIG_PR3K
			check_fec_neg_complete();
#endif
#ifdef CONFIG_SM100
			/* for debugging */
#if 0
			if (pa8pa7old != (pa8pa7new = (pm->im_ioport.iop_padat & RS_STATUS))) {
				if ((pa8pa7old & RS_STATUS_SCC1) != (pa8pa7new & RS_STATUS_SCC1))
					printk("\r\nSCC1 status changed from 0x%04x to 0x%04x", pa8pa7old & RS_STATUS_SCC1, pa8pa7new & RS_STATUS_SCC1);
				if ((pa8pa7old & RS_STATUS_SMC1) != (pa8pa7new & RS_STATUS_SMC1))
					printk("\r\nSMC1 status changed from 0x%04x to 0x%04x", pa8pa7old & RS_STATUS_SMC1, pa8pa7new & RS_STATUS_SMC1);
				//printk("\r\nSCC1 and SMC1 status changed from 0x%04x to 0x%04x", pa8pa7old, pa8pa7new);
				pa8pa7old = pa8pa7new;
			}
#endif
#endif
			if(!led_is_open) {
				return -ESPIPE;
			}
#ifdef CONFIG_PR3K
			if (*buf) {
				fpga->cpu_led &= ~(0x10);
			} else {
				fpga->cpu_led |= (0x10);
			}
#else
#ifdef CONFIG_SM100
			if (*buf) {
		 		pm->im_ioport.iop_padat &= ~(0x8000);		
			} else {
		 		pm->im_ioport.iop_padat |= 0x8000;	
			}
#else
			if (*buf) {
		 		pm->im_cpm.cp_pbdat &= ~(0x0001); 			
			} else {
		 		pm->im_cpm.cp_pbdat |= 0x0001; 			
			}
#endif
#endif
			break;
		default:
			return -ENODEV;
	}
	return 1;
}

/*
 *	Read reports the temperature in degrees Fahrenheit.
 */
 
static ssize_t wdt_read(struct file *file, char *buf, size_t count, loff_t *ptr)
{
        struct inode *inode;

        inode = file->f_dentry->d_inode;

	if (MINOR(inode->i_rdev) != TEMP_MINOR) {
		return -ESPIPE;
	}
	*buf = ((bd_t *)res)->hw_info.error_status;
	return 1;
}

static int wdt_open(struct inode *inode, struct file *file)
{
	switch(MINOR(inode->i_rdev))
	{
		case WATCHDOG_MINOR:
			if(wdt_is_open)
				return -EBUSY;
			wdt_is_open=1;
        	((bd_t *)res)->wdt |= 2;
			break;
		case TEMP_MINOR:
			if(led_is_open)
				return -EBUSY;
			led_is_open=1;
			break;
		default:
			return -ENODEV;
	}
	return 0;
}

static int wdt_release(struct inode *inode, struct file *file)
{
	switch(MINOR(inode->i_rdev))
	{
		case WATCHDOG_MINOR:
			if(!wdt_is_open)
				return -ESPIPE;
			wdt_is_open=0;
        	((bd_t *)res)->wdt &= 1;
			break;
		case TEMP_MINOR:
			if(!led_is_open)
				return -ESPIPE;
			led_is_open=0;
			break;
		default:
			return -ENODEV;
	}
	return 0;
}

/*
 *	Kernel Interfaces
 */
 
 
static struct file_operations wdt_fops = {
	wdt_llseek,
	wdt_read,
	wdt_write,
	NULL,		/* No Readdir */
	NULL,		/* No Select */
	NULL,		/* No ioctl */
	NULL,		/* No mmap */
	wdt_open,
	NULL,		/* flush */
	wdt_release
};

static struct miscdevice wdt_miscdev=
{
	WATCHDOG_MINOR,
	"PPC_wdt",
	&wdt_fops
};

static struct miscdevice led_miscdev=
{
	TEMP_MINOR,
	"CPU_led",
	&wdt_fops
};

int cy_wdt_init(void)
{
#ifndef CONFIG_PR3K
	volatile immap_t *pm = (volatile immap_t *)IMAP_ADDR;
#endif

#ifdef CONFIG_PR3K
	fpga->cpu_led = 0xf8;
#else
#ifdef CONFIG_SM100
    pm->im_ioport.iop_padir |=  0x8000;
    pm->im_ioport.iop_papar &= ~0x8000;
    pm->im_ioport.iop_padat |=  0x8000;

#else
    pm->im_cpm.cp_pbpar &= ~(0x0001);
    pm->im_cpm.cp_pbdir |= 0x0001;
    pm->im_cpm.cp_pbdat |= 0x0001;
#endif
#endif
	misc_register(&wdt_miscdev);
	misc_register(&led_miscdev);
	return 0;
}

