/*
 *
 *  hhnet/driver/hhnet.h, version 1.1
 *
 *  Copyright 2000, MontaVista Software, Inc.
 *
 *  This software may be used and distributed according to the terms of
 *  the GNU Public License, Version 2, incorporated herein by reference.
 *
 *  Contact:  <source@mvista.com>
 *
 */

#ifndef _HHNET_H_
#define _HHNET_H_

#define NDEBUG
#define HHNET_ETHERNET_ADDRLEN	6

extern int hhnet_probe(void);

typedef void (*hhnet_receive_t)(void *arg, void *buf);
typedef void * (*hhnet_bufalloc_t)(void *arg, unsigned int len, void **buf);

extern int
hhnet_open(
    void *addr,
    hhnet_receive_t recv,
    hhnet_bufalloc_t alloc,
    void *arg);

extern void hhnet_close(void);

typedef void (*hhnet_iodone_fn)(int result, void *arg);

extern int
hhnet_send(
    void *addr,
    void *data,
    unsigned int len,
    hhnet_iodone_fn iodone,
    void *arg);

extern int
hhnet_broadcast(
    void *data,
    unsigned int len,
    hhnet_iodone_fn iodone,
    void *arg);

struct hhnet_device;

struct hhnet_device_funcs {
    int (*listen)(void *dev, struct hhnet_device *netdev);
    int (*accept)(void *dev, unsigned long sysbuf, int devnum);
    int (*connect)(void *dev, struct hhnet_device *netdev, void *buf);
    int (*start)(void *dev);
    int (*update)(void *dev);
    void * (*open_conn)(
	void *dev,
	int num,
	int indirect,
	u32 mem_paddr,
	u32 csr_paddr,
	void **devconn);
    int (*send)(void *dev);
    void (*close_conn)(void *devconn);
    void (*stop)(void *dev);
    void (*disconnect)(void *dev);
    void (*close)(void *dev);
};

struct hhnet_funcs {
    int (*add_device)(struct hhnet_device_funcs *devfn, void *dev, int manager);
    int (*accept)(
	struct hhnet_device *netdev,
	void *addr,
	unsigned long csr_paddr,
	unsigned long mem_paddr);
    int (*connect)(struct hhnet_device *netdev, void *sysmem, int devnum);
    int (*start)(struct hhnet_device *netdev);
    void (*update)(struct hhnet_device *netdev);
    void (*receive)(struct hhnet_device *netdev);
    void (*stop)(struct hhnet_device *netdev);
    void (*disconnect)(struct hhnet_device *netdev);
};

#endif /* _HHNET_H_ */
