/*****************************************************************
* File: ipdu.c
*
* Copyright (C) 2003 Cyclades Corporation
*
* www.cyclades.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version
* 2 of the License, or (at your option) any later version.
*
* Description:
*
****************************************************************/

# include <stdio.h>
# include <sys/types.h>
# include <sys/stat.h>
# include <unistd.h>
# include <string.h>
# include <stdlib.h>
# include <ctype.h>
# include <server.h>
# include <pmd.h>

// These strings are used to parse the ipdu configuration file:

char needLogin[] = "needLogin";
char username[] = "username";
char password[] = "password";
char prompt[] = "prompt";
char logout[] = "logout";
char timeout[] = "timeout";
char admUser[] = "admUser";
char admPasswd[] = "admPasswd";
char confirm[] = "confirm";
char admConfirm[] = "admConfirm";
char multiOutCommand[] = "multiOutletCommand";
char logoutAfterCommandComplete[] = "logoutAfterCommandComplete";
char timeAfterFailedLogin[] = "timeAfterFailedLogin";
char entersAfterLogout[] = "entersAfterLogout";
char dcdDetect[] = "dcdDetect";
char psyslog[] = "syslog";
char initcomm[] = "initcomm";
char numberOutlets[] = "numberOutlets";
char numberIPDUs[] = "numberIPDUs";

char regular[] = "regular";
char admin[] = "admin";

char yes[] = "yes";
char no[] = "no";

/*******************************************************************
 * This routine fill the ipdu struct with the information from the
 * the buffer start.
 *******************************************************************/
static int fillIpduStruct( ipduInfo * ipdu, char * start) {
	char * s = start, * t, * u, * v;
	int menu = 0, open = 0, nitems = 0, line = 0, arg, fim;
	menuItem *item;

	while (* s) {
		t = s;
		/* Gets a line */
		while (*s && *s != '\n') s++;
		if (*s) *s++ = 0;
		/* Pass the spaces */
		while (isspace(*t)) t++;
		line ++;
		if (*t == 0 || *t == '#') {
			/* Comment - do nothing */
			continue;
		} else if (menu) {
			/* Read the menu */
			if (!open) {
				/* '{' not shown yet. */
				if (*t != '{') {
					return line;
				}
				t++;
				while (isspace(*t)) t++;
				open = 1;
			}
			if (*t == '}')	{
				/* menu closed */
				if (menu == 1) {
					ipdu->menuMax = nitems;
				} else if (menu == 2) {
					ipdu->admMax = nitems;
				}
				open = 0;
				menu = 0;
				continue;
			} else if (*t == '(') {
				/* Aditional set */
				if (menu == 1) {
					item = &ipdu->menuItems[nitems];
				} else if (menu == 2) {
					item = &ipdu->adminItems[nitems];
				} else {
					return line;
				}
				/* All parameters should be separated by 
				 * commas */
				t++;
				while (isspace(*t)) t++;
				u = t;
				while (*t && *t != ',') t++;
				if (*t) *t++ = 0;
				if ( strlen(u) > MAXLABELLEN ) {
					u[MAXLABELLEN] = 0;
				}
				item->label = u;
				while (isspace(*t)) t++;
				u = t;
				while (*t && *t != ',') t++;
				if (*t) *t++ = 0;
				item->command = u;
				/* Read help string */
				while (isspace(*t)) t++;
				if (*t != '"') return line;
				t++;
				u = t;
				while (*t != '"') {
					if (*t == 0) return line;
					t++;
				}
				*t++ = 0;
				item->help = u;
				while (isspace(*t)) t++;
				if (*t != ',') return line;
				t++;
				/* Read arguments */
				while (isspace(*t)) t++;
				u = t;
				arg = 0;
				if (*t == ')') {
					fim = 1; 
				} else {
					fim = 0;
				}
				while (!fim) {
					if (*t == 0) return line;
					if (!isspace(*t) && *t != '|' && *t != ')') {
						t++;
						continue;
					}
					if (isspace(*t)) {
							*t++ = 0;
							while (isspace(*t)) t++;
							if (*t != ')') return line;
							fim = 1;
					} else if (*t == ')') {
							fim = 1;
					}
					*t++ = 0;
					if (arg >= 4) {
						return line;
					} else if (! strcmp (u, "outlet")) {
						item->args[arg++] = ARG_OUTLET;
					} else if (! strcmp (u, "outlet1")) {
						item->args[arg++] = ARG_OUTLET1;
					} else if (! strcmp (u, "user")) {
						item->args[arg++] = ARG_USER;
					} else if (! strcmp (u, "onoff")) {
						item->args[arg++] = ARG_ONOFF;
					} else if (! strcmp (u, "ipdu")) {
						item->args[arg++] = ARG_IPDU;
					} else if (! strcmp (u, "current")) {
						item->args[arg++] = ARG_CURRENT;
					} else if (! strcmp (u, "name")) {
						item->args[arg++] = ARG_NAME;
					} else {
						return line;
					}
					u = t;
				}
				if (arg < 4) item->args[arg] = 0;
				nitems ++;
			}
		} else {
			/* u will be the parameter and v will be the value */
			u = t;
			while (*t && !isspace(*t) && *t!='=') t++;
			if (*t) *t++ = 0;
			if (!strcmp(u, needLogin) || 
				!strcmp(u, multiOutCommand) ||
				!strcmp(u, logoutAfterCommandComplete) ||
				!strcmp(u, timeAfterFailedLogin) ||
				!strcmp(u, entersAfterLogout) ||
				!strcmp(u, dcdDetect) ||
				!strcmp(u, regular) ||
				!strcmp(u, admin)) {
				while (isspace(*t) || *t == '=') t++;
				v = t;
				while (*t && !isspace(*t)) t++;
				if (*t) *t++ = 0;
			} else {
				while (*t && *t != '"') t++;
				if (*t == 0) {
					return line;
				}
				t ++;
				v = t;
				while (*t && *t != '"') t++;
				if (*t == 0) {
					return line;
				}
				*t++ = 0;
			}
			if (strcmp(u, needLogin) == 0) {
				if (strcmp(v, yes) == 0) {
					ipdu->needAuth = 1;
				} else if (strcmp(v, no) == 0) {
					ipdu->needAuth = 0;
				} else {
					return line;
				}
			} else if (strcmp(u, username) == 0) {
				ipdu->username = v;
			} else if (strcmp(u, password) == 0) {
				ipdu->password = v;
			} else if (strcmp(u, prompt) == 0) {
				ipdu->prompt = v;
			} else if (strcmp(u, logout) == 0) {
				ipdu->logout = v;
			} else if (strcmp(u, timeout) == 0) {
				ipdu->timeout = v;
			} else if (strcmp(u, confirm) == 0) {
				ipdu->confirm = v;
			} else if (strcmp(u, admUser) == 0) {
				ipdu->admUser = v;
			} else if (strcmp(u, admPasswd) == 0) {
#ifdef admPasswdAppend
				ipdu->admPasswd = malloc(strlen(v) + 
					strlen(admPasswdAppend) + 1);
				strcpy(ipdu->admPasswd, v);
				strcat(ipdu->admPasswd, admPasswdAppend);
#else
				ipdu->admPasswd = v;
#endif
			} else if (strcmp(u, admConfirm) == 0) {
				ipdu->admConfirm = v;
			} else if (strcmp(u, psyslog) == 0) {
				ipdu->syslog = v;
			} else if (strcmp(u, initcomm) == 0) {
				ipdu->initcomm = v;
			} else if (strcmp(u, numberOutlets) == 0) {
				ipdu->numberOutlets = v;
			} else if (strcmp(u, numberIPDUs) == 0) {
				ipdu->numberIPDUs = v;
			} else if (strcmp(u, multiOutCommand) == 0) {
				if (strcmp(v, yes) == 0) {
					ipdu->multiOutCommand = 1;
				} else if (strcmp(v, no) == 0) {
					ipdu->multiOutCommand = 0;
				} else {
					return line;
				}
			} else if (strcmp(u, logoutAfterCommandComplete) == 0) {
				if (strcmp(v, yes) == 0) {
					ipdu->logoutAfterCommandComplete = 1;
				} else if (strcmp(v, no) == 0) {
					ipdu->logoutAfterCommandComplete = 0;
				} else {
					return line;
				}
			} else if (strcmp(u, timeAfterFailedLogin) == 0) {
				if (sscanf(v, "%d", &ipdu->timeAfterFailedLogin) < 1 || 
					ipdu->timeAfterFailedLogin < 0) {
					return line;
				}
			} else if (strcmp(u, entersAfterLogout) == 0) {
				if (sscanf(v, "%d", 
					&ipdu->entersAfterLogout) < 1 || 
					ipdu->entersAfterLogout < 0) {
					return line;
				}
			} else if (strcmp(u, dcdDetect) == 0) {
				if (strcmp(v, yes) == 0) {
					ipdu->dcdDetectOff = 0;
				} else if (strcmp(v, no) == 0) {
					ipdu->dcdDetectOff = 1;
				} else {
					return line;
				}
			} else if (strcmp(u, regular) == 0 || 
				strcmp(u, admin) == 0) {
				if (strcmp(u, regular) == 0) menu = 1;
				if (strcmp(u, admin) == 0) menu = 2;
				nitems = 0;
				if (strcmp(v, "{") == 0) {
					open = 1;
				} else {
					open = 0;
				}
			}
		}
	}
	return 0;
}

/*******************************************************************
 * This gets the profile of the IPDU type and save it in the struct
 * ipdu, that is, the information contained in the file 
 * /etc/pm.<type>. It returns 0 on success and -1 on failure.
 ******************************************************************/
int initIpduStruct (ipduInfo * ipdu, char * type) {
	char filename[50], *s;
	struct stat st;
	char * buf = NULL;
	FILE * fp;
	int aux;

	// dolog(LOG_INFO, "initIpduStruct - starting");

	// If the type is NULL, it cannot work
	if (type == NULL) {
		//dolog(LOG_ERR, "ipdu.c:initIpduStruct  - null type. Exiting");
		dodebug("IPDU[%d]:initIpduStruct: null type. Exiting\n",ipdu->serialPort);
		return -1;
	}

	// Getting filename
	sprintf(filename, "/etc/pm.%s", type);
	for (s=filename; !isspace(*s) && *s; s++);
	*s = 0;

	// Checking if the config file exists
	if ( stat(filename, &st) == -1) {
		//dolog(LOG_ERR, "ipdu.c:initIpduStruct: %s file does not exist", filename);
		dodebug("IPDU[%d]:initIpduStruct: %s file does not exist\n", ipdu->serialPort,filename);
		return -1;
	}

	//open the configuration file
	fp = fopen(filename, "r");
	if ( fp == NULL ) {
		//dolog(LOG_ERR, "ipdu.c:initIpduStruct - Could not open file\r\n");
		dodebug("IPDU[%d]:initIpduStruct - Could not open %s file\n",ipdu->serialPort,filename);
		return -1;
	}

	// Allocating memory to read the file
	buf = (char *) malloc(st.st_size+1);
	if ( buf == NULL ) {
		//dolog(LOG_ERR, "ipdu.c:initIpduStruct - Could not alloc memory for buf");
		dodebug("IPDU[%d]:initIpduStruct - Could not alloc memory for buf\n",ipdu->serialPort);
		return -1;
	}
	bzero(buf, st.st_size);

	// Reading the file
	fread(buf, 1, st.st_size, fp);
	buf[st.st_size] = 0;

	// Fill the ipduStruct with the file information
	bzero(ipdu, sizeof(ipduInfo));
	aux = fillIpduStruct (ipdu, buf);
	if (aux) {
		//dolog(LOG_ERR, "ipdu.c:initIpduStruct - Config error in the line %d", aux);
		dodebug("IPDU[%d]:initIpduStruct - Config error in the line %d\n", ipdu->serialPort,aux);
		free(buf);
		return -1;
	}

	// Storing the buf position then we can free it later
	ipdu->buf = buf;
	ipdu->dcdStatus = 0xFF;

	// a small consistency
	if (! ipdu->needAuth) {
		ipdu->logoutAfterCommandComplete = 0;
	}
	return 0;
}

/*******************************************************************
 * This routine frees all the memory allocated for the ipdu struct
 ******************************************************************/
void cleanIpduStruct ( ipduInfo * ipdu ) {
	if (ipdu->userlist) free(ipdu->userlist);
#ifdef admPasswdAppend
	free(ipdu->admPasswd);
#endif
	free(ipdu->buf);
	free(ipdu);
}

/*******************************************************************
 * This routine retrieves the size of the buffer to be allocated to store
 * all the menu items.
 ******************************************************************/
static int getIpdu2room( ipduInfo *ipdu )
{
	int i, size = 0;
	for (i = 0; i < ipdu->menuMax; i++) {
		if (ipdu->menuItems[i].label) {
			size += strlen(ipdu->menuItems[i].label) + 1;
		}
		if (ipdu->menuItems[i].command) {
			size += strlen(ipdu->menuItems[i].command) + 1;
		}
		if (ipdu->menuItems[i].help) {
			size += strlen(ipdu->menuItems[i].help) + 1;
		}
	}
	for (i = 0; i < ipdu->admMax; i++) {
		if (ipdu->adminItems[i].label) {
			size += strlen(ipdu->adminItems[i].label) + 1;
		}
		if (ipdu->adminItems[i].command) {
			size += strlen(ipdu->adminItems[i].command) + 1;
		}
		if (ipdu->adminItems[i].help) {
			size += strlen(ipdu->adminItems[i].help) + 1;
		}
	}
	return size;
}

/*******************************************************************
 * This routine copies the strings into a pre-allocated buffer and 
 * points the pointer to the string copied.
 ******************************************************************/
static char *copyIpduString(char *s, char *from, char **to)
{
	if (from) {
		strcpy (s, from);
		*to = s;
		s += strlen(s) + 1;
	} else {
		*to = from;
	}
	return s;
}

/*******************************************************************
 * This routine allocates in the shared memory the IPDU menu information to
 * be shared with the clients.
 ******************************************************************/
ipduInfo2 *initIpdu2Struct (ipduInfo * ipdu) {
	unsigned char *s;
	int i;
	ipduInfo2 *ipdu2;
	ipdu2 = (ipduInfo2 *)shm_malloc(sizeof(ipduInfo2), TYPE_SHM_IPDU);
	if (!ipdu2) {
		return NULL;
	}
	s = ipdu2->buf = shm_malloc(getIpdu2room(ipdu), TYPE_SHM_IPDUBUF);
	if (!s) {
		shm_free_str((char *)ipdu2);
		return NULL;
	}
	ipdu2->layout = 1;
	ipdu2->multiOutCommand = ipdu->multiOutCommand;
	for (i = 0; i < ipdu->menuMax; i++) {
		s = copyIpduString(s, ipdu->menuItems[i].label, 
			&ipdu2->menuItems[i].label);
		s = copyIpduString(s, ipdu->menuItems[i].command, 
			&ipdu2->menuItems[i].command);
		s = copyIpduString(s, ipdu->menuItems[i].help, 
			&ipdu2->menuItems[i].help);
		*(int *)ipdu2->menuItems[i].args = 
			*(int *)ipdu->menuItems[i].args;
	}
	ipdu2->menuMax = ipdu->menuMax;
	for (i = 0; i < ipdu->admMax; i++) {
		s = copyIpduString(s, ipdu->adminItems[i].label, 
			&ipdu2->adminItems[i].label);
		s = copyIpduString(s, ipdu->adminItems[i].command, 
			&ipdu2->adminItems[i].command);
		s = copyIpduString(s, ipdu->adminItems[i].help, 
			&ipdu2->adminItems[i].help);
		*(int *)ipdu2->adminItems[i].args = 
			*(int *)ipdu->adminItems[i].args;
	}
	ipdu2->admMax = ipdu->admMax;
	return ipdu2;
}

/*******************************************************************
 * This routine frees all the shared memory allocated for the ipdu2 struct
 ******************************************************************/
void cleanIpdu2Struct ( ipduInfo2 * ipdu2 ) {
	shm_free_str((char *)(ipdu2->buf));
	shm_free_str((char *)ipdu2);
}
