/*****************************************************************
* File: ipdu.h
*
* Copyright (C) 2003 Cyclades Corporation
*
* www.cyclades.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version
* 2 of the License, or (at your option) any later version.
*
* Description:
*
****************************************************************/

// The max number of commands a menu can have
# define MAXCOMMANDS 32

// Max strlen for label strings
# define MAXLABELLEN 16

//TimeOuts defines
# define ECHOTO           2    // how many seconds pmd will wait for a command echo
# define COMTO            70   // how many seconds pmd will wait for a command answer
# define USRNAMETO        5    // how many seconds pmd will wait for the username string
# define PASSWDTO         5    // how many seconds pmd will wait for the password string
# define PROMPTTO         5    // how many seconds pmd will wait for the prompt string
# define SENTTO           5    // how many seconds pmd will wait for data to be sent to the client

// Other defines
# define MAXUSRNAMETRIES  3    // how many times pmd will try to receive the username string

// This struct holds information about a command
struct _menuItem {
	char *label;		// the string will be shown in the menu
	char *command;		// the command will be sent to the ipdu
	char *help;		// help string
	char  args[4];		// arguments for the command 
#define	ARG_OUTLET	1
#define ARG_OUTLET1	2
#define ARG_USER	3
#define ARG_ONOFF	4
#define ARG_IPDU	5
#define ARG_CURRENT	6
#define ARG_NAME	7
};
typedef struct _menuItem menuItem;

struct _commItem {
	int pmindex;
	char *command;
	char *outlet;
};

typedef struct _commItem commItem;

#define kMaxUserName 31

struct _ipduUserList {
	char username[kMaxUserName];
	char outlets[MAXOutletsPS];
};

typedef struct _ipduUserList IpduUserList;

// This struct holds information about a IPDU used only
struct _ipduInfo {
	int serialPort;                  // Serial port where the IPDU is 
					 // 	connected
	int auxflg;		// 1 if the port is aux port; 0 if the port is not
	int fd;                          // File descriptor to send and receive 
					 //	data
	int needAuth;                    // Tells if the ipdu needs 
					 // 	authentication
	char * prompt;                   // The prompt string of the ipdu
	char * username;                 // The string the ipdu send when it 
					 // 	wants the usermane
	char * password;                 // The string the ipdu send when it
					 // 	wants the password 
	char * confirm;                  // The string the ipdu send when it 
					 // 	wants a confirmation for the 
					 // 	command
	char * logout;                   // The logout command of the IPDU
	char * timeout;			 // The string the ipdu sends when it
					 // logs out by timeout. 
	char * admUser;                  // The admin user for the IPDU
	char * admPasswd;                // The password for the admin user
	char * admConfirm;		 // The confirmation string
	char * syslog;			 // The syslog header string
	char * initcomm;		 // The command to be issued after the
					 // IPDU logs in
	char * numberOutlets;		 // String to detect number of outlets
	char * numberIPDUs;		 // String to detect number of IPDUs
	int multiOutCommand;             // Tells if the IPDU accepts multiple 
					 //	outlet commands
	int logoutAfterCommandComplete;  // Logs out after each command is  
					 //	complete
	int timeAfterFailedLogin;	 // Waiting time after the login 
					 // attempt has failed
	int entersAfterLogout;		 // Number of enters that has to be 
					 // sent after logout to enter in the
					 // login prompt
	int dcdDetectOff;		 // DCD detection off
	int dcdStatus;			 // DCD last status
	
	menuItem menuItems[MAXCOMMANDS]; // List of commands for regular users
	int menuMax;                     // The total number of commands for
					 //	regular
	menuItem adminItems[MAXCOMMANDS];// A list of all possible commands for 
					 //	admin
	int admMax;                      // The total number of commands for 
	char * buf;                      // This is the place where all strings 
					 // 	are kept. It is allocated by 
	 				 //	the routine initIpduStruct and
                                         // 	is freed by the routine 
	 				 // 	cleanIpduStruct.
	int state;			 // State of this IPDU; options below
	time_t start_time;		 // Time when the command was sent to
					 // 	the IPDU.
	int retry;			 // Number of retries.
	int command_get;		 // Pointer for getting the command 
					 // 	from the buffer.
	int command_put;		 // Pointer for putting the command
					 // 	in the buffer.
#define MAXIPDUQUEUE	16
	commItem commands[MAXIPDUQUEUE]; // Queue of commands which come from
					 // 	the clients.
#define IPDUBUFSIZE	3072
	char buffer[IPDUBUFSIZE];	 // Buffer of data from the serial
					 // 	port
	int bufindex;			 // Index for the input buffer
#define IPDUCOMMSIZE	64
	char comm[IPDUCOMMSIZE];	 // Current command being processed
	int pmindex;			 // Index for the pmd session structure
					 // 	in the shared memory.
	//[RK]Mar/03/04 - check userName
	IpduUserList *userlist;
	int flgSendEnter;
	int pmupgidx;
	int gotNumOutlets;
#ifdef UPLOAD_COMMAND
	int speed;
#endif
};

typedef struct _ipduInfo ipduInfo;

// This struct holds information about a IPDU to be shared with the clients.
// This structures is stored in the shared memory.
struct _ipduInfo2 {
	char layout;		// 0-sess, 1-ipdu
	int multiOutCommand;             // Tells if the IPDU accepts multiple 
					 //	outlet commands 

	menuItem menuItems[MAXCOMMANDS]; // List of commands for regular users
	int menuMax;                     // The total number of commands for 
					 //	regular
	menuItem adminItems[MAXCOMMANDS];// A list of all possible commands for 
					 //	admin
	int admMax;                      // The total number of commands for 
					 // 	admin
	char * buf;                      // This is the place where all strings 
					 // 	are kept. It is allocated by 
	 				 //	the routine initIpdu2Struct and
                                         // 	is freed by the routine 
	 				 // 	cleanIpdu2Struct.
};

typedef struct _ipduInfo2 ipduInfo2;

struct _pmdStatus{
	int state;
	time_t start_time;
	int retry;
	int cmd_get;
	int cmd_put;
	commItem commands[MAXIPDUQUEUE];
	char comm[IPDUCOMMSIZE];
	int pmindex;
};

// IPDU states
enum {
	IPDU_INIT,
	IPDU_WAITUSERNAMESTR,
	IPDU_WAITPASSWDSTR,
	IPDU_WAITPROMPTSTR,
	IPDU_READY,
	IPDU_WAITECHO,
	IPDU_WAITANSWER,
	IPDU_WAITSYSLOG,
	IPDU_WAITUSERANS,
	IPDU_UPGRADE,
	IPDU_WAITUPGRADEANS,
	IPDU_INITCOMM,
	IPDU_WAITRESTART,
	IPDU_LOGINFAILED,
#ifdef UPLOAD_COMMAND
	IPDU_UPLOADPRE,
#endif
};

// IPDU events
#define	IPDU_GOTUSERNAMESTR	0x0001
#define	IPDU_GOTPASSWDSTR       0x0002
#define	IPDU_GOTPROMPTSTR       0x0004
#define	IPDU_GOTECHO            0x0008
#define	IPDU_GOTANSWER          0x0010
#define	IPDU_GOTSYSLOG          0x0020
#define	IPDU_GOTUPGOPTION       0x0040
#define	IPDU_GOTUPGSTART        0x0080
#define	IPDU_GOTUPGDONE         0x0100
#define IPDU_TIMEOUT		0x0200
#define	IPDU_GOTTIMEOUTSTR      0x0400
#define	IPDU_GOTCONFIRM         0x0800
#define	IPDU_ENDSYSLOG          0x1000

// Routines
int initIpduStruct (ipduInfo * ipdu, char * type);
void cleanIpduStruct( ipduInfo * ipdu);
ipduInfo2 *initIpdu2Struct (ipduInfo * ipdu);
void cleanIpdu2Struct( ipduInfo2 * ipdu2);
