/*****************************************************************
* File: pm.c
*
* Copyright (C) 2003 Cyclades Corporation
*
* www.cyclades.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version
* 2 of the License, or (at your option) any later version.
*
* Description:
* This program treats the commands pass from user to pm menu and 
* pmCommand. It takes the command, check the arguments and pass to
* pmd daemon, by coping command to shared memory and sending a signal
* to pmd indicating that there is a command to be executed.
*
****************************************************************/

#include "pmCommand.h"
#include <grp.h>

#define ADMIN_GROUP             "admin"
#define ADMIN_USER              "admin"
#define alias		serverfarm

//[GY]2005/Nov/11 
#include "pmheaders.h"
static unsigned char *outletCommands[] = {OUTLET_COMMANDS, NULL};

int admin=0,serial,pmcindex,multimode_numcomands=0;
char username[NAME]={0};
pmcInfo *pmc=0;

//[RK]Dec/09/05 - need to review in the next release
char mult_buf[5000];

#ifdef ONS
int auxflag = 0;
#endif

static pid_t my_pid;

/* Which software interrupts were detected */
int got_sig;

# define GOT_SIGUSR1	1	/* It is signalled by PMD, indicating that the
				   PMD is exiting */
# define GOT_SIGUSR2	2	/* It is signalled by PMD, indicating that 
				   there is data to be read from the shared
				   memory. */
# define GOT_SIGINT	4	/* It exits the pm/pmCommand process */
#define MAX_DEV         cy_shm->max_dev

char pm_syntax[] =
	"\r\nUse: pm <serial port number>\r\n";

char pm_line[] =
        "--------------------------------------------------------------------- \r\n";

char pm_prompt[] =
	"\r\nPlease choose an option: ";

#ifdef MULTI_OUT
char pm_menu_new_start[] =
        "--------------------------------------------------------------------- \r\n"
	"1. Exit\t2. individual ipdus\t3. multi-outlet device\t4.Info";

char pm_menu_new_start_desc[] =
        "--------------------------------------------------------------------- \r\n"
 	"If option 2  is chosen, then the  user then will be  asked about the\r\n"
	"serial  port number  (what in  previous versions  of this  application\r\n"
	"would have been  passed by a command line argument  only)and next will\r\n"
	"be lead  to the same  menu he  would have found  if he had  called the\r\n"
	"application  in the  traditional way,  except  for the  fact that  the\r\n"
	"option 1 for the menu that  will appear, that otherwise would be Exit,\r\n"
	"this time will be Return and will take the user back to this new first\r\n"
	"menu if chosen. Otherwise,  if he or she chooses option  3, he will be\r\n"
	"presented with a menu where  commands for multi-outlet entidities will\r\n"
	"be  available. For  more info  see the  specific help  for that  menu.\r\n"
	"Option 1 finishes this application, and option 4 shows this info.";
#endif

//[GY]2005/Nov/11  
/*
char pm_menu_multi_1[] = 
	"\r\n"
        "--------------------------------------------------------------------- \r\n"
	"1. Return\t4. Cycle\t6. Unlock\t\t8. Show \r\n"
	"2. On\t\t5. Lock\t\t7. Status\t\t9. Info \r\n"
	"3. Off"; 

char pm_menu_multi_1_desc[] =	
	"\r\n"
        "--------------------------------------------------------------------- \r\n"
	"1.Return: takes you back to the last menu\r\n"
	"2.On: Issue  an \"on\" on  a specified multi-device  outlet.  Its name\r\n"
	"will be asked if not passed already  as an argument (a string with the\r\n"
	"alias name). This command will be translated into multiple on commands\r\n"
	"respecting  the sequence  give in  the sXX.outlets  parameter for  the\r\n"
	"outlets in the same ipdu.\r\n"
	"3.Off: Will turn off all the outlets belonging  to this  multi-outlet.\r\n"
	"4.Cycle: executes a  cycle on the multi-outlet  device,  which has the\r\n"
	"same effect of an off followed by an on at the multi-outlet.\r\n"
	"5.Lock:  Will  block this  multi-outlet  from   issuing any  of  prior\r\n"
	"commands by an individual lock on each of the outlets\r\n"
	"6.Unlock: Will unlock all the outlets belonging  to this multi-outlet.\r\n" 
	"7.Status:  Will  issue  individual  status  commands  on each  of  the\r\n"
	"outlets belonging to this multi-outlet.\r\n"
	"8.Show:  Will  show  which  outlets  in  which  ipdus belong  to  this\r\n"
	"multi-outlet\r\n"
	"9.Info: Will show this help  text  explaining each one of the options.\r\n";
*/

int start_processing(char **cmdoptions, unsigned char multimode); 

#ifdef MULTI_OUT
int multioutlet_menu(void);
void multioutlet_do(char *cmd, char *header);
void multioutlet_issuecmd(char *list, char *cmd, char *header);
#endif
static int OutletNames2Numbers(char *command);
static int OutletName2Number(char *outlet);

#ifdef MULTI_OUT
//retrieve devices configured as IPDU
int multioutlet_getlist(char *devlist);
//process the show option of the multi-outlet menu
void multioutlet_show(void);
#endif

static int pm_treat_exit(struct _commands *pcomm); 
static int pm_show_help(struct _commands *pcomm);
static int pm_treat_whoami(struct _commands *pcomm);
 
struct _commands pmBasicCommands[] = {
	{0, "Exit", "exit", NULL, 
		"Exit", 
		NULL, pm_treat_exit},
	{0, "Help", "help", NULL, 
		"Show this help", 
		NULL, pm_show_help},
	{0, "Who Am I", "whoami", NULL, 
		"Display the username currently logged in", 
		NULL, pm_treat_whoami}
};
#define NUM_BASIC_COMMANDS	(sizeof(pmBasicCommands)/sizeof(struct _commands))

static unsigned char itemList[NUM_BASIC_COMMANDS + NUMBER_OF_COMMANDS];

static char *read_string(char *string, int size) 
{
	char *r, *s;
	fgets(string,size,stdin);
	string[strlen(string)-1]='\0';
	/* Treat the backspaces */
	for (r = s = string; *s; s ++) {
		if (*s != '\b') {
			*r ++ = *s;
		} else if (r > string) {
			r --;
		}
	}
	*r = '\0';
	return string;
}

static void usage() {
	puts(pm_syntax);
}

int check_pmd() {
	if (!cy_shm->PMD_pid) {
		printf ("\r\n[Error] - PMD has not started.\r\n");
		return 0;
	} 
	return 1;	
}

static int check_ipdu(int ipduport) {
	if (cy_shm->line_conf[ipduport-1].protocol != P_IPDU) {
			printf("\r\n[Error] - Protocol IPDU is not set.\r\n");
			return 0;
	} 
	return 1;
}

/* Check if user is admin, and if not, if there is an users allowed list,
 * if is, check if user are in list. 1 is success, 0 is fail */

static int check_user() {
	int uid;
	char *str, *token="\0";
	struct passwd *user;
	struct group *grpa; //[RK]Aug/13/04 - check admin group
	
	uid=getuid();
	user=getpwuid(uid);

	strcpy(username,user->pw_name);

	grpa = getgrnam(ADMIN_GROUP);

	if (grpa) {
		//[RK]Aug/13/04 - check group admin
		while (*grpa->gr_mem) {
			if (!strcmp(*grpa->gr_mem, username)) {
				admin=1;
				return 1;
			}
			grpa->gr_mem++;
		}
	}
	admin=0;

	if (lineconf[serial-1].pmusers) { //[RK]Aug/13/04 - check the pmusers not users
		//str=strdup(lineconf[serial-1].pmusers);
 		str=malloc(strlen(lineconf[serial-1].pmusers)+1);
 		strcpy(str,lineconf[serial-1].pmusers);
		token=strtok(str," :;");
		while (token != NULL) {
			if(strcmp(username,token)==0) 
				return 1;
			else 
				token=strtok(NULL," :;");
		}
		free(str);
	}
	return 0;
}

// treatSignals : set got_sig with corret signal received SIGUSR1 or SIGUSR2
static void treatSignals( int sig ) {
	struct sigaction act;
	memset((char*)&act,0,sizeof(struct sigaction));
	switch (sig) {
		case SIGUSR1:
			got_sig |= GOT_SIGUSR1;
			break;
		case SIGUSR2:
			got_sig |= GOT_SIGUSR2;
			break;
		case SIGINT:
		case SIGTERM:
			got_sig |= GOT_SIGINT;
			break;
	}
        act.sa_handler = treatSignals;
        sigaction(sig, &act, 0);
        unblock(sig);
}

static void setSignals() 
{
	struct sigaction act;

	memset((char*)&act,0,sizeof(struct sigaction));
	
	// Setting up the act structure
	act.sa_handler = treatSignals;
	sigemptyset(&act.sa_mask);
	act.sa_flags = 0;
	
	sigaction(SIGUSR1, &act, 0);
	sigaction(SIGUSR2, &act, 0);
	sigaction(SIGINT, &act, 0);
	sigaction(SIGTERM, &act, 0);
	unblock(SIGUSR1);
	unblock(SIGUSR2);
	unblock(SIGINT);
	unblock(SIGTERM);
}


static int  waitAnswer() {

	while(TRUE) {
		sleep(1);
		if (got_sig & GOT_SIGUSR1) {
			got_sig &= ~GOT_SIGUSR1;
			return 0;
		}
		if (got_sig & GOT_SIGUSR2) {
			got_sig &= ~GOT_SIGUSR2;
			return 1;
		}
		if (got_sig & GOT_SIGINT) {
			Exit();	
		}
	
	}

}

static pmcInfo * initPmc(int flag) 
{

	int sizeB;	// size of answer buf = 400 + (80 * number of outlets)
	char * buf;
	int noutlets;

	if (pmc && flag) {
		cy_shm->pmc_list[pmc->idx_pmclist] = NULL;
		shm_free_str(pmc->pmc_answerBuf);
		shm_free_str(pmc);
	}

	for (pmcindex=0;pmcindex<MAXPMDACCESS; pmcindex++) {
		if (cy_shm->pmc_list[pmcindex] == 0)
			break;
		pmc = cy_shm->pmc_list[pmcindex];
		if (kill(pmc->pmc_pid, 0) < 0) {
			if (errno == ESRCH) {
				cy_shm->pmc_list[pmcindex] = NULL;
				if (pmc->pmc_answerBuf) {
					shm_free_str(pmc->pmc_answerBuf);
				}
				shm_free_str((char *)pmc);
				break;
			}
		}
	}
	if (pmcindex==MAXPMDACCESS) {
		puts("Cannot find an empty position at pmc_list");
		exit(0);
	}

	noutlets = serial == 0 ? 10 : cy_shm->PMD_outlets[serial-1];
	sizeB = 400 + (80 * noutlets);
	buf = shm_malloc(sizeB,TYPE_SHM_PMCBUF);
	pmc = (pmcInfo *)shm_malloc(sizeof(pmcInfo), TYPE_SHM_PMC);

	if (!pmc || !buf) {
		if (pmc) shm_free_str((char *)pmc);
		if (buf) shm_free_str(buf);
		pmc = NULL;
		puts("Memory allocation error");
		exit(0);
	}
	shm_signal(CY_SEM_SHM);
	pmc->idx_pmclist = pmcindex;
	pmc->pmc_pid = my_pid;
#ifdef ONS
	pmc->pmc_aux = auxflag;
#endif
	pmc->pmc_ipdu = serial-1; //Checar isto
	pmc->pmc_status = PMC_IDLE;
	pmc->pmc_sizeAnswerBuf = sizeB;
	pmc->pmc_answerBuf = buf;
	pmc->next = NULL;
	cy_shm->pmc_list[pmcindex] = pmc;
	strcpy(pmc->pmc_username,username);
	shm_unsignal(CY_SEM_SHM);
	return pmc;

}

static void Exit() 
{
	shm_signal(CY_SEM_SHM);
	if (pmc) {
		cy_shm->pmc_list[pmcindex]=NULL;
		shm_free_str(pmc->pmc_answerBuf);
		shm_free_str((char *)pmc);
	}
	shm_unsignal(CY_SEM_SHM);
	exit(0);
}


static void sendCommand(char *command) 
{
//[GY]2005/Nov/11  (start)
        int i, out_flag = 0;

        for( i = 0; outletCommands[i] != NULL; i++ ) {
           if( !strncmp(command, outletCommands[i], strlen(outletCommands[i])) ) {
              out_flag = 1;
           }
        }
        if( out_flag || !strncmp(command, "name", 4) ) {
//[GY]2005/Nov/11  (end)

	   if(OutletNames2Numbers(command)){ //error, no outlet left
		puts("No valid outlet specified\n");
		puts(pm_outlet_desc);
		return;
	   }
        }
	strcpy(pmc->pmc_cmd,command);
	strcpy(pmc->pmc_username,username);
	pmc->pmc_status = PMC_REQUEST;
	if (!waitAnswer()) {
		puts("Error sending command");
	}
	else {
		printf("\r\n");
		printf(pmc->pmc_answerBuf);
	}
	pmc->pmc_answerBuf[0]=0;
	pmc->pmc_status = PMC_IDLE;
	memset(pmc->pmc_cmd,0,CMD);
	memset(pmc->pmc_username,0,NAME);
}

/***************************************************************************************************
 outletName2Number(char *command) substitutes, in the command give, each oultet name by its number
 Basically, the function sends a status command for that name, as root, for pmd, gets the answer,
 parses it for the outlet number, and stores it into the command, in place of the name. Notice that
 a similar aproach has been used before, in the function check_outlet_name, just below this one.

 command - buffer containing the command, each outlet name will be replaced by its number

 Returns 0 if at least one valid outlet name was substituted by its number, or -1 if all the names
 supplied were invalid (not known by pm)
 **************************************************************************************************/
static int OutletNames2Numbers(char *command)
{
	char localcommand[100];
	char cmdbuf[100], outletbuf[100],  *parser;
	int ind=0;
	char *cfs=NULL; //first space character position in command		

	//if it is a name command, we dont process the last argument
	unsigned char isname = 0 ;
	//used to mark special case: name command, having given old and new name of the outlet
	unsigned char name3=0; 
        unsigned char double_quote = 0, loop = 1; //[GY]2005/Nov/11 

//[GY] 2005/Sep/27 BUG#4790,4791
	//we dont process here any of these commands
//	if(!strncmp(command,"ver",3)) return 0; //version command, it doesnt take outlets
//	if(!strncmp(command,"temperature",11)) return 0; //temperature command
//	if(!strncmp(command,"current",7)) return 0; //current command

	if(!strncmp(command,"name",4)) isname = 1;

	bzero(cmdbuf, sizeof(cmdbuf));
	bzero(outletbuf, sizeof(outletbuf));
	bzero(localcommand, sizeof(localcommand));

	strncpy(localcommand, command, sizeof(localcommand));
	cfs = strstr(localcommand, " ");		
	
        if( cfs == NULL ) return 0;  //[GY]2005/Nov/11 
	parser = cfs + 1;

	if(!isname){

		//starts looking for an outlet name 
		//checks for anything different from number, comma or space until the end
		while ( ((isdigit(*parser)) || 
			(*parser == ' ')  || 
			(*parser == '-')  || //[RK]Oct/24/05
                        (*parser == ':')  || //[GY]2005/Nov/11 
			(*parser == ','))  && 
			(*parser)            ) parser++;

	}
	else{
		//checks, in the name command, if the outlet is being refered by
		//its previous name or by its number
		while(*parser && (*parser != ' ')){
			if(!isdigit(*parser)) name3 = 1;
			parser++;
		}
		//if it is a name command, but the outlet is being refered by its number
		//we have nothing to change
		if(!name3) return 0;
		
		//returns it to its original position, just after the space
		parser = cfs + 1; 
	}

	if(! *parser) return 0; //job finished, no names left
	//we found a name, so lets start work
	
	//copy into our local buffer, the left part of the command, before the outlet name
	strncpy(cmdbuf, localcommand, parser - localcommand);

	//copy the outlet name only here
        //[GY]2005/Nov/11  (start)
/*
	while( (*parser) && ((*parser != ' ') && (*parser != ',')) ){
		outletbuf[ind] = *parser;
		parser++;
		ind++;
	}
*/
        while( *parser && loop ) {
           if( (*parser == '"') )
              double_quote++;
           if( (*parser != ',') && (*parser != ':') ) {
              if( (double_quote == 0) && (*parser == ' ') )
                 loop = 0;
              else {
                 outletbuf[ind] = *parser;
                 if( (*parser == '"') && (double_quote == 2) )
                    double_quote = 0;
                 parser++;
                 ind++;
              }
           }
           else
              loop = 0;
        }
        //[GY]2005/Nov/11  (end)

	if(strstr(outletbuf,"all")) return 0;

	//transform the outlet name into its number
	OutletName2Number(outletbuf);

	//concatenates the number into the command and, finally, the rest of the command,
	//just after the command name
	strncat(cmdbuf, outletbuf, sizeof(cmdbuf) - 1);
	strncat(cmdbuf, parser, sizeof(cmdbuf) - 1);

	//check for further outlet names, doing the same job over again

	if(!isname) //if it is a name command, the last argument is the new name, so exit
		OutletNames2Numbers(cmdbuf); 

	//finally, rewrite the command, with all the names substituted
	strcpy(command, cmdbuf);

	//lets make sure that there is at least one outlet left after the processing
	cfs = strstr(command," ");
	if(! cfs[1]) return -1;
	return 0;
}

/***************************************************************************************************
 OutletName2Number - for an aoutlet name given, chages it to its number

 outlet - string containing the outlet name, will be rewritten by its number

 Returns the new size for outlet, which will be 0 if the name was not recognized
 **************************************************************************************************/
static int OutletName2Number(char *outlet)
{
	char outletlist[64];
	int i, count = 0;

	//set a status command over this name as root
	mountStatusCommand(pmc->pmc_cmd, outlet);
	strcpy(pmc->pmc_username, "root");
	pmc->pmc_cmd[CMD-1]='\0';
	pmc->pmc_status = PMC_REQUEST|PMD_NOAUTH;

	if(waitAnswer()){
		count = string2outlets(pmc->pmc_answerBuf, outletlist);
	}

	pmc->pmc_answerBuf[0] = 0;
	pmc->pmc_username[0] = 0;
	pmc->pmc_status = PMC_IDLE;
	memset(pmc->pmc_cmd,0,PMC_CMDSIZE);
	memset(pmc->pmc_outlet,0,PMC_OUTLETS);

	outlet[0] = 0;
	for (i = 0; i < count; i ++) {
		if (i) {
			strcat(outlet, ",");
		}
		sprintf(outlet + strlen(outlet), "%d", outletlist[i]);
	}
	return strlen(outlet);
}

#if 0
static int check_outlet_name(char *command) 
{
	int ok;
	char *nout;

	if(!strcmp(command,"all")) return 1;
	mountStatusCommand(pmc->pmc_cmd, command);
	pmc->pmc_cmd[CMD-1]='\0';
	pmc->pmc_status = PMC_REQUEST|PMD_NOAUTH;
	if(waitAnswer()) {
		if((nout=strstr(pmc->pmc_answerBuf,command))==NULL) {
			ok=0;
		} else {
			ok = getOutletNumber(nout);
			pmc->pmc_outlet[0] = ok;
			pmc->pmc_outlet[1] = 0;
			ok=1;
		}
	}
	else {
		printf("Error checking outlet string.\n");
		ok=0;
	}
	pmc->pmc_answerBuf[0]=0;
	pmc->pmc_status = PMC_IDLE;
	memset(pmc->pmc_cmd,0,PMC_CMDSIZE);
	memset(pmc->pmc_outlet,0,PMC_OUTLETS);
	return ok;
}

static int expandOutlets(char *command) 
{

	char *pb, *pe;
	long first,last;
	int count=0,index,ok;

	memset(pmc->pmc_outlet,0,PMC_OUTLETS);
	 if(strcmp(command,"all")!=0){
		pb = command;
		pe = command;
		last = strtol(pb,&pe,10);
		if(last == 0) {
			ok=check_outlet_name(command);
			if (!ok) {
				printf("\r\nPlease type a valid outlet name.\r\n");
				return (-1);
			} else {
				 return 1;
			}
		}
		while(*pb) {
			last = first = strtol(pb,&pe,10);
			if (first < 1 || first > 128) {
				printf("\r\nPlease type a valid first outlet number.\r\n");
				return (-1);
			}
			if(*pe == '-') {
				last=strtol(pe+1,&pb,10);
				if (last < 1 || last > 128) {
					printf("\r\nPlease type a valid last outlet number.\r\n");
					return (-1);
				}
				pe = pb;
			}
			if (first < 1 || last > 128 || first > last) {
				printf("\r\nPlease type a valid outlet range.\r\n");
				return (-1);
			}
			for (index=first;index<=last && count < PMC_OUTLETS;index++) {
				pmc->pmc_outlet[count] = index;
				count++;
			}
			if (*pe == ',') pe++;
			pb = pe;
		}
	 }
	return count;
}
#else
static int expandOutlets(char *command)
{
	int count = -1;
	memset(pmc->pmc_outlet,0,PMC_OUTLETS);
	if(! strcmp(command,"all")){
		return 0;
	}
	mountStatusCommand(pmc->pmc_cmd, command);
	pmc->pmc_status = PMC_REQUEST|PMD_NOAUTH;
	if(waitAnswer()) {
		count = string2outlets(pmc->pmc_answerBuf, pmc->pmc_outlet);
		if (count == 0) {
			printf("\r\nInvalid outlet string.\r\n");
			return(-1);
		}
	} else {
		printf("Error checking outlet string.\n");
	}
	pmc->pmc_answerBuf[0]=0;
	pmc->pmc_status = PMC_IDLE;
	memset(pmc->pmc_cmd,0,PMC_CMDSIZE);
	return count;
}
#endif

static int check_name_outlet(char *outlet) 
{
	int index;
	char aux[CMD]={0};
	char *token,*aux2;

	aux2=malloc(100);
	for(index=0;index < MAX_DEV; index++) {
		if (lineconf[index].pmoutlet) {
			strcpy(aux,lineconf[index].pmoutlet);
			token=strtok(aux," ");
#ifdef ONS
			if (lineconf[index].auxflg) {
				sprintf(aux2,"a%d.%s",(serial-AUX_OFFSET),outlet);
			} else {
				 sprintf(aux2,"s%d.%s",serial,outlet);
			}
#else
			sprintf(aux2,"%d.%s",serial,outlet);
#endif
			while (token) {
				if (strcmp(token,aux2)==0) {
					free(aux2);
					return 1;
				}
				token=strtok(NULL," ,");
			}

		}
	}
	free(aux2);
	return 0;
}

static void cleanOutlets (void)
{
	memset(pmc->pmc_outlet,0,PMC_OUTLETS);
}

static int isAdmin(void)
{
	return admin;
}

static pmcInfo *getPmc (void)
{
	return pmc;
}

static void print_menu(int multimode)
{
	unsigned char menuItems[NUM_BASIC_COMMANDS + NUMBER_OF_COMMANDS];
	char *comm;
	int i, j, k, nelem, nrows;
	/* First count the number of elements */
	nelem = 0;
	for (i = 0; i < NUM_BASIC_COMMANDS; i ++) {
		if (pmBasicCommands[i].restricted && !admin) continue;
		//[RK]Jun/23/06 - bug 7404
		itemList[nelem] = i ;
		menuItems[nelem++] = i;
	}
	for (i = 0; i < NUMBER_OF_COMMANDS; i ++) {
		if (pmCommands[i].restricted && !admin) continue;
		//[RK]Jun/23/06 - bug 7404
		itemList[nelem] = i + NUM_BASIC_COMMANDS;
		menuItems[nelem++] = i + NUM_BASIC_COMMANDS;
	}
	nrows = (nelem + 2) / 3;
	printf("\r\n------------------------------------------------------------------- \r\n");
	for (i = 0; i < nrows; i ++) {
		for (j = i; j < nelem; j += nrows) {
			k = menuItems[j];
			if (multimode && k == 0) {
				comm = "Return";
			} else if (k < NUM_BASIC_COMMANDS) {
				comm = pmBasicCommands[k].menuName;
			} else {
				comm = pmCommands[k-NUM_BASIC_COMMANDS].menuName;
			}
			k = printf ("%d. %s", j + 1, comm);
			for (; k < 26; k ++) putchar(' ');
		}
		printf("\r\n");
	}
}

static void show_short_help (struct _commands *pcomm)
{
	int i;
	char *s;
	for (s = pcomm->menuName; *s; s ++) putchar(*s);
	putchar(' ');
	for (i = 0; 
		i < (27 - strlen(pcomm->menuName)); 
		i ++) {
		putchar('-');
	}
	putchar(' ');
	for (s = pcomm->shortHelp; *s; s ++) putchar(*s);
	putchar('\r');
	putchar('\n');
}

static int pm_treat_exit(struct _commands *pcomm) 
{
	Exit();
	return 1;
}

static int pm_show_help(struct _commands *pcomm)
{
	int i;
	putchar('\r');
	putchar('\n');
	for (i = 0; i < NUM_BASIC_COMMANDS; i ++) {
		if (pmBasicCommands[i].restricted && ! admin) continue;
		show_short_help (&pmBasicCommands[i]);
	}
	for (i = 0; i < NUMBER_OF_COMMANDS; i ++) {
		if (pmCommands[i].restricted && ! admin) continue;
		show_short_help (&pmCommands[i]);
	}
	return 1;
}

static int pm_treat_whoami(struct _commands *pcomm) 
{
	printf("\n\r");
	system("whoami");
	return 1;
}

static int treats_pm(int option) {
	int i = itemList[option - 1];
	struct _commands *pcomm;

	if (i < NUM_BASIC_COMMANDS) {
		pcomm = &pmBasicCommands[i];
	} else if (i < NUM_BASIC_COMMANDS + NUMBER_OF_COMMANDS) {
		pcomm = &pmCommands[i - NUM_BASIC_COMMANDS];
	} else {
		puts("Invalid Option");
		return 0;
	}
	
	if (pcomm->restricted && !admin) {
		puts("Admin access only");
		if (! strcmp(pcomm->commandName, "getname")) {
			return 1;
		} else {
			return 0;
		}
	}
	if (pcomm->pmTreat) {
		while (! (pcomm->pmTreat)(pcomm));
	} else {
		sendCommand(pcomm->commandName);
	}
	return 0;
}

int main(int argc, char **argv){
	unsigned char multimode = 0;	

#if defined(KVM)
	if (argc < 2 ) {
		usage();
		exit(0);
	}
#endif

	if( argc > 1 ){ 
	
		if (!strcmp(argv[1],"--help")) {
			usage();
			exit(0);
		}
		multimode = 0;
		argv++; //skip program name if received command line option 
	}
#ifdef MULTI_OUT
	else multimode = 1;
#else
	else {
		usage();
		exit(0);
	}
#endif	
	// Starting the shared memory
	if (shm_init(-1) < 0) {
		printf ("\r\n[Error] - Could not start shared memory.\r\n");
		exit(0);
	}
	if (!check_pmd()) exit(0);

	my_pid = getpid();

	return start_processing(argv,multimode);
}

/***************************************************************************************************
 start_processing - this function effectivaly starts the true working, presenting the menus and
 answering to the user options. 

 cmdoptions - only used if the program received command line arguments. It will be an array of
 	      pointer for these arguments skipping the program name (literally argv + 1)

 multimode - can be any value between 0 to 2

 	0 - the user called the application with the tradition syntax (pm <serial>), so the menus
	    presented will be the classic ones
        1 - the user called the application with no arguments, what will allow him to choose 
            between two paths, the traditional mehu or a new menu with support for multi-outlet
	    devices.
	2 - this option is used by the function when it calls itself again to present the
	    traditional pm menu. Since this menu, in this case, will have a change (option 1 will
	    be Return and *not* exit), we need a way to differentiate it from when the menu was
	    reached through calling the application passing the serial port as a command line
	    argument. So this extra option is used for that.

The function returns the value to be later returned by the main() function.
**************************************************************************************************/
int start_processing(char **cmdoptions, unsigned char multimode) {
	int opt,quits=0,userPerm;
	char option[OPT];

	if(multimode == 0 || multimode == 2){
#ifdef ONS
    		if (memcmp(*cmdoptions, "a", 1) == 0) {
        		auxflag = 1;
        		serial = atoi(*cmdoptions+1) + AUX_OFFSET;
		} else if (!memcmp(*cmdoptions, "s", 1)) {
			auxflag = 0;
			serial = atoi(*cmdoptions+1);
		} else {
			printf("error in syntax\n");
			exit(0);
		}
#else
		serial=atoi(*cmdoptions);
#endif
		if (!check_ipdu(serial)) exit(0);
	}//if(multimode == 0 || multimode == 2)
#ifdef MULTI_OUT
	else {
		serial = 0;
	}
	userPerm=check_user();  //[GY]2005/Nov/23
        if( multimode != 1 ) {  //[GY]2005/Nov/17
	   if (!userPerm) {
	      puts("User have no access\r\n");
	      exit(0);
	   }
        }
#else
	userPerm=check_user();  //[GY]2005/Nov/23
	if (!userPerm) {
	    puts("User have no access\r\n");
	    exit(0);
	}
#endif
	setSignals();
	atexit(Exit);
	initPmc(1);
	setbuf(stdout,0);
	printf("%s",pm_header);

	if(multimode == 0 || multimode == 2){
		if(lineconf[serial-1].alias) {
			printf("PowerPort: %s\r\n",lineconf[serial-1].alias);
		}
		else {
#ifdef ONS
			printf("PowerPort: %s%d\r\n",
			auxflag ? "AuxPort" : "SerialPort",
			auxflag ? (serial-AUX_OFFSET) : serial);
#else
			printf("PowerPort: SerialPort%d\r\n",serial);
#endif
		}
		printf("Number of units: %d\r\n",cy_shm->PMD_pms[serial-1]);
#ifdef ONS
		printf("%s: %d\r\n",
			auxflag ? "Aux Port" : "Serial Port",
			auxflag ? (serial-AUX_OFFSET) : serial);
#else
		printf("Serial Port: %d\r\n",serial);
#endif
		printf("%s",pm_line);
		quits = 0;
		while(!quits) {
			puts(pm_prompt_header);
			if(lineconf[serial-1].alias) {
				printf("PowerPort: %s",lineconf[serial-1].alias);
			}
			else {
#ifdef ONS
				printf("PowerPort: %s%d",
				auxflag ? "AuxPort" : "SerialPort",
				auxflag ? (serial-AUX_OFFSET) : serial);
#else
			 	printf("PowerPort: SerialPort%d",serial);
#endif
			}
			print_menu(multimode);
//			if (admin){  
			        //if multimode then option 1 will be Return instead of Exit
//				puts(multimode ? pm_menu_admin2 : pm_menu_admin);
//			}	
//			else{ 
//				puts(multimode ? pm_menu_user2 : pm_menu_user);
//			}		
//
			printf("%s ",pm_prompt);


			if (read_string(option, OPT) == NULL) {
				continue; 
			}
			opt=atoi(option);
	
			//[RK]Nov/29/05 
			if (opt == 0) {
				puts("Invalid option\r\n");
				continue;
			}

			if (opt==1) { 
			         //takes the user back to the very first menu
				 if(multimode==1) start_processing(cmdoptions,1);
				 else quits=1; 
				 //if multimode = 0 then the app is being used in the traditional way
				 //so it must quit
				 //if multimode = 2 then this function was already called by itself
				 //so we must quit now to not enter in a sort of "infinite recursion"
				 //going back to the caller instance, the very first menu will
				 //be shown again
			}
			else {
				quits=treats_pm(opt);
			}
		}//while(!quits)
	}//if(multimode == 0 || multimode == 2)
#ifdef MULTI_OUT
	else{
		quits = 0;
		while(!quits){
			puts(pm_prompt_header);
			puts(pm_menu_new_start); //multidevice menu	
			printf("%s ",pm_prompt);
			if(read_string(option, OPT) == NULL){
				Exit();
			}
			opt = atoi(option);

			switch(opt){
				case 1: quits = 1;//quit this menu level, what will quit the app 
					break;
					
				case 2: printf("Give serial port number: ");
					read_string(cmdoptions[0],OUTL);
					start_processing(cmdoptions,2); 
					continue; 

				case 3: multioutlet_menu();
				        continue;
				
				case 4: //shows help
					puts(pm_menu_new_start_desc);
				        continue;
					
				default: 
					puts("Invalid option\r\n");
				        continue;
			}//switch

		}//while(1)
	}//else
#endif
	return 1;
}

#ifdef MULTI_OUT
/**************************************************************************************************
 multioutlet_menu - this function shows and processes the options for the multi outlet menu, which
 is the one that appears when the application is called with no arguments and then it is chosen
 option 3 in the first menu. 
 *************************************************************************************************/
int multioutlet_menu(void)
{
	int opt, quits=0;
	char option[OPT];
	int mypmci;
	//check if there is at least one device propertly configured
	//if there isnt any, there is no sense in showing this menu
	if(!multioutlet_getlist(mult_buf)){
		printf("There are no devices configured  with pmoutlet parameter in pslave.conf\r\n");
		printf("This is the main parameter to any device to be used as a multi-outlet device\r\n");
		return -1;
	}
	
	while (!quits){	
		//frees memory allocated during last execution for multoutlet command
#if 0
		for (mypmci=0; mypmci<MAXPMDACCESS; mypmci++) 
			if (cy_shm->pmc_list[mypmci] == 0)
				break;

		mypmci--;
		for( ;multimode_numcomands>0; multimode_numcomands--){		
		//	printf("liberei %d\n",multimode_numcomands);
			pmc = cy_shm->pmc_list[mypmci];
			cy_shm->pmc_list[mypmci] = NULL;
			if (pmc->pmc_answerBuf) {
				shm_free_str(pmc->pmc_answerBuf);
			}
			shm_free_str((char *)pmc);
			mypmci--;
			pmcindex--;
		}
#endif
		for (mypmci=0; mypmci<MAXPMDACCESS; mypmci++) {
			if (cy_shm->pmc_list[mypmci] == 0) continue;
			if (mypmci == pmcindex) continue;
			pmc = cy_shm->pmc_list[mypmci];
			if (pmc->pmc_pid != my_pid) continue;
			cy_shm->pmc_list[mypmci] = NULL;
			if (pmc->pmc_answerBuf) {
				shm_free_str(pmc->pmc_answerBuf);
			}
			shm_free_str((char *)pmc);
		}

		pmc = cy_shm->pmc_list[pmcindex];

		//standard for all menus in this application
		puts(pm_prompt_header);
		//menu for multi-outlet device
		puts(pm_menu_multi_1);
		//standard too
		printf("%s ",pm_prompt);
		read_string(option,OPT);
		opt = atoi(option);
		switch(opt){
			case 1: quits = 1;
				break;
				
			case 2: multioutlet_do("on","Turning on...\r\n");
				break;

			case 3: multioutlet_do("off","Turning off...\r\n");
				break;

#ifdef OEM3  //[GY]2005/Nov/11 
                        case 4: multioutlet_do("reboot","Rebooting...\r\n");
                                break;

                        case 5: multioutlet_do("status",
                                "These are the status for these outlets in the IPDU attached to ttyS%d\r\n");
                                break;

                        case 6: multioutlet_show();
                                break;

                        case 7: puts(pm_menu_multi_1_desc);
                                break;

#else
			case 4: multioutlet_do("cycle","Cycling...\r\n");
				break;

			case 5:	multioutlet_do("lock","Locking...\r\n");
				break;

			case 6:	multioutlet_do("unlock","Unlocking...\r\n"); 
				break;
				
			case 7:	multioutlet_do("status",
				"These are the status for these outlets in the IPDU attached to ttyS%d\r\n");
				break;

			case 8: multioutlet_show(); 
				break;

			case 9: puts(pm_menu_multi_1_desc);
				break;
#endif
			default: 
				puts( "Invalid option\r\n" );
				break;
		}
	}
	return 0;
}


/***************************************************************************************************
 multioutlet_show - shows an formated output with the list of devices configured as ipdus, as 
 well as its aliases and outlets. The formatting itself is made inside multioutlet_getlist()
 **************************************************************************************************/
void multioutlet_show(void){

	//this checking is redundant since the user should never get here if there are
	//no devices configured as IPDU... but who knows?
	if(!multioutlet_getlist(mult_buf))
		printf("No devices configured as IPDU were found\r\n");
	else
		puts(mult_buf);

}

/***************************************************************************************************
 multioutlet_getlist - retrieves in a buffer given the list of devices configured with pmoutlets,
 as well as its aliases and outlets.

 devlist - pointer to the buffer where the info will be written. If this parameter is set as NULL,
 then the function limits itself to return the number of devices configured as IPDU, what is 
 used to check if it makes sense allowing the user access the multi-outlet menu.

 return the number of devices found
 **************************************************************************************************/
int multioutlet_getlist(char *devlist){
	int i,n;
	int nfound = 0; //number of devices configured as ipdu
	int room = 5000;
	memset(devlist,0,room);
	for( i = 0; i < MAXVLINES; i++){
		if(lineconf[i].this != i ||
			lineconf[i].tty == NULL ||
			*lineconf[i].tty == 0) continue; //skip invalid

		//if there is one configuration pmoutlet
		if(lineconf[i].pmoutlet){
			nfound++; 
			n = snprintf(devlist, room, "alias: %s   port: %s   outlets: %s\r\n", 
				lineconf[i].alias, lineconf[i].tty, lineconf[i].pmoutlet);
			room -= n;
			if(room <= 1) break;
			devlist += n; //start next writting from the end of this
		}
	
	}
	return nfound;
}

/***************************************************************************************************
 multioutlet_do() - make the very first processing for any menu . Gets from the
 user the info about the desired multidevice, checks if it is valid, and even executes a show 
 function (the same that would be executed if the user had selected option 8), in case user
 is in doubt about what to type. Once the multidevice is chosen and confirmed as valid, 
 multioutlet_issuecmd() is called.

 See the header for multioutlet_issuecmd() function to details about cmd and header parameters.
 **************************************************************************************************/
void multioutlet_do(char *cmd, char *header){
	char binput[80]; //the user input for the desired device
	char portno[10] = "ttyS";
	int nmax;
	char *pdevice, *poutlist; 
	
	if(!multioutlet_getlist(mult_buf)){
		printf("No propertly configured devices found\r\n");
		return;
	}

	nmax = 3; //the user has up to 3 tries to use show here
	do{
		printf("Please supply the serial port number or the alias for the multi-outlet devices.\r\n"); //[GY]2005/Nov/11 
		printf("Serial port number or alias(? for a list of available devices): ");  //[GY]2005/Nov/11 
//		printf("If in doubt, type ? followed by enter and a list of available devices will be shown: ");
		binput[0]='\0';
		fflush(stdin);
		read_string(binput,79);
		if(binput[0] == '?'){ //if user didnt type 
			nmax--; //check for a maximum of tryes
			if(nmax){ 
				multioutlet_show(); //if below the limit, show the available devices
				continue;
			}
			else{ //if not, quits this menu
				puts("Returning to previous menu...\r\n");
				return;
			}
		}
	} while( binput[0] == '?' || (binput[0] == 0x00));  //[GY]2005/Nov/21

	snprintf(portno+4, 3, "%s", binput);
	if (!(pdevice = strstr(mult_buf,portno))) {
		if (!(pdevice = strstr(mult_buf,binput)) || strncmp(pdevice-7,"alias",5)) {
			printf("No such device\r\n");
			return;
		}
	}

	//found the port number or alias name asked for the user, now locate the pmoutlet list
	poutlist = strstr(pdevice, "outlets:");
	poutlist += 9; //skip the word outlets: plus the space

	multioutlet_issuecmd(poutlist,cmd,header);
		
}


/****************************************************************************************************
 multioutlet_issuecmd - this function sends a command for the outlets belonging to a multidevice
 The commands will be issued by serial port. It means that, for example, if the parameter
 pmoutlet in pslave.conf is 

 1.1 1.3 1.2 5.2 5.4 5.5

 there will be issued two comands:

 cmd 1 3 2 on serial port 1
 cmd 2 4 5 on serial port 5

 Another example, if the parameter is

 1.3 1.2 5.4 5.5 1.1

 Then three commands will be issued:

 cmd 3 2 on serial port 1
 cmd 4 5 on serial port 5
 cmd 1 on serial port 1 again

 list - list of outlets belonging to a multidevice, in the same format as would be found 
 	for the parameter pmoutlet in pslave.conf

 cmd -  this string contains any valid command for pm (on, off, lock, unlock, cycle, status...)
 header - this string will be shown just before performing the command. If the string includes
 a %d in the middle, this will be changed by the serial port number associated to the multidevice 
 the command is being issued for.
 ***************************************************************************************************/

void multioutlet_send_cmd(char *cmd, char *outlets, char *oserial, char *header)
{
	char command[CMD];

	//get the serial port attached to the ipdu
#ifdef ONS
	serial = atoi(oserial+1); //converts to integer
	if (*oserial == 'a') serial += AUX_OFFSET;
#else
	serial = atoi(oserial); //converts to integer
#endif
	initPmc(0);
	printf(header, serial);
	multimode_numcomands++; //used to check later how much buffers free
	sprintf(command, "%s %s", cmd, outlets);
	sendCommand(command);
}

void multioutlet_issuecmd(char *list, char *cmd, char *header)
{
	char *p, *sserial, *oserial;
	char outlets[CMD];
	char *pOut = outlets;
	int outn;

	sserial=oserial=strtok(list," ");

	memset(outlets,0,CMD);
	
	*pOut = 0;
	while(sserial){ 
		p = strchr(sserial,'.'); //locate the dot
		if (!p) break; // ????
		*p='\0'; //once the dot is replaced with the "zero" character
		if (strcmp(sserial, oserial)) {
			// outlet belongs to other ipdu, so send command
			multioutlet_send_cmd(cmd, outlets, oserial, header);
			oserial = sserial;
			pOut = outlets;
			*pOut = 0;
		}
		//[RK]Mar/01/06 = the outlet is a number
		outn = atoi(p+1); 
		pOut += sprintf(pOut,"%d,", outn);
		sserial=strtok(NULL," "); //go to next outlet
	}
	if (outlets[0])
		multioutlet_send_cmd(cmd, outlets, oserial, header);
}

#endif
