/*****************************************************************
* File: pm.h
*
* Copyright (C) 2003 Cyclades Corporation
*
* www.cyclades.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version
* 2 of the License, or (at your option) any later version.
*
* Description:
* This program treats the commands pass from user to pm menu and 
* pmCommand. It takes the command, check the arguments and pass to
* pmd daemon, by coping command to shared memory and sending a signal
* to pmd indicating that there is a command to be executed.
*
* This file holds the basic structures and messages strings
*
****************************************************************/

# include <stdio.h>
# include <pwd.h>
# include <sys/types.h>
# include <sys/stat.h>
# include <fcntl.h>
# include <errno.h>
# include <unistd.h>
# include <termios.h>
# include <linux/ioctl.h>
# include <stdlib.h>
# include <signal.h>
# include <string.h>
# include <ctype.h>

# include <linux/shm.h>
# include <linux/sem.h>
# include <server.h>

/* Extern variables */

extern int thisport;

/* Initialize banners, help messages and prompts */

/* pm menu session */

char pm_syntax[] =
	"\r\nUse: pm <serial port number>\r\n";

char pm_header[] = 
	"\r\n"
	"-----------------------------------------------------------------\r\n"
	"Cyclades Corporation \r\n"
	"Power Management Menu v1.0\r\n"
	"-----------------------------------------------------------------\r\n"; 
char pm_line[] =
	"-----------------------------------------------------------------\r\n";

char pm_prompt[] =
	"\r\n"
	"Please choose an option: ";

char pm_prompt_header[] =
	"\r\n"
	"------------------------------------------------------------------- \r\n"
	"Cyclades Power Management Menu \r\n";
	
char pm_menu_user[] = 
	"\r\n"
	"------------------------------------------------------------------- \r\n"
	"1. Exit\t\t5. Lock\t\t9. Name\t\t13.Who Am I \r\n"
	"2. On\t\t6. Unlock\t10. Current\t14. Help \r\n"
	"3. Off\t\t7. Status\t11. Temperature	 \r\n"
	"4. Cycle\t8. Interval\t12. Version  \r\n";

char pm_menu_admin[] =
	"\r\n"
	"--------------------------------------------------------------------- \r\n"
	"1. Exit\t\t7. Status\t13. Who Am I\t\t19. Restore \r\n"
	"2. On\t\t8. Interval\t14. Help\t\t20. Save \r\n"
	"3. Off\t\t9. Name\t\t15. Buzzer\t\t21. Syslog \r\n"
	"4. Cycle\t10. Current\t16. Current Protection\t22. Alarm \r\n"
	"5. Lock\t\t11. Temperature\t17. Factory Default \r\n"
	"6. Unlock\t12. Version\t18. Reboot ";

char pm_help_menu[] =
	"\r\n"
	"on|off --------------------- Turn on|off outlets \r\n"
	"lock|unlock ---------------- Lock|unlock outlets in current state \r\n"
	"cycle ---------------------- Power cycle outlets \r\n"
	"interval|buzzer ------------ Sets/reads the power up interval|buzzer \r\n"
	"syslog|alarm --------------- Sets/reads syslog notifications|alarm status \r\n"
	"temperature|current -------- Sets/reads/reset the temperature|current \r\n"
	"currentprotection ---------- Sets/reads the over current protection \r\n"
	"name ----------------------- Names an outlet \r\n"
	"status --------------------- Displays outlets state \r\n"
	"reboot  -------------------- Reboot the units in chain \r\n"
	"help ----------------------- Shows this help\r\n"
	"ver ------------------------ Shows the software and hardware version\r\n"
	"whoami --------------------- Display the current username\r\n"
	"exit ----------------------- Exit\r\n"
	"menu ----------------------- Starts the menu driven text interface(pm)\r\n"
	"factorydefaults ------------ Brings the unit to factory configuration\r\n"
	"restore -------------------- Restore the configuration in flash\r\n"
	"save ----------------------- Saves the current configuration in flash\r\n"
	"Type 'help <command>' to see details of <command>\r\n";

char pm_outlet_desc_n[] =
	"\r\n"
	"Please enter a outlet number, an outlet list or a outlet name. \r\n"
	"Outlet number is the outlet's number in IPDU. \r\n"
	"Outlet name is the name associated to an outlet in IPDU or the alias \r\n"
	"for server connected to this outlet\r\n"
	"An outlet list is a list of outlet numbers separated by comas \r\n"
	"(indicating outlets) or dash (indicating a range). \r\n"
	"Ex: 2,3,5 - indicates outlets 2, 3 and 5 \r\n"
       	"	2,3-5 - indicates outlets 2 and 3 throught 5 (3, 4 and 5) \r\n";

char pm_outlet_desc[] =
	"\r\n"
	"Please enter a outlet number or an outlet. \r\n"
	"Outlet number is the outlet's number in IPDU. \r\n"
	"An outlet list is a list of outlet numbers separated by comas \r\n"
	"(indicating outlets) or dash (indicating a range). \r\n"
	"Ex: 2,3,5 - indicates outlets 2, 3 and 5 \r\n"
       	"	2,3-5 - indicates outlets 2 and 3 throught 5 (3, 4 and 5) \r\n";

char pm_status_desc[] =
	"\r\n"
	"Please enter a outlet number or an outlet list. \r\n"
	"Outlet number is the outlet's number in IPDU. \r\n"
	"An outlet list is a list of outlet numbers separated by comas \r\n"
	"(indicating outlets) or dash (indicating a range). \r\n"
	"Ex: 2,3,5 - indicates outlets 2, 3 and 5 \r\n"
	"	2,3-5 - indicates outlets 2 and 3 throught 5 (3, 4 and 5) \r\n";

char pm_current_desc[] =
	"\r\n"
	"Please enter blank if you want check the value for current draw for all\r\n"
	" units or IPDU number in chain for a specific unit. \r\n"
	"Reset option is allowed for admin user only. \r\n";

char pm_temperature_desc[] =
	"Please enter blank if you want check the value for temperature for all \r\n"
	"units or IPDU number in chain for a specific unit.\r\n"
	"Reset option is allowed for admin user only.\r\n";

char pm_reboot_desc[] =
	"\r\n"
	"Please enter blank if you want reboot all units or IPDU number in chain for a specific unit. \r\n";


char pm_alarm_desc_1[] =
	"\r\n"
	"Please enter the IPDU number in chain or blank for check all units.\r\n";

char pm_alarm_desc_2[] =
	"\r\n"
	"Please enter the maximum allowed current for the unit. \r\n";
