/*****************************************************************
* File: pmdteste.c
*
* Copyright (C) 2003 Cyclades Corporation
*
* www.cyclades.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version
* 2 of the License, or (at your option) any later version.
*
* Description:
*
****************************************************************/

# include <stdio.h>
# include <sys/types.h>
# include <sys/stat.h>
# include <unistd.h>
# include <string.h>
# include <stdlib.h>
# include <ctype.h>
#include <signal.h>
# include <server.h>
# include <pmd.h>

#define MAX_DEV         cy_shm->max_dev

pmcInfo *pmc=0;
int got_sig;

#ifdef ONS
int auxflag = 0;
#endif

static void treatSignals( int sig ) {
	struct sigaction act;
	memset((char*)&act,0,sizeof(struct sigaction));
	switch (sig) {
		case SIGUSR1:
		case SIGUSR2:
			got_sig = sig;
			break;
	}
        act.sa_handler = treatSignals;
        sigaction(sig, &act, 0);
        unblock(sig);
}

static void initSig() {
	struct sigaction act;
	memset((char*)&act,0,sizeof(struct sigaction));

	// Setting up the act structure
	act.sa_handler = treatSignals;
	sigemptyset(&act.sa_mask);
	act.sa_flags = 0;

	// Trying to register the signals. If one of them fail it returns error
	sigaction(SIGUSR1, &act, 0);
	sigaction(SIGUSR2, &act, 0);
	unblock(SIGUSR1);
	unblock(SIGUSR2);
}

static void pmdSendReqSt(pmcInfo* pmc, int serialP)
{
	pmc->pmc_ipdu = serialP -1;
	strcpy(pmc->pmc_cmd,"pmdcycl");
	pmc->pmc_status = PMD_STATUS;
	kill(cy_shm->PMD_pid,SIGIO);
}

static pmcInfo *pmdGetPmc()
{
	pmcInfo *pmc;
	char *buf;
	int id;

	for (id = 0; id < MAXPMDACCESS; id++){
		if (cy_shm->pmc_list[id]==0)
			break;
	}
	
	if (id == MAXPMDACCESS) exit;
	
	pmc = (pmcInfo *) shm_malloc(sizeof(pmcInfo),TYPE_SHM_PMC);
	buf = shm_malloc(4000,TYPE_SHM_PMCBUF);
	if (!pmc || !buf) {
		return 0;
	}
	shm_signal(CY_SEM_SHM);
	pmc->idx_pmclist = id;
	pmc->pmc_pid = getpid();
	pmc->pmc_sizeAnswerBuf = 4000;
	pmc->pmc_answerBuf = buf;
	pmc->next = NULL;
	cy_shm->pmc_list[id] = pmc;
	shm_unsignal(CY_SEM_SHM);
	return pmc;
}

static void pmdFreePmc()
{
	if (pmc) {
		shm_signal(CY_SEM_SHM);
		cy_shm->pmc_list[pmc->idx_pmclist] = 0;
		shm_free_str(pmc->pmc_answerBuf);
		shm_free_str((char*) pmc);
		shm_unsignal(CY_SEM_SHM);
		pmc =0;
	}
}

char *ipdu_state[] =
{"IPDU INIT",
 "Wait UserName string from PM",
 "Wait Password string from PM",
 "Wait Prompt string from PM",
 "Ready",
 "Wait command echo from PM",
 "Wait answer to command from PM",
 "Wait the complete syslog message from PM",
 "Wait password from Client",
 "Doing Upgrade",
 "Wait Xmodem answer from PM",
 "Wait init comm",
 "Wait Restart after upgrade firmware",
};
static void pmdDisplayStatus(pmcInfo *pmc,int serial)
{
	struct _pmdStatus *ps;
	int pid,j,k;

	pmdSendReqSt(pmc,serial);

	while(1) {
		sleep(2);
		if (got_sig == SIGUSR2) break;
	}
	got_sig = 0;
	if (pmc->pmc_status != PMC_ANSOK) return;

	ps = (struct _pmdStatus *) pmc->pmc_answerBuf;

	printf ("\n\nStatus of IPDU #%d:\n",serial);
	printf(" state : %s   \n ",ipdu_state[ps->state]);
	if (ps->pmindex) {
		if (cy_shm->pmc_list[ps->pmindex] != 0) {
			pid = cy_shm->pmc_list[ps->pmindex]->pmc_pid;
			printf("command :[%s] from process[%d] at %s\n",ps->comm,pid,ctime(ps->start_time));
		} else {
			printf("last command :[%s]\n",ps->comm);
		}
	} else {
		printf("last command :[%s]\n",ps->comm);
	}
	if (ps->cmd_get != ps->cmd_put) {
		printf("\n commands in queue : \n");
		for (k=ps->cmd_get; k != ps->cmd_put; ) {
			j = ps->commands[k].pmindex;
			if (cy_shm->pmc_list[j] != 0) {
				pid = cy_shm->pmc_list[j]->pmc_pid;
				printf("command[%d]:[%s] out[%s] from process[%d]\n",j,
						ps->commands[k].command,ps->commands[k].outlet,pid);
			} else {
				printf("command[%d]:[%s] out[%s] process died\n",j,
						ps->commands[k].command,ps->commands[k].outlet);
			}	
			k = (k+1) % MAXIPDUQUEUE;
		}
	}
	
}

/*******************************************************************
 * This routine prints the information about a menu item.
 ******************************************************************/
static void printMenuItem(menuItem *pitem, int nitems)
{
	int i, j, k;
	menuItem *item;
	printf(
"Label       Command                 Description\r\n");
	printf(
"------------------------------------------------------------------------------\r\n");
	for (i = 0; i < nitems; i++) {
		item = pitem + i;
		j = printf(item->label);
		for (; j < 12; j++) printf(" ");
		j = printf(item->command);
		for (k = 0; k < 4; k++) {
			if (item->args[k] == 0) {
				break;
			} else if (item->args[k] == ARG_OUTLET) {
				j += printf(" <outlet(s)>");
			} else if (item->args[k] == ARG_OUTLET1) {
				j += printf(" <outlet>");
			} else if (item->args[k] == ARG_USER) {
				j += printf(" <user>");
			} else if (item->args[k] == ARG_ONOFF) {
				j += printf(" <on/off>");
			} else if (item->args[k] == ARG_IPDU) {
				j += printf(" <ipdu>");
			} else if (item->args[k] == ARG_CURRENT) {
				j += printf(" <current>");
			} else if (item->args[k] == ARG_NAME) {
				j += printf(" <name>");
			}
		}
		for (; j < 24; j++) printf(" ");
		printf("%s\r\n", item->help);
	}
}

/*******************************************************************
 * This routine prints the information about a ipdu type, 
 ******************************************************************/
static void printIpduMenu ( ipduInfo2 * ipdu, int serialP ) {
#ifdef ONS
	printf("IPDU Menu configuration[%s port %d] :\n\n",
		auxflag ? "aux" : "serial", auxflag ? (serialP-AUX_OFFSET) : serialP);
#else
	printf("IPDU Menu configuration[serial port %d] :\n\n",serialP);
#endif

	printf("Does your IPDU accept multiple outlet commands: ");

	if (ipdu->multiOutCommand) {
		printf("yes\n\n");
	} else {
		printf("no\n\n");
	}

	printf("List of Regular Menu commands:\n");
	printMenuItem(ipdu->menuItems, ipdu->menuMax);

	printf("\nList of Administration Menu commands:\n");
	printMenuItem(ipdu->adminItems, ipdu->admMax);
}

static void clearPmcEntries()
{
	int i;
	pmcInfo * pm;
	
	for (i = 0; i < MAXPMDACCESS; i ++) {
		if ((pm = cy_shm->pmc_list[i]) == NULL) {
			continue;
		}
		if (kill(pm->pmc_pid, 0) == -1) {
			shm_signal(CY_SEM_SHM);
			cy_shm->pmc_list[i] = 0;
			shm_free_str(pmc->pmc_answerBuf);
			shm_free_str((char*) pmc);
			shm_unsignal(CY_SEM_SHM);
		}
	}
}

static void printPmcAnswerBuf(int pmcIndex)
{
	int i,j,k;
	char ch, *s1;
	pmcInfo * pm;
	if ((pm = cy_shm->pmc_list[pmcIndex]) == NULL) {
		return;
	}
	printf("\r\nAnswer Buf : \n");
	s1 = pm->pmc_answerBuf;
	i = 0;
	while (1) {
		for (j = 0; j < 16; j ++) {
			ch = s1[i*16+j];
			if (ch == 0) {
				for (k = j; k < 16; k ++) {
					printf("   ");
				}
				break;
			}
			printf("%02x ", ch);
		}
		printf("   ");
		for (j = 0; j < 16; j ++) {
			ch = s1[i*16+j];
			if (ch == 0) {
				break;
			} else if (ch >= 0x20 && ch <= 0x7e) {
				printf("%c", ch);
			} else {
				printf(".");
			}
		}
		printf("\n");
		i ++;
		if (ch == 0) {
			break;
		}
	}
}

static void printPmcEntries()
{
	int i,j;
	pmcInfo * pm;
	
	printf ("\n\nList of PMC entries:\n"
" Index   PID     PMC Status  AnswSize Command/Outlet \n");
	for (i = 0; i < MAXPMDACCESS; i ++) {
		if ((pm = cy_shm->pmc_list[i]) == NULL) {
			continue;
		}
		printf ("\n %.5d %.5d %.5d ", pm->idx_pmclist, pm->pmc_pid, 
			pm->pmc_ipdu);
		switch (pm->pmc_status) {
			case PMC_IDLE:
				printf (" IDLE  ");
				break;
			case PMC_REQUEST:
			case (PMC_REQUEST | PMD_NOAUTH):
				printf ("REQUEST");
				break;
			case PMC_WAITANS:
				printf ("WAITANS");
				break;
			case PMC_ANSOK:
				printf (" ANSOK ");
				break;
			case PMC_UPGRADE:
				printf ("UPGRADE");
				break;
			case PMC_UPGRADE_END:
				printf ("UPG_END");
				break;
			case PMC_PASSWD:
				printf ("PASSWD ");
				break;
			case PMC_PASSWDOK:
				printf ("PASSWDO");
				break;
			case PMD_STATUS:
				printf ("STATUS ");
				break;
		}
		printf(" ");
		printf("%.8d",pm->pmc_sizeAnswerBuf);
		printf(pm->pmc_cmd);
		printf(" outlets[");
		for (j = 0; j < PMC_OUTLETS;j++) {
			if (pm->pmc_outlet[j]!=0) {
				printf("%d,",pm->pmc_outlet[j]);
			}
		}		
		if (pm->pmc_username[0])
			printf("] user[%s] ",pm->pmc_username);
		else
			printf("] user[root] ");
	}
	printf("\n");
	fflush(stdout);
}

static void usage()
{
	printf("\n\nUSAGE : \n\n"
	" -h 	: help\n"
	" -S    : the state of all IPDUs\n"
	" -d 	: change state of PMD debug \n"
	" -s <serial port number> : the IPDU state\n"
	" -m <serial port number> : the IPDU menu\n"
	" -p 	: display the pmc_info vector\n"
	" -a <index> : display the answer buffer\n"
	" -c 	: clear the pmc_info vector\n");
	
}

int main (int argc, char ** argv)
{
	ipduInfo2 *ipdu;
	int debug, serialP, dispPmc,clearPmc, menu,status;
	
	int i,pmcIndex=-1;
	struct line_cfg *line=0;  
	char *optstr;
	menu= serialP = dispPmc = clearPmc = status = 0;
	debug = 0;
	
	for (i=1; i < argc ; i++) {
		optstr = argv[i];
		if (*optstr != '-') continue;
		switch(*(optstr+1)) {
			case 'h' : usage();
				   return -1;
			case 'd' : 
				debug = 1;
				break;
			case 'm' :
				i++;
				if (i >=argc) {
					usage();
					return -1;
				}
#ifdef ONS
				if (!memcmp(argv[i], "a", 1)) {
					auxflag = 1;
					serialP = menu = atoi(argv[i]+1) + AUX_OFFSET;
				} else if (!memcmp(argv[i], "s" , 1)) {
					auxflag = 0;
					serialP = menu = atoi(argv[i]+1);
				} else {
					usage();
					return -1;
				}
#else
				serialP = menu = atoi(argv[i]);
#endif
				break;
			case 's' :
				status = 1;
				i++;
				if (i >=argc) {
					usage();
					return -1;
				}
#ifdef ONS
				if (!memcmp(argv[i], "a", 1)) {
					auxflag = 1;
					serialP = atoi(argv[i]+1) + AUX_OFFSET;
				} else if (!memcmp(argv[i], "s" , 1)) {
					auxflag = 0;
					serialP = atoi(argv[i]+1);
				} else {
					usage();
					return -1;
				}
#else
				serialP = atoi(argv[i]);
#endif
				break;
			case 'p' : 
				dispPmc = 1;
				break;
			case 'c' :
				clearPmc = 1;
				break;
			case 'S' :   status = 1;
				serialP = 0;
				break;
			case 'a' :
				i++;
				if (i >=argc) {
					usage();
					return -1;
				}
				pmcIndex = atoi(argv[i]);
				dispPmc = 1;
				break;
		}
	}
	
	if (shm_init(-1) < 0) {
		printf ("[Error] - Could not start shared memory.\n");
		return -1;
        }
	if (shm_get_set_val (0, &cy_shm->st_cy_ras, 0, CY_SEM_CONFIG) != 1) {
		printf ("[Error] - Configuration not ready.\n");
		return -1;
	}

	if (serialP) {
		line = &cy_shm->line_conf[serialP-1];
		if (line->protocol != P_IPDU) {
			printf("[Error] - Port is not configured as IPDU\n");
			return 0;
		}
		if (menu) {
			if ((ipdu = (ipduInfo2 *)(cy_shm->sess_list[serialP-1])) == NULL) {
				printf("[Error] - PMD is not detected this port as IPDU\n");
				return -1;
			}
			printIpduMenu(ipdu,serialP);
			return 0;
		}
	}	 

	if (dispPmc){
		if (pmcIndex!= -1) {
			printPmcAnswerBuf(pmcIndex);
			return 0;
		}
	       	printPmcEntries();
		return 0;
	}
	
	if (clearPmc) {
		clearPmcEntries();
		return 0;
	}

	if (debug) {
		kill(cy_shm->PMD_pid,SIGUSR1);
		return 0;
	}

	pmc = pmdGetPmc();
	
	if (!pmc) return -1;

	atexit(pmdFreePmc);

	initSig();	

	got_sig = 0;

	// display Status of IPDU
	printf ("\nPMD PID = %d\n\n", cy_shm->PMD_pid);
	if (serialP) {
		printf ("\n\nStatus of active IPDUs:\n");
		pmdDisplayStatus(pmc,serialP);
		return -1;
	}
	
	for ( i = 0; i < MAX_DEV; i++) {
		line = &cy_shm->line_conf[i];
		if (line->protocol != P_IPDU) {
			continue;
		}
		printf ("\n\nStatus of active IPDUs:\n");
		pmdDisplayStatus(pmc,i+1);
	}
	pmdFreePmc();
	return 0;	
}
