/*
 * auth.h	Header file for the authentication stuff.
 *		We only use radius, but anyway this is
 *		a try to hide the authentication protocol
 *		from the rest of the system.
 *
 * Version:	@(#)auth.h  1.00  05-Jul-1996  miquels@cistron.nl
 *
 */

#ifndef __auth_h
#define __auth_h

/*
 *	Types of connection.
 */
#define P_UNKNOWN	0
#define P_LOCAL		'L'
#define P_RLOGIN	'R'
#define P_SLIP		'S'
#define P_CSLIP		'C'
#define P_PPP		'P'
#define P_PPP_ONLY	'O'
#define P_AUTOPPP	'A'
#define P_TELNET	'E'
#define P_TCPCLEAR      'T'
#define P_TCPLOGIN      'U'
#define P_CONSOLE	'!'
#define P_SHELL		'X'
#define P_SSH		'H'
#define P_SSH2		'2'
#define P_SOCKET_SERVER		'W'
#define P_SOCKET_SERVER_RAW	'V'
#define P_SOCKET_CLIENT		'I'
#define P_SOCKET_SSH		'Y'
#define P_IPDU 'i'
#define P_MODBUS         'M'
#define P_BILLING        'B'
#define P_MODEM          'Z'
#define P_2WRAW          'D'

/*
 *	Types of authentication
 */
#define AUTH_NONE                   0x00
#define AUTH_RADIUS                 0x01
#define AUTH_TacacsPlus             0x02
#define AUTH_LDAP                   0x08
#define AUTH_REMOTE                 0x10
#define AUTH_LOCAL                  0x20
#define AUTH_RADIUS_LOCAL           0x31
#define AUTH_TacacsPlus_LOCAL       0x32
#define AUTH_LOCAL_RADIUS           0x41
#define AUTH_LOCAL_TacacsPlus       0x42
#define AUTH_RADIUS_DOWN_LOCAL      0x51
#define AUTH_TacacsPlus_DOWN_LOCAL  0x52
#define AUTH_LDAP_DOWN_LOCAL  	    0x58

#define AUTH_DSVIEW             0x04
#define AUTH_DSVIEW_LOCAL       0x34
#define AUTH_DSVIEW_DOWN_LOCAL  0x54

#define AUTH_SERVER					0x0F

/*
 *	Authentication info. This struct
 *	contains all info we get back from
 *	the authentication protocol.
 */
struct auth {
	/* Input */
#ifdef CYCLADES
	short  next_token;	// next token mode
	short  st_sess;		// session status 0=None 1=Read/Write, 2=Read/Only 
	char login[64];
	char passwd[64];
	CY_SESS *sess;
#else
	char login[256];
	char passwd[256];
#endif
	time_t start;
	/* Output */
	char session[32];
	short msn;			//message num
	short fln;			//filterid num
	short frn;			//framed_router num
	short len_state;	//lenght of state_info field
#ifdef CYCLADES
	char message[4][128];		//Up to 4 messages. Good?
	char filterid[16][32];		//Up to 16 filters. Good?
	char framed_route[16][48];	//Up to 16 routes. Good?
#else
	char message[16][256];		//Up to 17 messages. Good?
	char filterid[16][256];		//Up to 17 filters. Good?
	char framed_route[16][256];	//Up to 17 routes. Good?
#endif
	char state_info[128];
	char conn_info[128];
	int  proto;
	int  nasport;			/* NAS-Port attribute value	    */
	int  loginport;			/* Login-TCP-Port attribute value   */
	short authenticated;
	short  do_acct;
	unsigned int host;
	unsigned int address;		/* Address is in network byte order */
	unsigned int localip;
	unsigned int netmask;
	int mtu;
	int mru;
	int idletime;
	int sessiontime;
	int porttype;
	int sent_bytes;
	int recv_bytes;
#ifdef CYCLADES
	unsigned char sess_type;	/* 0 - normal session, 1-callback session */
	unsigned char callback_number[64]; /* set by Radius Server */
	unsigned char flag_admin_user; // 0 - no admin user, 1-admin user
#endif
};

int login_local(struct auth *pai);
int do_local_or_server_authentication(struct auth *pai, int ppp);
int rad_client(struct auth *ai, int ppp);
int rad_acct(struct auth *ai, int islogin);
unsigned int rad_sessionid(char *s);
int rad_init(int port, struct auth *ai, char *tty, int fproto);
char *dotted(unsigned int ipno);
struct realm_def *ckrealm(struct auth *ai);

#endif	/* __auth_h */
