/*
 *  Module:  billing.h (Billing feature header file)
 *
 *  Author:  Marcelo Peccin <mpeccin@cyclades.com>
 *
 *  Copyright:  (c) 2003 Cyclades Corp.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version
 *  2 of the License, or (at your option) any later version.
 *
 *  Date:  January 16, 2003
 *
 *  Comments:  
 *
 */

#ifndef BILLING_H
#define BILLING_H


/**************************************************/
/**********     Constant Definitions     **********/
/**************************************************/
#undef  TRUE
#define TRUE   1
#undef  FALSE
#define FALSE  (!TRUE)
#undef  NULL
#define NULL   ((void*)0)

#define ERROR     (-1)
#define ERRSCLOSE (-2)
#define OK         0
#define PROBLEM    1

#define PARITY_NONE  1
#define PARITY_ODD   2
#define PARITY_EVEN  3

#define FLOW_NONE  0
#define FLOW_HARD  1
#define FLOW_SOFT  2

#define BBUFFSIZE  512
#define MINRDISK   100
#define RDISKOK   (MINRDISK + 20)
#define FILENLEN    50


/**********************************************/
/**********     Type Definitions     **********/
/**********************************************/
typedef unsigned char uchar;

typedef struct
{
	int    port;
	int    speed;
	int    datasize;
	int    parity;
	int    stopbits;
	int    flowctrl;
	int    media;
	uchar  dev[15];
}
sconf;

typedef struct
{
	sconf  serial;
	int    maxrecords;
	int    timeout;
	uchar *record_end;
	uchar *collector_name;
	uchar *path;

}
billingconf;


/*************************************************/
/**********     Function Prototypes     **********/
/*************************************************/
static void mpfprint(const char *fmt, ...);
static int  config_port(int fd, sconf *pconf);
static void setbfname(uchar* collector);


/***********************************************/
/**********     Macro Definitions     **********/
/***********************************************/




#endif /* BILLING_H */

/* End of billing.h */
