 /*
 * main.c	Main part of the portslave.
 *
 * Version:	@(#)main.c  1.35  08-Nov-1997  MvS.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <termios.h>
#include <signal.h>
#include <ctype.h>
#include <errno.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/mount.h>
#include <sys/wait.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/time.h>
#include <sys/timeb.h>
#include <sys/times.h>  //[GY]2006/May/30
#include <sys/stat.h>  //[GY]2006/May/31

#define XDEBUG

#include "server.h"

//[RK]May/12/06 - event configuration
extern void writeevt(int, char* , ...);

//function prototypes  [GY]2006/May/30
extern void EscRedirectChar(int SockFd, int PortFd, BufferType *B);
extern int write_pending_buffer(int PortFd);
extern void nopenlog(const char *ident, int logstat, int logfac);
extern void nsyslog(int pri, const char *fmt, ...);

//#define DEBUG_CNT

#define GOT_SIG_HUP			1
#define GOT_SIG_INT			2
#define GOT_SIG_USR			4
#define GOT_SIG_FD			8
#define GOT_SIG_IGN			16
#define GOT_SIG_IO			32		//[RK]Sep/04/02 Dell
#define GOT_SIG_CHILD		64
#define GOT_SIG_DLS		128

static struct itimerval dlstimer = {{0,0},{0,0}};
static int dls_timer_on;

LOG_DATA_BUFFER log_buffer[MAXLINES] = {{0}};  //[GY]2006/May/30

static int vet_lines[MAXLINES];
static int st_lines[MAXLINES];
static int dtr_lines[MAXLINES]; 	//flag for DTR_reset
static int db_lines[MAXLINES];		//flag for DB_mode
static int time_start[MAXLINES];
static int max_lines;

#define SEL_TIMEOUT 500
static int elapsed;

extern int snmp_pipe_flag;  //[GY]2006/Jun/01
extern FILE *snmp_pipe_FP;  //[GY]2006/Jun/01
static char snmp_pipe[] = "/var/run/snmp_pipe";  //[GY]2006/May/31
static char DB_nfs[] = "/mnt/DB_nfs";

static char muntf_in[]  = "\r\n\
WARNING: New user connected to this port.  \r\n\
         Current number of users: %d   \r\n"; //same size; extra spaces are important
static char muntf_out[] = "\r\n\
WARNING: User disconnection from this port.\r\n\
         Current number of users: %d   \r\n"; //same size; extra spaces are important
static int muntf_len = sizeof(muntf_in) - 1/*for the Null termination*/;

static struct termios save_old_tio[MAXLINES];
static int tty_fd[MAXLINES];
static int file_fd[MAXLINES];
extern int pipe_fd[MAXLINES];
struct TmpBuf cyb_tmp[MAXLINES];
extern struct TmpBuf *tmp;  // defined in sock.c

/* Session Pointer */
extern CY_SESS *this_sess;  // defined in sock.c
struct line_cfg *pconf;
int DeviceFd;

#ifdef OEM1 
#include "pipe_alarm.h"
#endif 
 
static int got_int_flag = 0, last_sig = 0;

static void got_int(int sig)
{
	struct sigaction sa = {{0}};  //[GY]2006/May/30

	switch(sig) {
		case SIGUSR1:
			got_int_flag |= GOT_SIG_USR;
			break;
		case SIGUSR2:
			got_int_flag |= GOT_SIG_HUP;
			break;
		case SIGIO:	//[RK]Sep/04/02 Dell
			got_int_flag |= GOT_SIG_IO;
			break;
		case SIGINT:
		case SIGTERM:
			got_int_flag |= GOT_SIG_INT;
			break;
		case SIGCHLD:
			got_int_flag |= GOT_SIG_CHILD;
			break;
		case SIGALRM:
			got_int_flag |= GOT_SIG_DLS;
			break;
		default:
			got_int_flag |= GOT_SIG_IGN;
			last_sig = sig;
			break;
	}
	sa.sa_handler = got_int;
	sigaction(sig,  &sa, NULL);
	unblock(sig);
}

/* [RK]Mar/06/03
 * routines moved from getty.c
 */

static void shm_set_st_data_buffering(int port, int st)
{
	if (!(st == DB_ST_RESTART &&
	      cy_shm->st_data_buffering[port] != DB_ST_STOPPED))
		cy_shm->st_data_buffering[port] = st;
}

static int shm_get_st_data_buffering(int port)
{
	return cy_shm->st_data_buffering[port];
}

/* [RK]Mar/06/03
 * routines moved from rot_buf.c
 */
static int GetFreeBufferCushion(BufferType * B, int cushion)
{
	int free;

	if (B->WrPos >= B->RdPos) {
		free = BufferSize - B->WrPos + B->RdPos - 1;
	} else {
		free = B->RdPos - B->WrPos - 1;
	}
	return(free > cushion ? free - cushion : 0);
}


/* [RK]Mar/06/03
 * routines moved from rot_sess1.c
 */

static int max_sess_free_buf(int cnt, int port)
{
	int aux;
	CY_SESS *sess = cy_shm->sess_list[port];

	while (sess) {
		aux = GetFreeBufferCushion(&sess->OutBuffer, 10); //[RK]May/26/03 Performance Improvement
		if (cnt > aux) {
			cnt = aux;
		}
		sess = sess->next;
	}

	return(cnt);

}

static void set_flow_off_all_session(int port)
{
	CY_SESS *sess = cy_shm->sess_list[port];

	while (sess) {
		if (sess->flow_flag == 0) sess->flow_flag = 1;
		sess = sess->next;
	}

}

static void stop_port_buffering(int port, int flag);
static void got_hup(int sig);
static void got_usr(void);
static void handle_port(int port);
static void handle_session(int port);
static void start_port_buffering(int port);
static void re_open_db_files(void);
static void handle_dtr(int port);
static void parse_alert_strings(int port);  //[GY]2006/May/31

//[RK]Dec/22/04 - RFC2217 - notify modem state
static unsigned char cy_ModemState[MAXLINES];
static unsigned char cy_FirstModemStateSent[MAXLINES];
static Boolean cy_TCPCEnabled[MAXLINES]; 
static unsigned long cy_notif_time[MAXLINES];
extern Boolean TCPCEnabled; // defined in sock.c => RFC2217 is/not enabled
extern unsigned char ModemStateMask;

/* Return the status of the modem control lines (DCD, CTS, DSR, RNG) */
extern unsigned char GetModemState(int PortFd,unsigned char PMState);
/* Send the CPC command Command using Parm as parameter */
extern void SendCPCByteCommand(int SockFd, unsigned char Command, unsigned char Parm);

#define TNASC_NOTIFY_MODEMSTATE ((unsigned char) 107)

/* Check the port state and notify the client if it's changed */
static void cy_notif_rfc2217(int index)
{
	unsigned char ModemSt = 0;

	ModemSt = GetModemState(tty_fd[index],cy_ModemState[index]) & ModemStateMask;

	if (!cy_FirstModemStateSent[index] || (ModemSt != cy_ModemState[index])) {
		this_sess = cy_shm->sess_list[index];
		SendCPCByteCommand(-1,TNASC_NOTIFY_MODEMSTATE, ModemSt);
		nsyslog(LOG_DEBUG,"Port[%d] RFC2217 sent modem state: %x", index+1,(unsigned int)ModemSt);
		cy_ModemState[index] = ModemSt;
		cy_FirstModemStateSent[index] = 1; //mp[Fev/2004] Bug3112
	}
}

/*
 *	Open the logfile.
 */
static char ident[] = "Data Buffering";

static void init_log(void)
{
	int fac;

	fac = LOG_FAC(LOG_LOCAL0) + mainconf.DB_facility;
	nopenlog(ident, LOG_CONS | LOG_NDELAY, LOG_MAKEPRI(fac, 0));

	nsyslog(LOG_INFO, "SysLog Started");
}

void ExitFunction(void)
{
	int i;
	char cmd[256];

	if (getpid() != cy_shm->DB_pid) {
		return;
	}

#ifdef OEM1
	if( filtering_present) kill_filtering(SIGTERM);
#endif
	for(i=0; i < MAXLINES; i++) {
		if (shm_get_st_data_buffering(i) == DB_ST_ACTIVE) {
			stop_port_buffering(i, 1);
		}
	}

	cy_shm->DB_pid = 0;

	cy_shm->DB_status = DB_STATUS_RESET;

	sprintf(cmd, "umount %s", DB_nfs);
	if ( system(cmd) == -1 ) {
		XPRINTF("Error when umounting %s.\r\n",DB_nfs);
	}
}

#define OPEN_FLAG (O_RDWR|O_CREAT|O_NONBLOCK)
#define PERM_FLAG (S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH)

static int nfs_server_not_ok(void)
{
	int fd, cnt;
	char fname[64];

	while (shm_get_set_val(0, &cy_shm->st_cy_ras, 0, CY_SEM_CONFIG) != 1) {
		XPRINTF8("NFS poll : Shared memory not ok\r\n");
		sleep(1);
		continue;
	}

	if (mainconf.nfs_data_buffering == NULL ||
		*mainconf.nfs_data_buffering == 0) {
		exit(0);		// no data buffering in nfs server anymore
	}

	snprintf(fname, sizeof(fname), "%s/nfs_ok.data", DB_nfs);
	umask(0);

	if ((fd = open(fname, OPEN_FLAG, PERM_FLAG)) < 0) {
        return(1);
	}

	cnt = write(fd, "C++", 3);
	close(fd);
	return(cnt != 3);
}

static void poll_nfs_server(void)
{
	int pid;

	shm_signal(CY_SEM_NFS);

	if (!(cy_shm->DB_status & DB_STATUS_NFS_ERR)) {
		got_int_flag |= GOT_SIG_FD;
		cy_shm->DB_pid_poll = 0;
		shm_unsignal(CY_SEM_NFS);
		return;
	}

	if (cy_shm->DB_pid_poll == 0 || kill(cy_shm->DB_pid_poll, 0) < 0) {
		if ((pid = fork())) {
			if (pid > 0) {
				cy_shm->DB_pid_poll = pid;
			}
			shm_unsignal(CY_SEM_NFS);
			return;
		}
		// child
		while (nfs_server_not_ok()) {
			sleep(2);
		}
		shm_signal(CY_SEM_NFS);
		cy_shm->DB_status &= ~DB_STATUS_NFS_ERR;
		cy_shm->DB_status |= DB_STATUS_NFS_CHANGED;
		shm_unsignal(CY_SEM_NFS);
		exit(0);
	}

	shm_unsignal(CY_SEM_NFS);
}

int main(int argc, char **argv)
{
	/* Input fd set */
	fd_set InFdSet, auxFdSet;
	int i, j, max_fd=0, signals, snmp_pipe_fd;  //[GY]2006/May/30
	struct sigaction sa = {{0}};  //[GY]2006/May/30
#ifdef OEM1
	fd_set pipe_set, * pwrite_set;
	int sav_max_fd = 0;
#endif
	struct timeval BTimeout;

	if (argc >= 2) {
		set_xprintf(atoi(argv[1]));
	}

	if (shm_init(-1) < 0) {
		return(-1);
	}

	cy_shm->DB_pid = getpid();

//[RK]Nov/06/02 - Bug#53 - more information see the bug
	/*
 	 *	  Restore signals
 	 */
#if 1
	signal(SIGQUIT, SIG_DFL);
	signal(SIGHUP,  SIG_IGN);
	unblock(SIGQUIT);
	unblock(SIGHUP);

	sa.sa_handler = got_int;
	sigaction(SIGUSR1,  &sa, NULL);
	sigaction(SIGUSR2,  &sa, NULL);
	sigaction(SIGTERM,  &sa, NULL);
	sigaction(SIGINT,   &sa, NULL);
	sigaction(SIGIO,   &sa, NULL); 	//[RK]Sep/04/02 Dell
	sigaction(SIGCHLD, &sa, NULL);
	sigaction(SIGALRM, &sa, NULL);
	unblock(SIGUSR1);
	unblock(SIGUSR2);
	unblock(SIGTERM);
	unblock(SIGINT);
	unblock(SIGIO);			//[RK]Sep/04/02 Dell
	unblock(SIGCHLD);
	unblock(SIGALRM);
#else
	sa.sa_handler = got_int;
	for(i=1; i < 64; i++) {
		if (i == SIGPIPE) continue;
		if (i == SIGWINCH) continue;
		if (i == SIGSEGV) continue;
		if (i == SIGFPE) continue;
		if (i == SIGILL) continue;
		if (i == SIGKILL) continue;
		sigaction(i,  &sa, NULL);
		unblock(i);
	}
#endif

	while (shm_get_set_val(0, &cy_shm->st_cy_ras, 0, CY_SEM_CONFIG) != 1) {
		sleep(1);
		continue;
	}

	/* Redirect stdin, stdout, and stderr to /dev/null. */
	freopen("/dev/null", "r", stdin);
	freopen("/dev/null", "w", stdout);
	freopen("/dev/null", "w", stderr);

	atexit(ExitFunction); 

	init_log();

#ifdef OEM1
	init_pipes();
	if( ! filtering_present ) {
	  nsyslog(LOG_INFO, "filtering/alarm is not present");
	}
#endif
	for(i=0; i < MAXLINES; i++) {
		file_fd[i] = -1; 	//[RK]ALARM
		tty_fd[i] = -1;
		st_lines[i] = 0;
		dtr_lines[i] = 0;
		db_lines[i] = 0;
		time_start[i] = 0;
		cy_shm->st_data_buffering[i] = DB_ST_INACTIVE;
		cyb_tmp[i].ix_buf = 0;
		cyb_tmp[i].cnt_buf = 0;
		//[RK]Dec/22/04 - RFC2217 send notify
		cy_TCPCEnabled[i] = 0;
		cy_notif_time[i] = 0;
		cy_ModemState[i] = 0;
		cy_FirstModemStateSent[i] = 0; //mp[Fev/2004] Bug3112
	}

        //[GY]2006/May/31
        snmp_pipe_flag = 1;
        if( access(snmp_pipe, F_OK) ) {
           if( mkfifo(snmp_pipe, 0777) ) {
              snmp_pipe_flag = 0;
              nsyslog(LOG_ERR,"Named pipe %s could not be created.", snmp_pipe);
           }
        }
        if( snmp_pipe_flag ) {
           if( (snmp_pipe_fd = open(snmp_pipe, O_RDWR|O_NONBLOCK|O_NOCTTY)) == -1 ) {
              snmp_pipe_flag = 0;
              nsyslog(LOG_ERR,"Named pipe %s could not be opened.", snmp_pipe);
           }
           else {
              if( (snmp_pipe_FP = fdopen(snmp_pipe_fd, "w")) == NULL ) {
                 snmp_pipe_flag = 0;
                 nsyslog(LOG_ERR,"Named pipe %s could not be associated with a stream.", snmp_pipe);
              }
              else {
                 if( setvbuf(snmp_pipe_FP, NULL, _IONBF , 2048) ) {
                    snmp_pipe_flag = 0;
                    nsyslog(LOG_ERR,"Named pipe %s could not be assigned to a buffer.", snmp_pipe);
                 }
              }
           }
        }

	got_hup(0);
	got_int_flag |= GOT_SIG_FD;
	dls_timer_on = 0;

	while (1) {
		if (got_int_flag & GOT_SIG_IGN) {
			nsyslog(LOG_WARNING, "cy_buf got_int sig = %x", last_sig);	  
			XPRINTF("Cy_buffering Got signal : %d\r\n", last_sig);
			got_int_flag &= ~GOT_SIG_IGN;
		}

		if (got_int_flag & GOT_SIG_CHILD) {
			XPRINTF("buffering : got sig Child\r\n");
			wait(&i);
			got_int_flag &= ~GOT_SIG_CHILD;
			if ((cy_shm->DB_status & DB_STATUS_NFS_CHANGED) &&
				!(cy_shm->DB_status & DB_STATUS_NFS_ERR)) {
				cy_shm->DB_status &= ~DB_STATUS_NFS_CHANGED;
				got_int_flag |= GOT_SIG_FD;
				re_open_db_files();
			}
		}

		if (got_int_flag & GOT_SIG_INT) {
			XPRINTF("buffering : got sig int\r\n");
			break;
		}

		while (shm_get_set_val(0, &cy_shm->st_cy_ras, 0, CY_SEM_CONFIG) != 1) {
			XPRINTF8("buffering : Shared memory not ok\r\n");
			sleep(1);
			continue;
		}

		if (got_int_flag & GOT_SIG_HUP) {
			XPRINTF8("buffering : got sig hup\r\n");
			got_int_flag &= ~GOT_SIG_HUP;
			got_int_flag |= GOT_SIG_FD;
			got_hup(1);
		}

		if (got_int_flag & GOT_SIG_USR) {
			XPRINTF8("buffering : got sig usr\r\n");
			got_int_flag &= ~GOT_SIG_USR;
			got_int_flag |= GOT_SIG_FD;
			got_usr();
		}

		if (got_int_flag & GOT_SIG_IO) { 	//[RK]Sep/04/02 Dell
			XPRINTF8("buffering : got sig io\r\n");
			got_int_flag &= ~GOT_SIG_IO;
			got_int_flag |= GOT_SIG_FD;
			for(j=0; j < max_lines; j++) {
				i = vet_lines[j];
				if (cy_shm->sess_list[i]) {
					//[RK]Nov/07/03 Fixed bg#1045
					if ((st_lines[i]==0) && (db_lines[i]==0)){
						st_lines[i] = 1;
					}
					handle_session(i);
				} else {
					if (tty_fd[i] != -1) handle_dtr(i);
				}
			}
		}

		if (got_int_flag & GOT_SIG_FD) {
			/* Set up fd sets */
			FD_ZERO(&InFdSet);
			max_fd=-1;
			max_lines = 0;
			for(i=0; i < MAXLINES; i++) {
				if (lineconf[i].tty && *lineconf[i].tty &&
					!lineconf[i].fast && //[RK]May/22/03 Performance Improvement
					!lineconf[i].virtual && tty_fd[i] >= 0 &&
					(lineconf[i].protocol == P_SOCKET_SERVER ||
					 lineconf[i].protocol == P_SOCKET_SERVER_RAW ||
					 lineconf[i].protocol == P_SOCKET_SSH)) {
					if ((!(cy_shm->DB_status & DB_STATUS_NFS_ACTIVE) ||
					 	!(cy_shm->DB_status & DB_STATUS_NFS_ERR) ||
					 	(lineconf[i].data_buffering == 0)) &&
						(st_lines[i] == 1)) {
						FD_SET(tty_fd[i],&InFdSet);
						if (tty_fd[i] > max_fd) {
							max_fd = tty_fd[i];
						}
					}
					vet_lines[max_lines++] = i;
				}
			}
			if (max_lines == 0) {
				XPRINTF8("buffering : No tty to buffering\r\n");
				sleep(10);
				continue;
			}
			if (max_fd < 0) {
				/* When there is no port open, we still have to handle the 
				sessions. */
				usleep(100*1000);
				for(j=0; j < max_lines; j++) {
					i = vet_lines[j];
					if (cy_shm->sess_list[i]) {
						handle_session(i);
					} else {
						if( tty_fd[i] != -1) handle_dtr(i);
					}
				}
				if ((cy_shm->DB_status & DB_STATUS_NFS_ACTIVE) &&
					((cy_shm->DB_status & DB_STATUS_NFS_ERR) ||
					 cy_shm->DB_pid_poll)) {
					poll_nfs_server();
				}
				continue;
			}
			got_int_flag &= ~GOT_SIG_FD;
		}

		if (got_int_flag & GOT_SIG_DLS) {
			for(i=0; i < MAXLINES; i++) {
				if (pipe_fd[i] == -1) continue;
				if (log_buffer[i].dls_status & DLS_ST_TIMESEND) {
					if (log_buffer[i].dls_buffer_pos) {
						curr_log = &log_buffer[i]; //handle_dls_buffer() needs it
						handle_dls_buffer(NULL, 0, i);
					}
				} else {
					log_buffer[i].dls_status |= DLS_ST_TIMESEND; //preparing next pass
				}
			}
			got_int_flag &= ~GOT_SIG_DLS;
		}

		auxFdSet = InFdSet;

		BTimeout.tv_sec = 0;
		BTimeout.tv_usec = SEL_TIMEOUT * 1000;

#ifdef OEM1
	       	pwrite_set = NULL ;
		if( filtering_present ) {
		    for(j=0; j < max_lines; j++) {
		        int port = vet_lines[j];
		        int fd   = get_pipefd(port) ;
			if(fd >= 0){
			     if( has_remaining(port)){
		                 if( ! pwrite_set ){
			              pwrite_set = &pipe_set ;      
			              FD_ZERO(pwrite_set);
			              sav_max_fd = max_fd ;
		                 }
		                 FD_SET(fd,pwrite_set);
		                 if( fd > max_fd ) max_fd = fd ;
			     }
		        }
		   }
	        }
		if (select(max_fd+1,&auxFdSet,pwrite_set, NULL, &BTimeout) < 0) {
			XPRINTF8("buffering : select error\r\n");
			continue;
		}

		if( pwrite_set){
		  for(j=0; j < max_lines; j++) {
			int port = vet_lines[j];
			int fd =  get_pipefd(port);
			if( fd >= 0 )
			     if (FD_ISSET(fd, pwrite_set))
				  write_alarm_pipe(port,NULL,0);
		  }
		  max_fd = sav_max_fd ;
		}
#else
		if (select(max_fd+1,&auxFdSet, NULL, NULL, &BTimeout) < 0) {
			//[RK]nsyslog(LOG_ERR,"buffering select error :%m\r\n");
			continue;
		}
		
#endif		
		elapsed = (SEL_TIMEOUT * 1000 - BTimeout.tv_usec) / 1000;
		if (elapsed == 0)
			elapsed = 1;

		/* Check for data on the device fd */
		for(j=0; j < max_lines; j++) {
			i = vet_lines[j];
			
			if ((cy_shm->DB_status & DB_STATUS_NFS_ACTIVE) &&
				((cy_shm->DB_status & DB_STATUS_NFS_ERR) ||
				 cy_shm->DB_pid_poll) &&
				(lineconf[i].data_buffering != 0)) {
				poll_nfs_server();
			}
			
			if (FD_ISSET(tty_fd[i], &auxFdSet)) {
				handle_port(i);
				if ((cy_shm->DB_status & DB_STATUS_NFS_ACTIVE) &&
					(cy_shm->DB_status & DB_STATUS_NFS_ERR) &&
					(file_fd[i] != -1)) {
					close(file_fd[i]);
					file_fd[i] = -1;
				}
			} else if (cy_shm->sess_list[i] &&
					   (cy_shm->sess_list[i]->sess_flag & CY_SESS_FLAG_MUNTF_BIT)) {

				handle_port(i+MAXVLINES); //signals there is nothing to read from the port

			} else if (lineconf[i].DB_user_logs) { //mp: maybe, flagging another place
				                                   //we could avoid the while() loop here...
				if (cy_shm->sess_list[i]) {
					CY_SESS *saux = cy_shm->sess_list[i];
					while (saux) {
						if ((saux->sess_flag & CY_SESS_FLAG_DB_USRLOG) ||
							saux->user_away) {
							handle_port(i+MAXVLINES); //signals there is nothing to read from the port
							break;
						}
						saux = saux->next;
					}
				} else {
					if (!lineconf[i].fast && cy_shm->tty_user)
						handle_port(i+MAXVLINES); //signals there is nothing to read from the port
				}
			}

 			if (cy_shm->sess_list[i]) {
 				handle_session(i);
 			} else { //[RK]Dec/22/04 - RFC2217 - notify modem state
				cy_TCPCEnabled[i] = 0; // init - does not have socket
				cy_notif_time[i] = 0;
				cy_ModemState[i] = 0;
				cy_FirstModemStateSent[i] = 0; //mp[Fev/2004] Bug3112
			}

			if (tty_fd[i]!=-1){
				  handle_dtr(i);
				//[RK]Dec/22/04 - RFC2217 - notify modem state
				if (cy_shm->sess_list[i] && cy_TCPCEnabled[i]) {
					if (lineconf[i].poll_interval &&
				    ((times(NULL) - cy_notif_time[i]) * 10 >= lineconf[i].poll_interval)) {
						cy_notif_rfc2217(i);
						cy_notif_time[i] = times(NULL);
					}
				}
			}	  

			//[RK]Jun/08/06 - event notification
			if (lineconf[i].dcd) {
				ioctl(tty_fd[i],TIOCMGET,&signals);
				if (!(signals & TIOCM_DTR)) {
					signals = signals | TIOCM_DTR ; 
					ioctl(tty_fd[i],TIOCMSET,&signals);
					ioctl(tty_fd[i], TIOCMGET, &signals);
				}
				if (!(signals & TIOCM_CD) ) { // DCD is OFF
					if (cy_shm->dcd_status[i] == 1) // DCD was ON
						writeevt(10,"si",lineconf[i].serverfarm,i+1);
					cy_shm->dcd_status[i] = 0;
				} else {
					if (cy_shm->dcd_status[i] == 0) // DCD was OFF
						writeevt(9,"si",lineconf[i].serverfarm,i+1);
					cy_shm->dcd_status[i] = 1;
				}
				cy_shm->flag_dcd[i] = 0;
			}
		}
	}

	return(0);
}

static void stop_port_buffering(int port, int flag)
{
	int tot, i;
	char *buf;

	if (tty_fd[port] == -1) {
		return;
	}

	tot = log_buffer[port].tot_buf;
	buf = log_buffer[port].buffer;

	if (tot) { //[RK]ALARM
		curr_log = &log_buffer[port];
		check_file_size(file_fd[port], NULL, 0, 3, &lineconf[port], port);
	}

	if (flag == 2) {
		close(tty_fd[port]);
		shm_set_st_data_buffering(port, DB_ST_STOPPED);
	} else {
		serial_close(tty_fd[port],&save_old_tio[port]);
		shm_set_st_data_buffering(port, DB_ST_INACTIVE);
	}

	if (file_fd[port] != -1) {
		close(file_fd[port]); //[RK]ALARM
		file_fd[port] = -1;
	}

	if (pipe_fd[port] != -1) {
		close(pipe_fd[port]);
		pipe_fd[port] = -1;
		if (dls_timer_on) {
			for(i=0; i < MAXLINES; ) {
				if (pipe_fd[i] != -1) break;
				i++;
				if (i < MAXLINES) continue;
				dlstimer.it_value.tv_usec = 0; //stop timer
				setitimer(ITIMER_REAL, &dlstimer, NULL);
				dls_timer_on = 0;
			}
		}
	}

	tty_fd[port] = -1;
	st_lines[port] = 0;

#ifdef OEM1
	close_alarm_pipe(port);
#endif
	nsyslog(LOG_INFO, "Stopped on port S%d", port+1);
}

static void got_hup(int sig)
{
	int i, flag = (sig == 0);
#ifdef OEM1
	if( filtering_present) kill_filtering(SIGTERM);
#endif
	if (cy_shm->DB_status & DB_STATUS_FAC_CHANGED) {
		cy_shm->DB_status &= ~DB_STATUS_FAC_CHANGED;
		closelog();
		init_log();
	}

	if (cy_shm->DB_status & DB_STATUS_NFS_CHANGED) {
		flag = 1;
		ExitFunction();
		cy_shm->DB_pid = getpid();
	}

	if (flag && mainconf.nfs_data_buffering && *mainconf.nfs_data_buffering) {
		char cmd[256];

		sprintf(cmd, "mount %s %s", mainconf.nfs_data_buffering, DB_nfs);
		if ((i = system(cmd)) != -1 && i != 127) {
			//[RK]Nov/18/02 - Bug#77
			if ((WIFEXITED(i) == 1) && // the child is ended 
			    (WEXITSTATUS(i) == 0)){ // mount cmd is OK
				cy_shm->DB_status |= DB_STATUS_NFS_ACTIVE;
			}
		}
	}

	if (cy_shm->DB_status & DB_STATUS_NFS_ACTIVE) {
		strcpy(cy_shm->DB_path, DB_nfs);
	} else {
		strcpy(cy_shm->DB_path, "/var/run/DB");
	}

// [Fuji] - Feb/11/2003 - vet_chg will be treated by masks. 1 will be related 
// to cy_buffering.

	for(i=0; i < MAXLINES; i++) {
		if (!flag && sig && !(cy_shm->vet_chg[i] & 1)) {
			continue;
		}
		cy_shm->vet_chg[i] &= ~1;
		stop_port_buffering(i, 1);
		if (!lineconf[i].fast) start_port_buffering(i);
	}
	cy_shm->DB_status |= DB_STATUS_RD_ACTIVE;
#ifdef OEM1
	if( filtering_present )
	  if( exec_filtering() < 0 )
	    nsyslog(LOG_WARNING, "Alarm is not running");	  
#endif
}

static void got_usr(void)
{
	int i;
#ifdef OEM1
	if( filtering_present ) kill_filtering(SIGTERM);
#endif
	for(i=0; i < MAXLINES; i++) {
		switch (shm_get_st_data_buffering(i)) {
		case DB_ST_STOPPING:
			stop_port_buffering(i, 2);
			break;
		case DB_ST_RESTART:
			start_port_buffering(i);
			break;
		case DB_ST_SOCK_REST:
			stop_port_buffering(i, 1);
			sleep(1);
			start_port_buffering(i);
			break;
		}
	}
#ifdef OEM1
	if( filtering_present ) 
	  if ( exec_filtering() < 0 )
	    nsyslog(LOG_WARNING, "Alarm is not running");
#endif
}

static void handle_port(int port)
{
	int tot, cnt, nrflag=0, flag=1;	//[RK]Oct/07/02
	char *buf;
	CY_SESS *sess, *saux;
	char db_usrlog[80]; //username must be about 20 chars max.
	struct tm *tz;

	if (port >= MAXVLINES) { //signals there is nothing to read on the port
		nrflag = 1;
		port -= MAXVLINES;
	}
	sess = cy_shm->sess_list[port];

	curr_log = &log_buffer[port];

	if (curr_log->tot_buf && curr_log->read_buf) {
		cnt = curr_log->tot_buf - curr_log->read_buf + 1;
		goto check_file;
	}

	//[RK]Mar/17/03 - Do not read TTY data if in PM session.
	saux = sess;
	while (saux) {
		if (saux->sess_flag & CY_SESS_FLAG_PMD) {
			return;
		}
		saux = saux->next;
	}

	tot = curr_log->tot_buf;
	buf = curr_log->buffer;
	cnt = MAX_BUFFER - 1 - tot;

	if (sess) {
		cnt = max_sess_free_buf(cnt, port); //cnt will be the smallest space
                                            //among "sessions" and log_buffer
		if (!cnt || ((sess->sess_flag & CY_SESS_FLAG_MUNTF_BIT) &&
					 (cnt < muntf_len))) {
			set_flow_off_all_session(port) ;
			st_lines[port] = 0;
			got_int_flag |= GOT_SIG_FD;
			return;
		}

		if (sess->sess_flag & CY_SESS_FLAG_MUNTF_BIT)
			cnt -= muntf_len; //space for multiuser notification

	}

	/* If there is no session and data buffering is configured 
	   as linear, do not go on to read from the port. If data
   	   is being sent to the port, RTS will drop once data fills
	   up the 4k buffer in the kernel */
	if (!sess && (lineconf[port].data_buffering != 0) &&
		((int)lineconf[port].DB_mode==LIN_mode) && 
		(mainconf.nfs_data_buffering==NULL || *mainconf.nfs_data_buffering==0)) {
		// nothing will be read into the fd, thus select will sleep
		db_lines[port] = 1;	//set flag 
		st_lines[port] = 0;
		got_int_flag |= GOT_SIG_FD;
		return;
	}

	if (cnt && !nrflag && !(curr_log->dls_status & DLS_ST_PIPEFULL)) {
		cnt = read(tty_fd[port], &buf[tot], cnt);
		if (pipe_fd[port] != -1)
			curr_log->dls_status &= ~DLS_ST_TIMESEND;
	} else {
		cnt = 0; //nothing read
		if (curr_log->dls_status & DLS_ST_PIPEFULL)
			handle_dls_buffer(NULL, 0, port); //try to reestablish dls_pipe health
	}

	if (sess && (sess->sess_flag & CY_SESS_FLAG_MUNTF_BIT)) {
		char muntf[muntf_len + 1];
		int userno = 0;
		
		// count number of users
		saux = sess;
		while (saux) {
			userno++;
			saux = saux->next;
		}
		// prepare multiuser notification
		if ((sess->sess_flag & CY_SESS_FLAG_MUNTF_OUT_BIT)) {
			sprintf(muntf, muntf_out, userno);
		} else {
			sprintf(muntf, muntf_in, userno);
		}

		// Multiuser Notification goes to the buffer to be processed as normal serial input
		memcpy(&buf[tot+cnt], muntf, muntf_len);

		cnt += muntf_len;
	}

	if (cnt > 0) {

		pconf = &lineconf[port]; //mp: be careful, it's used in another place !!!

		if (EscWriteBuffer(-1, &buf[tot], cnt, 0, tty_fd[port], (int)sess)) {  //[GY]2006/May/30
			st_lines[port] = 0;
			got_int_flag |= GOT_SIG_FD;
		}

		if (sess && (sess->sess_flag & CY_SESS_FLAG_MUNTF_BIT)) {
			saux = sess;

			cnt -= muntf_len; //multiuser notification shouldn't go to data buffering file

            //mp[Sep2003] update multiuser notification status
			shm_signal(CY_SEM_SHM);
			while (saux) {
				saux->sess_flag &= ~CY_SESS_FLAG_MUNTF_M;
				saux->sess_flag |= CY_SESS_FLAG_MUNTF_NO;
				saux = saux->next;
			}
			shm_unsignal(CY_SEM_SHM);

		}

	}

	//[RK]Oct/17/02 BUG#36
	saux = sess;
	while (saux) {
		if (saux->sess_flag & CY_SESS_FLAG_W &&
			!(saux->sess_flag & CY_SESS_FLAG_DBMENU)) {
			flag = 0;
			kill(saux->vet_pid, SIGIO);
		} 
		saux = saux->next;
	}

	if(!lineconf[port].data_buffering &&
	   !lineconf[port].syslog_buffering &&
	   !lineconf[port].alarm) {
		XPRINTF4("buffering : tty(%d) buffering not config 1\r\n", port+1);
		return;
	}

#ifdef OEM1
	if( filtering_present ){
		if( write_alarm_pipe(port,&buf[tot],cnt) < 0 ){
			nsyslog(LOG_ERR, "Fifo Write (fifo index=%d) Error: %s",
					port, strerror(errno));
		}
	}
#endif

	/* If there is a session, and data buffering is configured as linear,
	   then read the port for data; however, do not send data to user-space
	   buffer. Data will just be outputted to user when user connects to port. */ 
	if (lineconf[port].data_buffering != 0 &&
		(int)lineconf[port].DB_mode==LIN_mode && (mainconf.nfs_data_buffering==NULL || *mainconf.nfs_data_buffering==0)) {
		return;		//if db is linear, don't go on to store data into userspace file done in check_file_size
	}

	// Check user connections and disconnections to log to buffering file
	if (lineconf[port].DB_user_logs && sess) {
		int userchg = FALSE;

		saux = sess;
		while (saux) {
			if (saux->sess_flag & CY_SESS_FLAG_DB_USRLOG) {
				// New user connected: prepare message to write to buffering file...
				tz = localtime(&saux->time);
				sprintf(db_usrlog, "\n<connect> %d-%02d-%02d %02d:%02d:%02d  %s  %s </connect>\n",
						tz->tm_year+1900, tz->tm_mon+1, tz->tm_mday,
						tz->tm_hour, tz->tm_min, tz->tm_sec, saux->tty_user,
						(saux->sess_flag & CY_SESS_FLAG_W) ? "Read/Write" : "Read_Only");

				memcpy(&buf[tot+cnt], db_usrlog, strlen(db_usrlog));
				cnt += strlen(db_usrlog);
				if (!userchg) userchg = TRUE;
			}				
			if (saux->user_away) {
				// A user just disconnect: prepare message to write to buffering file...
				tz = localtime(&saux->time_user_left);
				sprintf(db_usrlog, "\n<disconnect> %d-%02d-%02d %02d:%02d:%02d  %s </disconnect>\n",
						tz->tm_year+1900, tz->tm_mon+1, tz->tm_mday,
						tz->tm_hour, tz->tm_min, tz->tm_sec, saux->user_away);

				memcpy(&buf[tot+cnt], db_usrlog, strlen(db_usrlog));
				cnt += strlen(db_usrlog);
				if (!userchg) userchg = TRUE;
			}

			saux = saux->next;
		}
		if (userchg) {
			saux = sess;
			shm_signal(CY_SEM_SHM);
			while (saux) {
				saux->sess_flag &= ~CY_SESS_FLAG_DB_USRLOG;
				if (saux->user_away) shm_free_str(saux->user_away);
				saux->user_away = NULL;
				saux = saux->next;
			}
			shm_unsignal(CY_SEM_SHM);
		}

	} else if (!lineconf[port].fast && lineconf[port].DB_user_logs && cy_shm->tty_user[port]) {
		// Last user just disconnect: prepare message to write to buffering file...
		tz = localtime(&cy_shm->st_time[port]);
		sprintf(db_usrlog, "\n<disconnect> %d-%02d-%02d %02d:%02d:%02d  %s </disconnect>\n",
				tz->tm_year+1900, tz->tm_mon+1, tz->tm_mday,
				tz->tm_hour, tz->tm_min, tz->tm_sec, cy_shm->tty_user[port]);

		memcpy(&buf[tot+cnt], db_usrlog, strlen(db_usrlog));
		cnt += strlen(db_usrlog);

		shm_signal(CY_SEM_SHM);
		shm_free_str(cy_shm->tty_user[port]);
		cy_shm->tty_user[port] = NULL;
		shm_unsignal(CY_SEM_SHM);
	}

	if (!cnt) return;

check_file:
	check_file_size(file_fd[port], NULL, cnt, flag, &lineconf[port], port); //[RK]Oct/07/02
}

static void handle_dtr(int port)
{
	int old_dtr, new_dtr;

	/* If there is a session to the port, DTR should always be on. */
	if (cy_shm->sess_list[port]) {
		if (dtr_lines[port] != 1) {
		/* start DTR immediately */
		ioctl(tty_fd[port],TIOCMGET,&old_dtr);
		old_dtr |= TIOCM_DTR;    
		ioctl(tty_fd[port],TIOCMSET,&old_dtr);
		dtr_lines[port] = 1;
		}
		return;
	}

	// no socket session  -> treats DTR	
	if (dtr_lines[port]==1) {
#ifdef DEBUG_CNT
		print_counters(1);
#endif
		ioctl(tty_fd[port],TIOCMGET,&old_dtr);
		new_dtr = old_dtr;
		if (!lineconf[port].DTR_reset) {
			dtr_lines[port] = 0;
			if (old_dtr & TIOCM_DTR) {
				new_dtr &= ~TIOCM_DTR;  
			}
		} else { /* drop DTR if configured to */
			if (lineconf[port].DTR_reset >= 100) {
				new_dtr &= ~TIOCM_DTR;  
				dtr_lines[port] = 2; 		
				time_start[port] = 0;
			} else {
				dtr_lines[port] = 0;				
				if (!(new_dtr & TIOCM_DTR)) {
					new_dtr |= TIOCM_DTR;  
				}
			}
		}
		if (new_dtr != old_dtr) {
			ioctl(tty_fd[port], TIOCMSET, &new_dtr);
		}
	} else { 
		if (dtr_lines[port] == 2) {
			/* Start DTR only if the difference in time is greater 
			 * than DTR_reset and also data buffering must be on. 
			 * If it's not, then no need to turn DTR back. It will 
			 * come on when user establishes a session. */
			int tmpTime;

			time_start[port] += elapsed;
			tmpTime = (lineconf[port].DTR_reset < 100) ? 
						100 : lineconf[port].DTR_reset;
			if (time_start[port] >= tmpTime) {
				ioctl(tty_fd[port],TIOCMGET,&old_dtr);
				old_dtr |= TIOCM_DTR;	/* turn on DTR */
				ioctl(tty_fd[port],TIOCMSET,&old_dtr);
				dtr_lines[port] = 0;
			}
		}
	}

	if ((st_lines[port]==0) && (db_lines[port] == 0)) { 	 
		st_lines[port] = 1;
		got_int_flag |= GOT_SIG_FD;
	}
} 

static void handle_session(int port)
{
	CY_SESS *sess = cy_shm->sess_list[port];
	int flag = 0, old_dtr;

	/* If there is a session to the port, DTR should always be on. */
	if (dtr_lines[port] != 1 && tty_fd[port] != -1) {
		/* start DTR immediately */
		ioctl(tty_fd[port],TIOCMGET,&old_dtr);
		old_dtr |= TIOCM_DTR;    
		ioctl(tty_fd[port],TIOCMSET,&old_dtr);
		dtr_lines[port] = 1;
	}

	/* If there is session, and db_lines flag was set to not read, 
   	   reset the flag so that port can be read now */
	if (db_lines[port] == 1) {
		db_lines[port] = 0;
		st_lines[port] = 1;
       	got_int_flag |= GOT_SIG_FD;
	}
	if (sess->sess_flag & CY_SESS_FLAG_PMD) {
		// PM session
		if (bytesAvailable(&sess->InBuffer)) {
			//[RK]Dec/22/04 - RC2217
			TCPCEnabled = cy_TCPCEnabled[port]; 
		 	EscRedirectChar(-2, DeviceFd, &sess->InBuffer);
			cy_TCPCEnabled[port] = TCPCEnabled;
		}
		return;
	}

	tmp = &cyb_tmp[port];
	pconf = &lineconf[port];
	DeviceFd = tty_fd[port];

	if (!write_pending_buffer(DeviceFd)) {
	  while (sess) {
		if (sess->sendbreak) {
			//[RK]May/05/03 - break interval
			// sendbreak has the break interval in miliseconds
			tcsendbreak(DeviceFd, sess->sendbreak);
			sess->sendbreak = 0;
		}
		this_sess = sess;
		//[RK]Dec/22/04 - RC2217
		if (sess == cy_shm->sess_list[port]) {
			TCPCEnabled = cy_TCPCEnabled[port]; 
		}
		EscRedirectChar(-1, DeviceFd, &sess->InBuffer);
		if (sess == cy_shm->sess_list[port]) {
			cy_TCPCEnabled[port] = TCPCEnabled;
		}
		if (sess->flow_flag == 2) {
			sess->flow_flag = 0;
			if (!flag) flag = 2;
		}
		if (sess->flow_flag == 1) {
			flag = 1;
			break;
		}

		sess = sess->next;
	  }
	}

	if (flag == 2 && db_lines[port] == 0) {
		st_lines[port] = 1;
		got_int_flag |= GOT_SIG_FD;
	}
}

static void re_open_db_files(void)
{
	int port;

	for(port=0; port < MAXLINES; port++) {

		if (!lineconf[port].tty || !(*lineconf[port].tty) ||
			lineconf[port].virtual || lineconf[port].fast ||
			(lineconf[port].protocol != P_SOCKET_SERVER &&
		 	lineconf[port].protocol != P_SOCKET_SERVER_RAW &&
		 	lineconf[port].protocol != P_SOCKET_SSH)) {
			continue;
		}

		if (lineconf[port].data_buffering) {
			file_fd[port] = open_data_buffering(&lineconf[port]);
		}
	}
}

//[GY]2006/May/31
static void parse_alert_strings(int port)
{
   char *p, *s, flag = 0;
   int cnt = 0;

   if( lineconf[port].port_alert_strings && *lineconf[port].port_alert_strings ) {
      p = s = lineconf[port].port_alert_strings;
      while( s && *s ) {
         while( *s && *s != '"' && *s != ',' ) s++;
         if( *s == ',' ) {
            if( !flag ) {
               flag = 0xa;
               log_buffer[port].alert_strings[cnt][0] = 0;
               cnt++;
            }
            else {
               if( !(flag & 0x1) ) {
                  if( flag & 0x2 )
                     log_buffer[port].alert_strings[cnt][0] = 0;
                  flag |= 0x2;
                  cnt++;
               }
            }
            s++;
         }
         if( *s == '"' ) {
            if( !flag ) {
               flag = 0x5;
               p = ++s;
            }
            else {
               if( flag & 0x1 ) {
                  flag &= 0xe;
                  strncpy(log_buffer[port].alert_strings[cnt], p, s-p);
                  log_buffer[port].alert_strings[cnt][s-p] = 0;
                  s++;
               }
               else {
                  flag |= 0x1;
                  flag &= 0xd;
                  p = ++s;
               }
            }
         }
      }
      for( ; cnt < ALERT_STRINGS_QUANTITY; cnt++ )
         log_buffer[port].alert_strings[cnt+1][0] = 0;
   }
   else {
      for( cnt = 0; cnt < ALERT_STRINGS_QUANTITY; cnt++ )
         log_buffer[port].alert_strings[cnt][0] = 0;
   }
}

static void start_port_buffering(int port)
{
	int fd, cnt, MLines;
	CY_SESS *sess = cy_shm->sess_list[port];
	char PipeName[64];

	shm_set_st_data_buffering(port, DB_ST_INACTIVE);

	if (!lineconf[port].tty || !(*lineconf[port].tty) ||
		lineconf[port].virtual || lineconf[port].fast ||
		(lineconf[port].protocol != P_SOCKET_SERVER &&
		 lineconf[port].protocol != P_SOCKET_SERVER_RAW &&
		 lineconf[port].protocol != P_SOCKET_SSH)) {
		return;
	}

	if ((fd = serial_open(&lineconf[port], 1,&save_old_tio[port])) < 0) {
		nsyslog(LOG_ERR, "Serial [%s] Open error %m",
			lineconf[port].tty);
		return;
	}

	tty_fd[port] = fd;
	st_lines[port] = 1;
	log_buffer[port].tot_buf = 0;
	log_buffer[port].dls_buffer_pos = 0;
	log_buffer[port].dls_status = DLS_ST_OK;
	log_buffer[port].read_buf = 0;
	cyb_tmp[port].ix_buf = 0;
	cyb_tmp[port].cnt_buf = 0;

	/* This makes sure right from the beginning that DTR is off 
	if there is no session and data buffering is not On */
	if (tty_fd[port]!=-1) {
		dtr_lines[port] = 0;
		if (lineconf[port].DTR_reset==0 && sess  == NULL) {
			ioctl(tty_fd[port],TIOCMGET,&MLines);
			MLines &= ~TIOCM_DTR;
			if (ioctl(tty_fd[port], TIOCMSET, &MLines) == -1) {
				perror("TIOCMSET");
			}
		}
	}

	shm_set_st_data_buffering(port, DB_ST_ACTIVE); //[RK]Dec/13/02 Bug#145

	if(!lineconf[port].data_buffering && !lineconf[port].alarm &&
           !lineconf[port].syslog_buffering) { //[RK]Dec/18/02 Bug#105
		XPRINTF4("buffering : tty(%d) buffering not config\r\n", port+1);
		return;
	}

        //[GY]2006/May/31
        if( lineconf[port].alarm == 2 || lineconf[port].alarm == 3 )
           parse_alert_strings(port);

	if (lineconf[port].data_buffering) { //[RK]ALARM
//[RK]Dec/26/02 Bug#156 - delete the treatment of the error in routine open_data_buffering
//		if ((fd = open_data_buffering(&lineconf[port])) < 0) {
//			nsyslog(LOG_ERR,"Unable to create/open file (%s) %m",
//				lineconf[port].tty);
//			serial_close(tty_fd[port]);
//			tty_fd[port] = -1;
//			st_lines[port] = 0;
//			return;
//		}
//
		fd = open_data_buffering(&lineconf[port]);
		file_fd[port] = fd;
		if (!(cy_shm->DB_status & DB_STATUS_NFS_ACTIVE) && (fd != -1)) { //[RK]Dec/26/02 Bug#156
			cnt = lseek(fd, 0, SEEK_END);

			if (lineconf[port].data_buffering < cnt) {
				trunc_file(fd, cnt, cnt - lineconf[port].data_buffering, &lineconf[port]); 
			}
		}
		if ((int)lineconf[port].DB_mode & DLS_mode) { //covers DLS and DLS_LOCAL !!!
			snprintf(PipeName, sizeof(PipeName), "/var/run/DB/dls_pipe%d", port+1);
			if (access(PipeName, F_OK)) {
				if (mkfifo(PipeName, 0777)) {
					nsyslog(LOG_ERR,"Unable to create FIFO special file (%s) %m", PipeName);
					return;
				}
			}
			if ((pipe_fd[port] = open(PipeName, O_RDWR | O_NONBLOCK | O_NOCTTY | O_LARGEFILE)) < 0) {
				nsyslog(LOG_ERR,"Unable to open FIFO special file (%s) %m", PipeName);
				//It is not 100% correct to open a pipe as O_RDWR, but doing this
				//we can open a O_NONBLOCKing pipe even while it is not being read yet.
				//This is useful for process synchronization...
				return;
			}
			if (!dls_timer_on) {
				dlstimer.it_interval.tv_usec = DLS_TIMER * 1000;
				dlstimer.it_value.tv_usec = DLS_TIMER * 1000 * 2; //twice for the first period...
				setitimer(ITIMER_REAL, &dlstimer, NULL);
				dls_timer_on = 1;
			}
		}
	}
#ifdef OEM1
	open_alarm_pipe(port);
#endif
	nsyslog(LOG_INFO, "Started on port S%d (%s)", port+1, lineconf[port].tty);
//[RK]Dec/13/02 Bug#145	shm_set_st_data_buffering(port, DB_ST_ACTIVE);
	XPRINTF1("Port:%d TTY:%d FD:%d \r\n", port+1, tty_fd[port], file_fd[port]);
}
