/*****************************************************************
* File: cy_shm.c
*
* Copyright (C) 2003 Cyclades Corporation
*
* www.cyclades.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version
* 2 of the License, or (at your option) any later version.
*
* Description: Handles shared memory.
*
****************************************************************/

#include <sys/sem.h>
#include <sys/shm.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <termios.h>
#include <signal.h>
#include <ctype.h>
#include <errno.h>
#include <netdb.h>
#include <utmp.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <dirent.h>
#include <time.h>
#include <sys/stat.h>

#include "server.h"

#include "rot_nports.h"

#if _SEM_SEMUN_UNDEFINED
union semun {
	int val;					// value for SETVAL
	struct semid_ds *buf;		// buffer for IPC_STAT & IPC_SET
	unsigned short int *array;	// array for GETALL & SETALL
	struct seminfo *__buf;		// buffer for IPC_INFO
};
#endif

static int teste_main(int argc, char *argv[]);
static int w_cas_main(int argc, char *argv[]);
static int parse_restore_main(int argc, char *argv[]);
static int sess_mngt_main(int argc, char *argv[]);  //[GY]2006/Jun/01

int main(int argc, char *argv[])
{
	int i, shm_id, sem_id, size, dyn_size;
	struct shmid_ds shm_ctl;
	union semun sem_ctl;
	struct MachineType mt;
	char *pn;

	get_machine_type(&mt);

	if ((pn = strrchr(argv[0], '/')) == NULL) {
		pn = argv[0];
	} else {
		++pn;
	}

	if (!strcmp(pn, "teste")) {
		return(teste_main(argc, argv));
	}

	//[RK]Jan/31/03 - 
	if (!strcmp(pn, "w_cas")) {
		return(w_cas_main(argc, argv));
	}

	//[RK]Jan/31/03 - 
	if (!strcmp(pn, "parse_restore")) {
		return(parse_restore_main(argc, argv));
	}

        //[GY]2006/Jun/01
        if (!strcmp(pn, "sess_mngt")) {
                return(sess_mngt_main(argc, argv));
        }

	printf("Machine %s, Ports %d\n", mt.name, mt.nports);

	if (argc == 2) {
		if (!strcmp(argv[1], "remove")) {
			if ((shm_id = shm_init(-1)) < 0) {
				return -1;
			}
			if (shmctl(shm_id, IPC_STAT, &shm_ctl) < 0) {
				printf("\n%s : Could not get shared memory ctl\n", argv[0]); 
				return(-1);
			}
			if (shm_ctl.shm_nattch > 1) {
				printf("\n%s : Shared memory still in use\n", argv[0]); 
				return(-1);
			}
			if (semctl(cy_shm->sem_id, 0, IPC_RMID, sem_ctl) < 0) {
				printf("\n%s : Cant remove semaphore, ignored\n", argv[0]); 
			}
			if (shmctl(shm_id, IPC_RMID, &shm_ctl) < 0) {
				printf("\n%s : Could not remove shared memory : %s\n", argv[0],
					strerror(errno)); 
				return(-1);
			}
			return(0);
		}
	}

	if (mt.nports <= 16) {
		dyn_size = 128;
	} else {
		if (mt.nports == 32) {
			dyn_size = 256;
		} else {
			dyn_size = 384;
		}
	}
	
	if (shm_init(dyn_size) < 0) {
		return -1;
	}

	size = sizeof (CY_SHM) + (dyn_size * 1024);
	memset(cy_shm, 0, size);

	// Create and init semaphores
	if ((sem_id = semget(CY_ID_SEM,CY_MAX_SEM,IPC_CREAT|IPC_EXCL|SHM_PERM)) < 0)  {
		perror("semget");
		return(-1);
	}

	cy_shm->sem_id = sem_id;

	sem_ctl.val = 1; // semaphore openned (not blocked)

	for(i=0; i < CY_MAX_SEM; i++) {
		semctl(sem_id, i, SETVAL, sem_ctl);
	}

	// initializes max device
	cy_shm->max_dev = mt.nports;
	printf("Max device :%d ", cy_shm->max_dev );

	// Initialize string pool
	cy_shm->free_str = cy_shm->strings;
	((CY_SHM_STR *)cy_shm->free_str)->size = dyn_size * 1024;
	printf(" dynamic memory size : %d\n", ((CY_SHM_STR *)cy_shm->free_str)->size );

	initcfg(&cy_shm->main_conf, &cy_shm->all_conf, &cy_shm->line_conf[0]);

	readcfg(&cy_shm->main_conf, &cy_shm->all_conf, &cy_shm->line_conf[0]);

	cy_shm->id_shm = CY_ID_SHM;
	cy_shm->st_cy_ras = 3;

	return(0);

}

typedef union _x_types {
	char				**xpc;
	struct realm_def	**xr;
} x_types;

typedef union _y_types {
	int					*yi;
	char				*yc;
	char				**ypc;
	struct realm_def	**yr;
	x_types				yx;
} y_types;

static int listx = 0;

static void shm_semop(int sem, int sem_op)
{
    struct sembuf sops;
	union semun sem_ctl;

	if (sem_op) {
    	printf("SHM_SEMOP SEM: %d %s\n", sem, sem_op>0? "Inc" : "Dec");
    	sops.sem_num = sem;  /* semaphore number: 0 = first */
    	sops.sem_op = sem_op;   /* semaphore operation */
    	sops.sem_flg = 0;  /* operation flags */
    	if (semop(cy_shm->sem_id, &sops, 1) < 0) {
        	perror("shm_semop:");
    	}
	}

	if ((sem_op = semctl(cy_shm->sem_id, sem, GETVAL, sem_ctl)) < 0) {
		perror("semctl:");
	} else {
   		printf("SHM_SEMOP READ: %d VAL:%d\n", sem, sem_op);
	}
}

static void show_groups(char *groups)
{
	int i = 0;

	while (groups) {
		printf("\nGroup%d %s", ++i, groups+4);
		groups = *(char **)groups;
	}
	printf("\n");
}

static void print_list(int flag, char *txt, char *list)
{
	CY_SHM_STR *psh = (CY_SHM_STR *)list;
	static char *type_str[5] = {"STR", "REALM", "SESS", "LSTR", "IPDU"};

	printf("\nList %s:\n", txt);

	while (psh) {
		if (flag) {
            switch(psh->flag) {
			case TYPE_SHM_STR:
				printf("Addr: %x[s:%02d c:%02d]  str:[%.50s]\n",
					(int)psh, psh->size, psh->count,
					(char *)psh + sizeof(CY_SHM_STR));
				break;
			case TYPE_SHM_LSTR:
				printf("Addr: %x[s:%02d c:%02d] lstr:[%.50s]\n",
					(int)psh, psh->size, psh->count,
					(char *)psh + sizeof(CY_SHM_STR) + 4);
				break;
			default:
				printf("Addr: %x type: %s [s:%02d c:%02d]\n", (int)psh,
					psh->flag < (sizeof(type_str)/sizeof(char *)) ?
					type_str[psh->flag] : "Undef",
					psh->size, psh->count);
				break;
			}
		} else {
			printf("Addr: %x size: %d\n", (int)psh, psh->size);
		}
		psh = psh->next;
	}

}

static void show_shm(int flag)
{
	int i;

	printf("id_shm : 0x%x\n", cy_shm->id_shm);
	printf("sem_id : %d\n", cy_shm->sem_id);
	printf("max_dev : %d\n", cy_shm->max_dev);
	printf("st_ras : %d\n", cy_shm->st_cy_ras);
	printf("DB_pid : %d\n", cy_shm->DB_pid);
	printf("DB_status : %x\n", cy_shm->DB_status);
	printf("DB_path : [%s]\n", cy_shm->DB_path);
	printf("PMD_pid : %d\n", cy_shm->PMD_pid);

	if (flag == 2) {
		printf("Port_DB_ST:\n");
		for (i = 0; i < cy_shm->max_dev; i++)
			printf("%4d", cy_shm->st_data_buffering[i]);
		printf("\nVet_ppp:\n");
		for (i = 0; i < cy_shm->max_dev; i++)
			printf("%8d", cy_shm->vet_ppp[i]);
		printf("\nVet_pid:\n");
		for (i = 0; i < cy_shm->max_dev; i++)
			printf("%4d", cy_shm->vet_pid[i]);
		printf("\nVet_chg:\n");
		for (i = 0; i < cy_shm->max_dev; i++)
			printf("%4d", cy_shm->vet_chg[i]);
		printf("\nVet_pool:\n");
		for (i = 0; i < cy_shm->max_dev; i++)
			printf("%8d", cy_shm->vet_pool[i]);
		printf("\nflag_dcd:\n");
		for (i = 0; i < cy_shm->max_dev; i++)
			printf("%4d", cy_shm->flag_dcd[i]);
		printf("\ndcd_status:\n");
		for (i = 0; i < cy_shm->max_dev; i++)
			printf("%4d", cy_shm->dcd_status[i]);
		printf("\nPMD_pms:\n");
		for (i = 0; i < cy_shm->max_dev; i++)
			printf("%4d", cy_shm->PMD_pms[i]);
		printf("\nPMD_outlets:\n");
		for (i = 0; i < cy_shm->max_dev; i++)
			printf("%4d", cy_shm->PMD_outlets[i]);
		printf("\npmcList :\n");
		for (i = 0; i < MAXPMDACCESS; i++) {
			if (cy_shm->pmc_list[i]) {
				int j;
				pmcInfo * pmc=cy_shm->pmc_list[i];
				printf("%d: ", i);
				printf("pmc_pid : %d  ipdu : %d status : %d username: %s", 
					pmc->pmc_pid, pmc->pmc_ipdu, pmc->pmc_status, pmc->pmc_username);
				printf("\npmc_outlet :");
				for (j=0; pmc->pmc_outlet[j]!=0; j++)
					printf(" %d,", pmc->pmc_outlet[j]);
				printf("\npmc_cmd : %s\n",pmc->pmc_cmd);
			}
		}
		printf("\n");
	}
}

void show_sess(int ind)
{
	CY_SESS *sess = cy_shm->sess_list[ind];
	pmcInfo *pmc;
	int cnt = 1;

	if (sess && sess->layout) { // IPDU
		printf("\nSerial tty Port : s%d is an IPDU\n\n", ind+1);
		return;
	}

	printf("\nSerial tty Port : s%d\n\n", ind+1);
	while (sess) {
		printf("* Session %d: type: %s",cnt,(sess->sess_flag & CY_SESS_FLAG_W) ?
			"Read/Write " : "Sniff      ");
		printf(" Sess_flag: 0x%x ", sess->sess_flag); 
		printf(" Pid: %d  Id: %d ", sess->vet_pid,sess->id); 
		if (sess->tty_user) {
			printf(" User: [%s]\n", sess->tty_user);
		} else {
			printf(" User: [%s]\n", sess->vet_pid ?	"none" : "null");
		}
		printf("   Flow_flag: %d", sess->flow_flag);
		printf("   Inbuffer Rd=%d Wr=%d    OutBuffer Rd=%d Wr=%d\n",sess->InBuffer.RdPos,
			sess->InBuffer.WrPos, sess->OutBuffer.RdPos, sess->OutBuffer.WrPos);
		sess = sess->next;
		cnt++;
	}
	
	sess = cy_shm->sess_list[ind];

	if (sess && sess->pmc) { // PM session
		pmc = sess->pmc;
		while (pmc) {
			printf("\n* PM session : IPDU %d outlet %s\n  status : ",pmc->pmc_ipdu+1,pmc->pmc_outlet);
			switch (pmc->pmc_status) {
				case 0: printf("Idle    "); break;
				case 1: printf("Request "); break;
				case 2: printf("Wait    "); break;
				case 3: printf("Answer  "); break;
			}
			printf("   command : <%s>\n", pmc->pmc_cmd);
			printf("  answer : <%s>\n",pmc->pmc_answerBuf);
			if (pmc->pmc_menu[0]) {
				printf("  menu:<%s>\n",pmc->pmc_menu);
			} else {
				printf("  menu was not mounted.\n");
			}
			printf("  index in PMC list = %d \n",pmc->idx_pmclist);
			pmc = pmc->next;
		}	
	}
}


static int check_equal_all(struct conf *pc, char *p, char *pa)
{
	y_types	y, x;
	int port;

	y.yc = p+pc->offs;
	x.yc = pa+pc->offs;

	switch(pc->type) {
	case C_INDY:
	case C_IPDY:
		port = ((struct line_cfg*)p)->this;

		if (*(y.yi) == 0 || *(y.yi) == *(x.yi)+port) {
			return(1);
		}
		break;
	default:
		if (*(y.yi) == *(x.yi)) {
			return(1);
		}
		break;
	}

	return(0);
}

static int ixr = 0;

static char str_ch[4];
static char *control(int ch)
{
	if (!ch) {
		return "off";
	}
	if (ch >= 0x20) {
		str_ch[0] = ch;
		str_ch[1] = 0;
	} else {
		str_ch[0] = '^';
		str_ch[1] = '@'+ch;
		str_ch[2] = 0;
	}
	return(str_ch);
}

static void list_config(struct conf *pc, char *p, char *str, char *pa, char *n)
{
	int i;
	y_types	y;
	char ch[32], *pch;
	struct lst *lst;

	for( ; pc->name != NULL; pc++) {
		y.yc = p+pc->offs;
		if (n && strcmp(pc->name, n)) continue;
		switch(pc->type) {
		case C_XSTR:
			if (!listx) {
				break;
			}
		case C_STR:
		case C_MSTR:
		case C_CHAT:
			if (pa && check_equal_all(pc, p, pa)) {
				break;
			}
			if (n) {
				if (*(y.yx.xpc)) {
					printf("%s\n", *y.yx.xpc);
				}
				break;
			}
			if (*(y.yx.xpc)) {
				printf("%s.%-20.20s:[%.50s]\n", str, pc->name, *y.yx.xpc);
			} else {
				printf("%s.%-20.20s:[null]\n", str, pc->name);
			}
			break;
		case C_REALM:
			if (n) break;
			if (*(y.yx.xr)) {
				strncpy(ch, str, sizeof(ch)-8);
				if ((pch = strchr(ch, '.'))) {
					sprintf(pch, ".realm%d", ++ixr);
				} else {
					snprintf(ch, sizeof(ch), "%s.realm%d", str, ++ixr);
				}
				list_config(&realm_cfg[0], *(y.yx.xpc), ch, NULL, NULL);
			}
			break;
		case C_XIPNO:
			if (!listx) {
				break;
			}
		case C_IPNO:
		case C_IPDY:
		case C_HOST:
			if (pa && check_equal_all(pc, p, pa)) {
				break;
			}
			if (n) {
				if (*y.yi) {
					printf("%d.%d.%d.%d\n",
						y.yc[0], y.yc[1], y.yc[2], y.yc[3]);  
				}
				break;
			}
			printf("%s.%-20.20s:%d.%d.%d.%d\n", str, pc->name,
				y.yc[0], y.yc[1], y.yc[2], y.yc[3]);  
			break;
		case C_XINT:
			if (!listx) {
				break;
			}
		case C_CHAR:
			if (pa && check_equal_all(pc, p, pa)) {
				break;
			}
			if (n) {
				printf("%s\n", control(*y.yi));
				break;
			}
			printf("%s.%-20.20s:%s\n", str, pc->name, control(*y.yi));
			break;
		case C_INT:
		case C_INDY:
			if (pa && check_equal_all(pc, p, pa)) {
				break;
			}
			if (n) {
				printf("%d\n", *y.yi);
				break;
			}
			printf("%s.%-20.20s:%d\n", str, pc->name, *y.yi);
			break;
		case C_LIST:
			if (pa && check_equal_all(pc, p, pa)) {
				break;
			}
			lst = pc->opts;
			for(i = 0; lst[i].name; i++) {
				if (lst[i].value == *y.yi) {
					if (n) {
						printf("%s\n", lst[i].name);
						break;
					}
					printf("%s.%-20.20s:%s\n", str, pc->name, lst[i].name);
					break;
				}
			}
			if (lst[i].name == NULL && n == NULL) {
				printf("%s.%-20.20s:null\n", str, pc->name);
			}
			break;
		}
		if (n) break;
	}
}
static void show_utmp_entry(void) 
{
	struct utmp *ptr;
	
	setutent();

	printf("\r\n UTMP entries : ");
	
	while((ptr = getutent()) != NULL) {
		printf("\r\n PID[%d] device[%s] user[%s] host[%s] session[%ld]",ptr->ut_pid,
				ptr->ut_line,ptr->ut_user,ptr->ut_host,ptr->ut_session);
		printf("\r\n         Type = %d ",ptr->ut_type);
		switch (ptr->ut_type) {
			case INIT_PROCESS : printf(" INIT "); break;
			case LOGIN_PROCESS: printf(" LOGIN ");break;
			case USER_PROCESS: printf(" USER "); break;
			case DEAD_PROCESS: printf(" DEAD "); break;
		}
	}
	printf("\r\n\r\n");
	endutent();
}

static int teste_main(int argc, char *argv[])
{
	extern char *optarg;
	extern int optind;
	int opt, port=0, listu=0, listg = 0, list=0, sem=0, sem_op=0, flag_sem=0, listm = 0, listc=0,utmp=0;
	struct MachineType mt;
	char ch[8], *name = NULL;

	get_machine_type(&mt);

    /* Parse command-line arguments. */
    while ((opt = getopt(argc, argv, "xgC:caA:X:P:p:s:S:r:u:LlMmU")) != EOF) {
        switch (opt) {
        case 'M':
			listm = 2;
            break;
        case 'm':
			listm = 1;
            break;
        case 'L':
			list = 2;
            break;
        case 'l':
			list = 1;
            break;
        case 'g':
			listg = 1;
            break;
        case 'x':
			listx = 1;
            break;
        case 'C':
			listc = 16;
			name=optarg;
            break;
        case 'c':
			listc = 4;
            break;
        case 'a':
			listc = 2;
            break;
        case 'P':
			listc = 32;
			if ((port = atoi(optarg)) > MAXVLINES || port < 1) {
				goto xdefault;
			}
			if (argv[optind] == NULL) {
				goto xdefault;
			}
			name=argv[optind++];
            break;
        case 'X':
			listc = 8;
			if ((port = atoi(optarg)) > MAXVLINES || port < 1) {
				goto xdefault;
			}
            break;
        case 'U':
	    		utmp = 1;
            break;
        case 'u':
			listu = 1;
			if ((port = atoi(optarg)) > MAXVLINES || port < 1) {
				goto xdefault;
			}
            break;
        case 'p':
			listc |= 1;
			if ((port = atoi(optarg)) > MAXVLINES || port < 1) {
				goto xdefault;
			}
            break;
        case 's':
			flag_sem = 1;
			sem_op = 1;
			if ((sem = atoi(optarg)) >= CY_MAX_SEM) {
				goto xdefault;
			}
            break;
        case 'S':
			flag_sem = 1;
			sem_op = -1;
			if ((sem = atoi(optarg)) >= CY_MAX_SEM) {
				goto xdefault;
			}
            break;
        case 'r':
			flag_sem = 1;
			sem_op = 0;
			if ((sem = atoi(optarg)) >= CY_MAX_SEM) {
				goto xdefault;
			}
            break;
        case 'h':
		default:
xdefault:
			printf("\n\nUSAGE: %s options\n\n"
					"-m      : List shared memory\n"
					"-g      : List Groups\n"
					"-x      : List extented field\n"
					"-l      : List shared memory block [used/free]\n"
					"-L      : List shared memory block [free/used]\n"
					"-r semn : Read semaphore semn[0-%d]\n"
					"-S semn : Decrement semaphore semn [0-%d]\n"
					"-s semn : Increment semaphore semn [0-%d]\n"
					"-p port : List 's<port>.*' configuration [1-%d]\n"
					"-P p nm : List 's<p>.nm' parameter [1-%d]\n"
					"-X port : List specific 's<port>.*' configuration [1-%d]\n"
					"-a      : List 'all.*' configuration\n"
					"-c      : List 'conf.*' configuration\n\n"
					"-A name : List 'conf.name' parameter\n\n"
					"-u port : List User session [1-%d]\n\n"
					"-U      : List UTMP entries\n\n"
				, argv[0],
				CY_MAX_SEM-1,
				CY_MAX_SEM-1,
				CY_MAX_SEM-1,
				MAXVLINES,
				MAXVLINES,
				MAXVLINES,
				MAXVLINES);
				exit(1);
			break;
		}
	}

	if (shm_init(-1) < 0) {
		return -1;
	}

	if (listc & 16) {
		ixr = 0;
		list_config(&main_cfg[0], (char *)&mainconf, NULL, NULL, name);
	}

	if (listc & 4) {
		ixr = 0;
		printf("\nGlobal configuration parameters\n");
		list_config(&main_cfg[0], (char *)&mainconf, "conf", NULL, NULL);
		printf("\n");
	}

	if (listc & 2) {
		ixr = 0;
		printf("\nAll ports configuration parameters\n");
		list_config(&line_cfg[0], (char *)&allconf, "all", NULL, NULL);
		printf("\n");
	}

	if (listc & 32) {
		ixr = 0;
		list_config(&line_cfg[0], (char *)&lineconf[port-1], NULL, NULL, name);
	}

	if (listc & 1) {
		ixr = 0;
		sprintf(ch, "s%d", port);
		printf("\n%s port configuration parameters\n", ch);
		list_config(&line_cfg[0], (char *)&lineconf[port-1], ch, NULL, NULL);
		printf("\n");
	}

	if (listc & 8) {
		ixr = 0;
		sprintf(ch, "s%d", port);
		printf("\nSpecific %s port configuration parameters\n", ch);
		list_config(&line_cfg[0], (char *)&lineconf[port-1], ch, (char *)&allconf, NULL);
		printf("\n");
	}

	if (list == 2) {
		print_list(1, "alloc", cy_shm->alloc_str);
		print_list(0, "free", cy_shm->free_str);
	}
	if (list == 1) {
		print_list(0, "free", cy_shm->free_str);
		print_list(1, "alloc", cy_shm->alloc_str);
	}

	if (flag_sem) {
		shm_semop(sem, sem_op);
	}

	if (listm) {
		show_shm(listm);
	}

	if (listu) {
		show_sess(port-1);
		printf("\n");
	}

	if (listg) {
		show_groups(mainconf.groups);
	}

	if (utmp) {
		show_utmp_entry();
	}

	return(0);
}

#define PRG_SOCKET_PFX	"socket:["
#define PRG_SOCKET_PFXl	(strlen(PRG_SOCKET_PFX))

static long extract_socket_inode(const char lname[]) 
{
	/* If lname is of the form "socket:[12345]", extract the "12345"
	   as inode.  Otherwise, return -1 .
	*/
	
	long inode = -1;
	char inode_str[strlen(lname + 1)]; 
	const int inode_str_len = strlen(lname) - PRG_SOCKET_PFXl - 1;

	if (strlen(lname) >= PRG_SOCKET_PFXl+3) { 
		if (!memcmp(lname, PRG_SOCKET_PFX, PRG_SOCKET_PFXl)) {
			if (lname[strlen(lname)-1] == ']') { 
				strncpy(inode_str, lname+PRG_SOCKET_PFXl, inode_str_len);
				inode_str[inode_str_len] = '\0';
				inode = atol(inode_str);
			}
		}
	}
	return(inode);
}

static void print_cmdline(int pid, unsigned char *prline)
{
	int len,fd;
	char buf[100];

	sprintf(buf,"/proc/%d/cmdline",pid);

	if ((fd = open(buf,O_RDONLY)) < 0) return;

	len = read(fd,buf, sizeof(buf) -1);
	close(fd);
	buf[len] = 0x00;
	
	len = snprintf(prline,23,"%d/%s",pid,buf); 
}

static int check_print_rem(long inode, unsigned char * rem_ip_txt)
{
	FILE * fd;
	char line[256];
	int len;
	struct in_addr aux;
	unsigned long rxq, txq, time_len, retr, inode_r;
	int num, local_port, rem_port, d, state, uid, timer_run, timeout;
	char rem_addr[128], local_addr[128], more[128];

	strcpy(line,"/proc/net/tcp");

	if (!(fd = fopen(line,"r"))) return 0;

	fgets(line,sizeof(line),fd); 	// skip header 

	while (fgets(line,sizeof(line),fd)) {
    		num = sscanf(line,
    "%d: %64[0-9A-Fa-f]:%X %64[0-9A-Fa-f]:%X %X %lX:%lX %X:%lX %lX %d %d %ld %128s\n",
			 &d, local_addr, &local_port, rem_addr, &rem_port, &state,
			 &txq, &rxq, &timer_run, &time_len, &retr, &uid, &timeout, &inode_r, more);
	
		if ( (num >= 11) && (inode_r == inode)){
			sscanf(rem_addr,"%X",&aux.s_addr);
			if (aux.s_addr) {
				sprintf(rem_ip_txt,"%s:%d",inet_ntoa(aux),rem_port);
				return 1;
			}
		}
		inode_r = 0;
	}
	return 0;
}

static void print_client(int pid,unsigned char * rem_ip_txt)
{
	DIR *dirfd=NULL;
	struct dirent *direfd;
	int len;
	char lname[30],line[100];
	long inode;

	// open process directory
	sprintf(line,"/proc/%d/fd",pid);

	*rem_ip_txt = 0;

	if (!(dirfd = opendir(line))) return;
	
	while ((direfd = readdir(dirfd))) {
		sprintf(line,"/proc/%d/fd/%s",pid, direfd->d_name);
		len = readlink( line, lname, sizeof(lname)-1);
		lname[len] = 0x00;
		inode = extract_socket_inode(lname);
		if (inode > 0) {
			if (check_print_rem(inode,rem_ip_txt)) {
				break;
			}
		} 
	}
	closedir(dirfd);
}

void wcas_show_sess(int ind)
{
	CY_SESS *sess = cy_shm->sess_list[ind];
	unsigned char prline[81];
	unsigned char aux[32];
	int len;
	struct tm st_time;

	while (sess) {
		len = 0;
		len += sprintf(prline,"%-20s ",
			sess->tty_user ? sess->tty_user : sess->vet_pid ? "none" : "null");
		len += sprintf((prline + len),"ttyS%-2d ", ind+1);
		print_client(sess->vet_pid,aux);
		len += sprintf((prline + len), "%-23s ", aux);
		memcpy(&st_time,localtime(&sess->time),sizeof(st_time));
		len += strftime((prline + len),10,"%I:%M%P ",&st_time);
		print_cmdline(sess->vet_pid,prline+len) ;		
		sess = sess->next;
		printf("%s\n",prline);
	}
}

void wcas_show_fast(int ind)
{
	unsigned char prline[81];
	unsigned char aux[32];
	int len = 0;
	struct tm st_time;

	len += sprintf((prline + len), "%-20s ",
		cy_shm->tty_user[ind] ? cy_shm->tty_user[ind] : "none");
	len += sprintf((prline + len),"ttyS%-2d ", ind+1);
	print_client(cy_shm->vet_pid[ind],aux);
	len += sprintf((prline + len), "%-23s ", aux);
	memcpy(&st_time,localtime(&cy_shm->st_time[ind]),sizeof(st_time));
	len += strftime((prline + len),10,"%I:%M%P ",&st_time);
	print_cmdline(cy_shm->vet_pid[ind],prline+len) ;		
	printf("%s\n",prline);
}

void wcas_show_ipdu(pmcInfo *pmc)
{
	unsigned char prline[81];
	unsigned char aux[32];
	int len = 0;

	len += sprintf((prline + len),"%-20s ", pmc->pmc_username);
	len += sprintf((prline + len),"ttyS%-2d ", pmc->pmc_ipdu+1);
	print_client(pmc->pmc_ppid,aux);
	len += sprintf((prline + len), "%-23s   -     ", aux);
	print_cmdline(pmc->pmc_ppid,prline+len) ;		
	printf("%s\n",prline);
}
static int w_cas_main(int argc, char *argv[])
{
	int port,i , count = 0, ipdu=0 ;
	CY_SESS *sess;

	setbuf(stdout,0);

	if (shm_init(-1) < 0) {
		exit(-1);
	}

	// to count number of sessions opened 
	for (port=0; port < MAXVLINES ; port++) {
		//[RK]Jun/28/04 - SSH/Telnet to IPDU
		if (cy_shm->line_conf[port].protocol == P_IPDU) {
			if (cy_shm->line_conf[port].pmsessions != 0) {
				ipdu = 1;
			}
			continue;
		}
		if (cy_shm->line_conf[port].fast) {
			if (cy_shm->vet_pid[port]) {
				count++;
			}
			continue;
		}
		sess = cy_shm->sess_list[port];
		while (sess) { 
			count++;
			sess = sess->next;
		}
	}

	if (ipdu) {
		for (i=0; i < MAXPMDACCESS ; i++) {
			if (cy_shm->pmc_list[i] == 0) continue;
			if (cy_shm->pmc_list[i]->pmc_ppid)
				count++;
		}
	}

	if (!count) exit(0);

	printf("\nCAS users :  %d \n",count);
	printf("USER                 TTY    FROM                    LOGIN@  PID/Command\n");
	for (port=0; port < MAXVLINES ; port++) {
		//[RK]Jun/28/04 - SSH/Telnet to IPDU
		if (cy_shm->line_conf[port].protocol == P_IPDU) {
			continue;
		}
		if (cy_shm->line_conf[port].fast && 
		    cy_shm->vet_pid[port] ) {
			wcas_show_fast(port);
		} else {
			wcas_show_sess(port);
		}
	}

	if (ipdu) {
		for (i=0; i < MAXPMDACCESS ; i++) {
			if (cy_shm->pmc_list[i] == 0) continue;
			if (cy_shm->pmc_list[i]->pmc_ppid)
				wcas_show_ipdu(cy_shm->pmc_list[i]);
		}
	}

	return 1;
}

static int parse_restore_main(int argc, char *argv[])
{
	FILE *fd, *fd_tmp, *fd_list;
	unsigned char tmp[50];
	unsigned char line[200], *plin;
	int size, flag_cr=0;
	
	if (!(fd = fopen(argv[1], "r"))) {
		return -1;
	}
	
	if (!(fd_list = fopen(argv[2], "w+"))) {
		return -1;
	}
	
	if (argc > 3) flag_cr=1;
	
	memset(line, 0, 200);

	fd_tmp = 0;

	while (fgets(line, 199, fd)) {
		plin = line;
		size = strlen(plin) - 1;
		if (flag_cr) {
			while ((size >= 0) && (line[size] != '\r')) size--;
			if (line[size] == '\r') {
				line[size]='\n';
				line[size+1]=0x00;
			}
			size = strlen(plin) - 1;
		} else {
			if (line[size] != '\n') {
				size++;
				line[size] = '\n';
				line[size+1]=0x00;
			}
		}
		
		if ((size > 5) && (line[0] == '[') && (line[1] == '[')
		    && (line[size-1] == ']') && (line[size-2] == ']')) {
			// new file
			if (fd_tmp) {
				fclose(fd_tmp);
				memset(tmp,0,50);
			}
			line[size-2] = 0x00;
			strcpy(tmp,&line[2]);
			sprintf(line,"%s ",tmp);
			strcat(tmp,".tmp");
			if (!(fd_tmp = fopen(tmp, "w+"))) {
				continue;
			}
			fputs(line, fd_list);			
		} else { 
			if (fd_tmp) {
				fputs(line,fd_tmp);	
			}
		}
		memset(line,0,200);	
	}
	return(0);
}

//[GY]2006/Jun/01
void sess_mngt_show_sess(int ind, time_t now)
{
	CY_SESS *sess = cy_shm->sess_list[ind];
	unsigned char prline[81];
	unsigned char aux[32];
	int len, sec;

	while( sess ) {
		len = 0;
		len += sprintf((prline + len), "%-20s ",
			sess->tty_user ?  sess->tty_user : sess->vet_pid ? "none" : "null");
		len += sprintf((prline + len), "ttyS%-2d ", ind + 1);
		print_client(sess->vet_pid,aux);
		len += sprintf((prline + len), "%-23s ", aux);
		sec = difftime(now, sess->time);
		len += sprintf((prline + len), "%3d:%02d:%02d ", sec/3600, (sec%3600)/60, sec%60);
		len += sprintf((prline + len), "Serial ");
		len += sprintf((prline + len), "  %2s  ", sess->sess_flag & CY_SESS_FLAG_W ? "RW" : "RO"); 
		len += sprintf((prline + len), "%d", sess->vet_pid);
		sess = sess->next;
		printf("%s\n", prline);
	}
}
void sess_mngt_show_fast(int ind, time_t now)
{
	unsigned char prline[81];
	unsigned char aux[32];
	int len, sec;

	len = sprintf(prline, "%-20s ",
		cy_shm->tty_user[ind] ?  cy_shm->tty_user[ind] : "none");
	len += sprintf((prline + len), "ttyS%-2d ", ind + 1);
	print_client(cy_shm->vet_pid[ind],aux);
	len += sprintf((prline + len), "%-23s ", aux);
	sec = difftime(now, cy_shm->st_time[ind]);
	len += sprintf((prline + len), "%3d:%02d:%02d ", sec/3600, (sec%3600)/60, sec%60);
	len += sprintf((prline + len), "Serial   RW  ");
	len += sprintf((prline + len), "%d", cy_shm->vet_pid[ind]);
	printf("%s\n", prline);
}
void sess_mngt_show_ipdu(pmcInfo *pmc)
{
	unsigned char prline[81];
	int len;
	unsigned char aux[32];

	len = sprintf(prline, "%-20s ", pmc->pmc_username);
	len += sprintf((prline + len), "ttyS%-2d ", pmc->pmc_ipdu + 1);
	print_client(pmc->pmc_ppid,aux);
	len += sprintf((prline + len), "%-23s ", aux);
	len += sprintf((prline + len), "       Power    RW  ");
	len += sprintf((prline + len), "%d", pmc->pmc_ppid);
	printf("%s\n", prline);
}
static int sess_mngt_main(int argc, char *argv[])
{
	int port, i, count = 0, ipdu = 0;
	CY_SESS *sess;
        time_t now;

	setbuf(stdout,0);

	if( shm_init(-1) < 0 )
		return -1;

	// to count number of sessions opened 
	for( port = 0; port < MAXVLINES; port++ ) {
		// SSH/Telnet to IPDU
		if( cy_shm->line_conf[port].protocol == P_IPDU ) {
			if( cy_shm->line_conf[port].pmsessions != 0 )
				ipdu = 1;
			continue;
		}
		if( cy_shm->line_conf[port].fast ) {
			if( cy_shm->vet_pid[port] )
				count++;
			continue;
		}
		sess = cy_shm->sess_list[port];
		while( sess ) { 
			count++;
			sess = sess->next;
		}
	}

	if( ipdu ) {
		for( i = 0; i < MAXPMDACCESS; i++ ) {
			if( cy_shm->pmc_list[i] == 0)
                                continue;
			if( cy_shm->pmc_list[i]->pmc_ppid )
				count++;
		}
	}

	if( !count ) {
                printf("\nCAS users : 0\n");
                return 0;
        }

        // Get present time
        time(&now);

	printf("\nCAS users :  %d\n", count);
	printf("USER                 DEVICE FROM                     DURATION TYPE    mode PID\n");
	for( port = 0; port < MAXVLINES; port++ ) {
		// SSH/Telnet to IPDU
		if( cy_shm->line_conf[port].protocol == P_IPDU )
			continue;
		if( cy_shm->line_conf[port].fast && cy_shm->vet_pid[port] )
			sess_mngt_show_fast(port, now);
		else
			sess_mngt_show_sess(port, now);
	}

	if( ipdu ) {
		for( i = 0; i < MAXPMDACCESS; i++ ) {
			if( cy_shm->pmc_list[i] == 0 )
                                continue;
			if( cy_shm->pmc_list[i]->pmc_ppid )
				sess_mngt_show_ipdu(cy_shm->pmc_list[i]);
		}
	}

	return 0;
}
