#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <termios.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <ctype.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>

unsigned char normal_led[] = {2, 0};
unsigned char fw_error_led[] = {10, 0};
unsigned char config_error_led[] = {2, 10, 0};
unsigned char eth_error_led[] = {2, 2, 10, 0};
unsigned char link_error_led[] = {2, 2, 2, 10, 0};
unsigned char boot_error_led[] = {2, 2, 2, 2, 10, 0};
unsigned char rtc_error_led[] = {2, 2, 2, 2, 2,10, 0};

unsigned char   *code_seq;

unsigned char   led_on = 0;
unsigned char   actual_code = 0;
unsigned char   ledct1 = 0;
unsigned char   ledct2 = 0;

#define CONFIG_FLASH_ERROR              0x01
#define NO_DAUGHTER_BOARD_ERROR         0x02
#define ETHERNET_ERROR                  0x04
#define NET_BOOT_ERROR                  0x08
#define FW_FLASH_ERROR                  0x10
#define DRAM_ERROR                      0x20
#define RTC_ERROR                       0x40

/*-----------------------------------------------------------------------------
 *
 * set_led_sequence
 *
 * Procedure responsible for setting the appropriate led sequence
 * acccording to the value set in the variables ffail
 ------------------------------------------------------------------------------
*/
void set_led_sequence(unsigned char ffail)
{

        if (ffail == 0) {
                code_seq = normal_led;
                return;
        }

        if ((ffail & CONFIG_FLASH_ERROR) == CONFIG_FLASH_ERROR) {
                code_seq = config_error_led;
                return;
        }
        if ((ffail & FW_FLASH_ERROR) == FW_FLASH_ERROR) {
                code_seq = fw_error_led;
                return;
        }
        if ((ffail & NO_DAUGHTER_BOARD_ERROR) == NO_DAUGHTER_BOARD_ERROR) {
                code_seq = link_error_led;
                return;
        }
        if ((ffail & ETHERNET_ERROR) == ETHERNET_ERROR) {
                code_seq = eth_error_led;
                return;
        }
        if ((ffail & NET_BOOT_ERROR) == NET_BOOT_ERROR) {
                code_seq = boot_error_led;
                return;
        }
        if ((ffail & RTC_ERROR) == RTC_ERROR) {
                code_seq = rtc_error_led;
                return;
        }
}

unsigned char get_next_ledval(void)
{
        if (ledct1 == ledct2++) {
                /* the LED state (ON,OFF) must be changed */
                ledct2 = 0;
                if (led_on) {
                        /* LED should be turned off */
                        ledct1 = 2; /* off for 1 second */
                        led_on = 0;
                }
                else {
                        /* Let's get next code for the led */
                        if (code_seq[actual_code] == 0) {
                                actual_code = 0;
                        }
                        ledct1 = code_seq[actual_code];
                        led_on = 1;
                        actual_code++;
                }
        }
	return(led_on);
}

static int got_sig_term = 0;

void unblock(int sig)
{
	sigset_t set;

	sigemptyset(&set);
	sigaddset(&set, sig);
	sigprocmask(SIG_UNBLOCK, &set, NULL);
}

static void term_handler(int sig)
{
	got_sig_term = 1;
	signal(sig, SIG_IGN);
}
/*
 *	Main loop for WDT and CPU led.
 */
int main(int argc, char **argv)
{
	int fl, fw, count=0;
	char fwn[32], fln[32], ch[4] = {0};

	/* Disconnect from the controlling tty. */ 
    if ((fl = open("/dev/tty", O_RDWR)) >= 0) {
        signal(SIGHUP, SIG_IGN);
        ioctl(fl, TIOCNOTTY, (char *)1);
        signal(SIGHUP, SIG_DFL);
        close(fl);
    }

	/*
	 *    Check the syntax.
	 */
	if ((argc != 3) ||
	    (argv[1][0] != 'w' && argv[1][0] != 'l') ||
	    (argv[2][0] != 'w' && argv[2][0] != 'l')) {
		fprintf(stderr,	"Syntax : %s wdt-device led-device\n", argv[0]);
		exit(-1);
	}

	strcpy(fwn, "/dev/");
	strcpy(fln, "/dev/");

	if (argv[1][0] == 'w') {
		strcat(fwn, argv[1]);
		strcat(fln, argv[2]);
	} else {
		strcat(fwn, argv[2]);
		strcat(fln, argv[1]);
	}

	if ((fw = open(fwn, O_RDWR|O_NONBLOCK)) < 0) {
		fprintf(stderr,	"%s : ERROR can't open %s\n", argv[0], fwn);
			return -1;
        }

	if ((fl = open(fln, O_RDWR|O_NONBLOCK)) < 0) {
		fprintf(stderr,	"%s : ERROR can't open %s\n", argv[0], fln);
			return -1;
        }

	/* Redirect stdin, stdout, and stderr to /dev/null. */
	freopen("/dev/null", "r", stdin);
	freopen("/dev/null", "w", stdout);
	freopen("/dev/null", "w", stderr);

	signal(SIGTERM, term_handler);
	unblock(SIGTERM);

	read(fl, &ch[2], 1);
	set_led_sequence(ch[2]);

	while (1) {
		/* service wdt each 1 second */
		if (got_sig_term == 1) {
			close(fw);
			got_sig_term = 2;
		} else {
			if (!got_sig_term && count++ & 1) {
				write(fw, ch, 1);
		}	
		}

		/* blink cpu led each half second */
		if ((ch[0] = get_next_ledval()) != ch[1]) {
			write(fl, ch, 1);
			ch[1] = ch[0];
		}
		/* sleep half second */
		usleep(500000);
	}

	close(fw);
	close(fl);

	return(0);
}
