/*
 * lib.c         Generic functions.
 *
 * Version:      @(#)lib.c  1.36  13-Jan-1998  miquels@cistron.nl
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <errno.h>
#include <stdarg.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <values.h>
#include "server.h"

/*
 *	Change argv[0] to reflect what we're doing.
 */

static char **pargv = NULL;

void
title(int all, char **argv, char *fmt, ...)
{
	va_list ap;
    int len = 0, i;
	char *s;

	va_start(ap, fmt);

	if (argv) pargv = argv;

	if (!pargv) goto exit_rot;

	switch (all) {
	case 0:
	case 2:
		for(len=0, i=0; pargv[i]; i++) {
			if (i == 0 || all == 2)
				len += strlen(pargv[i]) + 1;		// should count '\0'
			for(s = pargv[i]; *s; s++)
				*s = ' ';
			pargv[i][0] = 0;
		}
		break;
	case 1:
		len = strlen(pargv[0]) + 1;		// should count '\0'
		break;
	}

	vsnprintf(pargv[0], len, fmt, ap);
exit_rot:
	va_end(ap);
}

#if 0
/*
 * Malloc and die if failed.
 */
void *xmalloc(int size)
{
	void *p;

	if ((p = malloc(size)) == NULL) {
		nsyslog(LOG_ERR, "Virtual memory exhausted.\n");
		exit(1);
	}
	memset(p, 0, size);
	return p;
}
#endif

/*
 * Signal functions.
 */
void block(int sig)
{
	sigset_t set;

	sigemptyset(&set);
	sigaddset(&set, sig);
	sigprocmask(SIG_BLOCK, &set, NULL);
}
void unblock(int sig)
{
	sigset_t set;

	sigemptyset(&set);
	sigaddset(&set, sig);
	sigprocmask(SIG_UNBLOCK, &set, NULL);
}

/*
 *	Print number in static buffer (itoa).
 */
//char *num(int i)
//{
//	static char buf[16];
//
//	snprintf(buf, sizeof (buf), "%d", i);
//	return buf;
//}

/*
 *	Print dotted IP address.
 */
char *dotted(unsigned int a)
{
		struct in_addr ia;

		ia.s_addr = a;
		return inet_ntoa(ia);
}

/*
 * Strips login name and writes it to buffer.
 */

static void strip_login (char *buffer, int buf_size, char *login) {
	char *ptr;

	if (strchr("PCS!L", login [0])) {
		snprintf (buffer, buf_size, "%s", login+1);
	} else {
		snprintf (buffer, buf_size, "%s", login);
		ptr = strrchr (buffer, '.');
		if (ptr) {
			if ((strcmp(ptr, ".slip")  == 0) ||
				(strcmp(ptr, ".cslip") == 0) ||
				(strcmp(ptr, ".ppp")   == 0)) {
				*ptr = 0;
			}
		}
	}
}

/*
 *
 * Expands a format string
 *
 * %l: login name
 * %L: stripped login name
 * %p: NAS port number
 * %P: protocol
 * %b: port speed
 * %i: local IP
 * %j: remote IP
 * %J: remote Port
 * %1: first byte (MSB) of remote IP
 * %2: second byte of remote IP
 * %3: third byte of remote IP
 * %4: fourth (LSB) byte of remote IP
 * %c: connect-info
 * %m: netmask
 * %t: MTU
 * %r: MRU
 * %I: idle timeout
 * %T: session timeout
 * %h: hostname
 * %%: %
 *
 * Everything else is copied unchanged.
 *
 */

void expand_format(char *buffer, size_t size, char *format, struct auth *ai) {
	char ch;
	int done;
	char str [256];
	unsigned long int byte_val;

	if (size-- == 0) {
		return;
	}
	while (size) {
		if ((ch = *format++) == '%') {
			switch (*format) {
				case 'l':
					snprintf (str, sizeof(str), "%s", ai->login);
					break;
				case 'L':
					strip_login (str, sizeof (str), ai->login);
					break;
				case 'p':
#ifndef CYCLADES
					snprintf (str, sizeof(str), "%03d", ai->nasport);
#else
					snprintf (str, sizeof(str), "%03d", ai->nasport+1);
#endif
					break;
				case 'P':
					snprintf (str, sizeof(str), "%c", (char) ai->proto);
					break;
				case 'b':
					snprintf (str, sizeof(str), "%d", lineconf[ai->nasport].speed);
					break;
				case 'H':
					if (ai->host) {
						snprintf (str, sizeof(str), "%s", dotted(ai->host));
						break;
					}
				case 'i':
					snprintf (str, sizeof(str), "%s", dotted(mainconf.ipno));
					break;
				case 'J':
					snprintf (str, sizeof(str), "%d", ai->loginport);
					break;
				case 'j':
					snprintf (str, sizeof(str), "%s", dotted(ai->address));
					break;
				case '1':
					byte_val = (ntohl (ai->address) >> (3*BITSPERBYTE)) & 0xFF;
					snprintf (str, sizeof(str), "%ld", byte_val);
					break;
				case '2':
					byte_val = (ntohl (ai->address) >> (2*BITSPERBYTE)) & 0xFF;
					snprintf (str, sizeof(str), "%ld", byte_val);
					break;
				case '3':
					byte_val = (ntohl (ai->address) >> BITSPERBYTE) & 0xFF;
					snprintf (str, sizeof(str), "%ld", byte_val);
					break;
				case '4':
					byte_val = ntohl (ai->address) & 0xFF;
					snprintf (str, sizeof(str), "%ld", byte_val);
					break;
				case 'c':
					snprintf (str, sizeof(str), "%s", ai->conn_info);
					break;
				case 'm':
					snprintf (str, sizeof(str), "%s", dotted(ai->netmask));
					break;
				case 't':
					snprintf (str, sizeof(str), "%d", ai->mtu);
					break;
				case 'r':
					snprintf (str, sizeof(str), "%d", ai->mru);
					break;
				case 'I':
					snprintf (str, sizeof(str), "%d", ai->idletime);
					break;
				case 'T':
					snprintf (str, sizeof(str), "%d", ai->sessiontime);
					break;
				case 'h':
					snprintf (str, sizeof(str), "%s", mainconf.hostname);
					break;
				case 'g':
					snprintf (str, sizeof(str), "%d", getpid());
					break;
				case '%':
					snprintf (str, sizeof(str), "%c", '%');
					break;
				default:
					*buffer++ = '?';
					--size;
					continue;
			}
			++format;
			if ((done = snprintf (buffer, size+1, "%s", str)) == -1) {
				return;
			}
			buffer += done;
			size -= done;
		} else {
			if ((*buffer = ch) == '\0') {
				return;
			}
			++buffer;
			--size;
		}
	}
	*buffer = '\0';
}

void set_remote_address(int s, struct auth *ai)
{
	struct sockaddr_in sn;
	size_t size = sizeof(sn);

	if (getpeername(s, (struct sockaddr *)&sn, &size)) {
		ai->loginport = 0;
		ai->address = 0;
		return;
	}

	ai->address = sn.sin_addr.s_addr;
	ai->loginport = sn.sin_port;
}

