/*
 *  Module:  modbus.h (Modbus/TCP header file)
 *
 *  Author:  Marcelo Peccin <marcelo.peccin@cyclades.com>
 *
 *  Copyright:  (c) 2002 Cyclades Corp.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version
 *  2 of the License, or (at your option) any later version.
 *
 *  Date:  June 13, 2002
 *
 *  Comments:  
 *
 */

#ifndef MODBUS_H
#define MODBUS_H


/**************************************************/
/**********     Constant Definitions     **********/
/**************************************************/
#undef  FALSE
#define FALSE      0
#undef  TRUE
#define TRUE    (!FALSE)
#undef  NULL
#define NULL   ((void*)0)

#define ERROR     (-1)
#define ERRORMIN  (-2)
#define ERRORMAX  (-3)
#define ERRHEADER (-4)
#define ERRORFCS  (-5)
#define ERRCLOSE  (-6)
#define OK         0
#define PROBLEM    1

#define INHSOCK      0
#define MODBUS_PORT  502
#define MODSERVICE   0
#define HOFFSET      6
#define LENGTHDISP   4
#define MAXTCPFRAME  270
#define MAXSERFRAME  448
#define TCPMIN       8
#define ASCIIMIN     6
#define RTUMIN       4
#define SERIALMIN    RTUMIN

#define ASCII  0
#define RTU    1

#define PARITY_NONE  1
#define PARITY_ODD   2
#define PARITY_EVEN  3

#define FLOW_NONE  0
#define FLOW_HARD  1
#define FLOW_SOFT  2


/**********************************************/
/**********     Type Definitions     **********/
/**********************************************/
typedef unsigned char uchar;

typedef enum
{
	EMPTY,
	WAIT_1,
	WAIT_2,
	TEST
}
sbuffstatus;

typedef struct
{
	int  speed;
	int  datasize;
	int  parity;
	int  stopbits;
	int  flowctrl;
	int  media;
}
sconf;

typedef struct
{
	sconf  serial;
	int    mode;
	int    idletimeout;
//	int    port;
}
modconf;

typedef struct
{
	ushort  inv_id;
	ushort  proto_id;
	ushort  length;
	uchar   unit_id;
}
MBAPheader;

typedef struct
{
	uchar  func_code;
	uchar  data[256];
}
modbusPDU;

typedef struct
{
	MBAPheader  header;
	modbusPDU   pdu;
}
tcpmsg;


/*************************************************/
/**********     Function Prototypes     **********/
/*************************************************/
void prepare_signals(void);
void resend(void);
void mpprint(const char *fmt, ...);
void mpfprint(const char *fmt, ...);
int  tcp2serial(uchar *tcpb, uchar *serialb, int len, int mode);
int  serial2tcp(const uchar *serialb, uchar *tcpb, int len, int mode);
int  check_CRC(const uchar *buff, int *len);
void build_CRC(uchar *buff, int *len);
int  check_LRC(uchar *buff, int *len, const uchar *lrcin);
void build_LRC(uchar *buff, int *len, uchar *res);
int  config_port(int fd, sconf *pconf);
int  set_frame_mode(int fd, int status, int mode);

/***********************************************/
/**********     Macro Definitions     **********/
/***********************************************/
#define ASCIIwrite(_a_,_b_)                  \
{                                            \
ushort tmp = (_b_);                          \
tmp >>= 4;                                   \
(tmp < 0xA) ? (tmp += 0x30) : (tmp += 0x37); \
tmp *= 0x100;                                \
(_a_) = tmp;                                 \
tmp = (_b_) & 0x0F;                          \
(tmp < 0xA) ? (tmp += 0x30) : (tmp += 0x37); \
(_a_) += tmp;                                \
(_a_) = htons(_a_);                          \
}


#define ASCIIread(_a_,_b_)                    \
{                                             \
ushort tmp = ntohs(_a_);                      \
tmp >>= 8;                                    \
(tmp < 0x41) ? (tmp -= 0x30) : (tmp -= 0x37); \
tmp *= 0x10;                                  \
(_b_) = tmp;                                  \
tmp = ntohs(_a_) & 0x00FF;                    \
(tmp < 0x41) ? (tmp -= 0x30) : (tmp -= 0x37); \
(_b_) += tmp;                                 \
}


#define SWAPcrc(_a_) \
((((_a_) & 0xFF00) >> 8) + (((_a_) & 0x00FF) << 8))



#endif /* MODBUS_H */

/* End of modbus.h */
