/*
 *  Module:  puconnect.h (TCP remote connection for digital inputs - header file)
 *
 *  Authors:  Marcelo Peccin <marcelo.peccin@cyclades.com>
 *            Sulamita Garcia <sulamita.garcia@cyclades.com>
 *
 *  Copyright:  (c) 2004 Cyclades Corp.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version
 *  2 of the License, or (at your option) any later version.
 *
 *  Date:  October 7, 2004
 *
 *  Comments:  The first release of this module is intended to the New York
 *             Fire Department. They need to communicate over the network
 *             when the monitored devices close a "dry contact".
 *             This will be achieved by raising the DTR signal on a remote
 *             RS232 serial interface (a remote console server).
 *
 *  Author:        Marcelo Peccin <marcelo.peccin@cyclades.com>
 *  Date:          December 28, 2004
 *  Enhancements:  Sockets now have the "keep alive" option to detect socket
 *                 related exceptions. To quickly detect network related
 *                 problems, a "peer monitoring" mechanism was implemented,
 *                 as well as a "recovery mechanism" to detect when connectivity
 *                 is reestablished. Also, both mechanisms described above are
 *                 optimized to allow resynchronization and to avoid duplication
 *                 when the destination server is the same for many inputs.
 */

#ifndef PUCONNECT_H
#define PUCONNECT_H


/**************************************************/
/**********     Constant Definitions     **********/
/**************************************************/
#undef  FALSE
#define FALSE      0
#undef  TRUE
#define TRUE    (!FALSE)
#undef  NULL
#define NULL   ((void*)0)

#define ERROR     (-1)
#define ERRCLOSE  (-2)  
#define OK          0
#define PROBLEM     1

#define PUCONN_OFF       0
#define PUCONN_ON        1
#define GOING_ON         2
#define CONNECTING       3
#define NET_DOWN         4
#define WAIT_RECOVER     5
#define GOING_OFF        6

#define NO_SOCK        (-1)

#define FIRSTDESTPORT  7001

#define PUCONFFILE     "/etc/puconnect.conf"


/**********************************************/
/**********     Type Definitions     **********/
/**********************************************/
typedef unsigned char  uchar;

typedef struct
{
	struct in_addr destip[9]; //position 0 is not used
	int    destport[9];       //position 0 is not used
	int    dref[9];           //position 0 is not used
}
puconf;


/*************************************************/
/**********     Function Prototypes     **********/
/*************************************************/
static void  mpfprint(const char *fmt, ...);


/***********************************************/
/**********     Macro Definitions     **********/
/***********************************************/
#undef Max
#define Max(x,y)  (((x) >= (y)) ? (x) : (y))


#endif /* PUCONNECT_H */

/* End of puconnect.h */
