#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <termios.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <ctype.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <linux/cy_sm.h>

static int got_sig_term = 0;
u_char bmc_msg_counter = 0x10;

void unblock(int sig)
{
	sigset_t set;

	sigemptyset(&set);
	sigaddset(&set, sig);
	sigprocmask(SIG_UNBLOCK, &set, NULL);
}

static int compareBMC_response(BMC_Rq *bmc_rq, BMC_Rs *bmc_rs)
{
	return ((bmc_rq->rq.rqSA == bmc_rs->rqSA) &&
			(bmc_rq->rq.cmd == bmc_rs->rs.cmd) &&
			(RESPONSE_NETFN(NETFN_OF(bmc_rq->netFn)) == NETFN_OF(bmc_rs->netFn)) &&
			(SEQ_OF(bmc_rq->rq.rqSeq) == SEQ_OF(bmc_rs->rs.rqSeq)));

}

static int ipmi_power_control(int fw, u_char action)
{
	char bmc_buf[128], ch[128];
	BMC_Rq *bmc_rq = (BMC_Rq *) &bmc_buf[0];
	char i, timeout = 2;
	int n_bytes;

	memset(&bmc_buf[0], 0, sizeof(bmc_buf));
	bmc_rq->rsSA = BMC_SA;
	bmc_rq->netFn = NETFN_LUN(APP_NETFN, BMC_LUN);
	bmc_rq->rq.rqSA = CY_SLAVE_ADDR;
	bmc_rq->rq.rqSeq = SEQ_LUN(0, 0);
	bmc_rq->rq.cmd = SET_WATCHDOG_TIMER;
	bmc_rq->rq.data[0] = WD_DONT_LOG | WD_SMS;
	bmc_rq->rq.data[1] = WD_PRE_ACT_NONE | action;
	/* following fields are already set to 0 by memset() */
	//bmc_rq->chk1 = 0;
	//bmc_rq->rq.data[2] = 0;
	//bmc_rq->rq.data[3] = 0;
	//bmc_rq->rq.data[4] = 0;
	//bmc_rq->rq.data[5] = 0;
	n_bytes = write(fw, &bmc_buf[0], 13);
	while ((n_bytes > 0) && timeout--) {
		n_bytes = read(fw, &ch[0], 8);
		if ((n_bytes > 0) && compareBMC_response(bmc_rq, (BMC_Rs *)&ch[0])) {
			break;
		}
		printf("Sent to /dev/cy_ipmi: ");
		for (i=0; i<13; i++)
			printf("%02x ", (u_char)bmc_buf[i]);
		printf("\r\n");
		printf("Got response from /dev/cy_ipmi: ");
		for (i=0; i<8; i++)
			printf("%02x ", (u_char)ch[i]);
		printf("\r\n");
		printf("Waiting reply on /dev/cy_ipmi. Retry countdonw: %d\r\n", timeout);
	}
	
	memset(&bmc_buf[0], 0, sizeof(bmc_buf));
	bmc_rq->rsSA = BMC_SA;
	bmc_rq->netFn = NETFN_LUN(APP_NETFN, BMC_LUN);
	/* following fields is already set to 0 by memset() */
	//bmc_rq->chk1 = 0;
	bmc_rq->rq.rqSA = CY_SLAVE_ADDR;
	bmc_rq->rq.rqSeq = SEQ_LUN(0, 0);
	bmc_rq->rq.cmd = RESET_WATCHDOG_TIMER;
	bmc_rq->rq.data[0] = 0;
	n_bytes = write(fw, &bmc_buf[0], 8);
	timeout = 2;
	while ((n_bytes > 0) && timeout--) {
		n_bytes = read(fw, &ch[0], 8);
		if ((n_bytes > 0) && compareBMC_response(bmc_rq, (BMC_Rs *)&ch[0])) {
			break;
		}
		printf("Sent to /dev/cy_ipmi: ");
		for (i=0; i<8; i++)
			printf("%02x ", (u_char)bmc_buf[i]);
		printf("\r\n");
		printf("Got response from /dev/cy_ipmi: ");
		for (i=0; i<8; i++)
			printf("%02x ", (u_char)ch[i]);
		printf("\r\n");
		printf("Waiting reply on /dev/cy_ipmi. Retry countdonw: %d\r\n", timeout);
	}
}

static int ipmi_chassis_control(int fw, u_char action)
{
	char bmc_buf[128], ch[128];
	BMC_Rq *bmc_rq = (BMC_Rq *) &bmc_buf[0];
	char i, timeout = 2;
	int n_bytes;

	memset(&bmc_buf[0], 0, sizeof(bmc_buf));
	bmc_rq->rsSA = BMC_SA;
	bmc_rq->netFn = NETFN_LUN(CHASSIS_NETFN, BMC_LUN);
	bmc_rq->rq.rqSA = CY_SLAVE_ADDR;
	bmc_rq->rq.rqSeq = SEQ_LUN(0, 0);
	bmc_rq->rq.cmd = CHASSIS_CONTROL;
	bmc_rq->rq.data[0] = action;
	/* following fields is already set to 0 by memset() */
	//bmc_rq->chk1 = 0;
	n_bytes = write(fw, &bmc_buf[0], 8);
	while ((n_bytes > 0) && timeout--) {
		n_bytes = read(fw, &ch[0], BMC_MAX_MSG_SIZE);
		if ((n_bytes > 0) && compareBMC_response(bmc_rq, (BMC_Rs *)&ch[0])) {
			break;
		}
		printf("Sent to /dev/cy_ipmi: ");
		for (i=0; i<13; i++)
			printf("%02x ", (u_char)bmc_buf[i]);
		printf("\r\n");
		printf("Got response from /dev/cy_ipmi: ");
		for (i=0; i<8; i++)
			printf("%02x ", (u_char)ch[i]);
		printf("\r\n");
		printf("Waiting reply on /dev/cy_ipmi. Retry countdonw: %d\r\n", timeout);
	}
}

static void term_handler(int sig)
{
	got_sig_term = 1;
	signal(sig, SIG_IGN);
}

int main(int argc, char **argv)
{
	int fl, fw, n_bytes;
	char fan[32], fin[32], ftn[32], ch[128] = {0}, aux[4], pw_mode, i;

	/* Disconnect from the controlling tty. */ 
    if ((fl = open("/dev/tty", O_RDWR)) >= 0) {
        signal(SIGHUP, SIG_IGN);
        ioctl(fl, TIOCNOTTY, (char *)1);
        signal(SIGHUP, SIG_DFL);
        close(fl);
    }

	/*
	 *    Check the syntax.
	 */
	if (argc < 2) {
		printf(	"Syntax error. Use '%s h' for help\r\n", argv[0]);
		exit(-1);
	}

	strcpy(fan, "/dev/sm_atx");
	strcpy(fin, "/dev/sm_ipmi");
	strcpy(ftn, "/dev/sm_sensor");

	switch (argv[1][0]) {
		case 'a':	/* atx */
			pw_mode = CMD_SM_MODE_ATX;
			if ((fw = open(fan, O_RDWR|O_NONBLOCK)) < 0) {
				printf(	"%s : ERROR can't open %s\n", argv[0], fan);
				return -1;
			}
			break;
		case 'i':	/* ipmi */
			pw_mode = CMD_SM_MODE_IPMI;
			if ((fw = open(fin, O_RDWR|O_NONBLOCK)) < 0) {
				printf(	"%s : ERROR can't open %s\n", argv[0], fin);
				return -1;
	        }
			break;
		case 'h':	/* help */
		    printf(	"Usage: %s <a|i|s|h> [options]\n", argv[0]);
		    printf(	"   or: %s <atx|ipmi|sensor|help> [options]\n\r", argv[0]);
		    printf(	"   Server Management options:\n");
//			hidden commands
//			printf(	"     atx mbe                     enables motherboard ATX power control\n\r");
//			printf(	"     atx mbd                     disables motherboard ATX power control\n\r");
//			printf(	"     atx sme                     enables SM100 ATX power control\n\r");
//			printf(	"     atx smd                     disables SM100 ATX power control\n\r");
//			printf(	"     atx active                  sets SM100 ATX power switch on\n\r");
//			printf(	"     atx inactive                sets SM100 ATX power switch off\n\r");
			printf(	"     atx on                      turns ATX power on\n\r");
			printf(	"     atx off                     turns ATX power off\n\r");
			printf(	"     atx cycle                   powers cycle ATX power\n\r");
			printf(	"     atx value                   returns ATX /dev/sm_atx hexadecimal value\n\r");
			printf(	"     atx status                  returns the ATX status\n\r");
			printf(	"     ipmi on                     turns power on\n\r");
			printf(	"     ipmi off                    turns power off\n\r");
			printf(	"     ipmi cycle                  powers cycle\n\r");
			printf(	"     ipmi reset                  hardware reset\n\r");
//			printf(	"     ipmi read SLAVE REG SIZE    returns the slave's register value\n\r");
			printf(	"     ipmi write MESSAGE          sends IPMB message and gets its response\n\r");
			printf(	"     sensor read REG             returns sensor register decimal value\n\r");
			printf(	"     sensor write REG VALUE      writes value on sensor register\n\r");
			printf(	"     sensor status               returns the temperature/voltage status\n\r");
			printf(	"     help                        for command help\n\r");
			printf(	"\n\r");
			return(0);
		case 's':	/* sensor */
			pw_mode = CMD_SM_MODE_SENSOR;
			if ((fw = open(ftn, O_RDWR|O_NONBLOCK)) < 0) {
				printf(	"%s : ERROR can't open %s\r\n", argv[0], ftn);
				return -1;
	        }
			break;
		default:
			printf(	"Syntax error. Use '%s h' for help\r\n", argv[0]);
			exit(-1);
	}

	/* Redirect stdin, stdout, and stderr to /dev/null. */
//	freopen("/dev/null", "r", stdin);
//	freopen("/dev/null", "w", stdout);

//	signal(SIGTERM, term_handler);
//	unblock(SIGTERM);

	if (argc < 3) {
		printf(	"Syntax error. Use '%s h' for help\r\n", argv[0]);
		close(fw);
		exit(-1);
	}

	switch (pw_mode) {
		case CMD_SM_MODE_ATX:
			if (argv[2][0] == 'a') {
				/* SM100 power switch active - just turn on power */
				read(fw, &ch[0], 1);
				if (*ch & POWER_MODE_STATUS) {
					printf("Power in IPMI mode\r\n");
				} 
				else {
					if ((~ch[0] & SM100_POWER_ENABLE) && 
						(~ch[0] & MB_POWER_ENABLE) &&
						(ch[0] & POWER_ON_MB)) {
						printf("Conflicting values. Command not accepted!\r\n");
					}
					else {
						ch[1] = ch[0] & ~SM100_POWER_ON;	/* set value */
						ch[0] = SM100_POWER_ON;				/* set mask */
						write(fw, &ch[0], 2);
					}
				}
				break;
			}
			if (argv[2][0] == 'c') {
				/* SM100 power cycle */
				read(fw, &ch[0], 1);
				if (*ch & POWER_MODE_STATUS) {
					printf("Power in IPMI mode\r\n");
				} 
				else {
					ch[1] = ch[0] & ~SM100_POWER_ENABLE;			/* enable SM100 */
					ch[1] |= MB_POWER_ENABLE|SM100_POWER_ON;		/* set value and disable MB */
					ch[0] = SM100_POWER_ON|SM100_POWER_ENABLE|MB_POWER_ENABLE;	/* set mask */
					write(fw, &ch[0], 2);
					/* sleep 3 seconds */
					usleep(3000000);
					ch[1] = ch[0] & ~(SM100_POWER_ON|SM100_POWER_ENABLE);	/* set value and enable SM100 */
					ch[1] |= MB_POWER_ENABLE;								/* disable MB */
					ch[0] = SM100_POWER_ON|SM100_POWER_ENABLE|MB_POWER_ENABLE;	/* set mask */
					write(fw, &ch[0], 2);
				}
				break;
			}
			if (argv[2][0] == 'i') {
				/* SM100 power switch inactive - just turn off power */
				read(fw, &ch[0], 1);
				if (*ch & POWER_MODE_STATUS) {
					printf("Power in IPMI mode\r\n");
				} 
				else {
					if ((~ch[0] & SM100_POWER_ENABLE) && 
						(~ch[0] & MB_POWER_ENABLE) &&
						(~ch[0] & POWER_ON_MB)) {
						printf("Conflicting values. Command not accepted!\r\n");
					}
					else {
						ch[1] = ch[0] | SM100_POWER_ON;		/* set value */
						ch[0] = SM100_POWER_ON;				/* set mask */
						write(fw, &ch[0], 2);
					}
				}
				break;
			}
			if ((argv[2][0] == 'm') && (argv[2][2] == 'd')) {
				/* Motherboard power control disable */
				read(fw, &ch[0], 1);
				if (*ch & POWER_MODE_STATUS) {
					printf("Power in IPMI mode\r\n");
				} 
				else {
					ch[1] = ch[0] | MB_POWER_ENABLE;		/* set value */
					ch[0] = MB_POWER_ENABLE;				/* set mask */
					write(fw, &ch[0], 2);
				}
				break;
			}
			if ((argv[2][0] == 'm') && (argv[2][2] == 'e')) {
				/* Motherboard power control enable */
				read(fw, &ch[0], 1);
				if (*ch & POWER_MODE_STATUS) {
					printf("Power in IPMI mode\r\n");
				} 
				else {
					if ((~ch[0] & SM100_POWER_ENABLE) && 
						((ch[0] & SM100_MB_POWER) != SM100_MB_POWER) && 
						((~ch[0] & SM100_MB_POWER) != SM100_MB_POWER)) {
						printf("Conflicting values. Command not accepted!\r\n");
					}
					else {
						ch[1] = ch[0] & ~MB_POWER_ENABLE;		/* set value */
						ch[0] = MB_POWER_ENABLE;				/* set mask */
						write(fw, &ch[0], 2);
					}
				}
				break;
			}
			if ((argv[2][0] == 'o') && (argv[2][1] == 'f')) {
				/* SM100 power switch off */
				read(fw, &ch[0], 1);
				if (*ch & POWER_MODE_STATUS) {
					printf("Power in IPMI mode\r\n");
				} 
				else {
					ch[1] = ch[0] & ~SM100_POWER_ENABLE;			/* enable SM100 */
					ch[1] |= MB_POWER_ENABLE|SM100_POWER_ON;		/* set value and disable MB */
					ch[0] = SM100_POWER_ON|SM100_POWER_ENABLE|MB_POWER_ENABLE;	/* set mask */
					write(fw, &ch[0], 2);
				}
				break;
			}
			if ((argv[2][0] == 'o') && (argv[2][1] == 'n')) {
				/* SM100 power switch on */
				read(fw, &ch[0], 1);
				if (*ch & POWER_MODE_STATUS) {
					printf("Power in IPMI mode\r\n");
				} 
				else {
					ch[1] = ch[0] & ~(SM100_POWER_ON|SM100_POWER_ENABLE);	/* set value and enable SM100 */
					ch[1] |= MB_POWER_ENABLE;								/* disable MB */
					ch[0] = SM100_POWER_ON|SM100_POWER_ENABLE|MB_POWER_ENABLE;	/* set mask */
					write(fw, &ch[0], 2);
				}
				break;
			}
			if ((argv[2][0] == 's') && (argv[2][2] == 'd')) {
				/* SM100 power control disable */
				read(fw, &ch[0], 1);
				if (*ch & POWER_MODE_STATUS) {
					printf("Power in IPMI mode\r\n");
				} 
				else {
					ch[1] = ch[0] | SM100_POWER_ENABLE;		/* set value */
					ch[0] = SM100_POWER_ENABLE;				/* set mask */
					write(fw, &ch[0], 2);
				}
				break;
			}
			if ((argv[2][0] == 's') && (argv[2][2] == 'e')) {
				/* SM100 power control enable */
				read(fw, &ch[0], 1);
				if (*ch & POWER_MODE_STATUS) {
					printf("Power in IPMI mode\r\n");
				} 
				else {
					if ((~ch[0] & MB_POWER_ENABLE) && 
						((ch[0] & SM100_MB_POWER) != SM100_MB_POWER) && 
						((~ch[0] & SM100_MB_POWER) != SM100_MB_POWER)) {
						printf("Conflicting values. Command not accepted!\r\n");
					}
					else {
						ch[1] = ch[0] & ~SM100_POWER_ENABLE;		/* set value */
						ch[0] = SM100_POWER_ENABLE;					/* set mask */
						write(fw, &ch[0], 2);
					}
				}
				break;
			}
			if (argv[2][0] == 's') {
				/* status */
				read(fw, &ch[0], 1);
				printf("Server Management Status\r\n");
				if (*ch & POWER_MODE_STATUS) {
					printf("  Power mode: IPMI\r\n");
				}
				else {
					printf("  Power mode: ATX\r\n  value: 0x%x\r\n  SM100 power control: ", (*(unsigned char *)ch & POWER_MODE_MASK));
					((*ch & SM100_POWER_ENABLE) ? printf("disabled") : printf("enabled")); 
					printf("\r\n  SM100 power switch: ");
					((*ch & SM100_POWER_ON) ? printf("off") : printf("on")); 
					printf("\r\n  Motherboard power control: ");
					((*ch & MB_POWER_ENABLE) ? printf("disabled") : printf("enabled")); 
					printf("\r\n  Motherboard power switch: ");
					((*ch & POWER_ON_MB) ? printf("off") : printf("on")); 
					printf("\r\n");
				}
				break;
			}
			if (argv[2][0] == 'v') {
				/* value */
				read(fw, &ch[0], 1);
				printf("%x\r\n", (*(unsigned char *)ch & POWER_MODE_MASK));
				break;
			}
			printf(	"Syntax error. Use '%s h' for help\r\n", argv[0]);
			break;		
		case CMD_SM_MODE_IPMI:
			if (argv[2][0] == 'c') {
				/* IPMI power cycle */
				ipmi_power_control(fw, WD_ACT_PWR_CYCLE);
				break;
			}
			if ((argv[2][0] == 'o') && (argv[2][1] == 'f')) {
				/* IPMI power off */
				ipmi_power_control(fw, WD_ACT_PWR_DOWN);
				break;
			}
			if ((argv[2][0] == 'o') && (argv[2][1] == 'n')) {
				/* IPMI power on */
				ipmi_chassis_control(fw, CH_PWR_UP);
				break;
			}
			if ((argv[2][0] == 'r') && (argc > 4)) {
				/* IPMI read registers */
				u_char a = 0, b = 0;
				memset(&ch[0], 0, sizeof(ch));
				while (argv[3][a] != 0) {
					if (argv[3][a] > '9') {
						ch[0] = ch[0] * 0x10 + ((argv[3][a++]|0x20) - 'a' + 0xa); 
					} else
					{
						ch[0] = ch[0] * 0x10 + (argv[3][a++] - '0'); 
					}
				}
				a = 0;
				while (argv[4][a] != 0) {
					if (argv[4][a] > '9') {
						ch[1] = ch[1] * 0x10 + ((argv[4][a++]|0x20) - 'a' + 0xa); 
					} else
					{
						ch[1] = ch[1] * 0x10 + (argv[4][a++] - '0'); 
					}
				}
				if (argc > 5) {
					a = 0;
					while (argv[5][a] != 0)
						b = b * 10 + (argv[5][a++] - '0'); 
				}
				else {
					b = 1;
				}
				read(fw, &ch[0], b);
				printf("Reading /dev/cy_ipmi: ");
				for (i=0; i<b; i++)
					printf("%02x ", (u_char)ch[i]);
				printf("\r\n");
				break;
			}
			if (argv[2][0] == 'r') {
				/* IPMI reset */
				ipmi_power_control(fw, WD_ACT_RESET);
				break;
			}
			if ((argv[2][0] == 'w') && (argc > 3)) {
				/* IPMI write registers */
				u_char a = 0, b = 0, buf_aux[128];
				memset(&ch[0], 0, sizeof(ch));

				while (argv[3][a] != 0) {
					if (argv[3][a] > '9') {
						ch[b] = ch[b] * 0x10 + ((argv[3][a]|0x20) - 'a' + 0xa); 
					} else
					{
						ch[b] = ch[b] * 0x10 + (argv[3][a] - '0'); 
					}
					b += a%2;
					a++;
				}
				n_bytes = write(fw, &ch[0], b);
				if (n_bytes > 0 ) {
					n_bytes = read(fw, &buf_aux[0], 32);
					printf("Sent: ");
					for (i=0; i<b; i++)
						printf("%02x ", (u_char)ch[i]);
					printf("\r\nGot:  ");
					for (i=0; i<n_bytes; i++)
						printf("%02x ", (u_char)buf_aux[i]);
					printf("\r\n");
				}
				break;
			}
			printf(	"Syntax error. Use '%s h' for help\r\n", argv[0]);
			break;
		case CMD_SM_MODE_SENSOR:
			if ((argv[2][0] == 'r') && (argc > 3)) {
				/* Sensor read registers */
				ch[0] = SENSOR_SLAVE_ADDRESS;
				ch[1] = 0;
				i = 0;
				while (argv[3][i] != 0) {
					if ((argv[3][i] >= '0') && (argv[3][i] <= '9')) {
						ch[1] = ch[1] * 0x10 + (argv[3][i++] - '0'); 
					} else
					if ((argv[3][i]|0x020 >= 'a') && (argv[3][i]|0x020 <= 'f')) { 
						ch[1] = ch[1] * 0x10 + ((argv[3][i++]|0x20) - 'a' + 0xa); 
					}
				}
				read(fw, &ch[0], 1);
				//printf("%02x\r\n", (u_char)ch[0]);
				printf("%d\r\n", (u_char)ch[0]);
				break;
			}
			if (argv[2][0] == 's') {
				/* Sensor status */
				memset(&ch[0], 0, sizeof(ch));
				/* sensor devices has a bug. Does not allow block read
				   Let's for now just read few registers
				*/
				aux[0] = SENSOR_SLAVE_ADDRESS;
				aux[1] = SENSOR_RRTE_REG;
				read(fw, &aux[0], 1);
				ch[SENSOR_RRTE_REG] = aux[0];

				aux[0] = SENSOR_SLAVE_ADDRESS;
				aux[1] = SENSOR_RRTE1_REG;
				read(fw, &aux[0], 1);
				ch[SENSOR_RRTE1_REG] = aux[0];

				aux[0] = SENSOR_SLAVE_ADDRESS;
				aux[1] = SENSOR_RV0_REG;
				read(fw, &aux[0], 1);
				ch[SENSOR_RV0_REG] = aux[0];

				aux[0] = SENSOR_SLAVE_ADDRESS;
				aux[1] = SENSOR_RV1_REG;
				read(fw, &aux[0], 1);
				ch[SENSOR_RV1_REG] = aux[0];

				aux[0] = SENSOR_SLAVE_ADDRESS;
				aux[1] = SENSOR_RV2_REG;
				read(fw, &aux[0], 1);
				ch[SENSOR_RV2_REG] = aux[0];

				printf("Server Management Status\r\n");
				printf("  Temperature: %d.%d C\r\n", ch[SENSOR_RRTE_REG], SENSOR_DEC_TEMP(ch[SENSOR_RRTE1_REG]));
				printf("  VCC +3.3: %d.%d V\r\n", SENSOR_VCC_3_3(ch[SENSOR_RV2_REG])/10000, SENSOR_VCC_3_3(ch[SENSOR_RV2_REG])%10000);
				printf("  VCC +5:   %d.%d V\r\n", SENSOR_VCC_5(ch[SENSOR_RV0_REG])/10000, SENSOR_VCC_5(ch[SENSOR_RV0_REG])%10000);
				printf("  VCC +12: %2d.%d V\r\n", SENSOR_VCC_12(ch[SENSOR_RV1_REG])/10000, SENSOR_VCC_12(ch[SENSOR_RV1_REG])%10000);
				//printf("  Status Byte1, Byte2: 0x%02x, 0x%02x\r\n", ch[SENSOR_RSL_REG], ch[SENSOR_RSL2_REG]);
				//printf("  Config Byte1, Byte2: 0x%02x, 0x%02x\r\n", ch[SENSOR_RCL_REG], ch[SENSOR_RCL2_REG]);
				break;
			}
			if ((argv[2][0] == 'w') && (argc > 4)) {
				/* Sensor write registers */
				ch[0] = SENSOR_SLAVE_ADDRESS;
				ch[1] = 0;
				ch[2] = 0;
				i = 0;
				while (argv[3][i] != 0) {
					if ((argv[3][i] >= '0') && (argv[3][i] <= '9')) {
						ch[1] = ch[1] * 0x10 + (argv[3][i++] - '0'); 
					} else
					if ((argv[3][i]|0x020 >= 'a') && (argv[3][i]|0x020 <= 'f')) { 
						ch[1] = ch[1] * 0x10 + ((argv[3][i++]|0x20) - 'a' + 0xa); 
					}
				}
				i = 0;
				while (argv[4][i] != 0) {
					if ((argv[4][i] >= '0') && (argv[4][i] <= '9')) {
						ch[2] = ch[2] * 0x10 + (argv[4][i++] - '0'); 
					} else
					if ((argv[4][i]|0x020 >= 'a') && (argv[4][i]|0x020 <= 'f')) { 
						ch[2] = ch[2] * 0x10 + ((argv[4][i++]|0x20) - 'a' + 0xa); 
					}
				}
				write(fw, &ch[0], 3);
				break;
			}
			printf(	"Syntax error. Use '%s h' for help\r\n", argv[0]);
			break;
		default:
			printf(	"Syntax error. Use '%s h' for help\r\n", argv[0]);
			break;
	}

#if 0
	while (1) {
		/* service wdt each 1 second */
		if (got_sig_term == 1) {
			close(fw);
			got_sig_term = 2;
		} else {
			if (!got_sig_term && count++ & 1) {
				write(fw, ch, 1);
		}	
		}
	}
#endif

	close(fw);
	return(0);
}

