#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/times.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <fcntl.h>
#include <termios.h>
#include <termio.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>

#include "server.h"

extern struct line_cfg		*pconf;
extern struct auth		*pai;

/*
 *	Main socket routine for socket_server port
 */
int main(int argc, char **argv)
{
	char *tty;
	int thisport, i;
	int ret;

	signal(SIGHUP, SIG_DFL);
	signal(SIGTERM, SIG_DFL);
	signal(SIGINT, SIG_DFL);
	signal(SIGQUIT, SIG_DFL);
	unblock(SIGHUP);
	unblock(SIGTERM);
	unblock(SIGINT);
	unblock(SIGQUIT);

	/*
	 *    Check the syntax.
	 */

	if (argc < 2) {
		printf("Socket SSH v%s\tGPL2\n"
			"(c) 1997-1999 Miquel van Smoorenburg, "
			"Dave Cinege, Vesselin Atanasov, et al.\n\n"
			"Usage: %s <tty | port | ipaddr>\n", VERSION, argv[0]);
		sleep(5);
		exit(1);
	}

	tty = argv[1];

	if (!strncmp(tty, "ttyS", 4)) {
		thisport = -6;
		goto sintax_ok;
	}

	for(i=0; tty[i]; i++) {
		if (tty[i] < '0' || tty[i] > '9') {
			goto hostname;
		}
	}

	if ((tty = (char *)atoi(tty)) < MAXVLINES) {
		thisport = (int)tty;
	} else {
		thisport = -3;
	}
	goto sintax_ok;

hostname:

	if ((tty = (char *)inet_addr(tty)) != (char *)-1) {
		thisport = -4;
		goto sintax_ok;
	}

	tty = argv[1];
	thisport = -5;

sintax_ok:

	/*
	 *    Read the config file.
	 */
	if ((thisport = rad_init(thisport, pai, tty, 1)) < 0) {
		if (thisport == -1) {
			printf("%s Radinit could not find the terminal, Param:%s\n", argv[0], argv[1]);
			nsyslog(LOG_ERR, "%s Radinit could not find the terminal, Param:%s\n", argv[0], argv[1]);
		} else {
			printf("%s Radinit could not find a free pool port, Param:%s\n", argv[0], argv[1]);
			nsyslog(LOG_ERR, "%s Radinit could not find a free pool port, Param:%s\n", argv[0], argv[1]);
		}
		fflush(NULL);
		sleep(5);
		exit(1);
	}

	pconf = &lineconf[thisport];
	
	nsyslog(LOG_INFO, "Socket-server started on port %d (%s)", (thisport+1),
		            pconf->tty);

	title(2, argv, "-Sock_srv %s", get_tty_text(pconf));

	ret = do_socket(pai);
	
	//cy_shm->vet_pid[pai->nasport] = 0;	/* unlock */
	
	return(ret);
}

