/*
 *  Module:  setConSpeed.c
 *
 *  Author:  Marcelo Peccin <marcelo.peccin@cyclades.com>
 *
 *  Copyright:  (c) 2005 Cyclades Corp.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version
 *  2 of the License, or (at your option) any later version.
 *
 *  Date:  February 23, 2005
 *
 *  Comments:  This programs creates "/etc/getty_ttyS0" file with the
 *             proper speed, so after boot time our Linux keeps working
 *             with the same "low level programmed" console speed.
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include "../linux/include/asm-ppc/flash.h"
#include "../linux/include/asm-ppc/cyc_config.h"
#include "../ras2000-12-25/src/bootconf.h"

#define FLASH_BLOCK	(64*1024)

char  buffer[FLASH_BLOCK];
int	  speeda[] = {4800, 9600, 19200, 38400, 57600, 115200};

int read_configuration_from_flash()
{
	int	n, fd;

	if ((fd = open("/proc/flash/config", O_RDONLY)) == -1) {
		printf("setConSpeed: Error %d reading flash (config) !!!\r\n", errno);
		return(0);
	}

	n = read(fd, buffer, FLASH_BLOCK);

	close(fd);
	return(n);
}

unsigned short calc_crc(unsigned long byte_n, unsigned short crc, char *buffer)
{
	int j;

	while (byte_n--) {
		crc = (((crc >> 8) ^ *buffer++) << 8) | (0xFF & crc) ;
		for(j = 0; j < 8; j++) {
			if (crc & 0x8000)  {
				crc <<= 1;
				crc ^= 0x1021;
			} 
			else {
				crc <<= 1;
			}
		}
	}
	return(crc);
}

char *access_type(FLASH_CONFIG_VECTOR_HEADER *p, int type, int tries)
{
	while (tries--) {
		if (p->type == 9999) {
			break;
		}
		if (p->type > 9999 || p->size == 0 || p->size > 10000) {
			break;
		}
		if (p->crc != calc_crc(p->size, 0, p->data)) {
			break;
		}
		if (p->type == type) {
			return(p->data);
		}
		p = (FLASH_CONFIG_VECTOR_HEADER *)((char *)p + p->size + 6);
	}
	printf("setConSpeed: Error accessing flash info !!!\r\n", errno);
	return(NULL);
}


int main(int argc, char *argv[])
{
	int config_size;
	FILE *gfile;
	struct HW_SETUP		*hw_setup;
	char sstr[] = "allspeeds";

	if (!(config_size = read_configuration_from_flash()))
		goto error_1;
	if (!(hw_setup = (struct HW_SETUP *)
		  access_type((FLASH_CONFIG_VECTOR_HEADER *)buffer, TYPE0068, 1000)))
		goto error_1;

	sprintf(sstr, "%d\n", speeda[hw_setup->console_speed]);
	if (!(gfile = fopen("/etc/getty_ttyS0", "w")))
		goto error_2;
	if (fwrite(sstr, strlen(sstr), 1, gfile) < 1) {
		fclose(gfile);
		goto error_2;
	}
	printf("setConSpeed: Linux console speed set to %s", sstr);
	fclose(gfile);
	return 0;

error_1:
	sprintf(sstr, "%d\n", 9600);
	if (!(gfile = fopen("/etc/getty_ttyS0", "w")))
		goto error_2;
	if (fwrite(sstr, strlen(sstr), 1, gfile) < 1) {
		fclose(gfile);
		goto error_2;
	}
	printf("setConSpeed: Default console speed file created !!!\r\n", errno);
	fclose(gfile);
	return -1;

error_2:
	printf("setConSpeed: Couldn't create console speed file !!!\r\n", errno);
	return -1;
}


/* End of setConSpeed.c */
