#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <termios.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <ctype.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <../../linux/include/linux/cy_sm.h>

static int got_sig_term = 0;
u_char bmc_msg_counter = 0x10;

void unblock(int sig)
{
	sigset_t set;

	sigemptyset(&set);
	sigaddset(&set, sig);
	sigprocmask(SIG_UNBLOCK, &set, NULL);
}

static void term_handler(int sig)
{
	got_sig_term = 1;
	signal(sig, SIG_IGN);
}

int TestAnalogPortsOnce(int fw)
{
	char ch[8] = {0};

	/* read I2C device via adc port */
	if (fw >= 0)
		read(fw, &ch[0], 2); 		/* 16-bit output reg */

	return((int) ((ch[0]<<8) | ch[1]));
}


int OpenAnalogPort(int *f)
{
	int fl, fw, status = 0;

	/* Disconnect from the controlling tty. */ 
    if ((fl = open("/dev/tty", O_RDWR)) >= 0) {
        signal(SIGHUP, SIG_IGN);
        ioctl(fl, TIOCNOTTY, (char *)1);
        signal(SIGHUP, SIG_DFL);
        close(fl);
    }

	/* open adc1 device */
	if ((f[0] = fw = open("/dev/adc1", O_RDWR|O_NONBLOCK)) < 0) {
		close(fw);
	}
	else {
		status += 1;
	}
	
	/* open adc2 device */
	if ((f[1] = fw = open("/dev/adc2", O_RDWR|O_NONBLOCK)) < 0) {
		close(fw);
	}
	else {
		status += 1;
	}
	return status;
}


void ResetAnalogPort (int *fw)
{
	if (fw[0] >= 0) close(fw[0]);
	if (fw[1] >= 0) close(fw[1]);
}

int TestDigitalPortsOnce(int fw)
{
	char ch[8] = {0};

	/* read gpio port */
	read(fw, &ch[0], 1);

	return(ch[0]);
}


int OpenDigitalPort(int *f)
{
	int fl, fw;

	/* Disconnect from the controlling tty. */ 
    if ((fl = open("/dev/tty", O_RDWR)) >= 0) {
        signal(SIGHUP, SIG_IGN);
        ioctl(fl, TIOCNOTTY, (char *)1);
        signal(SIGHUP, SIG_DFL);
        close(fl);
    }

	/* open gpio device */
	if ((fw = open("/dev/gpio", O_RDWR|O_NONBLOCK)) < 0) {
		close(fw);
		return 0;
	}
	*f = fw;
	return 1;
}


void ResetDigitalPort (int fw)
{
	if (fw >= 0) close(fw);
}


