# include <signal.h>
# include <errno.h>
# include <netdb.h>
# include <stdio.h>
# include <unistd.h>
# include <fcntl.h>
# include <stdlib.h>
# include <string.h>

# include <sys/time.h>
# include <sys/types.h>
# include <sys/stat.h>
# include <sys/socket.h>
# include <sys/ioctl.h>
# include <netinet/in.h>
# include <arpa/inet.h>

#define XDEBUG( P... ) if (debug) fprintf(stdout, ##P )

typedef struct _CONFMSG {
	unsigned int	id;
	unsigned int	op;
	unsigned int	ip;
	unsigned short	port;
	unsigned short	f1;
} CONFMSG;

unsigned short COMMAND_PORT = 0x7cf3;
unsigned short SOCKET_PORT = 7001;

int baudrate = 9600, op = 1;

CONFMSG cmd;

char *			IPserver;

char *			IPlocal;

char *			Pgname;

int			MpSocket = -1;

struct sockaddr_in	Mp_rem, Mp_loc;	

int len_sockaddr = sizeof(struct sockaddr_in);

int get_command_sock();
int free_command_sock();
int recv_command_udp(unsigned int);
int send_command_udp(unsigned int, int op);

void show_usage(void)
{
	fprintf(stdout, "Usage : %s  [-d] [-b baudrate] [-p UDPport] [-s SockPort] IPlocal IPserver\n", Pgname);
	exit(2);
}

static char strConn[] = "Put the loopback connector to start the burnin\r\n\n";

int start_tests(int debug)
{
	int flag = 0;
	fd_set InFdSet;
	char ch[64];
	struct timeval BTimeout;

	fflush(stdout);

	fprintf(stdout, "\n\rts1h_mon Server (%s) succefully contacted !!!\r\n\n",
		IPserver);

	while (1) {
		if (!flag) {
			fprintf (stdout, strConn);
			fflush(stdout);
		}

		FD_ZERO(&InFdSet);
		FD_SET(0,&InFdSet);

		BTimeout.tv_sec = 2;
		BTimeout.tv_usec = 0;

		if (select(1,&InFdSet,NULL,NULL,&BTimeout) == 1) {
			fgets(ch, sizeof(ch), stdin);
			if (!(flag&1) && strstr(ch, "loopback")) {
				flag |= 1;
			} else {
				flag |= 2;
			}
		} else {
			if (flag & 1) {
				break;
			} else {
				return(1);
			}
		}
	}

	freopen("/dev/null", "r", stdin);
	freopen("/dev/null", "w", stdout);
	freopen("/dev/null", "w", stdout);

	system("signal_ras start");

	sleep(3);

	return(2);
}

void set_tty_speed(void)
{
	speed_t Speed;
	struct termios tio;
	int MLines;

	if (op != 1) {
		return;
	}

	freopen("/dev/ttyS1", "r", stdin);
	freopen("/dev/ttyS1", "w", stdout);
	freopen("/dev/ttyS1", "w", stdout);

	tcgetattr(0, &tio);

	switch(baudrate) {
		case 1200:
			Speed = B1200;
			break;
		case 2400:
			Speed = B2400;
			break;
		case 4800:
			Speed = B4800;
			break;
		case 9600:
			Speed = B9600;
			break;
		case 19200:
			Speed = B19200;
			break;
		case 38400:
			Speed = B38400;
			break;
		case 57600:
			Speed = B57600;
			break;
		case 76800:
			Speed = B76800;
			break;
		case 115200:
			Speed = B115200;
			break;
		case 230400:
			Speed = B230400;
			break;
		case 460800:
			Speed = B460800;
			break;
		case 921600:
			Speed = B921600;
			break;
		default:
			Speed = B9600;
			break;
	}

	// Setting speed above 38400 correctly
	cfsetospeed(&tio, Speed);
	cfsetispeed(&tio, Speed);

	tio.c_lflag &= ~ECHO;

	tcsetattr(0, TCSANOW, &tio);

	ioctl(0,TIOCMGET,&MLines);
	MLines = MLines | TIOCM_DTR;
	ioctl(0,TIOCMSET,&MLines);

}

int main(int argc, char **argv)
{
	int fl, ret, debug = 0;
	unsigned int i = 0;
	extern int optind;
	extern char *optarg;

	Pgname = argv[0];

	while ((ret = getopt(argc, argv, "dp:b:s:")) != EOF) {
		switch(ret) {
		case 'd':
			debug = 1;
			break;
		case 's':
			if (!optarg || *optarg =='-') {
				show_usage();
			}
			SOCKET_PORT = atoi(optarg);
			break;
		case 'p':
			if (!optarg || *optarg =='-') {
				show_usage();
			}
			COMMAND_PORT = atoi(optarg);
			break;
		case 'b':
			if (!optarg || *optarg =='-') {
				show_usage();
			}
			baudrate = atoi(optarg);
			break;
		default:
			show_usage();
		}
	}

	if (argc - optind != 2) {
		show_usage();
	}

	signal(SIGHUP, SIG_IGN);
	signal(SIGPIPE, SIG_IGN);

	sleep(2);

	IPserver = argv[optind+1];
	IPlocal  = argv[optind+0];

	while (1) {

		if (MpSocket < 0 &&
			get_command_sock() != 0) {
			sleep(2);
			continue;
		}

		if (++i >= 0x7fffffff) {
			i = 1;
		}

		set_tty_speed();

		if (send_command_udp(i, op)) {
			close(MpSocket);
			MpSocket = -1;
			sleep(1);
			continue;
		}

		while ((ret = recv_command_udp(i)) == -2);

		if (ret == 0) {
			sleep (4);
			continue;
		}

		if (ret < 0) {
			sleep (1);
			continue;
		}

		set_tty_speed();

		if (ret == 11) {
			op = start_tests(debug);
		}

		if (ret == 13) {
			system("reboot");
		}

		if (ret == 14) {
			system("killall -int socket");
			send_command_udp(++i, 4);
		}

		sleep(4);
	}

	free_command_sock();

	exit(ret);
}

int get_command_sock()
{

	unsigned long inaddr;

	struct hostent *hp;

	if ((inaddr = inet_addr (IPlocal)) == INADDR_NONE) {
		if ((hp = gethostbyname (IPlocal)) == (struct hostent *) NULL) {
			fprintf (stdout, "%s : no such local host : %s\n", Pgname,
				IPlocal);
			return(1);
		}
		memcpy ((char *) &inaddr, hp->h_addr, sizeof (long));
	}

	Mp_loc.sin_family = AF_INET;
	Mp_loc.sin_addr.s_addr = inaddr;
	Mp_loc.sin_port = htons (COMMAND_PORT);

	if ((inaddr = inet_addr (IPserver)) == INADDR_NONE) {
		if ((hp = gethostbyname (IPserver)) == (struct hostent *) NULL) {
			fprintf (stdout, "%s : no such remote host : %s\n", Pgname,
				IPserver);
			return(1);
		}
		memcpy ((char *) &inaddr, hp->h_addr, sizeof (long));
	}

	Mp_rem.sin_family = AF_INET;
	Mp_rem.sin_addr.s_addr = inaddr;
	Mp_rem.sin_port = htons (COMMAND_PORT);

	if ((MpSocket = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1) {
		fprintf(stdout, "%s : Can't alocate a socket\n", 
				Pgname);
		perror("socket");
		return (-1);
	}

	if (bind(MpSocket, (struct sockaddr *)&Mp_loc, len_sockaddr) < 0) {
		fprintf(stdout, "%s : Can't bind socket IP %s Port %u\n",
			Pgname, IPlocal, COMMAND_PORT);
		perror("bind");
		return (-1);
	}

	return (0);
}

int free_command_sock()
{
	return (close (MpSocket));
}

int send_command_udp(unsigned int id, int op)
{
	int tam = sizeof(CONFMSG);

	memset((char *)&cmd, 0, sizeof(cmd));
	
	cmd.id = htonl(id);
	cmd.op = htonl(op);
	cmd.ip = Mp_loc.sin_addr.s_addr;
	cmd.port = htons (SOCKET_PORT);
	
	if (sendto (MpSocket, &cmd, tam, 0,
		(struct sockaddr *) &Mp_rem, len_sockaddr) == -1) {
		fprintf(stdout, "%s : Can't send message to %s\n", Pgname, IPserver);
		perror ("sendto");
		return (-1);
	}

	fprintf(stdout, "%s : send message id %u to %s\n", Pgname, id, IPserver);
	return (0);
}

int recv_command_udp(unsigned int id)
{
	int			ret;
	fd_set InFdSet;
	struct timeval BTimeout;
	struct sockaddr_in	Mp_aux;	

	extern int		errno;
	
	FD_ZERO(&InFdSet);
	FD_SET(MpSocket,&InFdSet);

	BTimeout.tv_sec = 4;
	BTimeout.tv_usec = 0;

	ret = select(MpSocket+1,&InFdSet,NULL,NULL,&BTimeout);

	if (ret == 0) {
		fprintf(stdout, "%s : No Response from %s id (%u) Timeout\n", Pgname,
					IPserver, id);
		return(-1);
	}

	if (ret < 0) {
		perror ("select");
		return (0);
	}

	ret = recvfrom (MpSocket, (char *) &cmd, sizeof (cmd), 0,
		(struct sockaddr *) &Mp_aux, &len_sockaddr);

	if (ret != sizeof (cmd)) {
		if (errno == EINTR) {
			return (-1);	/* User abort */
		}
		fprintf(stdout, "\r\n%s : Wrong message response from %s\n", Pgname, IPserver);
		perror ("recvfrom");
		return (0);
	}

	if (cmd.id && cmd.id != id) {
		fprintf(stdout, "%s : Received response Id:%u from %s is out of sequence\n", Pgname, cmd.id, IPserver);
		if (cmd.id < id) {
			return(-2);
		}
	}
	return (cmd.op);
}

/* Alarm received */

