/*
 * appl2.c -- Routines for the web pages (LINUX version)
 *
 */

/******************************** Description *********************************/

/*
 *	Routines for the web pages (asp/form/cgi). This module is reserved
 *      for individual configuration (not structured in tables). The files
 *      handled by this module are, for the time being, getty_ttyS0, issue,
 *      hostname, host.conf, resolv.conf and portslave/pslave.conf.
 */

/********************************* Includes ***********************************/

#include	"appl.h"
#include	<signal.h>
#include	<unistd.h> 
#include	<sys/types.h>
#include	<sys/wait.h>
#include	<pslave_cfg.h>
#include 	<src/server.h>
#include        <dirent.h>

#ifdef BOOT
#include	<sys/ioctl.h>
#include	<asm/flash.h>
#include	<asm/cyc_config.h>
#include	<net/if.h>
#endif
/******************************** Definitions *********************************/
#define SIZE_IPADD   16
#define MAX_PSLAVE_PORTS MAXVLINES
#define MAX_SERIAL_PORTS 64

/* Types of entries */
#define TYPE_NUMBER  'N' /* Number */
#define TYPE_IPADDR  'I' /* IP address */
#define TYPE_IPMASK  'M' /* IP mask */
#define TYPE_STRING  'S' /* String */
#define TYPE_LIST    'L' /* List of valid entries */
#define TYPE_YN      'Y' /* Yes or no */
#define TYPE_TEXT    'T' /* Text */
#define TYPE_RADIO   'R' /* Radio */
#define TYPE_MACADDR 'E' /* MAC Address */
#define TYPE_TTY     'Z' /* TTY - it can be a list of ttyS<n>, an IP Address */
						 /* or an IP Address with socket port */

#define FLAG_READ      1 /* The parameter was read from the file */
#define FLAG_PRINTED   2 /* The parameter was printed in the page */
#define FLAG_WRITTEN   4 /* The parameter was written in the file */
#define FLAG_ERROR     8 /* An error was detected in the parameter */
#define FLAG_INACTIVE 16 /* This parameter is temporarily inactive */
#define FLAG_MSG      32 /* This parameter will carry a string */

#define CONFIG_DONT_CLOSE_CELL 1 /* The table cell should not be closed */
#define CONFIG_DONT_OPEN_CELL  2 /* The table cell should be closed */
#define CONFIG_READ_ONLY 	   4 /* The parameter is read only */

/*Maybe we'll need to convert some characters into symbols. Just in case ...
struct symbolchar {
    unsigned char ch;
    unsigned char *str;
};
struct symbolchar symbolch[] = {
    {'', "&Agrave;"}, {'', "&agrave;"}, {'', "&Aacute;"}, {'', "&aacute;"},
    {'', "&Atilde;"}, {'', "&atilde;"}, {'', "&Acirc;"},  {'', "&acirc;"},
    {'', "&Eacute;"}, {'', "&eacute;"}, {'', "&Ecirc;"},  {'', "&ecirc;"},
    {'', "&Iacute;"}, {'', "&iacute;"}, {'', "&Oacute;"}, {'', "&oacute;"},
    {'', "&Ocirc;"},  {'', "&ocirc;"},  {'', "&Uacute;"}, {'', "&uacute;"},
    {'', "&Ccedil;"}, {'', "&ccedil;"}, {'`', "&grave;"},  {'~', "&tilde;"},
    {'"', "&quot;"},   {' ', "&nbsp;"}
};
*/

/***************************** Routine Definition ***************************/
typedef unsigned char *(getputroutine) (unsigned char *, unsigned char *, int, int);

/**************************** Structure Definition **************************/
/* List of valid entries */
struct combolist {
    unsigned char **elements;
    unsigned char **show;
    unsigned char nelements;
};
/* Configuration parameter structure */
struct cfglist {
    unsigned char *show;          /* name of the element as shown in the page */
    unsigned char *name;          /* name of the element as shown in the config file */
    unsigned char *def_value;     /* default value */
    unsigned char *value;         /* value of the element */
    void *param;                  /* parameters for the element */
    getputroutine *getputrt;      /* routine which extracts and inserts the values from fvalue */
    unsigned char *fvalue;        /* value in the file, when different */
    unsigned char type;           /* type of element - see definitions */
    unsigned char flag;           /* some flags READ/PRINTED/WRITTEN */
    unsigned char config;         /* configuration for this element */
    unsigned char index;          /* index for the element */
    unsigned short field_size;    /* size of the field in the HTTP page */
    unsigned short size;          /* size allocated for the element */
    unsigned short fsize;         /* size allocated for fvalue */
};

/**************************************************************************
 * Definitions for common initialized data
 */
/* Console/Serial Port Spped List */
unsigned char *consspeedlist[] = {
"50", "75", "110", "134", "150", "200", "300", "600", "1200", "1800",
"2400", "4800", "9600", "14400", "19200", "28800", "38400", "57600",
"76800", "115200", "230400", "460800", "921600"};
static struct combolist consspeedl = {consspeedlist, consspeedlist, 22}; /* [Fuji] Dec/19/02 - Don't accept 921600bps */

static unsigned char *numberlist[]= {"0", "1", "2", "3", "4", "5", "6", "7", "8"};
static unsigned char *onezerolist[] = {"1", "0"};
static unsigned char *activelist[] = {"active", "inactive"};
static unsigned char *enablelist[] = {"enabled", "disabled"};
static unsigned char *yesnolist[] = {"yes", "no"};
static unsigned char *onofflist[] = {"on", "off"};
static unsigned char *mulses_show[] =
	{"yes", "no", "RW Session", "Sniff Session"};
static unsigned char *mulses_list[] =
	{"yes", "no", "RW_session", "sniff_session"};
static struct combolist onezerol = {onezerolist, onezerolist, 2};
static struct combolist activel = {onezerolist, activelist, 2};
//static struct combolist inactivel = {numberlist, activelist, 2};
//static struct combolist enablel = {onezerolist, enablelist, 2};
static struct combolist disablel = {numberlist, enablelist, 2};
static struct combolist yesnol = {onezerolist, yesnolist, 2};
static struct combolist yesnol2 = {onofflist, yesnolist, 2};
static struct combolist noyesl = {numberlist, yesnolist, 2};
static struct combolist mulsesl = {mulses_list, mulses_show, 4};

//[RK] dhcpc list
static unsigned char *dhcpclist[] = {"inactive", "active", "act & restore last assigned"};
static struct combolist dhcpcl = {numberlist, dhcpclist, 3};

/**************************************************************************
 * Definitions for the host name configuration
 */

/* Parameter structure */
static struct cfglist hostnamecfg[] = {
    {"Hostname", "hostname",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 35, 40, 0}
};

/**************************************************************************
 * Definitions for the Issue configuration
 */

/* Parameter structure */
static struct cfglist issuecfg[] = {
    {"Console Banner", "issue",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_TEXT, 0, 0, 0, 40, 512, 0}
};

/**************************************************************************
 * Definitions for the Host configuration (host.conf)
 */

/* List of valid entries */
static unsigned char *ordercombo[] = {"", "hosts", "bind", "hosts,bind", "bind,hosts"};
static unsigned char *ordershow[] = {"None", "Hosts", "DNS", "Hosts/DNS", "DNS/Hosts"};
static struct combolist searchorder = {ordercombo, ordershow, 5};

static unsigned char *onoff[] = {"on", "off"};
static struct combolist onoffl = {onoff, onoff, 2};

/* Routines */
static unsigned char *ListGetOrder (unsigned char *str, unsigned char *value, int arg, int size);

/* Parameter structure */
static struct cfglist hostlist[] = {
    {"Access Order", "order",
    NULL, NULL, &searchorder, ListGetOrder, NULL,
    TYPE_LIST, 0, 0, 0, 0, 16, 16},
    {"Multiple IPs for One Host", "multi",
    NULL, NULL, &onoffl, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 4, 0}
};

/**************************************************************************
 * Definitions for the DNS configuration (resolv.conf)
 */

/* Routines - This routine should be used when the domain name is refered by
 * the argument search; not we don't need it. */
/*static unsigned char *ListGetArg (unsigned char *str, unsigned char *value, int arg, int size);*/

/* Parameter structure */
static struct cfglist resolvlist[] = {
    {"Primary DNS Server", "nameserver",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_IPADDR, 0, 0, 0, SIZE_IPADD, SIZE_IPADD, 0},
    {"Secondary DNS Server", "nameserver",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_IPADDR, 0, 0, 0x10, SIZE_IPADD, SIZE_IPADD, 0},
    {"Domain Name", "domain",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 35, 40, 0}
};

/**************************************************************************
 * Definitions for the Portslave configuration ("conf" word)
 */

/* List of valid entries - "numberlist" could also be used in the other
 * sessions */
static unsigned char *facilityshow[]= {"local0", "local1", "local2", "local3",
"local4", "local5", "local6", "local7"};
static struct combolist facilityl = {numberlist, facilityshow, 8};

#define CONF_INCL_FILE	5
#define CONF_HOSTNAME	4
#define CONF_DHCPC		6

/* Parameter structure */
static struct cfglist pslaveconflist[] = {
    {"Primary IP Address", "eth_ip",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_IPADDR, 0, 0, 0, SIZE_IPADD, SIZE_IPADD, 0},
    {"Network Mask", "eth_mask",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_IPMASK, 0, 0, 0, SIZE_IPADD, SIZE_IPADD, 0},
    {"Secondary IP Address", "eth_ip_alias",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_IPADDR, 0, 0, 0, SIZE_IPADD, SIZE_IPADD, 0},
    {"Network Mask", "eth_mask_alias",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_IPMASK, 0, 0, 0, SIZE_IPADD, SIZE_IPADD, 0},
    {"Hostname", "hostname",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 40, 45, 0},
    {"Common Configuration File Name", "include",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 40, 45, 0},
    {"DHCP Client", "dhcp_client",
    "0", NULL, &dhcpcl, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 3, 0},
    {"MTU", "eth_mtu",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_NUMBER, 0, 0, 0, 6, 6, 0},
/*    {"Default Gateway", "defgw",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_IPADDR, 0, 0, 0, SIZE_IPADD, SIZE_IPADD, 0}, */
/*[RK]    {"Syslog Server", "syslog",
 *   NULL, NULL, NULL, NULL, NULL,
 *   TYPE_STRING, 0, 0, 0, 35, 40, 0}, */
    {"Syslog Facility", "facility",
    NULL, NULL, &facilityl, NULL, NULL,
    TYPE_LIST, 0, 0, 0, 0, 2, 0},
    {"Remote NFS Path", "nfs_data_buffering",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 35, 50, 0},
    {"Data Buffering Facility", "DB_facility",
    NULL, NULL, &facilityl, NULL, NULL,
    TYPE_LIST, 0, 0, 0, 0, 2, 0},
};

/**************************************************************************
 * Definitions for the Portslave configuration (port specific)
 */

/* List of valid entries */
static unsigned char pvoid[] = "";
static unsigned char pinactive[] = "Inactive";
static unsigned char premote[] = "Remote";
static unsigned char ttystr[8 * MAX_SERIAL_PORTS];
static unsigned char *ttylist[MAX_SERIAL_PORTS+2];
static unsigned char *ttyshow[MAX_SERIAL_PORTS+2];
static struct combolist ttyl = {ttylist, ttyshow, MAX_SERIAL_PORTS+2};

static struct combolist datasizel = {numberlist+4, numberlist+4, 5};

static struct combolist stopbitl = {numberlist+1, numberlist+1, 2};

static unsigned char *paritylist[] = {"none", "odd", "even"};
static unsigned char *parityshow[] = {"None", "Odd", "Even"};
static struct combolist parityl = {paritylist, parityshow, 3};

static unsigned char *flowlist[] = {"none", "hard", "soft"};
static unsigned char *flowshow[] = {"None", "Hardware", "Software"};
static struct combolist flowl = {flowlist, flowshow, 3};

//[RK] static unsigned char *syslogshow[] = {"Disabled", "Alert", "Critical", "Error", "Warning", "Notice", "Info", "Debug"};
//[RK] static struct combolist syslogl = {numberlist, syslogshow, 8};

static unsigned char *authtypelist[] = {"none", "radius", "TacacsPlus", "local", "radius/local", "TacacsPlus/local", "local/radius", "local/TacacsPlus", "RadiusDownLocal", "TacacsPlusDownLocal", "ldap", "ldapDownLocal", "DSView", "DSView/local", "DSViewDownLocal"};
static unsigned char *authtypeshow[] = {"None", "RADIUS", "TacacsPlus","Local", "RADIUS/Local", "TacacsPlus/Local", "Local/RADIUS", "Local/TacacsPlus", "RadiusDownLocal", "TacacsPlusDownLocal", "LDAP", "LDAPDownLocal", "DSView", "DSView/local", "DSViewDownLocal"};
static struct combolist authtypel = {authtypelist, authtypeshow, 15};

//[RK]Aug/21/06 - removed support for rlogin protocol
static unsigned char *protocollist[] = {"login", "telnet", "ssh",
"ssh2", "slip", "cslip", "ppp", "ppp_only", "socket_client", "socket_server", 
"socket_ssh", "raw_data", "billing", "modem_replace",
#ifdef WEBS_PMD
"ipdu", 
#endif
#ifdef TS1H
"modbus",
#endif
};
static unsigned char *protocolshow[] = {"Login", "TELNET", "SSH",
"SSH2", "SLIP", "CSLIP", "PPP", "PPP Only", "Socket Client", "Socket Server", 
"Socket SSH", "Socket RAW", "Billing", "Modem Replace", 
#ifdef WEBS_PMD
"IPDU", 
#endif
#ifdef TS1H
"MODBUS", 
#endif
};
static struct combolist protocoll = {protocollist, protocolshow, 
	sizeof(protocollist)/sizeof(unsigned char *)};

static unsigned char *incremshow[] = {"Incremented", ""};
static unsigned char *incremlist[] = {"+", ""};
static struct combolist increml = {incremlist, incremshow, 2};

static unsigned char *snifmodeshow[] = {"Input", "Output", "Both","Not Sniff"};
static unsigned char *snifmodelist[] = {"in", "out", "i/o","no"};
static struct combolist snifmodel = {snifmodelist, snifmodeshow, 4};

static unsigned char *mediashow[] = {"RS232", "RS232 Half Duplex","RS232 Halfd with CTS ctrl", "RS485 Half Duplex", "RS422",
	"RS485 Full Duplex", "RS485 Half Duplex Terminator", 
	"RS485 Full Duplex Terminator"};
static unsigned char *medialist[] = {"rs232", "rs232_half","rs232_half_cts","rs485_half", "rs422", "rs485_full",
	"rs485_half_terminator", "rs485_full_terminator"};

static struct combolist medial = {medialist, mediashow, 8};

static unsigned char *smodeshow[] = {"ASCII", "RTU"};
static unsigned char *smodelist[] = {"ascii", "rtu"};
static struct combolist smodel = {smodelist, smodeshow, 2};

static unsigned char *noDBmenushow[] = {"Show Menu", 
		"Don't Show Menu and Ignore DBfile",
		"Don't Show Menu and Show DBfile",
		"Show Short Menu"};
static unsigned char *noDBmenulist[] = {"0", "1", "2", "3"};
static struct combolist noDBmenul = {noDBmenulist, noDBmenushow, 4};

static unsigned char *dbmodeshow[] = {"CIR", "LIN"};
static unsigned char *dbmodelist[] = {"cir", "lin"};
static struct combolist dbmodel = {dbmodelist, dbmodeshow, 2};

//[RK]Nov/20/02 Bug#88
static unsigned char *telnetmodeshow[] = {"Text", "Binary"};
static unsigned char *telnetmodelist[] = {"0", "1"};
static struct combolist telnetmodel = {telnetmodelist, telnetmodeshow, 2};

#ifdef WEBS_PMD
static unsigned char pmtypenames[16 * 20];
static unsigned char *pmtypelist[16] = {
        pmtypenames +  0*20, pmtypenames +  1*20, pmtypenames +  2*20,
        pmtypenames +  3*20, pmtypenames +  4*20, pmtypenames +  5*20,
        pmtypenames +  6*20, pmtypenames +  7*20, pmtypenames +  8*20,
        pmtypenames +  9*20, pmtypenames + 10*20, pmtypenames + 11*20,
        pmtypenames + 12*20, pmtypenames + 13*20, pmtypenames + 14*20,
        pmtypenames + 15*20};                            
static struct combolist pmtypel = {pmtypelist, pmtypelist, 16};
#endif

/* Routines */
static unsigned char *SeparateIncrement (unsigned char *str, unsigned char *value, int arg, int size);


/* Fixed positions for some configuration parameters. These parameters will
 * be changed according to the situation. */
#define PORT_TTY               0
#define PORT_REMADDR           1
#define PORT_REMADDR_INC       2
#define PORT_TCPPORT           3
#define PORT_TCPPORT_INC       4
#define PORT_DISCSPEC          5
#define PORT_SERVERFARM        6
#define PORT_MODBUS_SMODE      7
#define PORT_MEDIA             8

/* Parameter structure */
static struct cfglist pslaveportlist[] = {
    {"Serial Port", "tty",
    NULL, NULL, &ttyl, NULL, NULL,
    TYPE_TTY, 0, 0, 0, 20, 25, 0},
    {"Remote/Local IP Address", "ipno",
    NULL, NULL, NULL, SeparateIncrement, NULL,
    TYPE_IPADDR, 0, CONFIG_DONT_CLOSE_CELL, 0, SIZE_IPADD, SIZE_IPADD, SIZE_IPADD+1},
    {"incremented", "ipno",
    NULL, NULL, &increml, SeparateIncrement, NULL,
    TYPE_YN, 0, CONFIG_DONT_OPEN_CELL, 0x01, 0, 2, 0},
    {"Socket Port", "socket_port",
    NULL, NULL, NULL, SeparateIncrement, NULL,
    TYPE_NUMBER, 0, CONFIG_DONT_CLOSE_CELL, 0, 6, 6, 7},
    {"incremented", "socket_port",
    NULL, NULL, &increml, SeparateIncrement, NULL,
    TYPE_YN, 0, CONFIG_DONT_OPEN_CELL, 0x01, 0, 2, 0},
    {"Discard Previous Port-specific Parameters", "discard_spec",
    "0", NULL, &yesnol, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 2, 0},
    {"Server Farm", "serverfarm",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 35, 40, 0},
    {"MODBUS Serial Mode", "modbus_smode",
    "ascii", NULL, &smodel, NULL, NULL,
    TYPE_LIST, 0, 0, 0, 0, 6, 0},
    {"Media", "media",
    "rs232", NULL, &medial, NULL, NULL,
    TYPE_LIST, 0, 0, 0, 0, 22, 0},
    {"Speed (bps)", "speed",
    "9600", NULL, &consspeedl, NULL, NULL,
    TYPE_LIST, 0, 0, 0, 0, 7, 0},
    {"Data Size", "datasize",
    "8", NULL, &datasizel, NULL, NULL,
    TYPE_LIST, 0, 0, 0, 0, 2, 0},
    {"Stop Bit", "stopbits",
    "1", NULL, &stopbitl, NULL, NULL,
    TYPE_LIST, 0, 0, 0, 0, 2, 0},
    {"Parity", "parity",
    "none", NULL, &parityl, NULL, NULL,
    TYPE_LIST, 0, 0, 0, 0, 5, 0},
    {"Flow Control", "flow",
    "none", NULL, &flowl, NULL, NULL,
    TYPE_LIST, 0, 0, 0, 0, 5, 0},
    {"DCD Sensitive", "dcd",
    NULL, NULL, &yesnol, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 2, 0},
    {"DTR Off Time Interval", "DTR_reset",
    "100", NULL, NULL, NULL, NULL,
    TYPE_NUMBER, 0, 0, 0, 6, 6, 0},
/* [RK]    {"Syslog Console Message Level", "console_level",
 *   NULL, NULL, &syslogl, NULL, NULL,
 *   TYPE_LIST, 0, 0, 0, 0, 2, 0},
 *   {"Syslog Server Message Level", "syslog_level",
 *   NULL, NULL, &syslogl, NULL, NULL,
 *   TYPE_LIST, 0, 0, 0, 0, 2, 0}, */
    {"Write Login Records in Utmp", "sysutmp",
    NULL, NULL, &yesnol, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 2, 0},
/*    {"Write Login Records in Wtmp", "syswtmp",
    NULL, NULL, &yesnol, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 2, 0}, */
    {"Authentication Type", "authtype",
    NULL, NULL, &authtypel, NULL, NULL,
    TYPE_LIST, 0, 0, 0, 0, 25, 0},	//[RK]
    {"First RADIUS/TacacsPlus Authentication Server", "authhost1",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 35, 40, 0},
    {"First RADIUS Accounting Server", "accthost1",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 35, 40, 0},
    {"Second RADIUS/TacacsPlus Authentication Server", "authhost2",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 35, 40, 0},
    {"Second RADIUS Accounting Server", "accthost2",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 35, 40, 0},
    {"RADIUS/TacacsPlus Timeout", "radtimeout",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_NUMBER, 0, 0, 0, 6, 6, 0},
    {"RADIUS/TacacsPlus Secret", "secret",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 35, 40, 0},
    {"RADIUS/TacacsPlus Retries", "radretries",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_NUMBER, 0, 0, 0, 6, 6, 0},
    {"RADIUS Password Required", "radnullpass",
    NULL, NULL, &noyesl, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 2, 0},
    {"Protocol", "protocol",
    NULL, NULL, &protocoll, NULL, NULL,
    TYPE_LIST, 0, 0, 0, 0, 14, 0},
    {"Host", "host",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 35, 40, 0},
    {"Network Mask", "netmask",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_IPMASK, 0, 0, 0, SIZE_IPADD, SIZE_IPADD, 0},
    {"Automatic User", "userauto",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 35, 40, 0},
    {"Access Restriction on Users", "users",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 40, 256, 0},
    {"Banner", "issue",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_TEXT, 0, 0, 0, 40, 1500, 0},
    {"Login Prompt", "prompt",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 35, 40, 0},
    {"Terminal Type", "term",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 20, 20, 0},
    {"MTU", "mtu",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_NUMBER, 0, 0, 0, 6, 6, 0},
    {"MRU", "mru",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_NUMBER, 0, 0, 0, 6, 6, 0},
    {"Auto PPP Options", "autoppp",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_TEXT, 0, 0, 0, 40, 256, 0},
    {"Forced PPP Options", "pppopt",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_TEXT, 0, 0, 0, 40, 256, 0},
    {"Init Chat", "initchat",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_TEXT, 0, 0, 0, 40, 256, 0},
    {"Maximum Buffer Size (0-disabled)", "data_buffering",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_NUMBER, 0, 0, 0, 9, 9, 0},
    {"Buffer Size to Send Syslog<br>(40 to 255, 0-disabled)", "syslog_buffering",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_NUMBER, 0, 0, 0, 6, 6, 0},
    {"Data Buffering Menu", "dont_show_DBmenu",
    NULL, NULL, &noDBmenul, NULL, NULL,
    TYPE_LIST, 0, 0, 0, 0, 2, 0},
    {"Break Sequence", "break_sequence",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 16, 16, 0},
    {"Sniff Session Mode", "sniff_mode",
    "no", NULL, &snifmodel, NULL, NULL,
    TYPE_LIST, 0, 0, 0, 0, 4, 0},
    {"Administrative Users", "admin_users",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 40, 256, 0},
    {"Socket Transmission Interval (ms)", "tx_interval",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_NUMBER, 0, 0, 0, 6, 6, 0},
    {"Polling Interval (ms)", "poll_interval",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_NUMBER, 0, 0, 0, 6, 6, 0},
    {"Idle Timeout (min)", "idletimeout",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_NUMBER, 0, 0, 0, 6, 6, 0},
    //[RK] new parameters 1.3.3
    {"Records the Time Stamp in the Data Buffering File", "DB_timestamp",
    "0", NULL, &yesnol, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 2, 0},
    {"Escape Char from Sniff Mode", "escape_char",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 6, 6, 0},
    {"Allows Multiple Sniff Sessions", "multiple_sessions",
    "no", NULL, &mulsesl, NULL, NULL,
    TYPE_LIST, 0, 0, 0, 0, 13, 0},
    {"Multiple Sniff Session Notification", "multiuser_notif",
    "off", NULL, &yesnol2, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 4, 0},
    {"STTY Command", "sttyCmd",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 40, 180, 0},
    {"Alarm for Data Buffering", "alarm",
    "0", NULL, &yesnol, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 2, 0},
    {"Data Buffering Mode", "DB_mode",
    "cir", NULL, &dbmodel, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 6, 0},
	{"Data Buffer User Connection Log", "DB_user_logs",
    "off", NULL, &yesnol2, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 4, 0},
    {"Telnet Client Mode", "telnet_client_mode", //[RK]Nov/20/02 Bug#88
    "0", NULL, &telnetmodel, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 6, 0},
    {"Syslog Buffering at All Times", "syslog_sess", //[RK]Nov/20/02 new parameter
    "0", NULL, &noyesl, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 2, 0},
    {"Line Feed Suppression", "lf_suppress", 	//[RK]Nov/26/02 new parameter
    "0", NULL, &activel, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 2, 0},
    {"Input String to Auto Answer", "auto_answer_input",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 40, 180, 0},
    {"Output String to Auto Answer", "auto_answer_output",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 40, 180, 0},
    {"Pool Local IP Address", "pool_ipno",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_IPADDR, 0, 0, 0, SIZE_IPADD, SIZE_IPADD, 0},
    {"Pool Socket Port", "pool_socket_port",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_NUMBER, 0, 0, 0, 6, 6, 0},
    {"Pool Server Farm", "pool_serverfarm",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 35, 40, 0},
#ifdef WEBS_PMD
    {"IPDU type", "pmtype",
    "cyclades", NULL, &pmtypel, NULL, NULL,
    TYPE_LIST, 0, 0, 0, 0, 20, 0},
//[RK]Oct/21/04 - removed - not used
//    {"Number of Outlets", "pmNumOfOutlets",
//    NULL, NULL, NULL, NULL, NULL,
//    TYPE_NUMBER, 0, 0, 0, 6, 6, 0},
    {"IPDU users", "pmusers",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 40, 120, 0},
//    {"IPDU Administrative Password", "pmadmpasswd",
//    NULL, NULL, NULL, NULL, NULL,
//    TYPE_STRING, 0, 0, 0, 20, 30, 0},
    {"Power Management Outlets", "pmoutlet",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 40, 60, 0},
    {"Power Management Hotkey", "pmkey",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 6, 8, 0},
#endif
    {"Billing Records per File", "billing_records",  //mp.Jan2003: new parameter
    NULL, NULL, NULL, NULL, NULL,
    TYPE_NUMBER, 0, 0, 0, 6, 6, 0},
    {"Billing Timeout (min)", "billing_timeout",     //mp.Jan2003: new parameter
    NULL, NULL, NULL, NULL, NULL,
    TYPE_NUMBER, 0, 0, 0, 6, 6, 0},
    {"Billing Record Termination", "billing_eor",    //mp.Aug2003: new parameter
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 15, 33, 0},
    {"Billing Directory", "billing_dir",             //mp.Jan2004: new parameter
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, 40, 64, 0},
    {"Break Interval (ms)", "break_interval",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_NUMBER, 0, 0, 0, 6, 6, 0},
};

/**************************** Global Variables ******************************/
static unsigned char ncfgerr; /* Number of parameters with error */
static int pslaveconfigport;
static int tcp_port;
static unsigned char pslave_incl = 0; /* 1 = if conf.include is defined */

/**************************************************************************
 * AllocConfig - allocates a buffer which will store all the parameters of
 * the list.
 */
static int AllocConfig(struct cfglist *list, int nlist)
{
    int i, size=0;
    unsigned char *buf;
    /* Defines the size of the buffer */
    for (i=0; i<nlist; i++) {
        size+=(list+i)->size;
        size+=(list+i)->fsize;
    }
    /* Allocates the buffer and assign the pointers for the values */
    if ((buf = malloc (size)) == NULL) return 0;
    size=0;
    for (i=0; i<nlist; i++) {
        (list+i)->value = buf+size;
        size+=(list+i)->size;
        (list+i)->flag=0;
        if ((list+i)->fsize) {
            (list+i)->fvalue = buf+size;
            size+=(list+i)->fsize;
        } else {
            (list+i)->fvalue = (list+i)->value;
        }
    }
    return 1;
}

/**************************************************************************
 * DeallocConfig - deallocates the buffer allocated in AllocConfig.
 */
static void DeallocConfig(struct cfglist *list)
{
    free(list->value);
}

/**************************************************************************
 * DefaultConfig - Sets the parameters with the default value
 */

static void DefaultConfig(struct cfglist *list, int nlist)
{
    int i;
    for (i=0; i<nlist; i++) {
        if ((list+i)->def_value) {
            strncpy((list+i)->value, (list+i)->def_value, (list+i)->size);
        } else {
            *((list+i)->value) = 0;
        }
    }
}

/**************************************************************************
 * CheckConfig - Verifies whether the value is valid; if positive, copies it
 * in the list.
 */
static int CheckConfig (struct cfglist *list, unsigned char *value)
{
    unsigned int j, k, mask, p[6];
    unsigned int l[4], pt;
    struct in_addr b;
    struct combolist *combo;

	*list->value = 0;

    switch (list->type) {

    case TYPE_STRING:
    case TYPE_TEXT:
        /* Type string - don't need to check. */
        strncpy(list->value, value, list->size);
        return 1;
    break;

    case TYPE_MACADDR:
    	if (sscanf(value, "%x:%x:%x:%x:%x:%x",
    		&p[0], &p[1], &p[2], &p[3], &p[4], &p[5]) < 6) break;
    	return 1;

    case TYPE_NUMBER:
        /* Type number - tries to convert the string in a number and checks
         the limits if applicable */
		if (strlen(value) >= list->field_size) break;
        if (sscanf(value,"%d", &p[0])!= 1) break;
        if (list->param) {
            if (p[0] < *((unsigned int *)(list->param)) ||
                p[0] > *((unsigned int *)(list->param) + 1)) {
                    break;
            }
        }
        if (snprintf(list->value, list->size, "%d", p[0]) < 0) break;
        return 1;

    case TYPE_IPADDR:
        /* Type IP address - tries to convert the string in an IP number */
        if (inet_aton(value, &b)) {
            strcpy (list->value, inet_ntoa(b));
            return 1;
        }
        break;

    case TYPE_IPMASK:
        /* Type IP mask - tries to convert the string in an IP number and
         * checks whether it is a mask */
		if (inet_aton(value, &b)) {
			mask=0;
			for (j=0; j<4; j++) {
				for (k=0x80; k; k/=2) {
					if (b.s_addr == mask) {
						strcpy (list->value, inet_ntoa(b));
						return 1;
					}
					*((unsigned char *)&mask + j) += k;
				}
			}
			if (b.s_addr == mask) {
				strcpy (list->value, inet_ntoa(b));
					return 1;
			}
		}
		break;

    case TYPE_RADIO:
    case TYPE_YN:
    case TYPE_LIST:
        /* Type list - The value should match one of the valid entries */
        {
	    char *t;
            for (t = value; !isspace(*t) && *t; t++);
            *t = 0;
        }
        combo = list->param;
        for (j=0; j<combo->nelements; j++) {
            if (strcmp(combo->elements[j], value)) continue;
            strcpy (list->value, combo->elements[j]);
            return 1;
        }
        break;
		
    case TYPE_TTY:
        /* Type tty */
        combo = list->param;
        for (j=0; j<combo->nelements-1; j++) {
            if (strcmp(combo->elements[j], value)) continue;
            strcpy (list->value, combo->elements[j]);
            return 1;
        }
       	if (inet_aton(value, &b)) {
            strcpy (list->value, inet_ntoa(b));
       		return 1;
       	}
       	if (sscanf(value, "%d.%d.%d.%d:%d", &l[0], &l[1], &l[2], &l[3], &pt) >= 5) {
       		for (j=0; j<4; j++) if (l[j]>255) return 0;
       		if (pt<0 || pt>65025) return 0;
       		sprintf(list->value, "%d.%d.%d.%d:%d", l[0], l[1], l[2], l[3], pt);
       		return 1;
       	}
    }
    return 0;
}

/**************************************************************************
 * PrintConfig - Prints an input in the page, for the parameter
 */
static int PrintConfig(webs_t wp, struct cfglist *list, unsigned char *name)
{
    int j;
    struct combolist *combo;
    unsigned char *buf, *s, *t, flag1, flag2;

    /* if there is an error - write the name in red */
    if (list->config & CONFIG_DONT_OPEN_CELL) {
    	if (list->type != TYPE_YN) {
		    websWrite(wp, T("<b%s>%s:&nbsp;</b>"),
		    	(list->flag & FLAG_ERROR)?" class=\"err\"":"", list->show);
		}
    } else {
	    websWrite(wp, T("<tr><td><b%s>%s:</b></td><td>"),
	    	(list->flag & FLAG_ERROR)?" class=\"err\"":"", list->show);
	}
                            	
    switch (list->type) {

    case TYPE_STRING:
    case TYPE_NUMBER:
    case TYPE_IPADDR:
    case TYPE_IPMASK:
    case TYPE_MACADDR:
        /* For most types: input type text */
        /* It's necessary first replace all the '"''s for &quot;'s*/
        buf=malloc (strlen(list->value)*6+1);
        for (s=list->value, t=buf; *s; s++) {
            if (*s=='"') {
                strcpy(t, "&quot;");
                t+=6;
            } else {
                *t++=*s;
            }
        }
        *t=0;
        websWrite(wp, T("<input class=\"tx\" type=\"text\" name=\"%s\" value=\"%s\" size=\"%d\" maxlength=\"%d\">"), \
				  name, buf, list->field_size, list->size-1 );
        free(buf);
        break;

    case TYPE_LIST:
        /* For List types: select */
        combo=(struct combolist *)(list->param);
        websWrite(wp, T("<select name=\"%s\">"), name);
        for (j=0; j<combo->nelements; j++) {
            websWrite(wp, T("<option value=\"%s\"%s>%s"), combo->elements[j],
            	(strcmp(list->value, combo->elements[j]))?"":" selected", combo->show[j]);
        }
        websWrite(wp, T("</select>"));
        break;

    case TYPE_TTY:
    	/* For TTY */
        combo=(struct combolist *)(list->param);
        websWrite(wp, T("<select name=\"%s\">"), name);
        flag1=0;
        for (j=0; j<combo->nelements-1; j++) {
        	flag2 = strcmp(list->value, combo->elements[j]) == 0;
            websWrite(wp, T("<option value=\"%s\"%s>%s"), combo->elements[j],
            	(flag2)?" selected":"", combo->show[j]);
            flag1 |= flag2;
        }
        websWrite(wp, T("<option value=\"%s\"%s>%s"), combo->elements[j],
           	(flag1)?"":" selected", combo->show[j]);
        websWrite(wp, T("</select>"));
        websWrite(wp, T("<b>Remote Cluster : </b>"));
        websWrite(wp, T("<input class=\"tx\" type=\"text\" name=\"%s1\" value=\"%s\" size=\"%d\">"), \
            name, (flag1)?pvoid:list->value, list->field_size);
        break;

    case TYPE_TEXT:
        /* For text types: textarea */
        websWrite(wp, T("<textarea name=\"%s\" rows=\"3\" cols=\"%d\" wrap=\"off\">%s</textarea>"), \
            name, list->field_size, list->value);
        break;

    case TYPE_RADIO:
        /* For option types: radio */
        combo=(struct combolist *)(list->param);
        for (j=0; j<combo->nelements; j++) {
        	if (j) websWrite(wp, T("&nbsp;&nbsp;&nbsp;&nbsp;"));
            websWrite(wp, T("<input type=\"radio\" name=\"%s\" value=\"%s\"%s>&nbsp;%s"),
            	name, combo->elements[j], (strcmp(list->value, combo->elements[j]))?"":" checked",
            	combo->show[j]);
        }
        break;

    case TYPE_YN:
        /* For yes/no types: checkbox */
        combo=(struct combolist *)(list->param);
        websWrite(wp, T("<input type=\"checkbox\" name=\"%s\" value=\"%s\"%s>"),
            	name, combo->elements[0], (strcmp(list->value, combo->elements[0]))?"":" checked");
	    if (list->config & CONFIG_DONT_OPEN_CELL) {
		    websWrite(wp, T("&nbsp;<b%s>%s</b>"),
		    	(list->flag & FLAG_ERROR)?" class=\"err\"":"", list->show);
	    	}
        break;

    default:
    	return 0;
    }
    if (list->config & CONFIG_DONT_CLOSE_CELL) {
       	websWrite(wp, T("&nbsp;&nbsp;&nbsp;&nbsp;"));
    } else {
	    websWrite(wp, "</td></tr>");
	}
    return 1;
}

static void PrintResultTable(webs_t wp, char_t *title, char_t *msg, int flag) 
{
/* Creates a nice table where you can put either an error
   message or a normal message. You should close this table
   calling WriteStandardEnd() */

  //if 1, error flag; if 0, normal flag

  websWrite(wp, T("<br><br><center><table border align=center><tr><th class=\"tt\">%s</th></tr>"
  "<tr><td><pre "), title);
  if (flag ? websWrite(wp, "class=\"err\">") : websWrite(wp, ">"));
  websWrite(wp,T("%s"), msg);
  websWrite(wp,"</pre></td></tr></table></center>");
}

/**************************************************************************
 * TreatNullConfig - Handles the NULL value in the configuration.
 */
static void TreatNullConfig(struct cfglist *list)
{
    struct combolist *combo;
    switch (list->type) {
    case TYPE_YN:
        /* Type yes/no - assumes the "no" value */
        combo=(struct combolist *)(list->param);
        strcpy(list->value, combo->elements[1]);
        break;

    case TYPE_LIST:
    case TYPE_RADIO:
    case TYPE_TTY:
        list->value[0]=0;
	break;
	
    default:
        list->value[0]=0;
        list->flag |= FLAG_ERROR;
        ncfgerr++;
        break;
    }
}

/**************************************************************************
 * GetConfigFromForm - Gets the variables written by the user in the
 * configuration page and verifies the validity of them.
 */
static void GetConfigFromForm(webs_t wp, struct cfglist *list, int nlist)
{
    int i;
    unsigned char *value, back[256];

    for (i=0; i<nlist; i++) {
        if ((list+i)->flag & FLAG_INACTIVE) continue;
        sprintf (back, "%s%02x", (list+i)->name, (list+i)->index);
        (list+i)->flag &= ~FLAG_ERROR;
        value = websGetVar(wp, back, NULL);
        if (value) {
            if ((list+i)->type == TYPE_TTY && strcmp(value, premote)==0) {
                sprintf (back, "%s%02x1", (list+i)->name, (list+i)->index);
                value = websGetVar(wp, back, NULL);
	    }
	}
        if (value) {
            if (*value == 0) {
		(list+i)->value[0] = 0;
	    } else if (!CheckConfig (list+i, value)) {
                strncpy((list+i)->value, value, (list+i)->size);
                (list+i)->flag |= FLAG_ERROR;
                ncfgerr++;
            }
	} else {
            TreatNullConfig (list+i);
        }
    }
}

/**************************************************************************
 * SeekCheckConfig - Evaluates the values parsed from the file and looks for
 * the parameter(s) in the structure.
 */
static int SeekCheckConfig (unsigned char *name, unsigned char *value, struct cfglist *list, int nlist)
{
    int i, j;
    unsigned char order, index;
    unsigned char *back, *pback;
    /* There must be a non-empty value */
    if (!(value && *value)) {
        return 0;
    }
    j=0;
    order=0;
    for (i=0; i<nlist; i++) {
        /* The name must match with the parameter or be NULL */
        if (name && strcmp(name, (list+i)->name)) continue;
        /* The parameter should not have been read yet */
        if ((list+i)->flag & FLAG_READ) continue;
        /* index will be the order of use of the parameter in a certain
         * line. This will be useful when the same value read in the
         * configuration will be used by more than one parameter (example:
         * all.ipno in pslave.conf)*/
        index=(list+i)->index&0x0f;
        if (!j) {
            if (index) {
                continue;
            } else {
                /* order will be the order of use of the parameter when it
                 * is called in more than one line in the configuration.
                 * (example: nameserver, in resolv.conf) */
                order=(list+i)->index&0xf0;
            }
        }
        /* seeks only the parameters of that order... */
        if (order!=((list+i)->index&0xf0)) break;
        j++;
        /* sets FLAG_READ */
        (list+i)->flag |= FLAG_READ;
        /* getputrt is a routine which will use the value read from the
         * file (stored in fvalue) to get the value of the parameter. */
        if ((list+i)->getputrt) {
            if (!index) {
                strncpy ((list+i)->fvalue, value, (list+i)->fsize);
            }
            back = strdup(value);
            pback = (*((list+i)->getputrt)) (back, NULL, index, 0);
        } else {
            back = NULL;
            pback = value;
        }
        CheckConfig(list+i, pback);
        if (back) {
            free(back);
        }
    }
    return j;
}

/**************************************************************************
 * WriteConfig - Seeks the parameter called by the ASP routine and calls
 * PrintConfig.
 */
static int WriteConfig(webs_t wp, struct cfglist *list, int nlist, unsigned char *name)
{
    int i,ret;
    unsigned char back[256];

    for (i=0; i<nlist; i++) {
        sprintf (back, "%s%02x", (list+i)->name, (list+i)->index);
        if (name && strcmp(name, back)) continue;
        if ((list+i)->flag & FLAG_PRINTED) continue;
        if ((list+i)->flag & FLAG_INACTIVE) return 1;
		if ((list+i)->config & CONFIG_READ_ONLY) {
			websWrite(wp, T("<tr><td><b>%s:</b></td><td width=200>%s</td></tr>"),
	    		(list+i)->show,(*((list+i)->value)!=0) ? (list+i)->value : (char_t *)"&nbsp;");
		} else {
			ret = PrintConfig (wp, list+i, back);
		}
		if (ret) {
            (list+i)->flag |= FLAG_PRINTED;
            return 1;
        } else {
            websWrite(wp, T("<b class=\"err\">Config parameter %s error</b>"), name);
            return 1;
        }
    }
    websWrite(wp, T("<b class=\"err\">Config parameter %s not found</b>"), name);
    return 1;
}

/**************************************************************************
 * SeekParameterInTheList - For each command/argument from the configuration
 * file, seeks the list of parameters and overwrites the argument with the
 * value in the RAM. Sometimes it will be necessary to parse the argument and
 * replace the parameter in the right place. This is done using the routine
 * getputrt.
 */
static struct cfglist *SeekParameterInTheList (unsigned char *c, unsigned char *a,
                                               struct cfglist *list, int nlist)
{
    int i, j=0;
    unsigned char index, order;
    struct cfglist *vlist;
    for (i=0; i<nlist; i++) {
        /* The names should match. */
        if (strcmp(c, (list+i)->name)) continue;
	/* If it is the first item, it should be active */
        if (j == 0 && (list+i)->flag & FLAG_INACTIVE) continue;
        /* The value should not have been written yet. */
        if ((list+i)->flag & FLAG_WRITTEN) {
            if ((list+i)->fvalue[0]==0)
                return list+i;
#ifdef SEEK_ANOTHER_ENTRY //[RK]Aug/18/05 - workaround :fix problem with DSView
            else
                continue;
#endif
        }
        /* index will be the order of use of the parameter in a certain
         * line. This will be useful when the same value read in the
         * configuration will be used by more than one parameter (example:
         * all.ipno in pslave.conf)*/
        index=(list+i)->index&0x0f;
        if (!j) {
            if (index) {
                continue;
            } else {
                /* order will be the order of use of the parameter when it
                 * is called in more than one line in the configuration.
                 * (example: nameserver, in resolv.conf) */
                order=(list+i)->index&0xf0;
                vlist=list+i;
            }
        }
        if (order!=((list+i)->index&0xf0)) break;
        j++;
        (list+i)->flag |= FLAG_WRITTEN;
        /* getputrt is a routine which will take the value read from the
         * file (stored in fvalue) and update the value of the parameter. */
        if ((list+i)->getputrt) {
            if (!index) {
                strncpy(vlist->fvalue, a, vlist->fsize);
            }
            (*((list+i)->getputrt)) (vlist->fvalue, (list+i)->value, index, vlist->fsize);
        } else {
            break;
        }
    }
    if (j) {
        return vlist;
    } else {
        return NULL;
    }
}

/**************************************************************************
 * GetRemainingParameter - This routine will send to the file all the
 * parameters that weren't saved yet, because they weren't in the
 * configuration file before.
 */

static unsigned char *GetRemainingParameter (struct cfglist *list, struct cfglist *endlist)
{
    struct cfglist *vlist;
    unsigned char order;
    if (list->flag & FLAG_WRITTEN) return NULL;
    if (list->flag & FLAG_INACTIVE) return NULL;
    if (list->index & 0x0f) return NULL;
    order=list->index & 0xf0;
    if (list->getputrt) {
        *(list->fvalue)=0;
        for (vlist=list; vlist<endlist; vlist++) {
            if (strcmp (list->name, vlist->name)) continue;
            if (order!=(vlist->index & 0xf0)) break;
            vlist->flag |= FLAG_WRITTEN;
            (*(vlist->getputrt)) (list->fvalue, vlist->value, vlist->index & 0x0f, list->fsize);
        }
    } else {
        list->flag |= FLAG_WRITTEN;
    }
    return list->fvalue;
}

/*============================================================================
 * Routines for parsing/read/write the most common configuration file syntax
 * (host.conf, resolv.conf, cyras.conf).
 ============================================================================
 */
/**************************************************************************
 * ParseLineComArg - Parses line. The syntax of a configuration line is:
 *      <parameter name> <parameter value>
 */
static int ParseLineComArg(unsigned char *buf, unsigned char **argument)
{
    unsigned char *a, *s;
    s=buf;
    if (*s==0 || isspace(*s)) return 0;
    while (*s && !isspace(*s)) s++;
    if (*s) *s++=0;
    while (isspace(*s)) s++;
    if (*s==0) return 0;
    a=s;
    while (*s) s++;
    while (*s!=0 || isspace(*s)) s--;
    if (*s) *(s+1)=0;
    *argument=a;
    return 1;
}

/**************************************************************************
 * ReadWriteComArg - Reads(flag=0) or writes(flag=1) the file
 */
static int ReadWriteComArg(unsigned char *filename, struct cfglist *list , int nlist, int flag)
{
    FILE *arq;
    struct stat st;
    int i;
    struct cfglist *l;
    unsigned char *a, *buf, *t, *s;

    /* we will first read the file in a buffer */
    if (stat(filename, &st)) return 0;
    buf = (unsigned char *)malloc(st.st_size+1);
    if (!buf) return 0;
    arq = fopen (filename, "r");
    if (!arq) {
        free(buf);
        return 0;
    }
    fread(buf, 1, st.st_size, arq);
    *(buf+st.st_size) = 0;
    fclose(arq);
    /* flag=1 -> write / flag=0 ->read */
    if (flag) {
        arq = fopen (filename, "w");
    }
    s=buf;
    while (*s) {
	if (*s == '#') { // comments
		while (*s && *s!='\n') s++;
		continue;
	}
        t=s;
        while (*s && *s!='\n') s++;
        if (*s) *s++=0;
        if (!ParseLineComArg(t, &a)) continue;
        if (!flag) {
            SeekCheckConfig (t, a, list, nlist);
            continue;
        }
        /* seek the parameter in the list*/
        if ((l = SeekParameterInTheList (t, a, list, nlist)) != NULL) {
            if (l->fvalue[0]) {
                fprintf (arq, "%s %s\n", t, l->fvalue);
            }
        } else {
            fprintf (arq, t);
        }
    }

    if (flag) {
        for (i=0; i<nlist; i++) {
            if ((a = GetRemainingParameter (list+i, list+nlist))!=NULL) {
                if (*a) {
                    fprintf (arq, "%s %s\n", (list+i)->name, a);
                }
            }
        }
        fclose(arq);
    }
    free(buf);
    return 1;
}

/**************************************************************************
 * ReadWriteDNS - Reads(flag=0) or writes(flag=1) the file
 */
static int ReadWriteDNS(unsigned char *filename, struct cfglist *list , int nlist, int flag)
{
    FILE *arq;
    struct stat st;
    int i,search_flag;
    struct cfglist *l;
    unsigned char *a, *buf, *t, *s;

    /* we will first read the file in a buffer */
    if (stat(filename, &st)) return 0;
    buf = (unsigned char *)malloc(st.st_size+1);
    if (!buf) return 0;
    arq = fopen (filename, "r");
    if (!arq) {
        free(buf);
        return 0;
    }
    fread(buf, 1, st.st_size, arq);
    *(buf+st.st_size) = 0;
    fclose(arq);
    /* flag=1 -> write / flag=0 ->read */
    if (flag) {
        arq = fopen (filename, "w");
    }
    s=buf;
    while (*s) {
	if (*s == '#') { // comments
		while (*s && *s!='\n') s++;
		continue;
	}
        t=s;
        while (*s && *s!='\n') s++;
        if (*s) *s++=0;
	if (strncmp(t,"search",6) == 0) {
		t += 7;
		search_flag = 1;
	} else {
		search_flag = 0;
	}
        if (!ParseLineComArg(t, &a)) {
	    if (search_flag) fprintf(arq,"search ");
            fprintf (arq,t);
	    fprintf (arq,"\n");
		continue;
	}
        if (!flag) {
            SeekCheckConfig (t, a, list, nlist);
            continue;
        }
        /* seek the parameter in the list*/
        if ((l = SeekParameterInTheList (t, a, list, nlist)) != NULL) {
            if (l->fvalue[0]) {
		fprintf (arq, "%s%s %s\n",(search_flag)?"search ":"",
			       	t, l->fvalue);
            }
        } else {
	    if (search_flag) fprintf(arq,"search ");
            fprintf (arq,t);
	    fprintf (arq,"\n");
        }
    }

    if (flag) {
        for (i=0; i<nlist; i++) {
            if ((a = GetRemainingParameter (list+i, list+nlist))!=NULL) {
                if (*a) {
                    fprintf (arq, "%s %s\n", (list+i)->name, a);
                }
            }
        }
        fclose(arq);
    }
    free(buf);
    return 1;
}

/******************************************************************************
 * Routines to configure/retrieve hostname
 */
static int ReadHostName()
{
    FILE *arq;
    char conf_hostname[40];
    if ((arq = fopen("/etc/hostname", "r")) == NULL) return 0;
    fgets(conf_hostname, 40L, arq);
    SeekCheckConfig(NULL, conf_hostname, hostnamecfg, 1);
    fclose(arq);
    return 1;
}

static int SaveHostName(webs_t wp)
{
    FILE *arq;
    arq= fopen("/etc/hostname", "w");
    fprintf(arq,"%s",hostnamecfg[0].value);
	pslaveconflist[CONF_HOSTNAME].fvalue = 0;
    fclose(arq);
    return 1;
}

/******************************************************************************
 * Routines to configure/retrieve issue
 */
static int ReadIssue ()
{
    FILE *arq;
    char conf_issue[512];
    unsigned int size;
    if ((arq = fopen("/etc/issue", "r")) == NULL) return 0;
    size = fread(conf_issue, 1, 511, arq);
    conf_issue[size] = 0;
    SeekCheckConfig(NULL, conf_issue, issuecfg, 1);
    fclose(arq);
    return 1;
}

static int SaveIssue(webs_t wp)
{
    FILE *arq;
    arq = fopen("/etc/issue", "w");
    fprintf(arq,"%s",issuecfg[0].value);
    fclose(arq);
    return 1;
}

/******************************************************************************
 * Routines to retrieve/print/save parameters in host.conf
 */
/******************************************************************************
 * For the parameter order, from host.conf - The only options acceptable for
 * this configurator will be hosts and bind.
 */
static unsigned char *ListGetOrder (unsigned char *str, unsigned char *value, int arg, int size)
{
    unsigned char *t, *s, *u, *v, ch;
    if (value) {
        strncpy(str, value, size);
        return str;
    } else {
        s=t=str;
        while (1) {
            ch=*(++s);
            if (ch==',' || ch==0) {
                *s=0;
                if (t!=str) *(t-1)=',';
                if (strcmp(t, "hosts") && strcmp(t, "bind")) {
                    for (u=s+1,v=t; *(u-1); u++,v++) *v=*u;
                    s=t;
                } else {
                    s++;
                    t=s;
                }
            }
            if (ch==0) break;
        }
        return str;
    }
}

static int ReadHostConfig()
{
    return ReadWriteComArg("/etc/host.conf", hostlist, sizeof(hostlist) / sizeof (struct cfglist), 0);
}

static int SaveHostConfig (webs_t wp)
{
    return ReadWriteComArg("/etc/host.conf", hostlist, sizeof(hostlist) / sizeof (struct cfglist), 1);
}

/******************************************************************************
 * Routines to retrieve/print/save parameters in resolv.conf
 */
static int ReadResolvConfig()
{
    return ReadWriteDNS("/etc/resolv.conf", resolvlist, sizeof(resolvlist) / sizeof (struct cfglist), 0);
}

static int SaveResolvConfig (webs_t wp)
{
    return ReadWriteDNS("/etc/resolv.conf", resolvlist, sizeof(resolvlist) / sizeof (struct cfglist), 1);
}

/******************************************************************************
 * Routines to retrieve/print/save parameters in pslave.conf
 */
unsigned char pslave_filename_orig[] = "/etc/portslave/pslave.conf";
unsigned char *pslave_filename = pslave_filename_orig;

/******************************************************************************
 * For the parameters ipno and socket_port, from pslave.conf - separares the
 * parameter of increment from the IP addres or socket port. The syntax is:
 *    <ip number>[+] or <socket port>[+]
 */
static unsigned char *SeparateIncrement (unsigned char *str, unsigned char *value, int arg, int size)
{
    int inc = (*(str+strlen(str)-1) == '+')?1:0;
    if (value) {
        if (arg) {
            *(str+strlen(str)-inc)=0;
            strncat(str, value, size);
        } else {
            strncpy(str, value, size);
            if (inc) {
                strncat(str, "+", size);
            }
        }
        return str;
    } else {
        if (arg) {
            return str+strlen(str)-inc;
        } else {
            *(str+strlen(str)-inc)=0;
            return str;
        }
    }
}


/**************************************************************************
 * ParseLinePslave - Parses line. The syntax of a configuration line is:
 *      <word>.<command> <argument>
 * Lines which start with # or empty are disregarded; lines which end with \
 * will be linked with the next line.
 */
static int ParseLinePslave (unsigned char *buf, unsigned char **word,
                            unsigned char **command, unsigned char **argument)
{
    unsigned char *p, *s, *w, *c, *a;
    p = buf;
    if (p[0] == '#' || p[0] == '\n') return 0;
    s = p + strlen(p);
    if (s > p && *(s-1) == '\\') {
        return 2; //keep adding the line
    }
    if (buf[0] == '\n' || buf[0] == 0) {
        return 0;
    }
    /* Parse Line - w=word c=command a=argument */
    s = buf;
    for (w = s; *w; w++);
    while (--w > s && *w == '\n') *w = 0;
    while isspace (*s) s++;
    w = s;
    while (*s && !isspace(*s)) s++;
    if (*w==0) return 0;
    if (*s) *s++=0;
    while (isspace(*s)) s++;
    a = s;
    if ((c = strchr(w,'.')) == NULL || c[1] == 0) return 0;
    *c++ = 0;
    *word=w;
    *command=c;
    *argument=a;
    return 1;
}

/**************************************************************************
 * SetIncrementPslave - When the portslave port is not defined as "all", this
 * routine interprets the increment signal for ipno and socket_port parameters,
 * in the "all" configuration.
 */
static void SetIncrementIpno (struct cfglist *pslavelist, int port)
{
    if ((pslavelist+2)->value[0]=='+') {
        struct in_addr b;
        inet_aton((pslavelist+1)->value, &b);
        *((unsigned char *)(&b)+3) += port;
        strcpy ((pslavelist+1)->value, inet_ntoa(b));
        strcpy ((pslavelist+1)->fvalue, (pslavelist+1)->value);
    }
}

static void SetIncrementSocket (struct cfglist *pslavelist, int port)
{
    if ((pslavelist+4)->value[0]=='+') {
        int b;
        sscanf((pslavelist+3)->value, "%d", &b);
        sprintf((pslavelist+3)->value, "%d", b+port);
        strcpy ((pslavelist+3)->fvalue, (pslavelist+3)->value);
    }
}

/**************************************************************************
 * PrintLinePslave - Writes a line in the file
 */
static void PrintLinePslave (FILE *arq, unsigned char *w, unsigned char *c, unsigned char *a, unsigned char flag)
{
    unsigned char *s;
    if (*a && !flag) {
        for (s=a; *s; s++) {
            if (*s=='\r') *s='\\';
        }
        fprintf (arq, "%s.%s\t%s\n", w, c, a);
    }
}

/******************************************************************************
 * ReadWriteConfPslave - Reads (flag=0) or Writes (flag=1) the file pslave.conf.
 */
static void Dhcpc_2(void)
{
    FILE *cmd;
    unsigned char *file_conf = "/etc/config_files";
	unsigned char *file_cmd = "/etc/network/dhcpcd_cmd";
	unsigned char *file1 = "/etc/dhcpcd-eth0.save";
	unsigned char *temp = "/tmp/webtemp";
    struct stat st;
	int size;
	unsigned char buf[256], *s, *t;

    /* Read the file /etc/config_files */
    if (stat(file_conf, &st)) return;
	sprintf(buf,"grep \"%s\" %s > %s",file_cmd,file_conf,temp);
	system(buf);
	stat(temp, &st);
	if (st.st_size != strlen(file_cmd)+1) {
		sprintf(buf,"echo %s >> %s", file_cmd, file_conf);
		system(buf);
	}

	sprintf(buf,"grep \"%s\" %s > %s",file1,file_conf,temp);
	system(buf);
	stat(temp, &st);
	if (st.st_size != strlen(file1)+1) {
		sprintf(buf,"echo %s >> %s", file1, file_conf);
		system(buf);
	}

	if (stat(file_cmd, &st)) return;
	sprintf(buf,"grep \"\\-x\" %s >> %s",file_cmd,temp);
	system(buf);
	stat(temp, &st);
	system("rm -f /tmp/webtemp");
	if (st.st_size) return;
	cmd = fopen(file_cmd,"w");
    fprintf (cmd, "/sbin/dhcpcd -x -c /sbin/handle_dhcp");
	fclose(cmd);
}

static int ReadWriteConfPslave (int flag)
{
    FILE *arq;
    unsigned char *filename = pslave_filename;
    struct cfglist *l;
    unsigned char *s, *t, *a, *w, *c, *v, *grp="conf";
    int nparam, i, j, size;
    unsigned char *buf, *buf2;
    struct stat st;
	unsigned char flg_disc = 0;

    nparam=sizeof(pslaveconflist)/sizeof(struct cfglist);

	/* verify conf.include option */
	if (flag && pslaveconflist[CONF_INCL_FILE].fvalue[0]) {
		flg_disc = 1;
		nparam = CONF_INCL_FILE + 1;
	}

    /* Read the file and writes it to the buffer */
    if (stat(filename, &st)) return 0;
    size=st.st_size+1;
    if (flag) size*=2;
    buf = (unsigned char *)malloc(size);
    if (buf==NULL) return 0;
    arq = fopen(filename, "r");
    if (arq==NULL) {
        free(buf);
        return 0;
    }
    fread(buf, 1, st.st_size+1, arq);
    *(buf+st.st_size)=0;
    fclose(arq);

    /* Some things to initialize when the call is to write file: */
    if (flag) {
        /* buf2: spare buffer, used to write to the file */
        buf2 = buf+st.st_size+1;
        strncpy(buf2, buf, st.st_size+1);
	if (pslaveconflist[CONF_DHCPC].fvalue && 
		(pslaveconflist[CONF_DHCPC].fvalue[0] == '1' ||
		 pslaveconflist[CONF_DHCPC].fvalue[0] == '2')) {
		/* DHCPC active - comment all parameteres related to the Ethernet Interface */
		pslaveconflist[0].fvalue[0] = 0;	// ethernet ip address
		pslaveconflist[1].fvalue[0] = 0;
	//	pslaveconflist[2].fvalue[0] = 0;	[Fuji] Dec/19/02 - Don't comment the secondary IP/Mask.
	//	pslaveconflist[3].fvalue[0] = 0;
	}
    }

    if (flag) {
        arq=fopen(filename, "w");
    }

    t = s = buf;
    while (*s) {
        while (*s && *s!='\n') s++;
        if (*s) *s++=0;
        switch (ParseLinePslave (t, &w, &c, &a)) {
        case 2:
            *(s-2) = '\r';
            *(s-1) = '\n';
            break;

        case 1:
			if (strcmp(w, grp) == 0) { // word == conf.
            	if (!flag) { /* For read only ... */
                	SeekCheckConfig(c, a, pslaveconflist, nparam);
                	t = s;
                	break;
            	}

	            /* Seek the parameter in the list */
  		        if ((l = SeekParameterInTheList (c, a, pslaveconflist,nparam)) != NULL) {
        	         PrintLinePslave(arq, grp, c, l->fvalue, 0);
           		     t=s;
                	 break;
            	}
			}
            /* If not found, just writes the line */
        default:
            if (flag && !flg_disc) {
                fwrite (buf2+(t-buf), 1, (s-t), arq);
            }
            t = s;
            break;
        }
    }

    /* Writes the rest of the parameters which were not defined in the
     configuration file before. */
    if (flag) {
		if (flg_disc) { /* only conf.include */
			if (!(pslaveconflist[CONF_INCL_FILE].flag & FLAG_WRITTEN)) {
				PrintLinePslave(arq, grp, pslaveconflist[CONF_INCL_FILE].name,
								pslaveconflist[CONF_INCL_FILE].fvalue,0);
			}
		} else {
    		for (i=0; i<nparam; i++) {
        		if (v=GetRemainingParameter(pslaveconflist+i,pslaveconflist+i))
        			PrintLinePslave(arq, grp, (pslaveconflist+i)->name,v, 0);
			}
        }
        fclose(arq);
		//[RK] dhcpc new value
		if (pslaveconflist[CONF_DHCPC].fvalue[0] == 0x32) {
			// need to changes /etc/config_files and /etc/network/dhcpcd_cmd
			Dhcpc_2();
		} 
    }
    free(buf);
    return 1;
}

/**************************************************************************
 * ReadWritePslave - Reads (flag=0) or Writes (flag=1) the file pslave.conf.
 */
static int ReadWritePslave (int flag)
{
    FILE *arq;
    unsigned char filename[40];
    struct cfglist *pslavelist, *pslavelist2, *l;
    unsigned char group[8], *s, *t, *a, *w, *c, *v;
    int npslaveent, i, j, analyse_flag=1, discard_flag=0, size;
    unsigned char ttyactive[MAX_SERIAL_PORTS];
    unsigned char *buf, *buf2;
    struct stat st;
	unsigned char flg_find=0, hostname[40];
	unsigned char discard_spec=0;

	pslave_incl = 0;

	strcpy(filename, pslave_filename);
    /* For read, initialize the ttyl structure */
    if (!flag) {
        ttylist[0] = pvoid;
        ttyshow[0] = pinactive;
		i=0;
		if (pslaveconfigport <= MAX_SERIAL_PORTS) {
	        for (; i<MAX_SERIAL_PORTS; i++) {
    	        ttylist[i+1]=ttyshow[i+1]=ttystr+i*8;
        	}
		}
        ttylist[i+1] = ttyshow[i+1] = premote;
        ttyl.nelements = i+2;
    }
    /* Read the file pslave.conf and writes it to the buffer */
    if (stat(filename, &st)) return 0;
    size=st.st_size+1;
    if (flag) size*=2;
    buf = (unsigned char *)malloc(size);
    if (buf==NULL) return 0;
    arq = fopen(filename, "r");
    if (arq==NULL) {
        free(buf);
        return 0;
    }
    fread(buf, 1, st.st_size+1, arq);
    *(buf+st.st_size)=0;
    fclose(arq);

	/* verify conf.include */
	if (!flag) { 	
		hostname[0] = 0;
    	t = s = buf;
    	while (*s) {
        	while (*s && *s!='\n') s++;
        	if (*s) *s++=0;
        	if ((ParseLinePslave (t, &w, &c, &a) == 1) && 
				(strcmp(w, "conf")==0)) { // word == conf.
				if (strcmp(c,"include") == 0) {
					pslave_incl = 1;
					strcpy(filename,a);
				} else {
					if (strcmp(c,"hostname") == 0) { 
						strcpy(hostname,a);
					}
				}
     	    }
        	t = s;
		}
		if (pslave_incl) {
			/* get hostname */
			if (hostname[0] == 0) {
    			if ((arq = fopen("/etc/hostname", "r")) == NULL) return 0;
    			fgets(hostname, 40L, arq);
				fclose(arq);
			}
        	free(buf);
    		if (stat(filename, &st)) return 0;
    		size=st.st_size+1;
    		if (flag) size*=2;
    		buf = (unsigned char *)malloc(size);
    		if (buf==NULL) return 0;
			flag = 0;
			flg_find = 0;
        	(pslaveportlist+PORT_REMADDR_INC)->flag |= FLAG_INACTIVE; 
			/*ipno increment*/
        	(pslaveportlist+PORT_TCPPORT_INC)->flag |= FLAG_INACTIVE; 
			/*socket port increment*/
		}
	} else {

	    /* Some things to initialize when the call is to write file: */
        /* buf2: spare buffer, used to write to the file */
        buf2 = buf+st.st_size+1;
        strncpy(buf2, buf, st.st_size+1);
			
       	(pslaveportlist+PORT_DISCSPEC)->flag |= FLAG_WRITTEN;
		if ((pslaveportlist+PORT_DISCSPEC)->fvalue[0] == '1') {
			discard_spec=1;
		}
        if (pslaveconfigport>0 && pslaveconfigport<=MAX_PSLAVE_PORTS) {
            /* pslavelist2: backup of pslaveportlist, where the configuration
             * for "all" will be stored. This will be used to compare the port
             * configuration with the "all" configuration, and only the
             * parameters that are different will be written */
            pslavelist2=malloc (sizeof(pslaveportlist));
            if (pslavelist2==NULL) {
                free(buf);
                return 0;
            }
            memcpy (pslavelist2, pslaveportlist, sizeof(pslaveportlist));
            if (!AllocConfig(pslavelist2, sizeof(pslaveportlist)/sizeof(struct cfglist))) {
                free(pslavelist2);
                free(buf);
                return 0;
            }
            /* If the port is inactive, disregard all the parameters related
             * to this port */
            if ((pslaveportlist+PORT_TTY)->value[0]==0) discard_flag=1;
        }
    }

    /* Initializes the list, and the group name according to the variable
     * pslaveconfigport */
    pslavelist=pslaveportlist;
    npslaveent=sizeof(pslaveportlist)/sizeof(struct cfglist);
    if (pslaveconfigport) {
        sprintf(group, "s%d", pslaveconfigport);
        for (i=0; i<MAX_SERIAL_PORTS; i++) ttyactive[i]=0;
    } else {
        strcpy(group, "all");
    }

	for (i=0; i < npslaveent ; i++) {
    		if (pslave_incl) {
			(pslavelist + i)->config |= CONFIG_READ_ONLY;
		} else {
			(pslavelist + i)->config &= ~CONFIG_READ_ONLY;
		}
	}		

    arq = fopen(filename, "r");
    fread(buf, 1, st.st_size+1, arq);
    *(buf+st.st_size)=0;
    fclose(arq);

    if (flag) {
        arq=fopen(filename, "w");
	}
    /* Sequence to parse the pslave.conf file. It was copied from the
     file tslinux/ras/src/rwconf.c. */
    t = s = buf;
    while (*s) {
        while (*s && *s!='\n') s++;
        if (*s) *s++=0;
        switch (ParseLinePslave (t, &w, &c, &a)) {
        case 2:
            *(s-2) = '\r';
            *(s-1) = '\n';
            break;

        case 1:
			if (pslave_incl) {
				if (!strcmp(w, "conf") && !strcmp(c, "host_config")) {
					if (flg_find) {
						free(buf);
						return 1;
					}
					if (!strcmp(a, hostname)) {
						flg_find = 1;
					}
					break;					
				}
				if (!flg_find) break;
			}
			
            /* If there is no more need to analyse the line, we could just
             * copy it to the file */
            if (flag && !analyse_flag) {
				if (!(*w=='s') || !strcmp(c,"tty") || !discard_spec){
                	fwrite (buf2+(t-buf), 1, (s-t), arq);
				}
                t = s;
                break;
            }

            /* This logic is to verify the device where the logical port is
             connected. */
            if (*w=='s') {
				if (strcmp(c, "tty")==0) {
                	if (strcmp (w, group) == 0) {
                    	for (i=0; i<npslaveent; i++)
                        	(pslavelist+i)->flag &= ~FLAG_READ;
                	} else {
                    	for (i=0; i<MAX_SERIAL_PORTS; i++) {
                        	if (strcmp(a, ttystr+i*8) == 0) {
                            	ttyactive[i]=1;
                            	break;
                        	}
                    	}
                	}
				} else {
					if (flag && discard_spec) {
                       	(pslavelist+i)->flag |= FLAG_WRITTEN;
                		t = s;
                		break;
					}
				}
            }

            if (!flag) {
                /* For read only ... */
                if ((strcmp(w, group) == 0) || (strcmp(w, "all") == 0 && *group=='s')) {
                    SeekCheckConfig(c, a, pslavelist, npslaveent);
                    if (strcmp(w, "all")==0 && *group=='s') {
                    	if (strcmp(c, "ipno")==0) {
		                    SetIncrementIpno(pslavelist, pslaveconfigport-1);
                    	}
                    	if (strcmp(c, "socket_port")==0) {
       	                    SetIncrementSocket(pslavelist, pslaveconfigport-1);
                    	}
                    }
                }
                t = s;
                break;
            }

            /* if the file configuration is "all" and a singular port is being
             configured, read "all" configuration */
            if (strcmp(w, "all")==0 && *group=='s') {
                SeekCheckConfig (c, a, pslavelist2, npslaveent);
                if (strcmp(c, "ipno")==0) {
		            SetIncrementIpno(pslavelist2, pslaveconfigport-1);
                }
                if (strcmp(c, "socket_port")==0) {
       	          	SetIncrementSocket(pslavelist2, pslaveconfigport-1);
                }
            }
            /* When it is going to the next "phase" add the remaining parameters
             before going to the next one.*/
            if (pslaveconfigport==0 && strcmp(w, "conf") && strcmp(w, "all")) {
                analyse_flag=0;
                for (i=0; i<npslaveent; i++) {
                    if ((v = GetRemainingParameter(pslavelist+i, pslavelist+npslaveent))==NULL)
                        continue;
                    PrintLinePslave(arq, group, (pslavelist+i)->name, v, discard_flag);
                }
            }
            /* Seek the parameter in the list */
            if (strcmp(w, group) == 0) {
                if ((l = SeekParameterInTheList (c, a, pslavelist, npslaveent)) != NULL) {
                    if (*group!='s' || strcmp(l->fvalue, (pslavelist2+(l-pslavelist))->fvalue)) {
                        PrintLinePslave(arq, group, c, l->fvalue, discard_flag);
                    }
                    t=s;
                    break;
                }
            }
            /* If not found, just writes the line */
        default:
            if (flag) {
                fwrite (buf2+(t-buf), 1, (s-t), arq);
            }
            t = s;
            break;
        }
    }
    /* For read, assembles the new set of possible serial ports */
    if (!flag) {
        ttylist[0] = pvoid;
        ttyshow[0] = pinactive;
		j=1;
		if (pslaveconfigport<=MAX_SERIAL_PORTS) {
	        for (i=0; i<numsports; i++) {
    	        if (! ttyactive[i]) {
        	        ttylist[j]=ttyshow[j++]=ttystr+i*8;
            	}
			}
        }
        ttylist[j] = ttyshow[j] = premote;
        ttyl.nelements = j + 1;
    }
    /* Writes the rest of the parameters which were not defined in the
     configuration file before. */
    if (flag) {
        if (*group == 's') {
            for (i=0; i<npslaveent; i++) {
                if ((v = GetRemainingParameter(pslavelist+i, pslavelist+npslaveent))==NULL)
                    continue;
                if (strcmp(v, (pslavelist2+i)->fvalue)==0)
                    continue;
                PrintLinePslave(arq, group, (pslavelist+i)->name, v, discard_flag);
            }
        }
        if (pslaveconfigport>0 && pslaveconfigport<=MAX_PSLAVE_PORTS) {
            free(pslavelist2);
        }
        fclose(arq);
    }

    free(buf);
    return 1;
}

static int ReadPslaveConfig()
{
	if (pslaveconfigport == -1) {
		return ReadWriteConfPslave(0);
	} else {
    	return ReadWritePslave (0);
	}
}

static int SavePslaveConfig (webs_t wp)
{
	if (pslaveconfigport == -1) {
		return ReadWriteConfPslave(1);
	} else {
    	return ReadWritePslave (1);
	}
}

static void ShowTitle(int eid, webs_t wp, int argc, char_t **argv)
{
	unsigned char *value;
	value=websGetVar(wp, "name", "1");
	websWrite(wp, "%s", value);
}

static int ShowTemperatureApplet(int eid, webs_t wp, int argc, char_t **argv)
{
	unsigned char *value;

    value=websGetVar(wp, "port", "1");
    tcp_port=atoi(value) + 600;

	if (tcp_port == 602) {
		websWrite(wp, "<applet CODEBASE=\".\" ARCHIVE=\"temperature.jar\" CODE=\"GrapherApplet.class\" WIDTH=\"860\" HEIGHT=\"550\">"); 
	}
	if (tcp_port == 601) {
		websWrite(wp, "<applet CODEBASE=\".\" ARCHIVE=\"temp.jar\" CODE=\"GrapherApplet.class\" WIDTH=\"860\" HEIGHT=\"550\">"); 
	}
	websWrite(wp, "<param name=\"port\" value=\"%d\">" , tcp_port);
	websWrite(wp, "</applet>");

	return 0;
}

static int ShowApplet(int eid, webs_t wp, int argc, char_t **argv)
{
	unsigned char *value;
	unsigned char *header = "Serial Port Configuration";
	unsigned char *error1 = "Error - Current protocol configuration prohibits connecting to serial ports.";
	unsigned char *error2 = "Error - No socket_port configured for this port.";
	char_t *aname, nparam;

	nparam=ejArgs(argc, argv, T("%s"), &aname);

  value=websGetVar(wp, "port", "1");
  pslaveconfigport=atoi(value);

	if ((lineconf[pslaveconfigport-1].protocol != P_SOCKET_SERVER) &&
        (lineconf[pslaveconfigport-1].protocol != P_SOCKET_SSH) &&
        (lineconf[pslaveconfigport-1].protocol != P_SOCKET_SERVER_RAW)) {
			websWrite(wp, "<table border align=\"center\"> <tr><th bgcolor=\"#336699\">"
				"<font color=\"#FFFFFF\" face=\"Verdana, Arial, Helvetica, sans-serif\""
				"size=\"2\">%s</th></tr>"
				"<tr><td bgcolor=\"#CCCCCC\"><font color=\"#ff0000\"><b>"
				"<font face=\"Verdana, Arial, Helvetica, sans-serif\" size=\"2\">"
				"%s</font></b></font></td></tr></table>", header, error1);
        return 1;
    }

	if (lineconf[pslaveconfigport-1].protocol == P_SOCKET_SERVER ||
        lineconf[pslaveconfigport-1].protocol == P_SOCKET_SERVER_RAW) {
    if(!lineconf[pslaveconfigport-1].socket_port) {
			websWrite(wp, "<table border align=\"center\"> <tr><th bgcolor=\"#336699\">"
				"<font color=\"#FFFFFF\" face=\"Verdana, Arial, Helvetica, sans-serif\""
				"size=\"2\">%s</th></tr>"
				"<tr><td bgcolor=\"#CCCCCC\"><font color=\"#ff0000\"><b>"
				"<font face=\"Verdana, Arial, Helvetica, sans-serif\" size=\"2\">"
				"%s</font></b></font></td></tr></table>", header, error2);
			return 1;
		}
	}

#ifdef OEM1
	websWrite(wp, "<applet CODEBASE=\".\" ARCHIVE=\"jta20.jar\" CODE=\"de.mud.jta.Applet\" WIDTH=%s HEIGHT=%s>", strcmp(aname, "applet.conf") ? "590" : "590", strcmp(aname, "applet.conf") ? "670" : "460");
#else
	websWrite(wp, "<applet CODEBASE=\".\" ARCHIVE=\"jta25.jar\" CODE=\"de.mud.jta.Applet\" WIDTH=%s HEIGHT=%s>", strcmp(aname, "applet.conf") ? "590" : "590", strcmp(aname, "applet.conf") ? "670" : "460");
#endif
	websWrite(wp, "<param name=\"config\" value=\"%s\">"
		"<param name=\"proto\" value=\"%d\">"
		,aname, lineconf[pslaveconfigport-1].protocol);

	if (lineconf[pslaveconfigport-1].protocol == P_SOCKET_SSH) {
		websWrite(wp, "<param name=\"plugins\" value=\"%s\">", "Status,Socket,SSH,ButtonBar,Terminal");
		if(!lineconf[pslaveconfigport-1].socket_port) {
			if(!lineconf[pslaveconfigport-1].serverfarm) 
				websWrite(wp, "<param name=\"port\" value=\"%s\">", lineconf[pslaveconfigport-1].tty);
			else
				websWrite(wp, "<param name=\"port\" value=\"%s\">", lineconf[pslaveconfigport-1].serverfarm);
		} else {
			websWrite(wp, "<param name=\"port\" value=\"%d\">", lineconf[pslaveconfigport-1].socket_port);
		}
	} else if (lineconf[pslaveconfigport-1].protocol == P_SOCKET_SERVER ||
				lineconf[pslaveconfigport-1].protocol == P_SOCKET_SERVER_RAW) {
		websWrite(wp, "<param name=\"plugins\" value=\"%s\">", "Status,Socket,Telnet,ButtonBar,Terminal");
		if(!lineconf[pslaveconfigport-1].socket_port) {
			PrintResultTable(wp, "Serial Port Connection", "No socket_port configured.", 1);
		} else {
			websWrite(wp, "<param name=\"port\" value=\"%d\">", lineconf[pslaveconfigport-1].socket_port);
		}
	}
	websWrite(wp, "<param name=\"breakseq\" value=\"%s\">", lineconf[pslaveconfigport-1].break_seq);
	websWrite(wp, "</applet>");

	return 0;
}

/******************************************************************************
 * Routines related to the page config/general.asp
 */
static int AllocGeneralConfig()
{
    pslaveconfigport=-1;
    if (!AllocConfig(hostnamecfg,    sizeof(hostnamecfg)   /sizeof(struct cfglist))) return 0;
/*    if (!AllocConfig(consspdcfg,     sizeof(consspdcfg)    /sizeof(struct cfglist))) return 0; */
    if (!AllocConfig(issuecfg,       sizeof(issuecfg)      /sizeof(struct cfglist))) return 0;
/*    if (!AllocConfig(cyraslist,      sizeof(cyraslist)     /sizeof(struct cfglist))) return 0;*/
    if (!AllocConfig(hostlist,       sizeof(hostlist)      /sizeof(struct cfglist))) return 0;
    if (!AllocConfig(resolvlist,     sizeof(resolvlist)    /sizeof(struct cfglist))) return 0;
    if (!AllocConfig(pslaveconflist, sizeof(pslaveconflist)/sizeof(struct cfglist))) return 0;
    return 1;
}

static void DeallocGeneralConfig()
{
    DeallocConfig(hostnamecfg);
/*    DeallocConfig(consspdcfg); */
    DeallocConfig(issuecfg);
/*    DeallocConfig(cyraslist);*/
    DeallocConfig(hostlist);
    DeallocConfig(resolvlist);
    DeallocConfig(pslaveconflist);
}

static void GetGeneralConfigFromForm(webs_t wp)
{
    GetConfigFromForm(wp, hostnamecfg,    sizeof(hostnamecfg)   /sizeof(struct cfglist));
/*    GetConfigFromForm(wp, consspdcfg,     sizeof(consspdcfg)    /sizeof(struct cfglist)); */
    GetConfigFromForm(wp, issuecfg,       sizeof(issuecfg)      /sizeof(struct cfglist));
/*    GetConfigFromForm(wp, cyraslist,      sizeof(cyraslist)     /sizeof(struct cfglist));*/
    GetConfigFromForm(wp, hostlist,       sizeof(hostlist)      /sizeof(struct cfglist));
    GetConfigFromForm(wp, resolvlist,     sizeof(resolvlist)    /sizeof(struct cfglist));
    GetConfigFromForm(wp, pslaveconflist, sizeof(pslaveconflist)/sizeof(struct cfglist));
}

static int OpenGeneralConfig(int eid, webs_t wp, int argc, char_t **argv)
{
	int i;
        struct cyc_webs_header hdr;
	websCycHdrInit(&hdr);
        hdr.title = "General Configuration";
        hdr.form = "cfg/WriteGeneralConfig";
        websPageStart(wp, &hdr);

    if (ncfgerr) {
	    websWrite(wp, T("<b class=\"err\">Configuration Error - The wrong parameters are marked with red.</b><br><br>"));
        return 0;
    }

    if (!AllocGeneralConfig()) {
        websError(wp, 400, T("Allocation Error\n"));
        return -1;
    }


    DefaultConfig(hostnamecfg,    sizeof(hostnamecfg)   /sizeof(struct cfglist));
/*    DefaultConfig(consspdcfg,     sizeof(consspdcfg)    /sizeof(struct cfglist)); */
    DefaultConfig(issuecfg,       sizeof(issuecfg)      /sizeof(struct cfglist));
/*    DefaultConfig(cyraslist,      sizeof(cyraslist)     /sizeof(struct cfglist));*/
    DefaultConfig(hostlist,       sizeof(hostlist)      /sizeof(struct cfglist));
    DefaultConfig(resolvlist,     sizeof(resolvlist)    /sizeof(struct cfglist));
    DefaultConfig(pslaveconflist, sizeof(pslaveconflist)/sizeof(struct cfglist));

    ReadHostName();
/*    ReadConsSpeed(); */
    ReadIssue();
/*    ReadCyRasConfig();*/
    ReadHostConfig();
    ReadResolvConfig();
    ReadPslaveConfig();
    for (i = 0; i < sizeof(hostnamecfg)/sizeof(struct cfglist) ; i ++) {
	if (websGetPriority(wp) != PRIV_FULL) {
	    (hostnamecfg + i)->config |= CONFIG_READ_ONLY;
	} else {
	    (hostnamecfg + i)->config &= ~CONFIG_READ_ONLY;
	}
    }
    for (i = 0; i < sizeof(issuecfg)/sizeof(struct cfglist) ; i ++) {
	if (websGetPriority(wp) != PRIV_FULL) {
	    (issuecfg + i)->config |= CONFIG_READ_ONLY;
	} else {
	    (issuecfg + i)->config &= ~CONFIG_READ_ONLY;
	}
    }
    for (i = 0; i < sizeof(hostlist)/sizeof(struct cfglist) ; i ++) {
	if (websGetPriority(wp) != PRIV_FULL) {
	    (hostlist + i)->config |= CONFIG_READ_ONLY;
	} else {
	    (hostlist + i)->config &= ~CONFIG_READ_ONLY;
	}
    }
    for (i = 0; i < sizeof(resolvlist)/sizeof(struct cfglist) ; i ++) {
	if (websGetPriority(wp) != PRIV_FULL) {
	    (resolvlist + i)->config |= CONFIG_READ_ONLY;
	} else {
	    (resolvlist + i)->config &= ~CONFIG_READ_ONLY;
	}
    }
    for (i = 0; i < sizeof(pslaveconflist)/sizeof(struct cfglist) ; i ++) {
	if (websGetPriority(wp) != PRIV_FULL) {
	    (pslaveconflist + i)->config |= CONFIG_READ_ONLY;
	} else {
	    (pslaveconflist + i)->config &= ~CONFIG_READ_ONLY;
	}
    }
    return 0;
}

static int PrintGeneralConfig(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t *name, *param, nparam;
    int ret=0;
    nparam=ejArgs(argc, argv, T("%s %s"), &name, &param);
    if (nparam>=1) {
        if (strcmp(name, "hostname") == 0) {
            ret=WriteConfig(wp, hostnamecfg, 1, NULL);
/*        } else if (strcmp(name, "consspeed") == 0) {
            ret=WriteConfig(wp, consspdcfg, 1, NULL); */
        } else if (strcmp(name, "issue") == 0) {
            ret=WriteConfig(wp, issuecfg, 1, NULL);
/*        } else if (strcmp(name, "cyras") == 0) {
            if (nparam>=2)
                ret=WriteConfig(wp, cyraslist, sizeof(cyraslist)/sizeof(struct cfglist), param);*/
        } else if (strcmp(name, "host") == 0) {
            if (nparam>=2)
                ret=WriteConfig(wp, hostlist, sizeof(hostlist)/sizeof(struct cfglist), param);
        } else if (strcmp(name, "dns") == 0) {
            if (nparam>=2)
                ret=WriteConfig(wp, resolvlist, sizeof(resolvlist)/sizeof(struct cfglist), param);
        } else if (strcmp(name, "pslave") == 0) {
            if (nparam>=2) {
                ret=WriteConfig(wp, pslaveconflist, sizeof(pslaveconflist)/sizeof(struct cfglist), param);
            }
        }
    }
    if (!ret) {
        return -1;
    }
    return 0;
}

/*static char *ip_to_asc(char *p, int ipno)
{
   p += sprintf(p, "%d", (ipno >> 24) & 0xff);
   p += sprintf(p, ".%d", (ipno >> 16) & 0xff);
   p += sprintf(p, ".%d", (ipno >> 8) & 0xff);
   p += sprintf(p, ".%d", ipno & 0xff);
   return(p);
}*/

//[RK]Nov/19/02 Bug#31 - to synchronize hostname and IP addres with /etc/hosts file
static void synchosts(void)
{
	FILE *arq;
	unsigned char filename[] = "/etc/hosts";
	unsigned char *buf, *s, *t, *a;
	struct stat st;	
	int size;
	int flag = 0;
	char temp[128];
	//char *p = temp;

	//p = ip_to_asc(p, mainconf.eth_ip);
	
	if (stat(filename, &st)) return;
	size = st.st_size+1;
	buf = (unsigned char *)malloc(size);
	if (buf==NULL) return;
	arq = fopen(filename, "r");
	if (arq==NULL) {
		free(buf);
		return;
	}
	fread(buf, 1, st.st_size+1, arq);
	*(buf+st.st_size)=0;

	fclose(arq); 

	arq = fopen (filename, "w");

	s = buf;   
	while (*s) {
		if (*s == '#') { // comments
			fprintf(arq, "%s\n", s);   
			while (*s && *s!='\n') s++;
			continue;
		}			
		t=s;
		while (*t && !(isalnum(*t))) t++;
		while (*s && *s!='\n') s++;
		if (*s) *s++=0;
		if (ParseLineComArg (t, &a) == 1) { 
				if (!strcmp(a,hostnamecfg[0].value)) {
					if (!flag) {
						if (*pslaveconflist[0].value && *hostnamecfg[0].value)
							fprintf(arq, "%s	%s\n", pslaveconflist[0].value, hostnamecfg[0].value);
						flag = 1;
					}
				} else {
					if (!strcmp(t, pslaveconflist[0].value)) {
						if (!flag) {
							if (*pslaveconflist[0].value && * hostnamecfg[0].value)
								fprintf(arq, "%s	%s\n", pslaveconflist[0].value, hostnamecfg[0].value);
							flag = 1;
						}
					} else {
						fprintf(arq, "%s	%s\n", t, a);	
					}
				}
		 }
   	}
	if(!flag) {
		if (pslaveconflist[CONF_DHCPC].fvalue != NULL && pslaveconflist[CONF_DHCPC].fvalue[0] == 0x30)
			if (*pslaveconflist[0].value && *hostnamecfg[0].value)
				fprintf(arq, "%s  %s\n", pslaveconflist[0].value, hostnamecfg[0].value);
	}

	fclose(arq);
	free(buf);
}

static void SaveGeneralConfig(webs_t wp)
{
    SaveHostName(wp);
/*    SaveConsSpeed(wp); */
    SaveIssue(wp);
/*  SaveCyRasConfig(wp);*/
    SaveHostConfig(wp);
    SaveResolvConfig(wp);
    SavePslaveConfig(wp);
	//[RK]Nov/19/02 Bug#31 - to synchronize hostname and IP addres with /etc/hosts file
	synchosts();
}

static int CloseGeneralConfig(int eid, webs_t wp, int argc, char_t **argv)
{
    struct cyc_webs_header hdr;
    if (websGetPriority(wp) == PRIV_FULL) {
	websWrite(wp, "<br><center><input type=\"submit\" value=\"Submit\">"
	    "&nbsp;&nbsp;&nbsp;&nbsp;<input type=\"reset\" value=\"Reset\">"
	    "</center>");
    }
    ncfgerr=0;
    DeallocGeneralConfig();
	websCycHdrInit(&hdr);
        hdr.form = "cfg/WriteGeneralConfig";
	websPageEnd(wp, &hdr);
    return 0;
}

static void WriteGeneralConfig(webs_t wp, char_t *path, char_t *query)
{
    if (!AllocGeneralConfig()) return;
    ncfgerr=0;
    GetGeneralConfigFromForm(wp);
    if (!ncfgerr) {
        SaveGeneralConfig(wp);
        websRedirect(wp, T("/read/general.asp"), NULL);
        DeallocGeneralConfig();
    } else {
        websRedirect(wp, T("/read/general.asp"), NULL);
    }
}

/******************************************************************************
 * Routine related to the page /appl/temperaturePorts.asp
 */

static int PrintPortsforTemperature(int eid, webs_t wp, int argc, char_t **argv)
{
    int i;

    websWrite(wp, "<center><table border align=\"center\"><tr><th class=\"tt\">Analog Port to read sensor from"
										"</th></tr><tr><td>");
    websWrite(wp, T("<select name=\"port\">"));
    for (i=1; i<=2; i++) {
        websWrite(wp, T("<option value=\"%d\">Port %d"), i, i);
    }
    websWrite(wp, "</select>"
		"</td></tr></table></center>"
		"<br><center><input type=\"button\" value=\"Read\" onClick=\"newWindow(this.form)\"></center>");
    return 0;
}

/******************************************************************************
 * Routines related to the page /config/serports.asp
 */

static int PrintPslavePort(int eid, webs_t wp, int argc, char_t **argv)
{
    int i;
    websWrite(wp, T("<select name=\"port\">"));
    websWrite(wp, T("<option value=\"0\" selected>All ports"));
    for (i=1; i<=MAX_PSLAVE_PORTS; i++) {
        websWrite(wp, T("<option value=\"%d\">Port %d"), i, i);
    }
    websWrite(wp, T("</select>"));
    return 0;
}

/* Obtain the active serial ports configured in pslave and display it in web */
static int PrintPslavePortforConnection(int eid, webs_t wp, int argc, char_t **argv)
{
  int i;
	int ttyConfigured = FALSE;
	int firsttime = TRUE;
	struct line_cfg *pconf;
 
	for (i=0, pconf=&lineconf[0]; i < MAXVLINES; i++, pconf++) {

		if (pconf->this != i) {
			continue;
		}

		if (!pconf->tty || !*pconf->tty) {
			continue;
		} 

		if (firsttime) {
			websWrite(wp, "<center><table border align=\"center\"><tr><th class=\"tt\">Serial Port Connection"
										"</th></tr><tr><td>");
			websWrite(wp, T("<b>Logical Port: </b>"));
			websWrite(wp, T("<select name=\"port\">"));
			firsttime = FALSE;
		}
		ttyConfigured = TRUE;

		if(pconf->serverfarm) {
        	websWrite(wp, T("<option value=\"%d\">%s"), i+1, pconf->serverfarm);
		} else {
			if (pconf->virtual) 
       	    	websWrite(wp, T("<option value=\"%d\">Remote Port %d"), i+1, i+1);
			else
				websWrite(wp, T("<option value=\"%d\">Port %d"), i+1, i+1);
		}

	}
	if (!ttyConfigured) {
		PrintResultTable(wp, "Serial Port Connection", "No ports configured for serial port connection.", 1);
	} else {
		websWrite(wp, "</select>"
			"</td></tr></table></center>"
			"<br><center><input type=\"button\" value=\"Connect\" onClick=\"newWindow(this.form)\"></center>");
  }

    return 0;
}

static void SetPslavePort(webs_t wp, char_t *path, char_t *query)
{
    unsigned char *value;
    value=websGetVar(wp, "port", "0");
    pslaveconfigport=atoi(value);
    websRedirect(wp, T("/read/sportcfg.asp"), NULL);
}

/******************************************************************************
 * Routines related to the page /config/sportcfg.asp
 */
#ifndef OEM1
unsigned char *wizard_name[] = {"CAS", "TS", "Dial-In", 
#ifdef WEBS_PMD
	"AlterPath PM", 
#endif
#ifdef TS1H
	"Automation", 
#endif
};

unsigned char *wizard_filename[] = {
	"/etc/portslave/pslave.wiz.cas",
	"/etc/portslave/pslave.wiz.ts",
	"/etc/portslave/pslave.wiz.ras",
#ifdef WEBS_PMD
	"/etc/portslave/pslave.wiz.pm8",
#endif
#ifdef TS1H
	"/etc/portslave/pslave.wiz.auto"
#endif
};
unsigned char num_wizards = sizeof(wizard_filename)/sizeof(unsigned char *);
#endif

static int OpenSportConfig(int eid, webs_t wp, int argc, char_t **argv)
{
    unsigned char title[128];
    int i;
    struct cyc_webs_header hdr;
    if (pslaveconfigport<0 || pslaveconfigport>MAX_PSLAVE_PORTS) {
        websError(wp, 400, T("Wrong Serial Port"));
        return -1;
    }

    if (pslaveconfigport) {
        sprintf(title, "Serial Port #%d Configuration", pslaveconfigport);
    } else {
        sprintf(title, "Configuration - All Serial Ports");
    }
   	
	websCycHdrInit(&hdr);
        hdr.title = title;
        hdr.form = "cfg/WriteSportConfig";
	hdr.links = "&nbsp;<a href=\"/read/serports.asp\">"
		"[Serial Port Configuration]</a>";
	hdr.father = "/read/serports.asp";
        websPageStart(wp, &hdr);

    websWrite (wp, "<input type=\"hidden\" name=\"port\" value=\"%d\">", pslaveconfigport);

    if (ncfgerr) {
        websWrite(wp, T("<br><b class=\"err\"><center>"));
#ifndef OEM1
        if ((ncfgerr & 0xf8) == 0xf8) {
            websWrite(wp, T("The configuration of the %s profile was loaded."
				"<br>Check the parameters which are to be changed "
				"and press Submit."), wizard_name[ncfgerr & 0x07]);
        } else {
            websWrite(wp, T("Configuration Error - The wrong parameters "
				"are marked with red."));
        }
#endif
        websWrite(wp, T("</center></b><br><br>"));
    }
    if (!pslave_incl && websGetPriority(wp) == PRIV_FULL) {
        websWrite(wp, "<center>"
	    "<input type=\"submit\" name=\"sub\" value=\"Submit\">"
	    "&nbsp;&nbsp;&nbsp;&nbsp;"
	    "<input type=\"reset\" value=\"Reset\"></center><br>");
    }
	if (ncfgerr) {
        return 0;
	}
    if (!AllocConfig(pslaveportlist, sizeof(pslaveportlist)/sizeof(struct cfglist)))
    {
        websError(wp, 400, T("Allocation Error\n"));
        return -1;
    }
    if (pslaveconfigport) {
        (pslaveportlist+PORT_REMADDR_INC)->flag |= FLAG_INACTIVE; 
		/*ipno increment*/
        (pslaveportlist+PORT_TCPPORT_INC)->flag |= FLAG_INACTIVE; 
		/*socket port increment*/
        (pslaveportlist+PORT_DISCSPEC)->flag |= FLAG_INACTIVE; 
		/*discard specific param*/
    } else {
        (pslaveportlist+PORT_TTY)->flag |= FLAG_INACTIVE; /*tty*/
        (pslaveportlist+PORT_SERVERFARM)->flag |= FLAG_INACTIVE; /*server farm*/
        (pslaveportlist+PORT_DISCSPEC)->value[0] = '0'; 
		/*discard specific param*/
    }
    pslaveportlist->flag |= FLAG_MSG;

    //[RK]Jul/29/03
    if ((mt.board_type != 12) && (mt.board_type != 18)) { 
	/* it is not TS100 nor TS110 */
	medial.nelements = 3; 
        (pslaveportlist+PORT_MODBUS_SMODE)->flag |= FLAG_INACTIVE; 
    }

#ifdef WEBS_PMD
        /* Build the list of IPDU's */
        {
                int i, n, j = 0;
                struct dirent **namelist;
                if ((n = scandir ("/etc", &namelist, 0, alphasort)) >= 0) {
                        for (i = 0; i < n; i ++) {
                                if (strncmp(namelist[i]->d_name, "pm.", 3)
                                        == 0 && j < 16) {
                                        strncpy(pmtypenames + j * 20,
                                                namelist[i]->d_name + 3, 19);
                                        j++;
                                }
                                free(namelist[i]);
                        }
                        free(namelist);
                }
                pmtypel.nelements = j;
        }
#endif
    DefaultConfig(pslaveportlist, sizeof(pslaveportlist)/sizeof(struct cfglist));
    ReadPslaveConfig();
    for (i = 0; i < sizeof(pslaveportlist)/sizeof(struct cfglist) ; i ++) {
	if (websGetPriority(wp) != PRIV_FULL) {
	    (pslaveportlist + i)->config |= CONFIG_READ_ONLY;
	} else {
	    (pslaveportlist + i)->config &= ~CONFIG_READ_ONLY;
	}
    }
    return 0;
}

static int InsertWizardButton(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t i;
    int ret=0;

#ifndef OEM1
    if (!pslave_incl && websGetPriority(wp) == PRIV_FULL) {
		ret += websWrite(wp, "<tr><th class=\"tt\" colspan=\"2\">Wizards</th></tr>");

#ifdef WIZ_SELECT
		ret += websWrite(wp, "<input type=\"hidden\" name=\"config\" "
			"value=\"1\">\n"
			"<script language=\"JavaScript\">\n"
			"<!--\n"
			"function select_change () {\n"
			"\tdocument.forms[0].config.value = 0;"
			"\tdocument.forms[0].submit();\n"
			"}\n"
			"//-->\n"
			"</script>"
			"<tr><td><b>Profile:</b></td>\n"
			"<td><select name=\"prof\" "
			"onChange=\"select_change();\">\n"
			"<option value=\"none\" selected>None\n");
		for (i=0; i<num_wizards; i++) {
			ret += websWrite(wp, "<option value = \"%s\">%s\n",
				wizard_name[i], wizard_name[i]);
		}
		ret += websWrite(wp, "</select></td></tr>");
#else
		ret += websWrite(wp, "<tr><td colspan=\"2\"><center>");
		for (i=0; i<num_wizards; i++) {
			ret+=websWrite(wp, "&nbsp;&nbsp;<input type=\"submit\" "
				"name=\"sub\" value=\"%s profile\">&nbsp;&nbsp;", 
				wizard_name[i]);
		}
		ret += websWrite(wp, "</center></td></tr>");
#endif
	}
#endif
	return ret;
}

static int PrintSportConfig(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t *name, nparam;
    int ret=0;
    nparam=ejArgs(argc, argv, T("%s"), &name);
    if (nparam>=1) {
        ret=WriteConfig(wp, pslaveportlist, sizeof(pslaveportlist)/sizeof(struct cfglist), name);
    }
    if (!ret) {
        return -1;
    }
    return 0;
}

static int CloseSportConfig(int eid, webs_t wp, int argc, char_t **argv)
{
    struct cyc_webs_header hdr;
    if (!pslave_incl && websGetPriority(wp) == PRIV_FULL) {
        websWrite(wp, "<br><center>"
	    "<input type=\"submit\" name=\"sub\" value=\"Submit\">"
	    "&nbsp;&nbsp;&nbsp;&nbsp;"
	    "<input type=\"reset\" value=\"Reset\"></center>");
    }
    ncfgerr=0;
    DeallocConfig(pslaveportlist);
	websCycHdrInit(&hdr);
        hdr.form = "cfg/WriteSportConfig";
	websPageEnd(wp, &hdr);
    return 0;
}

static void WriteSportConfig(webs_t wp, char_t *path, char_t *query)
{
    /* The first thing to do is to calculate the portslave port */
    unsigned char *port, *sub, *config, *prof, i;
    port = websGetVar(wp, "port", "0");
#ifdef WIZ_SELECT
    config = websGetVar(wp, "config", "0");
    prof = websGetVar(wp, "prof", "none");
#else
    sub = websGetVar(wp, "sub", "Submit");
#endif
    sscanf (port, "%d", &pslaveconfigport);
    if (!AllocConfig(pslaveportlist, sizeof(pslaveportlist)/sizeof(struct cfglist)))
    {
        return;
    }
    pslaveportlist->flag |= FLAG_MSG;

    if (pslaveconfigport) {
	(pslaveportlist+PORT_REMADDR_INC)->flag |= FLAG_INACTIVE; 
	/*ipno increment*/
	(pslaveportlist+PORT_TCPPORT_INC)->flag |= FLAG_INACTIVE; 
	/*socket port increment*/
	(pslaveportlist+PORT_DISCSPEC)->flag |= FLAG_INACTIVE; 
	/*discard specific param*/
    } else {
	(pslaveportlist+PORT_TTY)->flag |= FLAG_INACTIVE; /*tty*/
	(pslaveportlist+PORT_SERVERFARM)->flag |= FLAG_INACTIVE; 
	/*server farm*/
	(pslaveportlist+PORT_DISCSPEC)->value[0] = '0'; 
	/*discard specific param*/
    }
    //[RK]Jul/29/03
    if ((mt.board_type != 12) && (mt.board_type != 18)) { 
	/* it is not TS100 nor TS110 */
	medial.nelements = 3; 
        (pslaveportlist+PORT_MODBUS_SMODE)->flag |= FLAG_INACTIVE; 
    }

    DefaultConfig(pslaveportlist, 
        sizeof(pslaveportlist)/sizeof(struct cfglist));
	
#ifndef OEM1
	if (!pslave_incl) {
		for (i=0; i<num_wizards; i++) {
#ifdef WIZ_SELECT
			if (strcmp(prof, wizard_name[i]) == 0) 
#else
			if (strncmp(sub, wizard_name[i], strlen(wizard_name[i])) == 0) 
#endif
			{
				pslave_filename = wizard_filename[i];
				ReadPslaveConfig();
				pslave_filename = pslave_filename_orig;
				ncfgerr = 0xf8 + i;
				websRedirect(wp, T("/read/sportcfg.asp"), NULL);
				return;
			}
		}
	}
#endif
#ifdef WIZ_SELECT
	if (strcmp(config, "1") == 0) 
#else
	if (strcmp(sub, "Submit") == 0)
#endif
	{
		ncfgerr = 0;
		GetConfigFromForm(wp, pslaveportlist, 
			sizeof(pslaveportlist)/sizeof(struct cfglist));
		if (!ncfgerr) {
			SavePslaveConfig(wp);
			websRedirect(wp, T("/read/serports.asp"), NULL);
			DeallocConfig(pslaveportlist);
		} else {
			websRedirect(wp, T("/read/sportcfg.asp"), NULL);
		}
	} 
#ifdef WIZ_SELECT
	else {
		ncfgerr = 0;
		GetConfigFromForm(wp, pslaveportlist, 
			sizeof(pslaveportlist)/sizeof(struct cfglist));
		ncfgerr=0xf7; /* The treatment for this ncfgerr was not implemented */
		websRedirect(wp, T("/read/sportcfg.asp"), NULL);
	}
#endif
}

#ifdef BOOT
/**************************************************************************
 * Definitions for the Boot Configuration
 */
/* List of valid entries */
static unsigned char *bootoptlist[] = {"flash", "network"};
static struct combolist bootoptl = {numberlist, bootoptlist, 2};

static unsigned char *boottypelist[] = {"bootp", "tftp", "both"};
static struct combolist boottypel = {numberlist+1, bootoptlist, 3};

static struct combolist consspdl = {numberlist, consspeedlist+11, 6};

static unsigned char *ftestlist[] = {"skip", "full"};
static struct combolist ftestl = {numberlist, bootoptlist, 2};

static unsigned char *rtestlist[] = {"skip", "quick", "full"};
static struct combolist rtestl = {numberlist, bootoptlist, 3};

static unsigned char *fethlist[] = {"0", "1", "3", "4", "2"};
static unsigned char *fethshow[] = {"Auto Negotiation",
									"100BaseT Half-Duplex",
									"100BaseT Full-Duplex",
									"10BaseT Half-Duplex",
									"10BaseT Full-Duplex"};
static struct combolist fethl = {fethlist, fethshow, 5};

/* Parameter structure */
static struct cfglist bootconflist[] {
	{"MAC Address assigned to Ethernet:", "ethmac",
	NULL, NULL, NULL, NULL, NULL,
	TYPE_MACADD, 0, 0, 0, 17, 17, 0},
	{"IP Address Assigned to Ethernet:", "ethip",
	NULL, NULL, NULL, NULL, NULL,
	TYPE_IPADDR, 0, 0, 0, SIZE_IPADD, SIZE_IPADD, 0},
	{"Watchdog Timer:", "wdt",
	NULL, NULL, &activel, NULL, NULL,
	TYPE_RADIO, 0, 0, 0, 0, 2, 0},
    {"Unit boot from:", "boot",
    NULL, NULL, &bootoptl, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 2, 0},
    {"Boot Type:", "btype",
    NULL, NULL, &boottypel, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 2, 0},
    {"Boot File Name:", "bfile",
    NULL, NULL, NULL, NULL, NULL,
    TYPE_STRING, 0, 0, 0, PATH_LEN, PATH_LEN, 0},
	{"Server's IP Address:", "srvip",
	NULL, NULL, NULL, NULL, NULL,
	TYPE_IPADDR, 0, 0, 0, SIZE_IPADD, SIZE_IPADD, 0},
    {"Console Speed:", "consspd",
    "1", NULL, &consspdl, NULL, NULL,
    TYPE_LIST, 0, 0, 0, 0, 7, 0}
    {"Flash Test:", "ftest",
    NULL, NULL, &ftestl, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 2, 0},
    {"RAM Test:", "rtest",
    NULL, NULL, &rtestl, NULL, NULL,
    TYPE_RADIO, 0, 0, 0, 0, 2, 0},
    {"Fast Ethernet:", "feth",
    NULL, NULL, &fethl, NULL, NULL,
    TYPE_LIST, 0, 0, 0, 0, 2, 0},
};

static unsigned char bootconf_n; /* number of elements of boot_conf */

/******************************************************************************
 * Routines related to the page bootconf.asp
 */

extern struct MachineType mt;

static int     speeda[] = {4800, 9600, 19200, 38400, 57600, 115200};
static unsigned char def_hwmac[6] = {0x00, 0x60, 0x2e, 0x00, 0x00, 0x01};
static unsigned char def_ethip[4] = {192, 162, 160,10};
static unsigned char def_srvip[4] = {192, 162, 160,1};

static unsigned short calc_crc(unsigned long byte_n, unsigned short crc, char *buffer)
{
	int j;

	while (byte_n--) {
		crc = (((crc >> 8) ^ *buffer++) << 8) | (0xFF & crc) ;
		for(j = 0; j < 8; j++) {
			if (crc & 0x8000)  {
				crc <<= 1;
				crc ^= 0x1021;
			} else {
				crc <<= 1;
			}
		}
	}
	return(crc);
}

static int set_default_config(unsigned char *macaddr)
{
	FLASH_CONFIG_VECTOR_HEADER	*v;
	struct HW_SETUP *hw_setup;
	struct BOOT_MASK *boot_mask;
	int ret = 0, fd;
	unsigned char *buffer, *s;
	
	if ((buffer = malloc(64*1024)) != NULL) {
		s = buffer;
		
		// TYPE0000
		v = (FLASH_CONFIG_VECTOR_HEADER	*)s;
		v->type = TYPE0000;
		v->size = sizeof(struct type0000);
		memset(v->data, 0, v->size);
		strcpy(v->data, "CYCLADES");
		v->crc = calc_crc(v->size, 0, v->data);
		s += 6 + v->size;

		// TYPE0068
		v = (FLASH_CONFIG_VECTOR_HEADER *)s;
		v->type = TYPE0068;
		v->size = sizeof(struct HW_SETUP);

		hw_setup = (struct HW_SETUP *)(v->data);
		memset(v->data, 0, v->size);
		if (memcmp(def_hwmac, macaddr,3)) {
			memcpy(hw_setup->physical_addr, def_hwmac, 6);
		} else {
			memcpy(hw_setup->physical_addr, macaddr, 6);
		}
		*(unsigned long *)hw_setup->itf_ip_addr = *(unsigned long *)def_ethip;
        hw_setup->enable_wdt = 1;
		hw_setup->console_speed = 1;
		hw_setup->op_code_crc = 1;
		hw_setup->mem_test = 2;
		hw_setup->boot_option = 0;
		strcpy(hw_setup->tftp_path, "zvmppcts.bin");
		v->crc = calc_crc(v->size, 0, v->data);
		s += 6 + v->size;

		// TYPE0015
		v = (FLASH_CONFIG_VECTOR_HEADER *)s;
		v->type = TYPE0015;
		v->size = sizeof(struct BOOT_MASK);
		boot_mask = (struct BOOT_MASK *)(v->data);
		memset(v->data, 0, v->size);
		boot_mask->boot_type = 2;
		*(unsigned long *)boot_mask->bootserver = *(unsigned long *)def_srvip;
		v->crc = calc_crc(v->size, 0, v->data);
		s += 6 + v->size;
		
		v = (FLASH_CONFIG_VECTOR_HEADER *)s;
		v->type = 9999;
		v->size = 0;
		v->crc = 0;
		s += 6 + v->size;
		
		if ((fd = open("/proc/flash/config", O_WRONLY|O_CREAT)) != -1) {
			if (write(fd, buffer, s-buffer)==s-buffer) {
				ret = 1;
			}
			close(fd);
		}
		free(buffer);
	}
	return ret;
}


static char *access_type(char *buf, int type)
{
	FLASH_CONFIG_VECTOR_HEADER	*p = (FLASH_CONFIG_VECTOR_HEADER *)buf;

	while (p->type != 9999) {
		FLASH_CONFIG_VECTOR_HEADER	*p = (FLASH_CONFIG_VECTOR_HEADER *)buf;
		if (p->type == 9999) {
			break;
		}
		if (p->crc != calc_crc(p->size, 0, p->data)) {
			break;
		}
		if (p->type == type) {
			return(p->data);
		}
		buf += p->size + 6;
	}
	return(NULL);
}


static int GetBootConfiguration(struct HW_SETUP *hwsetup, struct BOOT_MASK *boot_mask)
{
	unsigned char *buffer, *s1, *s2, *s3;
	int fd, ret=0, i;
	if ((buffer = malloc(64*1024)) != NULL) {
		if ((fd = open("/proc/flash/config", O_RDONLY|O_CREAT)) != -1) {
			if (read(fd, buffer, 64*1024)>0) {
				if ((s1 = access_type(buffer, TYPE0000)) != NULL &&
					(s2 = access_type(buffer, TYPE0068)) != NULL &&
					(s3 = access_type(buffer, TYPE0015)) != NULL) {
					
					memcpy((unsigned char *)hwsetup, s2, sizeof(struct HW_SETUP));
					memcpy((unsigned char *)boot_mask, s3, sizeof(struct BOOT_MASK));
					ret = 1;
				}
			}
			close(fd);
		}
		free(buffer);
	}
	return ret;
}

static int SetBootConfiguration(struct HW_SETUP *hwsetup, struct BOOT_MASK *boot_mask)
{
	unsigned char *buffer, *s1, *s2, *s3;
	int fd, ret=0, n;
	FLASH_CONFIG_VECTOR_HEADER	*v;
	if ((buffer = malloc(64*1024)) != NULL) {
		if ((fd = open("/proc/flash/config", O_RDONLY|O_CREAT)) != -1) {
			n = read(fd, buffer, 64*1024);
			close(fd);
			if (n>0) {
				if ((s1 = access_type(buffer, TYPE0000)) != NULL &&
					(s2 = access_type(buffer, TYPE0068)) != NULL &&
					(s3 = access_type(buffer, TYPE0015)) != NULL) {
					
					memcpy(s2, (unsigned char *)hwsetup, sizeof(struct HW_SETUP));
					v = (FLASH_CONFIG_VECTOR_HEADER *)(s2 - 6);
					v->crc = calc_crc(v->size, 0, v->data);
					
					memcpy(s3, (unsigned char *)boot_mask, sizeof(struct BOOT_MASK));
					v = (FLASH_CONFIG_VECTOR_HEADER *)(s3 - 6);
					v->crc = calc_crc(v->size, 0, v->data);
				
					if ((fd = open("/proc/flash/config", O_WRONLY|O_CREAT)) != -1) {
						if (write(fd, buffer, n)==n) {
							ret = 1;
						}
						close(fd);
					}
				}
			}
		}
		free(buffer);
	}
	return ret;
}

static void check_speed_console(int console_speed)
{
	FILE *arq;
	int file_speed;

	if ((arq = fopen("/etc/getty_ttyS0", "rw")) != NULL) {
		fscanf(arq, "%d", &file_speed);
	}
	
	if (speeda[console_speed] != file_speed) {
		rewind(arq);
		fprintf(arq, "%d", speeda[console_speed]);
		system("sh /bin/updatefiles /etc/getty_ttyS0 > /dev/ttyS0");
	}
	fclose(arq);
}

static int check_ethernet_operation(struct HW_SETUP *hw_setup)
{
	struct ifreq ifr;
	int sockfd, ret=0;
	unsigned char aux[128];

	if (!strcmp(mt.name, "CPLD")) {
		if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) >= 0) {
			strcpy(ifr.ifr_name, "eth0");
			ifr.ifr_ifru.ifru_flags = 1;
			ifr.ifr_ifru.ifru_ivalue = hw_setup->mii_operation;
        	if (ioctl(sockfd, SIOCDEVPRIVATE, &ifr) >= 0) {
        		ret = 1;
			}
		}
		close(sockfd);
	}
	if (ret) {
		sprintf(aux, "sleep 10; config_eth0 %s %02x:%02x:%02x:%02x:%02x:%02x",
			inet_ntoa (*(struct in_addr *)(hw_setup->itf_ip_addr)),
			hw_setup->physical_addr[0],	hw_setup->physical_addr[1],
			hw_setup->physical_addr[2],	hw_setup->physical_addr[3],
			hw_setup->physical_addr[4],	hw_setup->physical_addr[5]);
		system(aux);
	}
	exit(0);
}

static int OpenBootConfig(int eid, webs_t wp, int argc, char_t **argv)
{
	struct HW_SETUP hw_setup;
	struct BOOT_MASK boot_mask;
	int flash_ok = GetBootConfiguration(&hw_setup, &boot_mask), i;
	unsigned char value[32];

	struct cyc_webs_header hdr;
	websCycHdrInit(&hdr);
        hdr.title = "Boot Configuration";
        hdr.form = "cfg/WriteBootConfig";
        websPageStart(wp, &hdr);

    websWrite(wp, T("<br><br>"));
		
    if (ncfgerr) {
        websWrite(wp, T("<b class=\"err\"><center>Configuration Error - The wrong parameters are marked with red.</center></b><br><br>"));
        return 0;
    }
    if (!AllocConfig(bootconflist, bootconf_n)) {
        websError(wp, 400, T("Allocation Error\n"));
        return -1;
    }
    DefaultConfig(bootconflist, bootconf_n);
	sprintf(bootconflist[0].value, "%02x:%02x:%02x:%02x:%02x:%02x",
		hw_setup.physical_addr[0], hw_setup.physical_addr[1],
		hw_setup.physical_addr[2], hw_setup.physical_addr[3],
		hw_setup.physical_addr[4], hw_setup.physical_addr[5]);
	strncpy(bootconflist[1].value, inet_ntoa(*(struct in_addr *)hw_setup.itf_ip_addr), bootconflist[1].size);
	sprintf(bootconflist[2].value, "%d", hw_setup.enable_wdt);
	sprintf(bootconflist[3].value, "%d", hw_setup.boot_option);
	sprintf(bootconflist[4].value, "%d", boot_mask.boot_type);
	strncpy(bootconflist[5].value, hw_setup.tftp_path, bootconflist[5].size);
	strncpy(bootconflist[6].value, inet_ntoa(*(struct in_addr *)boot_mask.bootserver), bootconflist[6].size);
	sprintf(bootconflist[7].value, "%d", hw_setup.console_speed);
	sprintf(bootconflist[8].value, "%d", hw_setup.op_code_crc);
	sprintf(bootconflist[9].value, "%d", hw_setup.mem_test);
	if (bootconf_n > 10) {
		sprintf(bootconflist[10].value, "%d", hw_setup.mii_operation);
	}
	websWrite(wp, "<input type=\"%s\" name=\"%s\" value=\"%s\"%s>",
		"hidden", "flashok", flash_ok?"1":"0", "");

    return 0;
}

static int PrintBootConfig(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t *name, nparam;
    int ret=0;
    if (ejArgs(argc, argv, T("%s"), &name)>=1) {
        ret=WriteConfig(wp, bootconflist, bootconf_n, name);
    }
    if (!ret) {
        return -1;
    }
    return 0;
}

static int CloseBootConfig(int eid, webs_t wp, int argc, char_t **argv)
{
    ncfgerr=0;
    DeallocConfig(bootconflist);
	struct cyc_webs_header hdr;
	websCycHdrInit(&hdr);
        hdr.form = "cfg/WriteBootConfig";
        websPageEnd(wp, &hdr);
    return 0;
}

static void WriteBootConfig(webs_t wp, char_t *path, char_t *query)
{
	struct HW_SETUP hw_setup;
	struct BOOT_MASK boot_mask;
	unsigned char *flashok = websGetVar(wp, "flashok", ""), *value;
	int action, readerr, writeerr, flash_ok, x[6], i;
	unsigned int aux;
	unsigned char eth_chg = 0;
	struct in_addr eth_aux;
	struct cyc_webs_header hdr;
	
	if (*flashok != '1') {
		/* The flash is not OK. Check whether we should set configuration to defaults */
		value = websGetVar(wp, "def", "");
		if (*value == '1') {
			action = 1; /* Set defaults */
		} else {
			action = 0; /* Cancel */
		}
	} else {
		value = websGetVar(wp, "opt", "");
		if (strcmp(value, "Save to Flash") == 0) {
			action = 2; /* Save configuration to flash */
		} else if (strcmp(value, "Set to Defaults") == 0) {
			action = 1; /* Set Defaults*/
		} else if (strcmp(value, "Cancel") == 0) {
			action = 0;	/* Cancel */
		}
	}
	flash_ok = GetBootConfiguration(&hw_setup, &boot_mask);
	if (action == 2) {
		if (flash_ok) {
		    if (!AllocConfig(bootconflist, bootconf_n)) {
        		return;
    		}
			ncfgerr=0;
    		GetConfigFromForm(wp, bootconflist, bootconf_n);
		    if (ncfgerr) {
		    	websRedirect(wp, "/read/bootconf.asp", NULL);
    	        DeallocConfig(bootconflist);
		    	return;
		    } else {
		    	sscanf(bootconflist[0].value, "%x:%x:%x:%x:%x:%x",
					&x[0], &x[1], &x[2], &x[3], &x[4], &x[5]);
				for (i=0; i<6; i++) hw_setup.physical_addr[i] = x[i];
				inet_aton(bootconflist[1].value,
								(struct in_addr*)(hw_setup.itf_ip_addr));
				sscanf(bootconflist[2].value, "%d", &aux);
				hw_setup.enable_wdt = aux;
				sscanf(bootconflist[3].value, "%d", &aux);
				hw_setup.boot_option = aux;
				sscanf(bootconflist[4].value, "%d", &aux);
				boot_mask.boot_type = aux;
				strncpy(hw_setup.tftp_path, bootconflist[5].value, PATH_LEN-1);
				hw_setup.tftp_path[PATH_LEN-1] = 0;
				inet_aton(bootconflist[6].value, (struct in_addr *)(boot_mask.bootserver));
				sscanf(bootconflist[7].value, "%d", &aux);
				hw_setup.console_speed = aux;
				sscanf(bootconflist[8].value, "%d", &aux);
				hw_setup.op_code_crc = aux;
				sscanf(bootconflist[9].value, "%d", &aux);
				hw_setup.mem_test = aux;
				if (bootconf_n > 10) {
					sscanf(bootconflist[10].value, "%d", &aux);
					hw_setup.mii_operation = aux;
			    }
       	        DeallocConfig(bootconflist);
				/* Save configuration in flash */
				writeerr = (SetBootConfiguration(&hw_setup, &boot_mask));
				/* Check Console Speed in /etc/getty_ttyS0 */
				check_speed_console(hw_setup.console_speed);
				/* Check Ethernet */
				check_ethernet_operation(&hw_setup);
			}
		}
	}
	if (action==1) {
		set_default_config(hw_setup.physical_addr);
	}
	
	websCycHdrInit(&hdr);
	hdr.need_header = 1;
	hdr.title = "Boot Configuration Result";
	hdr.links = "&nbsp;<a href=\"/goform/ShowBootConf\">"
		"[Boot Configuration]</a>";
        hdr.father = "/goform/ShowBootConf";
        websPageStart(wp, &hdr);
	websWrite(wp, "<center><table border align=\"center\"><tr><th class=\"tt\"%s>%s</th></tr>",
		"", "Boot Configuration Result");
	websWrite(wp, "<tr><td>");
	if (action == 0) {
		websWrite(wp, "<b>Boot configuration was cancelled.</b>");
	} else if (action == 1) {
		websWrite(wp, "<b>Boot configuration was set to default.</b>");
	} else if (action == 2) {
		if (writeerr == 0) {
			websWrite(wp, "<b class=\"err\">ERROR - %s</b>", "Configuration could not be written in the flash.");
		} else {
			websWrite(wp, "<b>Configuration was written successfully.</b>");
		}
	}
	websWrite(wp, "</td></tr></table></center>");
        websPageEnd(wp, &hdr);
}
#endif

/******************************************************************************/
/*
 *	Initialize the asps/forms/cgi's.
 */

void DefineWebAspForm7()
{
	int i;
	for (i=0; i<numsports; i++) {
	  sprintf (ttystr+8*i, "ttyS%d", i+1);
	}

	websAspDefine(T("OpenGeneralConfig"), OpenGeneralConfig);
	websAspDefine(T("PrintGeneralConfig"), PrintGeneralConfig);
	websAspDefine(T("CloseGeneralConfig"), CloseGeneralConfig);
	websFormDefine(T("cfg/WriteGeneralConfig"), WriteGeneralConfig);

	websAspDefine(T("PrintPslavePortforConnection"), PrintPslavePortforConnection);
	websAspDefine(T("PrintPortsforTemperature"), PrintPortsforTemperature);

	websAspDefine(T("ShowApplet"), ShowApplet);
	websAspDefine(T("ShowTemperatureApplet"), ShowTemperatureApplet);
	websAspDefine(T("ShowTitle"), ShowTitle);
	websAspDefine(T("PrintSportConfig"), PrintSportConfig);
	websAspDefine(T("CloseSportConfig"), CloseSportConfig);

#ifdef BOOT
	bootconf_n = sizeof(bootconflist)/sizeof(struct cfglist);
	if  (strcmp(mt.name, "CPLD")) bootconf_n--;

	websAspDefine(T("OpenBootConfig"), OpenBootConfig);
	websAspDefine(T("PrintBootConfig"), PrintBootConfig);
	websAspDefine(T("CloseBootConfig"), CloseBootConfig);
	websFormDefine(T("cfg/WriteBootConfig"), WriteBootConfig);
#endif
}

void DefineWebAspForm2()
{
    int i;
    for (i=0; i<numsports; i++) {
        sprintf (ttystr+8*i, "ttyS%d", i+1);
    }

    websAspDefine(T("OpenGeneralConfig"), OpenGeneralConfig);
    websAspDefine(T("PrintGeneralConfig"), PrintGeneralConfig);
    websAspDefine(T("CloseGeneralConfig"), CloseGeneralConfig);
    websFormDefine(T("cfg/WriteGeneralConfig"), WriteGeneralConfig);

    websAspDefine(T("PrintPslavePort"), PrintPslavePort);
    websFormDefine(T("SetPslavePort"), SetPslavePort);

    websAspDefine(T("OpenSportConfig"), OpenSportConfig);
	if (!pslave_incl)
		websAspDefine(T("InsertWizardButton"), InsertWizardButton);
    websAspDefine(T("PrintSportConfig"), PrintSportConfig);
    websAspDefine(T("CloseSportConfig"), CloseSportConfig);
    websFormDefine(T("cfg/WriteSportConfig"), WriteSportConfig);

#ifdef BOOT
    bootconf_n = sizeof(bootconflist)/sizeof(struct cfglist);
    if  (strcmp(mt.name, "CPLD")) bootconf_n--;

    websAspDefine(T("OpenBootConfig"), OpenBootConfig);
    websAspDefine(T("PrintBootConfig"), PrintBootConfig);
    websAspDefine(T("CloseBootConfig"), CloseBootConfig);
    websFormDefine(T("cfg/WriteBootConfig"), WriteBootConfig);
#endif
}
