/*
 * main.c -- Main program for the GoAhead WebServer (LINUX version)
 *
 * Copyright (c) GoAhead Software Inc., 1995-2000. All Rights Reserved.
 *
 * See the file "license.txt" for usage and redistribution license requirements
 *
 * $Id: main.c,v 1.71 2006/06/21 13:44:19 regina Exp $
 */

/******************************** Description *********************************/

/*
 *	Main program for for the GoAhead WebServer. This is a demonstration
 *	main program to initialize and configure the web server.
 */

/********************************* Includes ***********************************/

#include 	"appl.h"
#include	<signal.h>
#include	<unistd.h> 
#include	<sys/types.h>
#include	<sys/wait.h>
#include	<netdb.h>

#ifdef WEBS_SSL_SUPPORT
#include	"../websSSL.h"
#endif

#ifdef USER_MANAGEMENT_SUPPORT
void	formDefineUserMgmt(void);
#endif

#ifdef SM100
#include <linux/cy_sm.h>
#endif

extern char PROG_RAS[];

int  numsports;
struct MachineType mt;

/*********************************** Locals ***********************************/
/*
 *	Change configuration here
 */

static char_t		*rootWeb = T("/web");			/* Root web directory */
static char_t		*password = T("");				/* Security password */
static int			port = 80;						/* Server port */
static int			retries = 5;					/* Server port retries */
static int			finished;						/* Finished flag */

unsigned char io_ports[] = "Analog/Digital Ports";

/****************************** Forward Declarations **************************/

static int  initWebs();
static int  websHomePageHandler(webs_t wp, char_t *urlPrefix, char_t *webDir,
	int arg, char_t *url, char_t *path, char_t *query);
static void DefineWebAspForm();

#ifdef B_STATS
	static void printMemStats(int handle, char_t *fmt, ...);
	static void memLeaks();
#endif

void defaultErrorHandler(int etype, char_t *msg);
void defaultTraceHandler(int level, char_t *buf);

#ifdef SM100
	unsigned char server_mgt[] = "Server Management";
	unsigned char pw_ipmi[] = "/adm/sm_ipmi.asp";
	unsigned char pw_atx[] = "/adm/sm_atx.asp";
#endif 

struct item_menu {
	unsigned char privilege;
	unsigned char *name;
	unsigned char *link;
	unsigned char *comments;
};

struct item_menu menuitems[] = {
	{PRIV_USER, "Applications", NULL, "This section contains the user applications."},
	{PRIV_USER, "Logout", "/goform/appl/Logout", "Exits the Web Management Service."},
	{PRIV_USER, "Connect to Serial Ports", "/appl/connectPorts.asp", "Telnet/SSH connection to Portslave."},
#ifdef TS1H
	{PRIV_USER, "Read Sensor", "/appl/temperaturePorts.asp", "Read Sensor from analog ports."},
#endif
	{PRIV_MONITOR, "Configuration", NULL, "This section contains the configuration tools."},
	{PRIV_MONITOR, "General", "/read/general.asp", "Unit description, Ethernet, DNS, Name Service Access, Data Buffering."},
	{PRIV_MONITOR, "Syslog", "/goform/LinkToEditTextFile?filename=/etc/syslog-ng/syslog-ng.conf", "Configuration for the syslog-ng."},
	{PRIV_MONITOR, "SNMP", "/goform/LinkToEditTextFile?filename=/etc/snmp/snmpd.conf", "Configuration for the SNMP server."},
	{PRIV_MONITOR, "Serial Ports", "/read/serports.asp", "Configuration of Portslave package."},
	{PRIV_MONITOR, "Serial Port Groups", "/goform/ListSPUGroupTable", "Configuration of User Groups for Serial Ports."},
	{PRIV_MONITOR, "Host Table", "/goform/ListHostTable", "Table of hosts in /etc/hosts."},
	{PRIV_MONITOR, "Static Routes", "/goform/ListRouteTable", "Static Routes defined in /etc/network/st_routes."},
	{PRIV_MONITOR, "IP Chains", "/goform/ListChainTable", "Static Filter Chains in /etc/network/ipchains."},
	{PRIV_FULL, "Boot Configuration", "/goform/ShowBootConf", "Configuration of parameters used in the boot process."},
	{PRIV_MONITOR, "Edit Text File", "/read/editfile.asp", "Tool to edit any configuration file."},
	{PRIV_MONITOR, "System Users", "/goform/ShowSysUsrTable", "Management of system users defined in /etc/passwd"},
	{PRIV_MONITOR, "System Groups", "/goform/ShowSysGrpTable", "Management of system groups defined in /etc/groups"},
#ifdef OEM1
	{PRIV_FULL, "Menu Configuration", "/read/menucfg.asp", "Configuration for Creating Custom Menus for System Users or Groups"}, 
	{PRIV_FULL, "Auto-Alert Configuration", "/read/filtercfg.asp", "Configuration for Creating Auto-Alerts and Notifications"},
#endif
	{PRIV_MONITOR, "Administration", NULL, "This section constains the administration tools."},
	{PRIV_ADMIN, "Reboot", "/adm/reboot.asp", "Resets the equipment."},
//[RK]removed	{PRIV_ADMIN, "Send Message", "/adm/sendmsg.asp", "Send messages to the users logged or to a determined serial port."},
//[Fuji]removed	{PRIV_ADMIN, "Port Conversation", "/adm/portconv.asp", "Does a Port Conversation through a determined serial port."},
	{PRIV_FULL, "Download/Upload Image", "/adm/zimage.asp", "Uses a FTP server to load/save the kernel image."},
	{PRIV_FULL, "Load/Save Config", "/adm/config.asp", "Uses the flash or a FTP server to load/save the unit's configuration."},
	{PRIV_FULL, "Run Configuration", "/adm/runconfig.asp", "Makes the configuration changes effective."},
	{PRIV_ADMIN, "Set Date/Time", "/adm/datetime.asp", "Set the unit's date and time."},
	{PRIV_MONITOR, "Active Sessions", "/goform/ShowActiveSessions", "Shows the active sessions."},
	{PRIV_MONITOR, "CAS Sessions", "/goform/ShowCASSessions", "Shows the CAS sessions."},
	{PRIV_MONITOR, "Process Status", "/goform/ShowProcessStatus", "Shows the processes running and allows the administrator to kill them."},
	{PRIV_ADMIN, "Restart Processes", "/adm/restproc.asp", "Allows the administrator to Start and Stop some specific processes."},
#ifdef WEBS_PMD
	{PRIV_ADMIN, "Power Management", "/goform/PowerMgmt", "Controls server's power (on|off|cycle|lock|unlock)."},
#endif
#ifdef SM100
	{PRIV_ADMIN, server_mgt, NULL, "Controls server's power (on|off|cycle|reset) and monitors its health."},
#endif
	{PRIV_MONITOR, "Web User Management", NULL, "This section contains the functions to manage the web users."},
	{PRIV_MONITOR, "Users", "/read/webuser.asp", "List of users allowed to access the web server."},
	{PRIV_MONITOR, "Groups", "/read/webgroup.asp", "List of possible access groups."},
	{PRIV_MONITOR, "Access Limits", "/read/weblimit.asp", "List of access limits to specific URL's."},
	{PRIV_FULL, "Load/Save Web Config", "/um/umconfig.asp", "Load/Save configuration in /etc/websum.conf."},
	{PRIV_MONITOR, "Information", NULL, "This section contains the information tools, including status and statistics."},
	{PRIV_MONITOR, "Interface Statistics", "/goform/ShowItfTable", "Shows statistics for active interfaces"},
	{PRIV_MONITOR, "DHCP Client", "/goform/ShowDHCPCInfo", "Shows host information from DHCP"},
	{PRIV_MONITOR, "Serial Ports", "/goform/ShowTtyInfo", "Shows status of all the serial ports"},
	{PRIV_MONITOR, io_ports, "/read/io_ports.asp", "Shows status of all analog and digital ports"}, //TS110 only
	{PRIV_MONITOR, "Routing Table", "/goform/ShowRouteTable", "Shows routing table and allows the administrator to add/delete routes"},
	{PRIV_MONITOR, "ARP Cache", "/goform/ShowARPTable", "Shows the ARP cache."},
	{PRIV_MONITOR, "IP Chains", "/goform/ShowIPChainsTable", "Shows the entries defines as IP chains."},
	{PRIV_MONITOR, "IP Rules", "/goform/ShowIPRulesTable", "Shows the IP Chain rules"},
	{PRIV_MONITOR, "IP Statistics", "/goform/ShowIPStatTable", "Shows the statistics concerning IP protocol."},
	{PRIV_MONITOR, "ICMP Statistics", "/goform/ShowICMPStatTable", "Shows the statistics concerning ICMP protocol."},
	{PRIV_MONITOR, "TCP Statistics", "/goform/ShowTCPStatTable", "Shows the statistics concerning TCP protocol."},
	{PRIV_MONITOR, "UDP Statistics", "/goform/ShowUDPStatTable", "Shows the statistics concerning UDP protocol."},
	{PRIV_MONITOR, "Ram Disk Usage", "/goform/ShowRamDiskUsage", "Shows the unit's File System status."},
	{PRIV_MONITOR, "System Information", "/read/sysinfo.asp", "Kernel, Time, CPU, Memory."},
	{0, NULL, NULL, NULL}};

/*********************************** Code *************************************/
/*
 *	Main -- entry point from LINUX
 */

int main(int argc, char** argv)
{
/*
 *	Initialize the memory allocator. Allow use of malloc and start 
 *	with a 1 Mb heap.  For each page request approx 8KB is allocated.
 *	1 Mb allows for several concurrent page requests.  If more space
 *	is required, malloc will be used for the overflow.
 */
	bopen(NULL, (1024 * 1024), B_USE_MALLOC);
	signal(SIGPIPE, SIG_IGN);

/*
 *	Initialize the web server
 */
	if (initWebs() < 0) {
		return -1;
	}

	if (shm_init(-1) < 0) {
        return -1;
    }


#ifdef WEBS_SSL_SUPPORT
	websSSLOpen(getSslVersion());
#endif

/*
 *	Basic event loop. SocketReady returns true when a socket is ready for
 *	service. SocketSelect will block until an event occurs. SocketProcess
 *	will actually do the servicing.
 */
	while (!finished) {
		if (socketReady(-1) || socketSelect(-1, 1000)) {
			socketProcess(-1);
		}
		websCgiCleanup();
		emfSchedProcess();
	}

#ifdef WEBS_SSL_SUPPORT
	websSSLClose();
#endif

#ifdef USER_MANAGEMENT_SUPPORT
	umClose();
#endif

/*
 *	Close the socket module, report memory leaks and close the memory allocator
 */
	websCloseServer();
	socketClose();
#ifdef B_STATS
	memLeaks();
#endif
	bclose();
	return 0;
}

/******************************************************************************/

#ifdef WEBS_SSL_SUPPORT
/*
 *	Get the SSL version
 */

static int getSslVersion(void)
{
	FILE *fp;
	char buf[1024], *p;
	int len;

	if ((fp = fopen("/etc/ssl_version.conf", "r")) == NULL) {
		return (0);
	}

	while(fgets(buf, sizeof(buf) - 1, fp)) {
		if ((p = strtok(buf, " #\r\n"))) {
			if (!(len = strlen(p))) {
				continue;
			}
			if (!strcmp(p, "SSLv3") && len == 5) {
				fclose(fp);
				return(3);
			}
			if (!strcmp(p, "SSLv2") && len == 5) {
				fclose(fp);
				return(2);
			}
		}
	}
	fclose(fp);
	return (0);
}

#endif

/*
 *	Initialize the web server.
 */

static int initWebs()
{
	struct hostent	*hp;
	struct in_addr	intaddr;
	struct servent	*sv;
	char			host[128];
//	char			dir[128], webdir[128];
	char			*cp;
	char_t			wbuf[128];
	struct item_menu *item;

#ifdef SM100
	unsigned char buf[4] = {0};
	int arq;
	if ((arq = open("/dev/sm_atx", O_RDWR|O_NONBLOCK)) >= 0) {
		read(arq, buf, 1);
		buf[0] &= POWER_MODE_STATUS;
		close(arq);
	}

	for (item=menuitems; ; item++) {
		if (item->name == NULL) break;
		if (!strcmp(item->name, server_mgt)) {
			if (buf[0]) {
				/* IPMI Power control */
				item->link = pw_ipmi;
			}
			else {
				/* ATX Power control */
				item->link = pw_atx;
			}			
			break;
		}
	}
#endif

/*
 *	Get machine type
 */

	get_machine_type(&mt);

 /* 
  * Skip Analog/Digital menu option, if not TS110
  */
 
	if (mt.board_type != 18) {	// not BOARD_TS110
		int item_ctn;
		int total_menu_items = sizeof(menuitems)/sizeof(struct item_menu);
		for (item=menuitems, item_ctn=1; ; item++, item_ctn++) {
			if (item->name == NULL) break;
			if (!strcmp(item->name, io_ports)) {
				/* found menu option valid only for TS110, let's skip it */
				memcpy(item, item+1, ((total_menu_items - item_ctn) * sizeof(struct item_menu)));
				break;
			}
		}
	}

/*
 *	Initialize the socket subsystem
 */
	socketOpen();

#ifdef USER_MANAGEMENT_SUPPORT
/*
 *	Initialize the User Management database
 */
	umOpen();
	umRestore(T("/etc/websum.conf"));
#endif

/*
 *	Define the local Ip address, host name, default home page and the 
 *	root web directory.
 */
	if (gethostname(host, sizeof(host)) < 0) {
		error(E_L, E_LOG, T("Can't get hostname, 'localhost assumed..."));
		strcpy(host, "localhost");
	}
	if ((hp = gethostbyname(host)) == NULL) {
		error(E_L, E_LOG, T("Can't get host address, '127.0.0.1' assumed..."));
		intaddr.s_addr = 0x7f000001;
	} else {
		memcpy((char *) &intaddr, (char *) hp->h_addr_list[0],
			(size_t) hp->h_length);
	}

/*
 *	Set ../web as the root web. Modify this to suit your needs
 */
//	getcwd(dir, sizeof(dir)); 
//	if ((cp = strrchr(dir, '/'))) {
//		*cp = '\0';
//	}
//	sprintf(webdir, "%s/%s", dir, rootWeb);
//
/*
 *	Configure the web server options before opening the web server
 */
	websSetDefaultDir(rootWeb);
	cp = inet_ntoa(intaddr);
	ascToUni(wbuf, cp, min(strlen(cp) + 1, sizeof(wbuf)));
	websSetIpaddr(wbuf);
	ascToUni(wbuf, host, min(strlen(host) + 1, sizeof(wbuf)));
	websSetHost(wbuf);

/*
 *	Configure the web server options before opening the web server
 */
	websSetDefaultPage(T("default.asp"));
	websSetPassword(password);

/* 
 *	Open the web server on the given port. If that port is taken, try
 *	the next sequential port for up to "retries" attempts.
 */
	if ((sv = getservbyname("www", "tcp")) != NULL) {
		port = sv->s_port;
	}

	websOpenServer(port, retries);

/*
 * 	First create the URL handlers. Note: handlers are called in sorted order
 *	with the longest path handler examined first. Here we define the security 
 *	handler, forms handler and the default web page handler.
 */
	websUrlHandlerDefine(T(""), NULL, 0, websSecurityHandler, 
		WEBS_HANDLER_FIRST);
	websUrlHandlerDefine(T("/goform"), NULL, 0, websFormHandler, 0);
	websUrlHandlerDefine(T("/cgi-bin"), NULL, 0, websCgiHandler, 0);
	websUrlHandlerDefine(T(""), NULL, 0, websDefaultHandler, 
		WEBS_HANDLER_LAST); 

/*
 *	Now define two test procedures. Replace these with your application
 *	relevant ASP script procedures and form functions.
 */
	numsports = mt.nports;
#ifdef OEM1
	strcpy(PROG_RAS, mt.vendor_name);
	lowerCase(PROG_RAS);
	strcpy(&PROG_RAS[4], "_ras");
#endif
	DefineWebAspForm();
	DefineWebAspForm1();
	DefineWebAspForm2();
	DefineWebAspForm3();
	DefineWebAspForm4();
	DefineWebAspForm5();
	DefineWebAspForm6();
	DefineWebAspForm7();
#ifdef OEM1
    DefineWebAspForm20();
    DefineWebAspForm21();
#endif

/*
 *	Create the Form handlers for the User Management pages
 */
#ifdef USER_MANAGEMENT_SUPPORT
	formDefineUserMgmt();
#endif

/*
 *	Create a handler for the default home page
 */
	websUrlHandlerDefine(T("/"), NULL, 0, websHomePageHandler, 0); 
	return 0;
}

/******************************************************************************//*
 *	Home page handler
 */

static int websHomePageHandler(webs_t wp, char_t *urlPrefix, char_t *webDir,
	int arg, char_t *url, char_t *path, char_t *query)
{
/*
 *	If the empty or "/" URL is invoked, redirect default URLs to the home page
 */
	if (*url == '\0' || gstrcmp(url, T("/")) == 0) {
		websRedirect(wp, T("/home.asp"), NULL);
		return 1;
	}
	return 0;
}

/******************************************************************************/
/*
 *	Default error handler.  The developer should insert code to handle
 *	error messages in the desired manner.
 */

void defaultErrorHandler(int etype, char_t *msg)
{
#if 0
	write(1, msg, gstrlen(msg));
#endif
}

/******************************************************************************/
/*
 *	Trace log. Customize this function to log trace output
 */

void defaultTraceHandler(int level, char_t *buf)
{
/*
 *	The following code would write all trace regardless of level
 *	to stdout.
 */
#if 0
	if (buf) {
		write(1, buf, gstrlen(buf));
	}
#endif
}

/******************************************************************************/
/*
 *	Returns a pointer to an allocated qualified unique temporary file name.
 *	This filename must eventually be deleted with bfree();
 */

char_t *websGetCgiCommName()
{
	char_t	*pname1, *pname2;

	pname1 = tempnam(NULL, T("cgi"));
	pname2 = bstrdup(B_L, pname1);
	free(pname1);
	return pname2;
}

/******************************************************************************/
/*
 *	Launch the CGI process and return a handle to it.
 */

int websLaunchCgiProc(char_t *cgiPath, char_t **argp, char_t **envp,
					  char_t *stdIn, char_t *stdOut)
{
	int	pid, fdin, fdout, hstdin, hstdout, rc;

	fdin = fdout = hstdin = hstdout = rc = -1; 
	if ((fdin = open(stdIn, O_RDWR | O_CREAT, 0666)) < 0 ||
		(fdout = open(stdOut, O_RDWR | O_CREAT, 0666)) < 0 ||
		(hstdin = dup(0)) == -1 ||
		(hstdout = dup(1)) == -1 ||
		dup2(fdin, 0) == -1 ||
		dup2(fdout, 1) == -1) {
		goto DONE;
	}
		
 	rc = pid = fork();
 	if (pid == 0) {
/*
 *		if pid == 0, then we are in the child process
 */
		if (execve(cgiPath, argp, envp) == -1) {
			printf("content-type: text/html\n\n"
				"Execution of cgi process failed\n");
		}
		exit (0);
	} 

DONE:
	if (hstdout >= 0) {
		dup2(hstdout, 1);
	}
	close(hstdout);
	if (hstdin >= 0) {
		dup2(hstdin, 0);
	}
	close(hstdin);
	if (fdout >= 0) {
		close(fdout);
	}
	if (fdin >= 0) {
		close(fdin);
	}
	return rc;
}

/******************************************************************************/
/*
 *	Check the CGI process.  Return 0 if it does not exist; non 0 if it does.
 */

int websCheckCgiProc(int handle)
{
/*
 *	Check to see if the CGI child process has terminated or not yet.  
 */
	if (waitpid(handle, NULL, WNOHANG) == handle) {
		return 0;
	} else {
		return 1;
	}
}

/******************************************************************************/

#ifdef B_STATS
static void memLeaks() 
{
	int		fd;

	if ((fd = gopen(T("leak.txt"), O_CREAT | O_TRUNC | O_WRONLY, 0666)) >= 0) {
		bstats(fd, printMemStats);
		close(fd);
	}
}

/******************************************************************************/
/*
 *	Print memory usage / leaks
 */

static void printMemStats(int handle, char_t *fmt, ...)
{
	va_list		args;
	char_t		buf[256];

	va_start(args, fmt);
	vsprintf(buf, fmt, args);
	va_end(args);
	write(handle, buf, strlen(buf));
}
#endif

unsigned char websGetPriority(webs_t wp)
{
	unsigned char appl = 0;

	/* Check if the user is a regular user */
	char_t *username;
	if (username = websGetRequestUserName (wp)) {
		char_t *group;
		if (group = umGetUserGroup (username)) {
			appl = umGetGroupPrivilege (group);
		}
	}
	return appl;
}

#ifdef OEM1

void WriteStandardStart(webs_t wp,
						unsigned char flag,
						unsigned char *title,
						unsigned char *form,
						unsigned char *links,
						unsigned char *addheader,
						unsigned char *father)
{
	struct item_menu *item;
	unsigned char *pathsel = (father)?father:(unsigned char *)wp->path;
	unsigned char appl = websGetPriority(wp);
    /* Title */
	websWrite(wp, T("<html><head><title>Digi PortServer CM %d"), numsports);
	websWrite(wp, T("</title></head><link rel=\"stylesheet\" href=\"/normal_ws.css\" type=\"text/css\">"
					"<body leftmargin=\"0\" topmargin=\"0\" marginwidth=\"0\" marginheight=\"0\">"
					"<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" align=\"center\"><tr>"
					"<td class=\"mn\" colspan=\"2\"><img src=\"/banner.jpg\" width=\"762\" height=\"80\"></td>"
					"</tr><tr>"
					"<td class=\"mn\" width=\"120\"><table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">"));

	for (item=menuitems; ; item++) {
		if (item->name == NULL) break;
		if (appl < item->privilege) continue;
		if (item->link == NULL) {
			websWrite(wp, "<tr><td class=\"mn\" colspan=\"2\">"
					"<b class=\"it\">%s</b></td></tr>", item->name);
		} else {
			websWrite(wp, "<tr><td class=\"mn\" width=\"10\">&nbsp;</td><td class=\"mn\">"
				"<a class=\"%s\" href=\"%s\">%s</a></td></tr>", (strcmp(item->link, pathsel))?"is":"ic", item->link, item->name);
		}
	}
	websWrite(wp, "</table></td><td class=\"bd\" width=\"670\">");

    /* Links to the previous pages and form */
    if (links) websWrite (wp, links);
    if (form && *form) websWrite(wp, T("<form method=POST action=/goform/%s>"), form);

    /* Header *
    websWrite(wp, T("<br><center><h1>"));
    if (title && *title) {
	    websWrite(wp, title);
		if (addheader && *addheader) {
		    websWrite(wp, "<br>%s", addheader);
	    }
    }
    websWrite(wp, T("</h1></center>")); */
    websWrite(wp, T("<br>"));
}

void WriteStandardEnd(webs_t wp, unsigned char flag)
{
	websWrite (wp, "</td></tr></table>%s</body>",
		(flag & 2)?"</form>":"");
	websFooter(wp);
}

#endif

void websCycHdrInit(struct cyc_webs_header *hdr)
{
	memset(hdr, 0, sizeof(struct cyc_webs_header));
}

void websPageStart(webs_t wp, struct cyc_webs_header *hdr)
{
	unsigned char appl = websGetPriority(wp);
	struct item_menu *item;
	unsigned char *pathsel = (hdr->father)?hdr->father:
		(unsigned char *)wp->path;
	if (hdr->need_header) {
		 websHeader(wp, (char_t *)hdr->cookie);
	}
    	/* Title */
	websWrite(wp, T("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 "
		"Transitional//EN\">"
		"<html><head><title>"));
#ifdef OEM1
	websWrite(wp, T("Digi PortServer CM %d"), numsports);
#else
#ifdef SM100
	websWrite(wp, T("%s %s"), mt.prod_line, mt.board_name); 
#else
	websWrite(wp, T("%s-%s"), mt.vendor_name, mt.board_name); 
#endif
	if (hdr->title && *(hdr->title)) {
		websWrite(wp, T(" %s"), hdr->title);
	}
#endif
	websWrite(wp, T("</title>"));
	if (hdr->refresh_secs) {
		websWrite(wp, T("<meta http-equiv=\"refresh\" content=\"%d, "
			"url=%s\">"), hdr->refresh_secs, 
			(hdr->refresh_url) ? hdr->refresh_url : 
				(unsigned char *)wp->path);
	}
	websWrite(wp, T("</head>"
		"<link rel=\"stylesheet\" href=\"/normal_ws.css\" "
		"type=\"text/css\">"
                "<body leftmargin=\"0\" topmargin=\"0\" marginwidth=\"0\" "
                "marginheight=\"0\">"
                "<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" "
                "align=\"center\"><tr>"));
#ifdef OEM2
	websWrite(wp, T("<td class=\"mn1\" colspan=\"2\"><div align=\"center\">"
		"<img src=\"/banner.jpg\" width=\"300\" height=\"57\"></td>"));
#elif OEM1
	websWrite(wp, T("<td class=\"mn\" colspan=\"2\">"
		"<img src=\"/banner.jpg\" width=\"762\" height=\"80\"></td>"));
#else
	websWrite(wp, T("<td class=\"mn\" colspan=\"2\">"
    "<a href=\"/goform/MainPageTable\">"
		"<img src=\"/banner.jpg\" border=\"0\" width=\"800\" height=\"57\"></a></td>"));
#endif
	websWrite(wp, T("<tr><td class=\"mn\" width=\"168\">"
		"<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">"));
	for (item=menuitems; ; item++) {
		if (item->name == NULL) break;
		if (appl < item->privilege) continue;
		if (item->link == NULL) {
			websWrite(wp, "<tr><td class=\"mn\" colspan=\"2\">"
				"<b class=\"it\">%s</b></td></tr>", item->name);
		} else {
			websWrite(wp, "<tr><td class=\"mn\" width=\"10\">"
				"&nbsp;</td><td nowrap class=\"mn\">"
				"<a class=\"%s\" href=\"%s\">&nbsp;%s</a></td></tr>", 
				(strcmp(item->link, pathsel))?"is":"ic", 
				item->link, item->name);
		}
	}
	websWrite(wp, "</table></td>"
		"<td class=\"bd\" width=\"670\" height=\"690\">");

	/* Links to the previous pages and form */
	if (hdr->links) {
		websWrite (wp, hdr->links);
	}
	if (hdr->form && *(hdr->form)) {
		websWrite(wp, T("<form method=POST action=/goform/%s>"), 
			hdr->form);
	}
	websWrite(wp, T("<br>"));
}


void websPageEnd(webs_t wp, struct cyc_webs_header *hdr)
{
	websWrite (wp, "<center><p><b class=\"mn\">");
	websWrite (wp, "NOTE: After submitting changes, remember to click on<br>
            Run Configuration to make the changes active. 
            To save the <br>configuration to flash, click on Load/Save Config. </b></p></center>");
	websWrite (wp, "</td></tr></table>%s</body>",
		(hdr->form && *(hdr->form))?"</form>":"");
	websFooter(wp);
	if (hdr->need_header) {
		 websDone(wp, 200);
	}
}

void websPageNoNoteEnd(webs_t wp, struct cyc_webs_header *hdr)
{
  websWrite (wp, "</td></tr></table>%s</body>",
    (hdr->form && *(hdr->form))?"</form>":"");
  websFooter(wp);
  if (hdr->need_header) {
     websDone(wp, 200);
  }
}

void websPageMessage(webs_t wp, struct cyc_webs_header *hdr,
	unsigned char *message, unsigned char error)
{
	hdr->need_header = 1;
	websPageStart(wp, hdr);
	websWrite(wp, "<center><table border align=center>"
		"<tr><th class=\"tt\">%s</th></tr>"
		"<tr><td><pre%s>%s</pre></td></tr>"
		"</table></center>",
		hdr->title, (error)?" class=\"err\"":"", message);
//[RK]Oct/01/04 - bug 2625	websPageEnd(wp, hdr);
	websPageNoNoteEnd(wp,hdr);
}

static void MainPageTable(webs_t wp, char_t *path, char_t *query)
{
	struct cyc_webs_header hdr;
	struct item_menu *item;
	unsigned char appl = 0;

	/* Check if the user is a regular user */
	char_t *username;
	if (username = websGetRequestUserName (wp)) {
		char_t *group;
		if (group = umGetUserGroup (username)) {
			appl = umGetGroupPrivilege (group);
		}
	}
	
	websCycHdrInit(&hdr);
	hdr.need_header = 1;
	hdr.title = "Web Management Service";
	websPageStart(wp, &hdr);
	websWrite(wp, "<center><table border align=\"center\">");
	for (item=menuitems; ; item++) {
		if (item->name == NULL) break;
		if (appl < item->privilege) continue;
		if (item->link == NULL) {
			websWrite(wp, "<tr><th class=\"tt\" colspan=\"2\">%s"
				"</th></tr>", item->name);
			websWrite(wp, "<tr><td colspan=\"2\"><b>%s</b>"
				"</td></tr>", item->comments);
		} else {
			websWrite(wp, "<tr><td><a href=\"%s\">%s</a></td>"
				"<td>%s</td></tr>",
				item->link, item->name, item->comments);
		}
	}
	websWrite(wp, "</table></center>");
	websPageNoNoteEnd(wp, &hdr);
}

static int PrintHeader(int eid, webs_t wp, int argc, char_t **argv)
{
	unsigned char *title, *form, *links, *father, nargs;
	struct cyc_webs_header hdr;
	nargs = ejArgs(argc, argv, T("%s %s %s %s"), 
		&title, &form, &links, &father);
	if (nargs < 3) {
		links = NULL;
		father = NULL;
	}
	websCycHdrInit(&hdr);
	hdr.title = title;
	hdr.form = form;
	hdr.links = links;
	hdr.father = father;
	websPageStart(wp, &hdr);
	return 0;
}

static int PrintFooter(int eid, webs_t wp, int argc, char_t **argv)
{
	unsigned char *form = NULL;
	unsigned char *note = NULL;
	struct cyc_webs_header hdr;
	ejArgs(argc, argv, T("%s %s"), &form, &note);
	websCycHdrInit(&hdr);
	hdr.form = form;

	if (atoi(note))
		websPageEnd(wp, &hdr);
	else
		websPageNoNoteEnd(wp, &hdr);
	return 0;
}

#define TXT_BUFFER_SIZE	128
static int websFillBuffer(webs_t wp, unsigned char *buffer, unsigned char *str)
{
	int count = 0;
	if (strlen(buffer) + strlen(str) >= TXT_BUFFER_SIZE) {
		count = websWrite(wp, buffer);
		*buffer = 0;
	}
	strcat (buffer, str);
	return count;
}

int websWriteTextFile(webs_t wp, FILE *arq, char flag, int *bcount)
{
        unsigned char ch;
	unsigned char chstr[2];
	unsigned char buffer[TXT_BUFFER_SIZE];
        int count = 0;
        if (arq == NULL) return 0;
        if (bcount) *bcount = 0;
	*buffer = 0;
	chstr[1] = 0;
        while (!feof(arq)) {
                ch = fgetc(arq);
                if (ch == (unsigned char)EOF) break;
                switch (ch) {
                        case '%':
                                count += websFillBuffer(wp, buffer, "%%");
                                break;
                        case '<':
                                count += websFillBuffer(wp, buffer, "&lt;");
                                break;
                        case '>':
                                count += websFillBuffer(wp, buffer, "&gt;");
                                break;
                        case '&':
                                count += websFillBuffer(wp, buffer, "&amp;");
                                break;
                        case ' ':
                                if (flag) {
                                        count += websFillBuffer(wp, buffer, 
						"&nbsp;");
                                        break;
                                }
                        case '\n':
                                if (flag) {
                                        count += websFillBuffer(wp, buffer, 
						"<br>");
                                        break;
                                }
                        default:
                                if ((ch < 0x20 || ch > 0x7e) && ch != '\t' &&
                                        ch != '\r' && ch != '\n') {
                                        if (bcount) *bcount += 1;
                                } else {
					chstr[0] = ch;
                                        count += websFillBuffer(wp, buffer, 
						chstr);
                                }
                                break;
                }
        }
	if (*buffer) {
		count += websWrite(wp, buffer);
	}
        return count;
}



//int getURL(unsigned char *link, unsigned char *url)
//{
//	unsigned char *accessLimit;
//	int secure = 0;
//	accessLimit = umGetAccessLimit(link);
//	if (accessLimit != NULL) {
//		secure = umGetAccessLimitSecure(accessLimit);
//	}
//	sprintf(url, "http%s://%s%s", (secure)?"s":"", websIpaddr, link);
//}
//
//static int sethttp(int eid, webs_t wp, int argc, char_t **argv)
//{
//	unsigned char *link, url[64];
//	unsigned char *accessLimit;
//	int secure = 0;
//		
//	ejArgs(argc, argv, T("%s"), &link);
//	getURL(link, url);
//	websWrite(wp, url);
//	return 0;
//}

static int GetModel(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef OEM1
	websWrite(wp, T("Digi PortServer CM %d"), numsports);
#else
    websWrite(wp, T("%s"), mt.board_name);
#endif
}

static int GetProdLine(int eid, webs_t wp, int argc, char_t **argv)
{
#ifdef SM100
    websWrite(wp, T("%s %s"), mt.prod_line, mt.board_name);
#else
    websWrite(wp, T("%s-%s"), mt.vendor_name, mt.board_name);
#endif
}

static int GetNPorts(int eid, webs_t wp, int argc, char_t **argv)
{
    websWrite(wp, T("%d"), numsports);
}

static char files_gif[19][8] = {
	"unknown",
	"unknown",
	"unknown",
	"unknown",
#ifdef OEM1
	"dgcm16",
	"dgcm32",
	"dgcm4",
	"dgcm8",
	"dgcm48",
#else
	"ts1000",
	"ts2000",
	"ts400",
	"ts800",
	"ts3000",
#endif
	"unknown",
	"unknown",
	"pl1000",
	"ts100",
	"nl1000",
	"ts1100",
	"ts2100",
	"sm100",
	"unknown",
	"ts110"
};

static int GetPictureFile(int eid, webs_t wp, int argc, char_t **argv)
{
    char *aux;

    if (mt.board_type >= sizeof(files_gif)/8) {
		aux = files_gif[0];
	} else {
		aux = files_gif[mt.board_type];
	}
    websWrite(wp, T("%s.gif"), aux);
}

static int GetHostName(int eid, webs_t wp, int argc, char_t **argv)
{
	FILE *arq;
    char conf_hostname[40];
    if ((arq = fopen("/etc/hostname", "r")) == NULL) return 0;
    fgets(conf_hostname, 40L, arq);
    websWrite(wp, conf_hostname);
    fclose(arq);
	return 0;
}

static int GetSWVersion(int eid, webs_t wp, int argc, char_t **argv)
{
	websWrite(wp, mt.version);
	return 0;
}

static void DefineWebAspForm()
{
    websAspDefine(T("PrintHeader"),    PrintHeader);
    websAspDefine(T("PrintFooter"),    PrintFooter);
//    websAspDefine(T("sethttp"),      sethttp);
    websAspDefine(T("GetModel"),       GetModel);
    websAspDefine(T("GetProdLine"),    GetProdLine);
    websAspDefine(T("GetNPorts"),      GetNPorts);
    websAspDefine(T("GetPictureFile"), GetPictureFile);
    websAspDefine(T("GetHostName"),    GetHostName);
    websAspDefine(T("GetSWVersion"),   GetSWVersion);
    websFormDefine(T("MainPageTable"), MainPageTable);
}

void lowerCase(char *z) {

    while(*z != '\0') {
        *z = tolower(*z);
        ++z;
    }
}

